Package: kutils
Type: Package
Title: Project Management Tools
Version: 1.73
Date: 2023-09-17
Encoding: UTF-8
Authors@R: c(person("Paul","Johnson",email="pauljohn@ku.edu",role = c("aut", "cre")),
		   person("Benjamin","Kite",email="bakite@ku.edu",role="aut"),
		   person("Charles","Redmon",email="redmon@ku.edu",role="aut"),
		   person("Jared","Harpole",email="jared.harpole@gmail.com", role="ctb"),
  		   person("Kenna","Whitley",role="ctb"),
		   person("Po-Yi","Chen",role="ctb"), 
                   person("Shadi", "Pirhosseinloo",role="ctb")) 
Maintainer: Paul Johnson <pauljohn@ku.edu>
Description: Tools for data importation, recoding, and inspection.
    There are functions to create new project
    folders, R code templates, create uniquely named output
    directories, and to quickly obtain a visual summary for each
    variable in a data frame.  The main feature here is the systematic
    implementation of the "variable key" framework for data
    importation and recoding.  We are eager to have community feedback
    about the variable key and the vignette about it. In version 1.7,
    the function 'semTable' is removed. It was deprecated since 1.67.
    That is provided in a separate package, 'semTable'.
License: GPL-2
Depends: R (>= 3.3.0)
Imports: stats, utils, methods, foreign, xtable, plyr, openxlsx, RUnit
Suggests: rockchalk
RoxygenNote: 7.2.3
LazyData: TRUE
NeedsCompilation: no
Packaged: 2023-09-17 14:13:02 UTC; pauljohn
Author: Paul Johnson [aut, cre],
  Benjamin Kite [aut],
  Charles Redmon [aut],
  Jared Harpole [ctb],
  Kenna Whitley [ctb],
  Po-Yi Chen [ctb],
  Shadi Pirhosseinloo [ctb]
Repository: CRAN
Date/Publication: 2023-09-17 15:50:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(all.equal,key)
S3method(all.equal,keylong)
S3method(print,keyDiff)
S3method(print,keycheck)
S3method(print,likert)
export(alphaOnly)
export(assignMissing)
export(assignRecode)
export(checkCoercion)
export(cleanDataFrame)
export(colnamesReplace)
export(deduper)
export(deleteBogusColumns)
export(deleteBogusRows)
export(dev.create)
export(dir.create.unique)
export(dms)
export(dts)
export(escape)
export(file.backup)
export(importQualtrics)
export(initProject)
export(is.data.frame.simple)
export(isNA)
export(keyApply)
export(keyCheck)
export(keyDiff)
export(keyImport)
export(keyLookup)
export(keyRead)
export(keySave)
export(keyTemplate)
export(keyTemplateSPSS)
export(keyTemplateStata)
export(keyUpdate)
export(keysPool)
export(keysPoolCheck)
export(likert)
export(long2wide)
export(mergeCheck)
export(mgsub)
export(modifyVector)
export(n2NA)
export(padW0)
export(peek)
export(qualtricsBlockStack)
export(removeMatches)
export(reverse)
export(safeInteger)
export(shorten)
export(starsig)
export(stringbreak)
export(truncsmart)
export(updatePackages)
export(varlabTemplate)
export(wide2long)
export(writeCSV)
export(zapspace)
import(datasets)
importFrom(foreign,read.dta)
importFrom(foreign,read.spss)
importFrom(grDevices,dev.off)
importFrom(grDevices,devAskNewPage)
importFrom(grDevices,pdf)
importFrom(graphics,barplot.default)
importFrom(graphics,hist.default)
importFrom(graphics,par)
importFrom(graphics,strwidth)
importFrom(methods,as)
importFrom(openxlsx,addWorksheet)
importFrom(openxlsx,getSheetNames)
importFrom(openxlsx,read.xlsx)
importFrom(openxlsx,readWorkbook)
importFrom(openxlsx,saveWorkbook)
importFrom(openxlsx,writeDataTable)
importFrom(plyr,mapvalues)
importFrom(plyr,rbind.fill)
importFrom(stats,na.omit)
importFrom(stats,reshape)
importFrom(tools,file_ext)
importFrom(tools,file_path_sans_ext)
importFrom(tools,package_dependencies)
importFrom(utils,available.packages)
importFrom(utils,head)
importFrom(utils,install.packages)
importFrom(utils,installed.packages)
importFrom(utils,modifyList)
importFrom(utils,packageDescription)
importFrom(utils,read.csv)
importFrom(utils,type.convert)
importFrom(utils,update.packages)
importFrom(utils,write.csv)
importFrom(utils,write.table)
importFrom(xtable,print.xtable)
importFrom(xtable,xtable)

Paul Johnson
20171005


mydf.templ.csv: generated from keyTemplate
mydf.key.csv: manual revision of mydf.templ.csv

mydf.key_long.csv: a long version of a key that is supposed to be
equivalent to mydf.key.csv.  Because of mis-matches, we use
wide2long() to generate this file.

mydf.key_long2.csv: a manually edited version of mydf.key_long.csv
that has additional variables.

library(kutils)

mydf.path <-  system.file("extdata", "mydf.csv", package = "kutils")
mydf.key.path <-  system.file("extdata", "mydf.key_new.csv", package = "kutils")
mydf.keylong.path <- system.file("extdata", "mydfkey.long_new.csv", package = "kutils")

mydf.keylist1 <- keyImport(mydf.key.path)

mydf.key <- read.csv(mydf.key.path, stringsAsFactors = FALSE)
mydf.keylist2 <- keyImport(mydf.key, long = FALSE)

identical(mydf.keylist1, mydf.keylist2)


dframe <- read.csv(mydf.path, stringsAsFactors = FALSE)

keyApply(dframe, mydf.keylist1)



klistlong1 <- keyImport(mydf.keylong.path, long = TRUE)

keylong2 <- read.csv(mydf.keylong.path, stringsAsFactors = FALSE)
klistlong2 <- keyImport(keylong2, long = TRUE)


identical(klistlong1, klistlong2)


dfnew <- keyApply(dframe = dframe, keylist = klistlong2)

dfnew2 <- keyApply(dframe = dframe, keylist = klistlong2, diagnostic = TRUE)


debug(keyApply)

library(kutils)
mydf.key.path <- system.file("extdata", "mydf.key.csv", package = "kutils")
mydf.key <-  keyImport(mydf.key.path)
mydf.path <- system.file("extdata", "mydf.csv", package = "kutils")

## Use if testing new functions in R code
## assignInNamespace("makeKeylist", kutils:::makeKeylist, "kutils")
## assignInNamespace("keyApply", keyApply, "kutils")

mydf <- read.csv(mydf.path, stringsAsFactors = FALSE)

## insert variable not in key
mydf$g1 <- 2 * mydf$x6
mydf2a <- keyApply(mydf, mydf.key, drop = FALSE)
mydf2b <- keyApply(mydf, mydf.key, drop = TRUE)

## erase some value_new in key
mydf.key.long <- wide2long(mydf.key)
mydf.key.long2 <- mydf.key.long
mydf.key.long2["x1.x1.3", "value_new"] <- "."
mydf.key.2 <- long2wide(mydf.key.long2)

mydf3a.wide <- keyApply(mydf, mydf.key.2, drop = FALSE)
mydf3b.wide <- keyApply(mydf, mydf.key.2, drop = TRUE)
## Should be equal, except for presence of column g1 in mydf3a.long
for(jj in colnames(mydf3b.long)) print(all.equal(mydf3a.wide[ , jj], mydf3b.wide[ , jj]))

mydf3a.long <- keyApply(mydf, mydf.key.long, drop = FALSE)
mydf3b.long <- keyApply(mydf, mydf.key.long, drop = TRUE)
## Should be equal, except for presence of column g1 in mydf3a.long
for(jj in colnames(mydf3b.long)) print(all.equal(mydf3a.long[ , jj], mydf3b.long[ , jj]))
all.equal(mydf3a.long, mydf3a.wide)
all.equal(mydf3b.long, mydf3b.wide)

## Now omit that value from both value_new and value_old
mydf.key.long.3 <- mydf.key.long[-match("x1.x1.3", rownames(mydf.key.long)), ]
mydf.key.3 <- long2wide(mydf.key.long.3)

mydf3a <- keyApply(mydf, mydf.key, drop = FALSE)
mydf3b <- keyApply(mydf, mydf.key, drop = TRUE)


## erase same line from key


## Creates more transformations
mydf.key.3 <-  keyImport("mydf2.csv")
mydf3 <- keyApply(mydf, mydf.key.3, drop = FALSE)
mydf3 <- keyApply(mydf, mydf.key.3, drop = c("vals"))

## Re-read data, 
mydf$x1 <- factor(mydf$x1)

mydf3 <- keyApply(mydf, mydf2.key, drop = FALSE)

mydf3 <- keyApply(mydf, mydf2.key, drop = TRUE)
mydf3 <- keyApply(mydf, mydf2.key, drop = "vars")
mydf3 <- keyApply(mydf, mydf2.key, drop = "vals")
## Should fail:
mydf3 <- keyApply(mydf, mydf2.key, drop = "typo")

##' Remove rows in which the proportion of missing data exceeds
##' a threshold.
##'
##' If cases are mostly missing, delete them. It often happens that
##' when data is imported from other sources, some noise rows exist at
##' the bottom of the input.  Anything that is missing in more than,
##' say, 90\% of cases is probably useless information.  We invented
##' this to deal with problem that MS Excel users often include a
##' marginal note at the bottom of a spread sheet.
##'
##' @param dframe A data frame or matrix
##' @param pm "proportion missing data" to be tolerated.
##' @param drop Default FALSE: if data frame result is reduced to one
##'     row, should R's default drop behavior "demote" this to a
##'     column vector.
##' @param verbose Default TRUE. Should a report be printed
##'     summarizing information to be delted?
##' @param n Default 25: limit on number of values to print in verbose
##'     diagnostic output. If set to NULL or NA, then all of the
##'     column values will be printed for the bogus rows.
##' @return a data frame, invisibly
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' mymat <- matrix(rnorm(10*100), nrow = 10, ncol = 100,
##'                dimnames = list(1:10, paste0("x", 1:100)))
##' mymat <- rbind(mymat, c(32, rep(NA, 99)))
##' mymat2 <- deleteBogusRows(mymat)
##' mydf <- as.data.frame(mymat)
##' mydf$someFactor <- factor(sample(c("A", "B"), size = NROW(mydf), replace = TRUE))
##' mydf2 <- deleteBogusRows(mydf, n = "all")
deleteBogusRows <- function (dframe, pm = 0.9, drop = FALSE,
                             verbose = TRUE, n = 25){
    if (is.null(n) || is.na(n)) n <- NCOL(dframe)
    if (n > NCOL(dframe)) n <- NCOL(dframe)
    rowna <- apply(dframe, 1, function(x){sum(is.na(x))})
    badrows <- rowna > pm * NCOL(dframe)
    if (any(badrows)){
        if (verbose){
            cat(paste("deleteBogusRows Diagnostic\n"))
            cat(paste("These rows from the data frame: ",
                      deparse(substitute(dframe)), "\n are being purged: "))
            cat(paste(which(badrows), "\n"))
            cat(paste("The bad content was\n"))
            print(dframe[badrows, 1:n, drop = drop])
        }
        dframe <- dframe[!badrows, , drop = drop]
        return(dframe)
    }
    print(paste("No bogus rows were found in: ", deparse(substitute(dframe))))
    invisible(dframe)
}


##' Remove columns in which the proportion of missing data exceeds
##' a threshold.
##'
##' This is a column version of \code{deleteBogusRows}. Use the pm
##' argument to set the proportion of missing required before a column
##' is flagged for deletion
##' 
##' @param dframe A data frame or matrix
##' @param pm "proportion missing data" to be tolerated.
##' @param drop Default FALSE: if data frame result is reduced to one
##'     column, should R's default drop behavior "demote" this to a
##'     column vector.
##' @param verbose Default TRUE. Should a report be printed
##'     summarizing information to be delted?
##' @param n Default 25: limit on number of values to print in
##'     diagnostic output. If set to NULL or NA, then all of the
##'     column values will be printed for the bogus rows.
##' @return a data frame, invisibly
##' @seealso \code{deleteBogusRows}
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
deleteBogusColumns <- function (dframe, pm = 0.9, drop = FALSE,
                                verbose = TRUE, n = 25)
{
    if (is.null(n) || is.na(n)) n <- NROW(dframe)
    if (n > NROW(dframe)) n <- NROW(dframe)
    colna <- apply(dframe, 2, function(x){sum(is.na(x))})
    badcols <- colna > pm * NROW(dframe)
    if (any(badcols)){
        if (verbose){
            cat(paste("deleteBogusColumns Diagnostic\n"))
            cat(paste("These columns from the data frame: ", deparse(substitute(dframe)), "\n are being purged: "))
            cat(paste(colnames(dframe)[badcols], "\n"))
            cat(paste("The bad content was:\n"))
            print(dframe[1:n, badcols, drop = drop])
        }
        newdframe <- dframe[, !badcols, drop = drop]
        return(newdframe)
    }
    print(paste("No bogus columns were found in: ", deparse(substitute(dframe))))
    invisible(dframe)
}


## TODO: make deleteBogus a generic, then write a method for data
## frames and a method for data tables.

##' Create project directories, initialize a git repo, create README.md
##' ChangeLog, and R template file in R directory
##'
##' This creates folders for the separate parts of a project. It tries
##' to be clever about which directories are created and where they are
##' placed. Please see details for 3 scenarios for which we have
##' planned. If a directory already exists, it will not be damaged or
##' re-created.
##'
##' If the dir argument is NULL, as default, then the current working
##' directory will be the place where new directories and the git
##' repository will be created.  Assuming the current working
##' directory's base name is not "R", then folders named "R", "data",
##' and so forth will be created in the current working directory.
##' 
##' If one has a current R working directory with a basename "R"
##' (suppose it is \code{"/tmp/whatever/R"}), and the user runs
##' \code{initProject()}, something different happens. The function
##' assumes we don't want to create subdirectories inside R. We don't
##' want to end up with \code{"/tmp/whatever/R/R"}. We don't
##' want \code{"/tmp/whatever/R/data"} either.  Instead, it assumes we
##' want the new directories created on same level as R, so it creates
##' \code{"/tmp/whatever/data"}, \code{"/tmp/whatever/workingdata"},
##' and so forth.  From within the R directory, these new directories
##' are seen as \code{"../data"}, \code{"../workingdata"}, and so
##' forth. That is, we should end up with directories and a git repo
##' in \code{"/tmp/whatever"}.
##'
##' If the \code{dir} argument is provided by the user, then that is
##' used as the folder in which directories \code{"R"}, \code{"data"},
##' \code{"workingdate"}, and so forth are created.  All materials are
##' created in \code{dir}, no matter what the current working
##' directory is named (even if it is \code{"R"}).
##'
##' The examples demonstrate all three of these scenarios.
##' @param dir Default NULL, otherwise a legal directory name to serve
##'     as the top level directory for this project
##' @param ddir Data directory, place where "read only" unadjusted
##'     data files are kept. Default is "data". If user sets it as NA
##'     or NULL, the directory will not be created.
##' @param wdir Working data directory, where recorded, revised, and
##'     cleaned data files are kept. Default is "workingdata".
##' @param odir Output directory. Default is "output".
##' @param tdir Temporary directory, where trash files can be kept for
##'     safe keeping. Default is "tmp".
##' @param ldir Literature directory, where material about the project
##'     can be stored. Default is "lit".
##' @param writedir The folder where the project writeup will be
##'     stored. Default is "writeup".
##' @param rdir The name to be used for the R files. Defaults to "R".
##' @param ... A list of other directories that the user would like to
##'     create. For example, \code{adir = "admin"}, \code{cdir =
##'     "client_provided"}, \code{bdir = "codebooks"}, \code{sdir =
##'     "Stata"}, \code{mdir = "Mplus"}. These may be grouped in a
##'     named vector or list, if user convenience dictates.
##' @param gitArgs This function tries to run "git init" and in our
##'     center we add "--shared=group" on a network file server. If
##'     that is undesirable in a user's context, put the argument
##'     gitArgs as "".
##' @importFrom stats na.omit
##' @export
##' @return Name of project top level directory. Leaves the R
##'     working directory unchanged.
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' projdir1 <- file.path(tempdir(), "test1")
##' dir.create(projdir1, recursive = TRUE)
##' initProject(dir = projdir1)
##' list.files(projdir1, all.files = TRUE)
##' projdir2 <- file.path(tempdir(), "test2")
##' dir.create(projdir2, recursive = TRUE)
##' ## demonstrate ability to create other directories
##' initProject(dir = projdir2, admin = "admin", clientfiles = "client")
##' list.files(projdir2, all.files = TRUE)
##' ## demonstrate ability to nullify standard directories
##' projdir3 <- file.path(tempdir(), "test3")
##' dir.create(projdir3, recursive = TRUE)
##' initProject(projdir3, odir = NA, tdir = NA, writedir = NA)
##' list.files(projdir3, all.files = TRUE)
##' unlink(c("projdir1", "projdir2", "projdir3"), recursive = TRUE)
initProject <- function(dir = NULL, ddir = "data",
                    wdir = "workingdata", odir = "output",
                    tdir = "tmp", ldir = "lit",
                    writedir = "writeup",
                    rdir = "R", ..., gitArgs = "--shared=group")
{
    wd <- getwd()

    dots <- unlist(list(...))
    
    dirs <- c(ddir, wdir, odir, tdir, ldir, writedir)
    if (length(dots > 0)){
        dirs <- c(dirs, dots)
    }
    dirs <- na.omit(dirs)
    
    ## Only create rdir if dir NULL or not now in "R"
    if (!is.null(dir)){
        dirs <- c(dirs, rdir)
        dirs <- paste(dir, dirs, sep = "/")
        if (!file.exists(dts(dir))) dir.create(dts(dir), recursive = TRUE)
    } else {
        ## is name of topdir for records
        if (basename(wd) == "R"){
            dirbase <- basename(dirname(wd))
            dir <- normalizePath("..")
            dirs <- paste0("../", dirs)
        } else {
            dirbase <- basename(wd)
            dir <- normalizePath(".")
            dirs <- c(dirs, rdir)
        }
    }
    
    for (i in dirs){
        if (!file.exists(i)) {
            dir.create(i, recursive = TRUE)
            messg <- paste("Creating:", i, "\n")
            cat(messg)
        }
    }

    setwd(dir)
    sysinfo <- Sys.info()
    systime <- Sys.time()

    fileheader <- paste0("## ", sysinfo[["user"]], "  \n", "##", format(systime,"%Y%m%d  \n"))
    cat(fileheader, file = paste("README.md"))

    ## TODO 20161207: Think harder on following, which assumes
    ## all of those directories have not been set at NA by user.
    rheader <- paste0(fileheader, 
                      paste0("\n\nwdir <- \"../", wdir, "\"\n",
                             "ddir <- \"../", ddir, "\"\n",
                             "odir <- \"../", odir, "\"\n",
                             "tdir <- \"../", tdir, "\"\n\n",
                             "library(kutils)\n\n",
                             "pdf.options(onefile=FALSE, family=\"Times\", paper=\"special\", height=4,\n",
                             "            width=6, pointsize=10)\n"))
                             
    if (!is.na(rdir)){
        cat(rheader,
            file = paste0(rdir, "/template.R"))
    }
    changelog <- paste(format(systime, "%Y-%m-%d"), sysinfo[["user"]], "\n\n")
    changelog <- paste0(changelog, "\t* " , dir, "(initProject):\n")
    cat(changelog, file = paste("ChangeLog"))

    git <- Sys.which("git")
    if (git != ""){
        makeGit <- paste("git init", gitArgs)
        gitout <- system(makeGit, intern = TRUE)
        
        messg1 <- "git add README.md ChangeLog"
        log1 <- system(messg1, intern = TRUE)
        messg2 <- paste("git commit -a -m \"Initialized project in", dir, "\"")
        log2 <- system(messg2, intern = TRUE)
        messg3 <- "Please consider creating a remote repository to which this repo should be linked"
        
        cat(gitout, log1, log2, messg3, fill = TRUE)
    } else {
        messg4 <- paste("The git executable was not found.",
                        "Thus, a git repo was not created in the working directory.",
                        "Please install put git's executable in your path.")
        cat(messg4)
    }
    ## Reset user's working directory
    setwd(wd)
    dir
}

    

##' Create a uniquely named directory. Appends number & optionally date to directory
##' name.
##' 
##' Checks if the requested directory exists. If so, will create new
##' directory name. My favorite method is to have the target directory
##' with a date-based subdirectory, but set usedate as FALSE if you
##' don't like that. Arguments showWarnings, recursive, and mode are
##' passed along to R's dir.create, which does the actual work here.
##'
##' Default response to dir = "../output/" fixes the directory name
##' like this, "../output/20151118-1/" because usedate is assumed
##' TRUE. If usedate = FALSE, then output names will be like
##' "../output-1/", "../output-2/", and so forth.
##' @param path A character string for the base name of the directory.
##' @param usedate TRUE or FALSE: Insert YYYYMMDD information?
##' @param showWarnings default TRUE. Show warnings? Will be passed on
##'     to dir.create
##' @param recursive default TRUE. Will be passed on to dir.create
##' @param mode Default permissions on unix-alike systems. Will be
##'     passed on to dir.create
##' @export
##' @return a character string with the directory name
##' @author Paul E Johnson <pauljohn@@ku.edu>
dir.create.unique <- function(path, usedate = TRUE,
                              showWarnings = TRUE,
                              recursive = TRUE,
                              mode = "0777")
{
    dts <- function(name) gsub("/$", "", name)
    if (!file.exists(dts(path))){
        res <- dir.create(path, recursive = recursive, showWarnings = showWarnings,
                          mode = mode)
        if (!isTRUE(res)) {
            messg <- paste("dir.create.unique: directory", path, "could not be created")
            stop(messg)
        } 
    } else {
        today <- format(Sys.time(), "%Y%m%d")
        j <- 1
        ocandidate <- paste0(dts(path),
                             if(usedate) paste0("/", today), "-",  j)
        while (file.exists(dts(ocandidate))) {
            j <- j + 1
            ocandidate <- paste0(dts(path),
                                 if (usedate) paste0("/", today), "-",  j)
        }
        path <- ocandidate
        res <- dir.create(path, recursive = recursive, showWarnings = showWarnings,
                          mode = mode)
        if (!isTRUE(res)) {
            messg <- paste("dir.create.unique: directory", path, "could not be created")
            stop(messg)
        } 
    }
    return(paste0(dts(path), "/"))
}

##' Import Qualtrics survey files, apply clean column names
##'
##' Defaults are based on most common format received from Qualtrics
##' downloads to CSV or XLSX (MS Excel) formats.  We assume that the
##' file has the column names in row 1 and that 3 rows are skipped
##' before the real data begins. If the parameter questrow is used, it
##' designates a row that is interpreted as the survey questions
##' themselves.  Often, this is in row 2.
##' @param file file name (including path if in another directory) of
##'     a CSV or XLSX file from Qualtrics.
##' @param namerow Row number for variable names.  Default 1, the
##'     information to be used as column names (same as HEADER row in
##'     R's read.table function)
##' @param questionrow Row number to be treated as the questions in the
##'     survey. Default is 2. If questions do not seem to be present
##'     in this row, there will be a warning.
##' @param importidrow Row number to be treated as Qualtrics meta
##'     data. Default is 3. Many CSV created by Qualtrics will have
##'     row 3 with a character string such as
##'     \code{"{""ImportId"":""QID1303_4""}"}.  If importids are not
##'     present in this row, there will be a warning.
##' @param skip Number of rows that are meta data. Current Qualtrics
##'     CSV files will usually have 3 metadata rows, 1 = name, 2 =
##'     question, 3 = ImportId. This function will try to guess how
##'     many rows of metadata are present. skip should be at least
##'     as large as \code{max(namerow, questions, and importids)}
##' @param dropTEXT Default TRUE, columns ending in "_TEXT" are
##'     omitted.
##' @param stringsAsFactors Default FALSE, same meaning as R's
##'     read.csv. Does not affect importation of Excel files.
##' @importFrom openxlsx read.xlsx
##' @importFrom utils type.convert
##' @export
##' @return Data frame that has attribute "meta"
##' @author Paul Johnson <pauljohn@@ku.edu>
importQualtrics <- function (file, namerow = 1, questionrow = 2,
                             importidrow = 3, skip = 3, dropTEXT = TRUE,
                             stringsAsFactors = FALSE){
    if (length(grep("csv$", tolower(file))) > 0) {
        dat1 <- read.csv(file, allowEscapes=TRUE, fill=TRUE,
                         header = FALSE, stringsAsFactors = stringsAsFactors)
 
    } else {
       dat1 <- read.xlsx(file, skipEmptyCols = FALSE,
                          colNames = FALSE)
    }

    importrow <- grep("\\{\"ImportId", dat1[ , 1])
    if((length(importrow) > 0)){
        if (importrow != importidrow){
            MESSG <- paste("ImportId information appears to be in row", importrow, ".\n")
            warning(MESSG)
            importidrow <- importrow
        } else {
            if (skip < importrow) skip <- importrow
        }
    } else {
        MESSG <- paste("Data does not include an ImportId metadata line.\n",
                       "We are omitting", skip, "rows, but this may throw\n",
                       "away some data that you want. Check your input,\n",
                       "re-specify the skip  parameter")
        warning(MESSG)
    }

    if (dropTEXT){
        ## Remove columns with variable names ending in "_TEXT"
        dat1[ , grep("TEXT$", dat1[namerow, ])] <- NULL
    } 
    
    meta <- as.data.frame(t(dat1[1:skip, ]), stringsAsFactors = FALSE)
    colnames(meta) <- paste0("row", 1:skip)
    colnames(meta)[namerow] <- "name"
    colnames(meta)[questionrow] <- "question"
    colnames(meta)[importidrow] <- "ImportId.orig"
    meta$name.clean <- make.names(meta[ , "name"], unique = TRUE)
    colnames(dat1) <- meta$name.clean
    meta$col.number <- 1:NROW(meta)
    if((length(importrow) > 0)){
    ## could not find 1 regex to handle whole problem
        meta$importid <- gsub("\\{\"(.*)\":\"(.*)\".*\\}", "\\2",
                              gsub("(.*),(.*)", "\\1}", meta[ , "ImportId.orig"]))
    }
    
    d2.1 <- dat1[ -(1:skip), ]
    dat2 <- as.data.frame(lapply(d2.1, type.convert, as.is = TRUE), stringsAsFactors = stringsAsFactors)
                
    if(NROW(dat2) != NROW(dat1) - skip){
        MESSG2 <- paste("\nNOTE:\n",
                        "Please verify number of rows in imported data.\n")
        warning(MESSG2)
    } else {
        MESSG3 <- paste("\n NOTE:\n",
                        "Number of imported rows appears correct.\n",
                        "Nevertheless, please verify against original data.\n")
        warning(MESSG3)
    }


    if(!is.null(meta)){
        attr(dat2, "meta") <- meta   
    }
    dat2
}


##' Create meta data frame to align identical questions
##'
##' Qualtrics returns a data frame that has vertical "blocks", one for
##' each "treatment condition" in an experimental condition.
##' Researchers often want to align the questions from the blocks
##' vertically, essentially converting the Qualtrics "wide" format to
##' a "long" format. This is a helper function that identifies
##' questions that may need to be stacked together.  The input is a
##' meta data structure (can be retrieved as an attribute from
##' importQualtrics). It will find out which questions are identical
##' and prepare to re-align ("stack") the columns.
##' @param meta A meta data structure retrieved from importQualtrics
##' @param questionname Character string for name of column in meta data that holds the questions
##' @return A new meta data table that horizontally aligns equivalent questions.
##' @export
##' @importFrom stats reshape
##' @author Paul Johnson
qualtricsBlockStack <- function(meta, questionname = "question"){
    meta$first.match <- match(meta[ , questionname], meta[ , questionname])
    
    ## data.table method was causing errors, so forget about it
    metasplit <- split(meta, f = meta$first.match)
    meta <- do.call("rbind", lapply(metasplit, function(x) {x$sect <- 1:NROW(x); x}))
      
    reslt <- reshape(meta, direction="wide",
            v.names = c("col.number", "name", "name.clean", "importid"),
            timevar = "sect", idvar = "first.match", drop = c("ImportId.orig"))
    reslt[ , "first.match"] <- NULL
    reslt
}

## 
##' Check if values can be safely coerced without introduction of
##' missing values
##'
##' This might be named "coercesSafely" or such.  If values cannot be
##' coerced into class specified, then values must be incorrect.
##' @param value Character vector of values, such as value_new or
##'     value_old for one variable in a key.
##' @param targetclass R class name
##' @param na.strings Values that should be interpreted as R NA.
##'     These are ignored in the coercion check.
##' @return either TRUE, or a vector of values which are not
##'     successfully coerced
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' x1 <- c("TRUE", "FALSE", FALSE, TRUE, NA, ".", "N/A", " ", "")
##' checkCoercion(x1, "logical")
##' x1 <- c(x1, "TRUE.FALSE", "Has a space")
##' ## Should fail:
##' checkCoercion(x1, "logical")
##' x2 <- c(4, 5, 6, 9.2, ".", " ")
##' ## Should fail
##' checkCoercion(x2, "logical")
##' x3 <- factor(c("bob", "emily", "bob", "jane", "N/A", " ", NA, "NA"))
##' checkCoercion(x3, "ordered")
##' checkCoercion(x3, "integer")
##' ## Should fail:
##' checkCoercion(x3, "logical")
##' 
checkCoercion <- function(value, targetclass,
                          na.strings = c("\\.", "", "\\s+",  "N/A") ){
    value.lab <-  deparse(substitute(value))
    ## previous error, which did not use regular expression
    ismissing <- grepl(paste0("^", paste0(na.strings, collapse="$|^"), "$"), value)
    value[ismissing] <- NA
    value <- na.omit(value)
    mytext <- paste0("as.", targetclass, "(value)")
    res <- eval(parse(text = mytext))
    if(any(is.na(res))){
        MESSG <- paste("checkValue: fails coercion of",
                       value.lab, "as", targetclass, "\n")
        cat(MESSG)
        return(value[is.na(res)])
    }
    TRUE
}

##' Replace column names with new names from a named vector
##'
##' A convenience function to alter column names. Can be called from
##' code cleanup in the variable key system.
##' @param dat a data frame
##' @param newnames Can be a named vector of the form c(oldname1 =
##'     "newname1", oldname2 = "newname") or it may be simply
##'     c("newname1", "newname2") to correspond with the oldname
##'     vector.
##' @param oldnames Optional. If supplied, must be same length as
##'     newnames.
##' @param ... Additional arguments that will be passed to R's
##'     \code{gsub} function, which is used term-by-term inside this
##'     function.
##' @param lowercase Default FALSE. Should all column names be
##'     converted to lower case.
##' @param verbose Default FALSE. Want diagnostic output about column
##'     name changes?
##' @return a data frame
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' set.seed(234234)
##' N <- 200
##' mydf <- data.frame(x5 = rnorm(N), x4 = rnorm(N), x3 = rnorm(N),
##'                    x2 = letters[sample(1:24, 200, replace = TRUE)],
##'                    x1 = factor(sample(c("cindy", "bobby", "marsha",
##'                                         "greg", "chris"), 200, replace = TRUE)),
##'                    x11 = 7,
##'                    x12 = 18,
##'                    x13 = 33,
##'                    stringsAsFactors = FALSE)
##' mydf2 <- colnamesReplace(mydf, newnames = c("x4" = "GLOPPY"))
##' mydf2 <- colnamesReplace(mydf, newnames = c("x4" = "GLOPPY", "USA" = "Interesting"), verbose = TRUE)
##' colnames(mydf2)
##' head(mydf3 <- colnamesReplace(mydf, newnames = c(x11 = "x12", x12 = "x13", x13 = "x20")))
##' head(mydf4 <- colnamesReplace(mydf, newnames = c(x12 = "x11", x11 = "x99", x13 = "x20")))
colnamesReplace <- function(dat, newnames, oldnames = NULL, ...,  lowercase = FALSE, verbose = FALSE){
    if (is.null(oldnames)) {
        if (is.null(names(newnames))){
            stop("The newname vector should be a named vector, using the oldnames as the names")
        }
        oldnames <- names(newnames)
    } else {
        if (length(newnames) != length(oldnames)){
            messg <- paste("newnames has", length(newnames), "elements")
            messg <- c(messg, paste("oldnames has", length(oldnames), "elements"))
            messg <- c(messg, "If oldnames is supplied, must be same length as newnames")
            stop(messg)
        }
        names(newnames) <- oldnames
    }
    namez <- colnames(dat)

    if (any(oldnames %in% namez)){
        mapidx <- match(namez, oldnames)
        mapidxNA <- is.na(mapidx)
        namez[!mapidxNA] <- newnames[mapidx[!mapidxNA]]
        ## 20181204: mgsub replaced by mapvalues style code
        ## newnamez <- mgsub(oldnames, newnames, colnames(dat), ...)
        colnames(dat) <- if (lowercase) tolower(namez) else namez
        if (verbose){
            cat(paste("colnamesReplace Diagnostic: These names replacements are requested:\n"))
            print(c("oldnames", "newnames"))
            print(cbind(oldnames, newnames ))
            cat(paste("These names were actually replaced: ",
                      paste(oldnames[oldnames %in% namez], collapse=",")))
        }
        return(dat)
    } else {
        cat(paste("colnamesReplace Diagnostic:  No column names were altered\n"))
        return(dat)
    }
}

##' Reverse the levels in a factor
##'
##' Simple literal reversal. Will stop with an error message if x is
##' not a factor (or ordered) variable.
##'
##' Sometimes people want to
##' reverse some levels, excluding others and leaving them at the end
##' of the list. The "eol" argument sets aside some levels and puts
##' them at the end of the list of levels.
##'
##' The use case for the \code{eol} argument is a factor
##' with several missing value labels, as appears in SPSS. With
##' up to 18 different missing codes, we want to leave them
##' at the end. In the case for which this was designed, the
##' researcher did not want to designate those values as
##' missing before inspecting the pattern of observed values.
##' 
##' @param x a factor variable
##' @param eol values to be kept at the end of the list. Does not
##'     accept regular expresssions, just literal text strings
##'     representing values.
##' @export
##' @return a new factor variable with reversed values
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' ## Consider alphabetication of upper and lower
##' x <- factor(c("a", "b", "c", "C", "a", "c"))
##' levels(x)
##' xr1 <- reverse(x)
##' xr1
##' ## Keep "C" at end of list, after reverse others
##' xr2 <- reverse(x, eol = "C")
##' xr2
##' y <- ordered(x, levels = c("a", "b", "c", "C"))
##' yr1 <- reverse(y)
##' class(yr1)[1] == "ordered"
##' yr1
##' ## Hmm. end of list amounts to being "maximal".
##' ## Unintended side-effect, but interesting.
##' yr2 <- reverse(y, eol = "C")
##' yr2
##' ## What about a period as a value (SAS missing)
##' z <- factor(c("a", "b", "c", "b", "c", "."))
##' reverse(z)
##' z <- factor(c(".", "a", "b", "c", "b", "c", "."))
##' reverse(z)
##' ## How about R NA's
##' z <- factor(c(".", "a", NA, "b", "c", "b", NA, "c", "."))
##' z
##' reverse(z)
##' z <- ordered(c(".", "a", NA, "b", "c", "b", NA, "c", "."))
##' z
##' str(z)
##' ## Put "." at end of list
##' zr <- reverse(z, eol = ".")
##' zr
##' str(zr)
##' z <- ordered(c(".", "c", NA, "e", "a", "c", NA, "e", "."),
##'          levels = c(".", "c", "e", "a"))
##' reverse(z, eol = ".")
##' reverse(z, eol = c("a", "."))
reverse <- function(x, eol = c("Skip", "DNP")){
    if (!is.factor(x)) stop("your variable is not a factor")
    class_old <- class(x)
    rlevels <- rev(levels(x))
    if (length(eol) > 0){
        for (jj in eol){
            if (length(yyy <- grep(jj, rlevels, fixed = TRUE))){
                rlevels <- c(rlevels[-yyy], jj)
            }
        }
    }
    factor(x, levels = rlevels, ordered = is.ordered(x))
}

##' Create a backup version of a file by renaming it.
##' 
##' Inserts the date-time of the most recent modification at the
##' end of the file name, before the extension.
##'
##' Return is the new file name that was created, using whatever
##' path information was provided in the file's original name. However,
##' the fullpath argument can be set to TRUE, so a path with the
##' full directory name will be created and returned.
##' 
##' @param name A character string for the name of the file.
##' @param fullpath Return the full directory path to the
##'     file. Default FALSE, return only the file name.
##' @param keep.old If FALSE (default), rename the file. Otherwise, keep old copy.
##' @param verbose If TRUE, output warnings and list the files in the output directory when done.
##' @author Shadi Pirhosseinloo <shadi@@ku.edu> Paul Johnson <pauljohn@@ku.edu>
##' @return The name of the newly created file.
##' @importFrom tools file_ext
##' @importFrom tools file_path_sans_ext
##' @export
##' @examples
##' tdir <- tempdir()
##' owd <- getwd()
##' 
##' setwd(tdir)
##' system("touch test.1.txt")
##' system("touch test.2.txt")
##' system("touch test.3.txt")
##' system("touch test.4.txt")
##' system("touch test.5.txt")
##' ## note: no extension next
##' system("touch test.6")
##' list.files()
##' file.backup("test.1.txt")
##' file.backup("test.2.txt", fullpath=TRUE)
##' list.files()
##' setwd(owd)
##' file.backup(file.path(tdir, "test.3.txt"))
##' ## Next should be same path because input had a full path
##' file.backup(file.path(tdir, "test.4.txt"), fullpath=TRUE)
##' file.backup(file.path(tdir, "test.5.txt"), fullpath = TRUE, verbose = TRUE)
##' file.backup(file.path(tdir, "test.6"))
file.backup <- function(name, fullpath = FALSE, keep.old = FALSE, verbose = FALSE){
    if(!file.exists(name)){
        MESSG <- paste("file", name, "does not exist. No backup created.")
        if(verbose) warning(MESSG)
        return(NULL)
    }
    dir.source <- dirname(normalizePath(name))
    
    date_cr <- base::format(base::file.info(name)$mtime, "%Y%m%d-%H%M")
    ext_name <- tools::file_ext(name)
    noext_name <- tools::file_path_sans_ext(name)
    new_name <- paste0(noext_name, "-", date_cr,
                       if(!ext_name == "") {paste0( ".", ext_name)})
   
    ## Abort if new file name already exists
    if (file.exists(new_name)){
        MESSG <- paste("backup file already exists. No new backup created.")
        if(verbose) warning(MESSG)
        return(new_name)
    }
    ret <- if(keep.old){
               file.copy(name, new_name, overwrite = TRUE,
                         copy.mode = TRUE, copy.date = TRUE)
           }else{
               file.rename(name, new_name)
           }
    if(!ret) {
        MESSG <- paste("file.rename(", name, ",", new_name, "failed")
        stop(MESSG)
    }
    if(verbose){
        cat("File list of:", dir.source, "\n")
        print(list.files(dir.source))
        cat("End of file list\n")
    }
    if(fullpath){
        new_name_fullpath <- normalizePath(new_name)
        return(new_name_fullpath)
    }
    
    ## else, give back adjusted with same path information it had
    new_name
}

##' If a numeric variable has only integer values, then
##' make it an integer.
##'
##' Users often accidentally create floating point numeric variables
##' when they really mean integers, such as c(1, 2, 3), when they
##' should have done c(1L, 2L, 3L). Before running \code{as.integer()}
##' to coerce the variable, we'd rather be polite and ask the variable
##' "do you mind being treated as if you are an integer?"  This
##' function checks to see if the variable is "close enough" to being
##' an integer, and then coerces as integer. Otherwise, it returns
##' NULL. And issues a warning.
##'
##' First, calculate absolute value of differences between \code{x}
##' and \code{as.integer(x)}. Second, find out if the sum of those
##' differences is smaller than \code{tol}. If so, then x can
##' reasonably be coerced to an integer.
##'
##' Be careful with the return. The correct return value for variables
##' that should not be coerced as integer is uncertain at this
##' point. We've tested various strategies, sometimes returning FALSE,
##' NULL, or just the original variable.
##' @param x a numeric variable
##' @param tol Tolerance value. Defaults to Machine$double.eps. See
##'     details.
##' @param vmax Maximum value allowed for an integer. Defaults to
##'     Machine$integer.max.
##' @param digits Digits value passed to the zapsmall
##'     function. Defaults to 7.
##' @param verbose Default FALSE: print warnings about x
##' @export
##' @return Either an integer vector or the original variable
##' @author Paul Johnson <pauljohn@@ku.edu> and Ben Kite
##'     <bakite@@ku.edu>
##' @examples
##' x1 <- c(1, 2, 3, 4, 5, 6)
##' is.integer(x1)
##' is.double(x1)
##' is.numeric(x1)
##' (x1int <- safeInteger(x1))
##' is.integer(x1int)
##' is.double(x1int)
##' is.numeric(x1int)
##' x2 <- rnorm(100)
##' x2int <- safeInteger(x2)
##' head(x2int)
##' x3 <- factor(x1, labels = c(LETTERS[1:6]))
##' x3int <- safeInteger(x3)
##'
safeInteger <- function(x, tol = .Machine$double.eps,
                        digits = 7, vmax = .Machine$integer.max,
                        verbose = FALSE)
{
    if(!is.numeric(x)) {
        if (verbose) {
            messg <- paste("safeInteger: x must be numeric or integer.")
            warning(messg)
        }
        return(NULL)
    }

    if(max(x, na.rm = TRUE) > vmax) {
        messg <- paste0("Values in x exceed the maximum integer value of ",
                        vmax, ". ", "safeInteger cannot be used safely ",
                        "for this variable. Original value is returned.")
        warning(messg)
        return(x)
    }

    if(is.integer(x)){
        return(x)
    } else {
        x <- zapsmall(x, digits)
        xnew <- as.integer(x)
        if (sum(abs(x - xnew), na.rm = TRUE) < tol) {
            return(xnew)
        } else {
            if (verbose) {
                messg <- paste("asInteger x:", paste(head(x), collapse=", "),
                               "... is not close enough to an integer")
                warning(messg)
            }
            return(NULL)
        }
    }
    messg <- paste0("safeInteger should not have reached this point")
    stop(messg)
}
NULL

##' Set missing values
##'
##' The missings values have to be carefully written, depending on the
##' type of variable that is being processed.
##'
##' Version 0.95 of kutils introduced a new style for specification of
##' missing values.
##'
##' @param x A variable
##' @param missings A string vector of semi-colon separated values,
##'     ranges, and/or inequalities.  For strings and factors, only an
##'     enumeration of values (or factor levels) to be excluded is
##'     allowed. For numeric variables (integers or floating point
##'     variables), one can specify open and double-sided intervals as
##'     well as particular values to be marked as missing. One can
##'     append particular values and ranges by
##'     "1;2;3;(8,10);[22,24];> 99;< 2". The double-sided interval is
##'     represented in the usual mathematical way, where hard
##'     bracketes indicate "closed" intervals and parentheses indicate
##'     open intervals.\enumerate{
##'
##' \item "(a,b)" means values of x greater than a and smaller than b
##'     will be set as missing.
##'
##' \item "[a,b]" is a closed interval, one which includes the
##'     endpoints, so a <= x <= b will be set as NA
##'
##' \item "(a,b]" and "[a,b)" are acceptable.
##' \item "< a"  indicates all values smaller than a will be missing
##' \item  "<= a" means values smaller than or equal to a will be
##'     excluded
##' \item "> a" and ">= a" have comparable
##'     interpretations.
##' \item "8;9;10" Mark off specific values by
##' an enumeration. Be aware, however, that this is
##' useful only for integer variables.  As demonstrated in the
##' example, for floating point numbers, one must specify intervals.
##' \item For factors and character variables, the argument missings
##' can be written either as "lo;med;hi" or "c('lo','med','hi')" }
##' @param sep A separator symbol, ";" (semicolon) by default
##' @return A cleaned column in which R's NA symbol replaces values
##'     that should be missing
##' @export
##' @importFrom utils head
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' ## 1.  Integers.
##' x <- seq.int(-2L, 22L, by = 2L)
##' ## Exclude scores 8, 10, 18
##' assignMissing(x, "8;10;18")
##' ## Specify range, 4 to 12 inclusive
##' missings <- "[4,12]"
##' assignMissing(x, missings)
##' ## Not inclusive
##' assignMissing(x,  "(4,12)")
##' ## Set missing for any value smaller that 7
##' assignMissing(x, "< 7")
##' assignMissing(x, "<= 8")
##' assignMissing(x, "> 11")
##' assignMissing(x, "< -1;2;4;(7, 9);> 20")
##'
##'
##' ## 2. strings
##' x <- c("low", "low", "med", "high")
##' missings <- "low;high"
##' assignMissing(x, missings)
##' missings <- "med;doesnot exist"
##' assignMissing(x, missings)
##' ## Test alternate separator
##' assignMissing(x, "low|med", sep = "|")
##'
##' ## 3. factors (same as strings, really)
##' x <- factor(c("low", "low", "med", "high"), levels = c("low", "med", "high"))
##' missings <- "low;high"
##' assignMissing(x, missings)
##' ## Previous same as
##' missings <- c("low", "high")
##' assignMissing(x, missings)
##'
##' missings <- c("med", "doesnot exist")
##' assignMissing(x, missings)
##' ## ordered factor:
##' x <- ordered(c("low", "low", "med", "high"), levels = c("low", "med", "high"))
##' missings <- c("low", "high")
##' assignMissing(x, missings)
##'
##' ## 4. Real-valued variable
##' set.seed(234234)
##' x <- rnorm(10)
##' x
##' missings <- "< 0"
##' assignMissing(x, missings)
##' missings <- "> -0.2"
##' assignMissing(x, missings)
##' ## values above 0.1 and below 0.7 are missing
##' missings <- "(0.1,0.7)"
##' assignMissing(x, missings)
##' ## Note that in floating point numbers, it is probably
##' ## futile to specify specific values for missings. Even if we
##' ## type out values to 7 decimals, nothing gets excluded
##' assignMissing(x, "-0.4879708;0.1435791")
##' ## Can mark a range, however
##' assignMissing(x, "(-0.487971,-0.487970);(0.14357, 0.14358)")
##' x
assignMissing <- function(x, missings = NULL, sep = ";"){
    if (is.null(missings)) return(x)
    if (is.character(missings)) missings <- zapspace(missings)
    missings <- unlist(strsplit(missings, split = sep, fixed = TRUE))
    missings <- na.omit(missings)
    missings <- zapspace(missings)
    missings <- gsub("-", " -", missings,  fixed = TRUE)
    ## If no NA  missings to remove, then return
    if (is.null(missings) | length(missings) == 0) return(x)

    if (is.character(x)){
        x[x %in% missings] <- NA
        return(x)
    }

    if (is.logical(x)){
        missings <- as.logical(missings)
        x[x %in% missings] <- NA
        return(x)
    }
    
    if (is.factor(x)){
        levels(x)[which(levels(x) %in% missings)] <- NA
        return(x)
    }

    ## 20170203: TODO following is hard coded to look at first character,
    ## if space first, fails, used zapspace to address above, but that's
    ## stupid bandaid, better regex work would be solid.
    if (is.numeric(x)){
        ## is  numeric includes integer and double and numeric
        ## separate the elements that inequality signs
        hasineq <- missings[substr(missings, 1, 1) %in% c(">", "<", "(", "[")]
        hasnoineq <- setdiff(missings, hasineq)
        hasnoineq <- if (length(hasnoineq) > 0) paste("x == ", hasnoineq)

        ## (9 ->   x > 9
        ## [9 ->   x >= 9
        ins <- c(">", "<" , "(",     "[",     ")",     "]",  ",")
        outs <- c("x >", "x <", "x >", "x >= ", " > x", " >= x", " & ")
        ranges <- mgsub(ins, outs, hasineq, fixed = TRUE)

        for(rr in c(ranges, hasnoineq)){
            eval(parse(text = paste0("x[", rr, "] <- NA")))
        }
        return(x)
    }

    messg <- "Sorry, no missings assigned because variable type was unhandled"
    warning(messg)
    ## return unchanged input, didn't see what to do
    x
}




##' A variable is transformed in an indicated way
##'
##' In the variable key framework, the user might request
##' transformations such as the logarithm, exponential, or square
##' root. This is done by including strings in the recodes column,
##' such as "log(x + 1)" or "3 + 1.1 * x + 0.5 * x ^ 2". This
##' function implements the user's request by parsing the character
##' string and applying the indicated re-calculation.
##'
##' In the variable key framework, this is applied to the raw data,
##' after missings are imposed.
##' @param x A column to be recoded
##' @param recode A character string using placeholder "x". See
##'     examples
##' @return A new column
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' set.seed(234234)
##' x <- rpois(100, lambda = 3)
##' x <- x[order(x)]
##' str1 <- "log(x + 1)"
##' xlog <- assignRecode(x, recode = str1)
##' plot(xlog ~ x, type = "l")
##' mean(xlog, na.rm = TRUE)
##' str2 <- "x^2"
##' xsq <- assignRecode(x, recode = str2)
##' plot(xsq ~ x, type = "l")
##' str3 <- "sqrt(x)"
##' xsrt <- assignRecode(x, recode = str3)
assignRecode <- function(x, recode = NULL){
    y <- eval(parse(text = recode))
}


##' Check and Clean data.frame for usage with variable key functions
##'
##' Checks that the data.frame is made up of simple individual
##' columns. Checks numeric columns to find out if they are acceptable
##' to treat as integers. If they are acceptable to treat as integers,
##' then convert those numeric to integer class variables.
##' @param dframe A data frame
##' @param safeNumericToInteger Default TRUE: Should we treat values
##'     which appear to be integers as integers? If a column is
##'     numeric, it might be safe to treat it as an integer.  In many
##'     csv data sets, the values coded c(1, 2, 3) are really
##'     integers, not floats c(1.0, 2.0, 3.0). See \code{safeInteger}.
##' @param trimws Defaults as "both", in meaning of \code{which}
##'     argument in \code{trimws} function.  Set as NULL if character
##'     variables must not be trimmed to eliminate white
##'     space. Otherwise, value should be one of \code{c("left",
##'     "right", "both")}.
##' @export
##' @return A checked and cleaned data frame
##' @keywords internal
##' @author Paul Johnson <pauljohn@@ku.edu>
cleanDataFrame <- function(dframe, safeNumericToInteger = TRUE, trimws = "both"){
    if(!is.data.frame(dframe)){
        messg <- paste("keyUpdate: The dframe object must be a data frame")
        stop(messg)
    }

    if (!(res <- is.data.frame.simple(dframe))) {
        messg <- paste(paste("cleanDataFrame checked if dframe elements are not single columns.",
                       "This frame has some elements that are not single columns.",
                       "The troublesome elements are:", collapse = ""),
                       paste(attr(res, "not_a_simple_column"), collapse = ", "))
        stop(messg)
    }

    ## If integer-like columns exist, turn them into integers
    if (safeNumericToInteger){
        for(i in colnames(dframe)){
            if(is.numeric(dframe[ , i])
               && !is.null(tmp <- safeInteger(dframe[ , i])))
                dframe[ , i] <- tmp
        }
    }

    ## Clean characters that have leading spaces to delete space
    if (!is.null(trimws)){
        for(i in colnames(dframe)){
            if(is.character(dframe[ , i])){
                dframe[ , i] <- trimws(dframe[ , i], which = trimws)
            } else if (is.factor(dframe[ , i])){
                levels(dframe[ , i]) <- trimws(levels(dframe[ , i]))
            }
        }
    }
    
    dframe
}


##' Check if a data frame is a simple collection of columns (no lists
##' or matrices within)
##'
##' Checks for the existence of dimensions within the data
##' frame. Returns FALSE if any object within dframe has non-null dim
##' value.
##'
##' See: http://stackoverflow.com/questions/38902880/data-frame-in-which-elements-are-not-single-columns
##'
##' @param dframe A data frame
##' @return Boolean, TRUE or FALSE. An attribute "not_a_simple_column"
##'     is created, indicating which of the elements in the dframe
##'     have dimensions
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @export
##' @examples
##' N <- 100
##' mydf <- data.frame(x5 = rnorm(N),
##'                    x4 = rpois(N, lambda = 3),
##'                    x3 = ordered(sample(c("lo", "med", "hi"),
##'                    size = N, replace=TRUE)))
##' is.data.frame.simple(mydf)
##' mydf$amatr <- matrix(0, ncol = 2, nrow = NROW(mydf))
##' is.data.frame.simple(mydf)
##' mydf$amatr <- NULL
##' is.data.frame.simple(mydf)
##' mydf$adf <- mydf
##' is.data.frame.simple(mydf)
is.data.frame.simple <- function(dframe){
    no.dims <- function(x) {!is.null(dim(x))}
    elemdims <- sapply(dframe, no.dims)
    res <-  if(sum(elemdims) > 0) FALSE else TRUE
    attr(res, "not_a_simple_column") <- names(which(elemdims))
    res
}



##' Compare observed values with the values listed
##' (presumably from a variable key).
##'
##' This is purely diagnostic. It prints warnings in
##' either of 2 cases. 1)  observed data has values that
##' are not in the value_old, or 2) that value_old has
##' values that are not in the data.
##'
##' @param x a variable, either character or factor
##' @param value_old a vector of old values for which we are checking
##' @param xname character string to use for x's name when printing output
##' @param diagnostic prints messages about variables if TRUE
##' @keywords internal
##' @return NULL
##' @author Paul Johnson <pauljohn@@ku.edu>
checkValue_old <- function(x, value_old, xname, diagnostic = FALSE){
    if (!is.factor(x) && !is.character(x)) return(NULL)
    ## if value_old is length 1 and is equal to NA, don't bother
    if(all(value_old %in% c("NA", NA))) return(NULL)
    xobs <- unique(x)
    keynotinobs <- value_old[!value_old %in% xobs]
    keynotinobs <- na.omit(keynotinobs)
    if (diagnostic && length(keynotinobs) > 0){
        messg <- paste("Data Check (variable", xname, ":)\n",
                     "Key values were not observed in the input data: ",
                     paste(keynotinobs, collapse = ", "), "\n")
        cat(messg)
    }
    tobsnotinkey <- xobs[!xobs %in% value_old]
    tobsnotinkey <- na.omit(tobsnotinkey)
    if (diagnostic && length(tobsnotinkey) > 0) {
        messg <- paste("Data Check (variable", xname, ":)\n",
                       "These values in the input data were not in value_old: ",
                       paste(tobsnotinkey, collapse = ", "), "\n")
        cat(messg)
    }
    NULL
}



##' Check if values are R NA symbol or any one of the na.strings
##' elements
##'
##' A value vector in the key will generally be a character
##' vector.  This utility is used to check if the characters
##' are either R missing or values in a list of characters that
##' represent missings.
##' 
##' @param x Input data vector
##' @param na.strings Vector of string values to be considered as
##'     missing. Defaults will match values that are equal to ., empty
##'     string, any number of white space elements, or charcter
##'     string N/A. We do not include `NA` by default because some
##'     projects use NA to mean "not appropriate".
##' @return Logical vector, TRUE if a value is either NA or in
##'     na.strings.
##' @export 
##' @examples
##' x1 <- c("TRUE", "FALSE", FALSE, TRUE, NA, "NA", ".", "N/A", " ", "")
##' x1na <- kutils:::isNA(x1)
##' cbind(x1, x1na)
isNA <- function(x, na.strings = c("\\.", "", "\\s+",  "N/A")){
    ismissing <- grepl(paste0("^", paste0(na.strings, collapse="$|^"), "$"), x)
    ismissing[is.na(x)] <- TRUE
    ismissing
}

##' Create variable key template (in memory or in a file)
##'
##' A variable key is a human readable document that describes the
##' variables in a data set. A key can be revised and re-imported by R
##' to recode data. This might also be referred to as a
##' "programmable codebook."  This function inspects a data frame,
##' takes notice of its variable names, their classes, and legal
##' values, and then it creates a table summarizing that
##' information. The aim is to create a document that principal
##' investigators and research assistants can use to keep a project
##' well organized.  Please see the vignette in this package.
##'
##' The variable key can be created in two formats, wide and long.
##' The original style of the variable key, wide, has one row per
##' variable. It has a style for compact notation about current values
##' and required recodes.  That is more compact, probably easier for
##' experts to read, but perhaps more difficult to edit. The long
##' style variable key has one row per value per variable.  Thus, in a
##' larger project, the long key can have many rows. However, in a
##' larger project, the long style key is easier to edit with a spread
##' sheet program.
##'
##' After a key is created, it should be re-imported into R with the
##' \code{kutils::keyImport} function.  Then the key structure can
##' guide the importation and recoding of the data set.
##'
##' Concerning the varlab attribute. Run \code{attr(key, "varlab"} to
##' review existing labels, if any.
##'
##' Storing the variable labels in files requires some care because
##' the \code{rds}, \code{xlsx}, and \code{csv} formats have different
##' capabilities.  The \code{rds} storage format saves all attributes without
##' difficulty. In contrast, because \code{csv} and \code{xlsx} do not save
##' attributes, the varlabs are stored as separate character
##' matrices. For \code{xlsx} files, the varlab object is saved as a second
##' sheet in \code{xlsx} file, while in \code{csv} a second file suffixed
##' "-varlab.csv" is created. 
##'
##' @param dframe A data frame
##' @param long Default FALSE.
##' @param sort Default FALSE. Should the rows representing the
##'     variables be sorted alphabetically? Otherwise, they appear in
##'     the order in which they were included in the original dataset.
##' @param file DEFAULT NULL, meaning no file is produced. Choose a
##'     file name ending in either "csv" (for comma separated
##'     variables), "xlsx" (compatible with Microsoft Excel), or "rds"
##'     (R serialization data). The file name will be used to select
##'     among the 3 storage formats. XLSX output requires the openxlsx
##'     package.
##' @param max.levels How high is the limit on the number of values
##'     for discrete (integer, character, and Date) variables?
##'     Default = 15. If observed number exceeds max.levels, we
##'     conclude the author should not assign new values in the key
##'     and only the missing value will be included in the key as a
##'     "placeholder". This does not affect variables declared as
##'     factor or ordered variables, for which all levels are included
##'     in all cases.
##' @param missings Values in exising data which should be treated as
##'     missing in the new key. Character string in format acceptable
##'     to the \code{assignMissing} function. Can be a string with
##'     several missing indicators"1;2;3;(8,10);[22,24];> 99;< 2".
##' @param missSymbol Default ".".  A character string used to
##'     represent missing values in the key that is created.  Relevant
##'     (mostly) for the key's \code{value_new} column. Default is the
##'     period, ".". Because R's symbol \code{NA} can be mistaken for
##'     the character string \code{"NA"}, we use a different
##'     (hopefully unmistakable) symbol in the key.
##' @param varlab A key can have a companion data structure for
##'     variable labels. Default is FALSE, but the value may also be
##'     TRUE or a named vector of variable labels, such as
##'     \code{c("x1" = "happiness", "x2" = "wealth")}. The labels
##'     become an attribute of the key object. See Details for
##'     information on storage of varlabs in saved key files.
##' @param safeNumericToInteger Default TRUE: Should we treat values
##'     which appear to be integers as integers? If a column is
##'     numeric, it might be safe to treat it as an integer.  In many
##'     csv data sets, the values coded c(1, 2, 3) are really
##'     integers, not floats c(1.0, 2.0, 3.0). See \code{safeInteger}.
##' @param trimws Default is "both", user can change to "left", "right", or
##'     set as NULL to avoid any trimming.
##' @return A key in the form of a data frame. May also be saved on
##'     disk if the file argument is supplied. The key may have an
##'     attribute "varlab", variable labels.
##' @export
##' @importFrom utils write.csv
##' @importFrom methods as
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' set.seed(234234)
##' N <- 200
##' mydf <- data.frame(x5 = rnorm(N),
##'                    x4 = rpois(N, lambda = 3),
##'                    x3 = ordered(sample(c("lo", "med", "hi"),
##'                    size = N, replace=TRUE),
##'                    levels = c("med", "lo", "hi")),
##'                    x2 = letters[sample(c(1:4,6), N, replace = TRUE)],
##'                    x1 = factor(sample(c("cindy", "bobby", "marcia",
##'                                         "greg", "peter"), N,
##'                    replace = TRUE)),
##'                    x7 = ordered(letters[sample(c(1:4,6), N, replace = TRUE)]),
##'                    x6 = sample(c(1:5), N, replace = TRUE),
##'                    stringsAsFactors = FALSE)
##' mydf$x4[sample(1:N, 10)] <- 999
##' mydf$x5[sample(1:N, 10)] <- -999
##'
##' ## Note: If we change this example data, we need to save a copy in
##' ## "../inst/extdata" for packacing
##' dn <- tempdir()
##' write.csv(mydf, file = file.path(dn, "mydf.csv"), row.names = FALSE)
##' mydf.templ <- keyTemplate(mydf, file = file.path(dn, "mydf.templ.csv"),
##'                           varlab = TRUE)
##' mydf.templ_long <- keyTemplate(mydf, long = TRUE,
##'                             file = file.path(dn, "mydf.templlong.csv"),
##'                             varlab = TRUE)
##'
##' mydf.templx <- keyTemplate(mydf, file = file.path(dn, "mydf.templ.xlsx"),
##'                             varlab = TRUE)
##' mydf.templ_longx <- keyTemplate(mydf, long = TRUE,
##'                              file = file.path(dn, "mydf.templ_long.xlsx"),
##'                              varlab = TRUE)
##' ## Check the varlab attribute
##' attr(mydf.templ, "varlab")
##' mydf.tmpl2 <- keyTemplate(mydf,
##'                          varlab = c(x5 = "height", x4 = "age",
##'                          x3 = "intelligence", x1 = "Name"))
##' ## Check the varlab attribute
##' attr(mydf.tmpl2, "varlab")
##' 
##' ## Try with the national longitudinal study data
##' data(natlongsurv)
##' natlong.templ <- keyTemplate(natlongsurv,
##'                           file = file.path(dn, "natlongsurv.templ.csv"),
##'                           max.levels = 15, varlab = TRUE, sort = TRUE)
##'
##' natlong.templlong <- keyTemplate(natlongsurv, long = TRUE,
##'                    file = file.path(dn, "natlongsurv.templ_long.csv"), sort = TRUE)
##' if(interactive()) View(natlong.templlong)
##' natlong.templlong2 <- keyTemplate(natlongsurv, long = TRUE,
##'                       missings = "<0", max.levels = 50, sort = TRUE,
##'                       varlab = TRUE)
##' if(interactive()) View(natlong.templlong2)
##' 
##' natlong.templwide2 <- keyTemplate(natlongsurv, long = FALSE,
##'                       missings = "<0", max.levels = 50, sort = TRUE)
##' if(interactive()) View(natlong.templwide2)
##'
##' all.equal(wide2long(natlong.templwide2), natlong.templlong2)
##'
##' head(keyTemplate(natlongsurv, file = file.path(dn, "natlongsurv.templ.xlsx"),
##'              max.levels = 15, varlab = TRUE, sort = TRUE), 10)
##' head(keyTemplate(natlongsurv, file = file.path(dn, "natlongsurv.templ.xlsx"),
##'              long = TRUE, max.levels = 15, varlab = TRUE, sort = TRUE), 10)
##'
##' list.files(dn)
##'
keyTemplate <-
    function(dframe, long = FALSE, sort = FALSE,
             file = NULL, max.levels = 15, missings = NULL, missSymbol = ".",
             safeNumericToInteger = TRUE, trimws = "both", 
             varlab = FALSE)
{
    if (class(dframe)[1] != "data.frame"){
        MESSG <- paste("Warning: keyTemplate is intended for an R *data.frame*,",
                       "not subclasses like tibbles or data.tables.",
                       "keyTemplate coerced your input",
                       "with as.data.frame().\n")
        if (inherits(dframe, "data.frame")){
            ## Coerce without warning, but print message
            cat(MESSG)
        } else {
            ## Elevate the message to a warning
            warning(MESSG)
        }
        dframe <- as.data.frame(dframe)
    }
            
    dframe <- cleanDataFrame(dframe, safeNumericToInteger = safeNumericToInteger,
                             trimws = trimws)

    df.class <- sapply(dframe, function(x)class(x)[1])
    cn <- colnames(dframe)

    ## Keep at most max.levels elements, but always add an NA symbol
    ## if there is none.  Also, always keep value that are missing
    ## according to missings
    shortenValues <- function(x, max.levels, missings){
        xmiss <- assignMissing(x, missings)
        xnotmissing <- if (length(temp <- x[-which(is.na(xmiss))]) > 0) temp else NULL
        xismissing <- unique(x[which(is.na(xmiss))])
        if (!any(vapply(xismissing, is.na, logical(1)))) xismissing <- c(xismissing, NA)
        if (length(xnotmissing) <= max.levels) {
            xnotmissing <- xnotmissing[1:min(max.levels, length(xnotmissing))]
            df <- data.frame(value_old = c(xnotmissing, xismissing),
                             value_new = c(xnotmissing, rep(NA, length(xismissing))),
                             stringsAsFactors = FALSE)
            return(df)
        } else {
            return(data.frame(value_old = NA, value_new = NA,
                              stringsAsFactors = FALSE))
        }
    }

    ## Returns all unique values, inserts NA at end if not present
    getUnique <- function(xname){
        ## For discrete variables integer, character, logical:
        if (df.class[[xname]] %in% c("integer", "character", "logical")){
            val <- unique(dframe[ , xname, drop = TRUE])
            ##pj 20170926: new sort method keeps missing on end, sets NA as missSymbol
            ##pj 20180502: tibble fails on following order function.
            val.sort <- val[order(val)]
            if(!NA %in% val.sort) val.sort <- c(val.sort, NA)
            return(val.sort)
        }
        ## For discrete variables factor, ordered:
        if (df.class[[xname]] %in% c("factor", "ordered")){
            return(c(levels(dframe[ , xname]), NA))
        }
        ## if coercion check passes, then use "as" to coerce the missing
        if (checkCoercion(c(NA), df.class[[xname]])) {
            value <- NA
            mytext <- paste0("as.",  df.class[[xname]], "(value)")
            res <- eval(parse(text = mytext))
            return(res)
        } else{
            ## Give up trying to cast the NA type
            return(c(NA))
        }
    }

    ## First, make a long key
    ## Generate a small key for one variable
    smallTemplate <- function(xname, missings = NULL){
        value_old <- getUnique(xname)
        valoldnew <- shortenValues(value_old,
                                   max.levels = max.levels,
                                   missings = missings)
        valoldnew[is.na(valoldnew)] <- missSymbol
        keysmall <- data.frame(name_old = xname, name_new = xname, 
                               class_old = df.class[[xname]],
                               class_new = df.class[[xname]],
                               value_old = as.character(valoldnew[ , "value_old"]),
                               value_new = as.character(valoldnew[ , "value_new"]),
                               missings = "",
                               recodes = "", 
                               stringsAsFactors = FALSE)
        keysmall
    }
        
    keyList <- lapply(cn, smallTemplate, missings = missings)

    key <- do.call("rbind", keyList)

    if(isTRUE(long)){
        if (sort) key <- key[order(key$name_old, key$name_new), ]
        class(key) <- c("keylong", class(key))
     } else {
        ## else !long, so make a wide key
        key <- long2wide(key)
        rownames(key) <- key$name_old
        if (sort) key <- key[order(key$name_old), ]
        ## put in order of original data columns
        else key <- key[cn, ]
    }

    attr(key, "missSymbol") <- missSymbol
    if (!missing(varlab) && !identical(varlab, FALSE)) {
        attr(key, "varlab") <- varlabTemplate(key, varlab)
        varlab <- TRUE
    }
   
    if (!missing(file) && !is.null(file)){
        keySave(key, file, na_ = missSymbol, varlab = varlab)
    }
    key
}
NULL

##' Create Variable Label Template
##'
##' Receive a key, create a varlab object, with columns
##'     \code{name_old} \code{name_new}, and \code{varlab}.
##'
##' If not specified, a matrix is created with empty variable labels.
##' @param obj A variable key
##' @param varlab Default NULL, function will start from clean slate,
##'     a set of column labels that match \code{name_new}. User can
##'     specify values by providing a named vector of labels, e.g.,
##'     \code{c("x1" = "happiness", "x2" = "wealth")}, where the names
##'     are values to be matched against "name_new" in key.
##' @return Character matrix with columns \code{name_new} and \code{varlab}.
##' @export
##' @author Paul Johnson
##' @examples
##' mydf.path <- system.file("extdata", "mydf.csv", package = "kutils")
##' mydf <- read.csv(mydf.path, stringsAsFactors=FALSE)
##' mydf.keywide1 <- keyTemplate(mydf, long = FALSE, sort = FALSE,
##'                     varlab = TRUE)
##' attr(mydf.keywide1, "varlab")
##' mydf.keywide2 <- keyTemplate(mydf, long = FALSE, sort = FALSE,
##'                     varlab = c("x3" = "fun"))
##' attr(mydf.keywide2, "varlab")
##' attr(mydf.keywide2, "varlab") <- varlabTemplate(mydf.keywide2,
##'                   varlab = c("x5" = "wealth", "x10" = "happy"))
##' attr(mydf.keywide2, "varlab")
##' attr(mydf.keywide2, "varlab") <- varlabTemplate(mydf.keywide2,
##'                   varlab = TRUE)
##' attr(mydf.keywide2, "varlab")
##' ## Target we are trying to match:
##' mydf.keylong <- keyTemplate(mydf, long = TRUE, sort = FALSE, varlab = TRUE)
##' attr(mydf.keylong, "varlab")
##' attr(mydf.keylong, "varlab") <- NULL
##' varlabTemplate(mydf.keylong)
##' attr(mydf.keylong, "varlab") <- varlabTemplate(mydf.keylong,
##'                    varlab = c("x3" = "wealth", "x10" = "happy"))
##' attr(mydf.keylong, "varlab")
##' attr(mydf.keylong, "varlab") <- varlabTemplate(mydf.keylong, varlab = TRUE)
##' attr(mydf.keylong, "varlab")
varlabTemplate <- function(obj, varlab = TRUE){
    if (identical(varlab, FALSE)){
        return(NULL)
    }
    varlabs.orig <- attr(obj, "varlab")
    if(is.null(varlabs.orig)){
        if (isTRUE(varlab)) {
            varlabs <- unique(obj[ , "name_new", drop = TRUE])
            names(varlabs) <- varlabs
            return(varlabs)
        } else if (is.vector(varlab)){
            return(varlab)
        } else {
            MESSG <- paste("varlabTemplate input:",
                           paste(varlab, collapse = " "),
                           "is not understandable")
            stop(MESSG)
        }
    }

    varlabs.all <- unique(obj[ , "name_new"])
    names(varlabs.all) <- varlabs.all
    if (isTRUE(varlab)){
        ## don't change existing varlabs, only new ones
        ## replace from varlabs.orig into varlabs.all
        varlabs <- varlabs.all
        varlabs.addtokey <- varlabs.all[!names(varlabs.all) %in% names(varlabs.orig)]
        return(c(varlabs.orig, varlabs.addtokey))
    } else if (is.vector(varlab)){
        ## keep if in original and not in varlab
        keepinkey <- varlabs.orig[!names(varlabs.orig) %in% names(varlab)]
        ## add from varlab to replace
        replaceinkey <- varlab[names(varlab) %in% names(varlabs.orig)]
        addtokey <- varlab[!names(varlab) %in% names(varlabs.orig)]
        res <- c(keepinkey, replaceinkey, addtokey)
        if(any(duplicated(names(res)))) stop("varlabTemplate fail1")
        return(c(keepinkey, replaceinkey, addtokey))
    } else {
            MESSG <- paste("varlabTemplate input:",
                           paste(varlab, collapse = " "),
                           "is not understandable")
            stop(MESSG)
    }
}
NULL

##' Save key as file after deducing type from suffix
##'
##' This is specialized to saving of key objects, it is not a
##' general purpose function for saving things.  It scans the
##' suffix of the file name and then does the right thing.
##'
##' In updates 2017-09, a varlab element was introduced.  The varlab
##' attribute of the object is saved.  The files created incorporate
##' the variable labels object in different ways. 1) XLSX: variable
##' labels a worksheet named "varlab" 2) CSV: variable labels saved in
##' a separate file suffixed "-varlab.csv". 3) RDS: varlab is an
##' attribute of the key object.
##' @param obj a variable key object
##' @param file file name. must end in "csv", "xlsx" or "rds"
##' @param na_ Value to insert to represent a missing score. Default
##'     ".".
##' @param varlab FALSE or TRUE. Default is FALSE, no new labels will
##'     be created. If a key object has a varlab already, it is saved
##'     with the key, always. This parameter controls whether a new
##'     varlab template should be created when the object is saved.
##'     If TRUE and obj has no varlab attribute, a new varlab template
##'     is created by the \code{varlabTemplate} function. If TRUE and
##'     a varlab attribute currently exists, but some variables are
##'     missing labels, then \code{varlabTemplate} is called to fill
##'     in new variable labels.
##' @return NULL if no file is created. Otherwise, a key object with
##'     an attribute varlab is returned.
##' @importFrom openxlsx addWorksheet writeDataTable saveWorkbook
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
keySave <- function(obj, file, na_ = ".", varlab){
    obj[is.na(obj)] <- "."
    if (!missing(varlab) && !varlab %in% c(TRUE, FALSE)){
        MESSG <- "keySave varlab argument must be TRUE or FALSE"
        stop(MESSG)
    }
    
    if (!is.null(attr(obj, "varlab"))){
        varlab <- TRUE
    }  else if (!missing(varlab) && !identical(varlab, FALSE)
                && is.null(attr(obj, "varlab"))) {
     ## varlab neither provided with key nor varlab == FALSE, so create
        attr(obj, "varlab") <- varlabTemplate(obj, varlab = TRUE)
        varlab <- TRUE
    } else {
        varlab <- FALSE
    }
      
    if (length(grep("csv$", tolower(file))) > 0){
        write.csv(obj, file = file, na = na_, row.names = FALSE)
        if(!identical(varlab, FALSE) && !is.null(attr(obj, "varlab"))){
            varlab.orig <- attr(obj, "varlab")
            varlab.mat <- cbind("name_new" = names(varlab.orig),
                                "varlab" = varlab.orig)
            write.csv(varlab.mat,
                      file = gsub(".csv$", "-varlab.csv", file),
                      row.names = FALSE, na = na_)
        }
    } else if (length(grep("xlsx$", tolower(file)))){
        wb <- openxlsx::createWorkbook()
        addWorksheet(wb, "key")
        writeDataTable(wb, sheet = "key", x = obj)
        if(!identical(varlab, FALSE) && !is.null(attr(obj, "varlab"))){
            varlab.orig <- attr(obj, "varlab")
            varlab.mat <- data.frame("name_new" = names(varlab.orig),
                                "varlab" = varlab.orig)
            addWorksheet(wb, "varlab")
            writeDataTable(wb, sheet = "varlab", x = varlab.mat)
        }
        saveWorkbook(wb, file, overwrite = TRUE)
    } else if (length(grep("rds$", tolower(file)))){
        saveRDS(obj, file = file)
    } else {
        warning("keySave: unrecognized suffix. No file created")
        NULL
    }
    invisible(obj)
}
NULL

##' Read file after deducing file type from suffix.
##'
##' If the input is XLSX, sheets named "key" and "varlab" are
##' imported if the exist. If input is CSV, then the key
##' CSV file is imported and another file suffixed with "-varlab" is
##' imported if it exists.
##'
##' The variable lables are a named vector saved as an attribute of
##' the key object.
##' @param file name of file to be imported, including path to
##'     file. file name must end in "csv", "xlsx" or "rds"
##' @param ... additional arguments for read.csv or read.xlsx.
##' @param na.strings Values to be converted to R missing symbol
##'     NA. Default is white space, "\\s+".
##' @return A data frame or matrix.
##' @importFrom utils read.csv
##' @importFrom openxlsx read.xlsx getSheetNames readWorkbook
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
keyRead <- function(file, ..., na.strings = c("\\s+")){
    ## TODO: implement code to sort out the dots arguments, find
    ## which are aimed at read.xlsx or read.csv, and divide them. See
    ## peek() function example.
    dots <- list(...)
    readxlsxFormals <- names(formals(openxlsx::read.xlsx))
    readcsvFormals <- names(formals(read.csv))
    dotsforxlsx <- dots[readxlsxFormals[readxlsxFormals %in% names(dots)]]
    dotsforcsv <- dots[readcsvFormals[readcsvFormals %in% names(dots)]]
    if (!is.character(file) || !file.exists(file)){
        messg <- paste("keyRead: 'file' not found")
        stop(messg)
    } else {
        ## key is file name, so scan for suffix
        if (length(grep("xlsx$", tolower(file))) > 0){
            sheetNames <- getSheetNames(file)
            xlsxargs <- list(xlsxFile = file, sheet = "key", colNames = TRUE,
                             check.names = FALSE, na.strings = na.strings)
            xlsxargz <- modifyList(xlsxargs, dotsforxlsx)
            key <- do.call("readWorkbook", xlsxargz)
            ## Force columns to be of type "character"
            ## replace NAs with empty strings
            for(i in colnames(key)){
                if (!inherits(key[ , i], "character")) key[ , i] <- as.character(key[ , i])
                key[which(is.na(key[ ,i])), i] <- "."
            }
            if ("varlab" %in% sheetNames){
                xlsxargz[["sheet"]] <- "varlab"
                varlab.mat <- tryCatch(do.call("readWorkbook", xlsxargz),
                                       finally = NULL)
                varlab <- varlab.mat[ , "varlab"]
                names(varlab) <- varlab.mat[ , "name_new"]
                attr(key, "varlab") <- varlab
            }
        } else if (length(grep("csv$", tolower(file))) > 0){
            csvargs <- list(file = file, stringsAsFactors = FALSE,
                            colClasses = "character", na.strings = na.strings)
            csvargz <- modifyList(csvargs, dotsforcsv)
            key <- do.call("read.csv", csvargz)
            filevarlab <- gsub("csv$", "-varlab.csv", file)
            if (file.exists(filevarlab)){
                csvargs[["file"]] <- filevarlab
                varlab.mat <- do.call("read.csv", csvargz)
                varlab <- varlab.mat[ , "varlab"]
                names(varlab) <- varlab.mat[ , "name_new"]
                attr(key, "varlab") <- varlab
            }
        } else if (length(grep("rds$", tolower(file))) > 0){
            key <- readRDS(file)
        }
    }
    invisible(key)
}
NULL

##' Convert nothing to R missing(NA).
##'
##' By "nothing", we mean white space or other indications of
##' nothingness.  Goal is to find character strings that users
##' might insert in a key to indicate missing values. Those things,
##' which are given default values in the argument nothings, will be
##' changed to NA.
##'
##' Using regular expression matching, any value that has nothing
##' except for the indicated "nothing" values is converted to NA.  The
##' "nothing" values included by default are a period by itself (A SAS
##' missing value), an empty string, or white space, meaning " ", or
##' any number of spaces, or a tab.
##' @param x A character vector. If x is not a character vector, it is
##'     returned unaltered without warning.
##' @param nothings A vector of values to be matched by regular
##'     expressions as missing.  The default vector is c("\\.",
##'     "\\s"), where "\\." means a literal period (backslashes needed
##'     to escape the symbol which would otherwise match anything in a
##'     regular expression).
##' @param zapspace Should leading and trailing white space be
##'     ignored, so that, for example " . " and "." are both treated
##'     as missing.
##' @return A vector with "nothing" values replaced by R's NA symbol.
##'     Does not alter other values in the vector. Previous version
##'     had applied zapspace to non-missing values, but it no longer
##'     does so.
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @export
##' @examples
##' gg <- c("", " ", "   ", "\t", "\t some", "some\t", " space first", ".",
##'        " . ")
##' n2NA(x = gg)
##' n2NA(x = gg, zapspace = FALSE)
##' n2NA(x = gg, nothings = c("\\s"), zapspace = FALSE)
##' n2NA(x = gg, nothings = c("\\."), zapspace = TRUE)
##' n2NA(x = gg, nothings = c("\\."), zapspace = FALSE)
n2NA <- function(x, nothings = c("\\.", "\\s"), zapspace = TRUE){
    if (!is.character(x)) {
        return(x)
    }
    if (!zapspace){
        for(j in seq_along(nothings)){
            x[grep(paste0("^", nothings[j], "*$"), x)] <- NA
        }
    } else {
        for(j in seq_along(nothings)){
            x[grep(paste0("^\\s*", nothings[j], "*\\s*$"), x)] <- NA
        }
    }
    x
}
NULL

##' Convert leading or trailing white space and tab characters to nothing.
##'
##' This eliminates any characters matched by the regular expression
##' `\\s` if they appear at the beginning of a string or at its
##' end. It does not alter spaces in the interior of a string. This
##' was created when I was not aware of R's \code{trimws} and the purpose
##' is the same. On our TODO list, we intend to eliminate this function
##' and replace its use with \code{trimws}
##' @param x A character vector
##' @return If x is a character vector, return is a character vector
##'     with leading and trailing white space values removed. If x is
##'     not a character vector, an unaltered x is returned.
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @export
##' @examples
##' x <-  c("", " ", "   ", "\t", "\t some", "some\t", " space first")
##' zapspace(x)
zapspace <- function(x){
    if (!is.character(x)){
        return(x)
    }
    y <- gsub("^\\s*", "", x)
    y <- gsub("\\s*$", "", y)
    y
}
NULL


##' Import/validate a key object or import/validate a key from a file.
##'
##' After the researcher has updated the key by filling in new names
##' and values, we import that key file. This function can import the
##' file by its name, after deducing the file type from the suffix, or
##' it can receive a key object from memory.
##'
##' This can be either a wide or long format key file.
##'
##' This cleans up variables in following ways.  1) \code{name_old}
##' and \code{name_new} have leading and trailing spaces removed 2)
##' \code{value_old} and \code{value_new} have leading and trailing
##' spaces removed, and if they are empty or blank spaces, then new
##' values are set as NA.
##'
##' Policy change concerning empty "value_new" cells in input keys
##' (20170929).
##' 
##' There is confusion about what ought to happen in a wide key when
##' the user leaves value_new as empty or missing. Literally, this
##' means all values are converted to missing, which does not seem
##' reasonable. Hence, when a key is wide, and value_new is one of the
##' na.strings elements, we assume the value_new is to be copied
##' from value_old. That is to say, if value_new is not supplied,
##' the values remain same as in old data.
##'
##' In a long key, the behavior is different.  Since the user can
##' specify each value for a variable in a separate row, the na.strings
##' appearing in value_new are treated as missing scores in the new
##' data set to be created.
##' 
##' @param key A key object (class key or keylong) or a file name
##'     character string (ending in csv, xlsx or rds).
##' @param ignoreCase In the use of this key, should we ignore
##'     differences in capitalization of the "name_old" variable?
##'     Sometimes there are inadvertent misspellings due to changes in
##'     capitalization. Columns named "var01" and "Var01" and "VAR01"
##'     probably should receive the same treatment, even if the key
##'     has name_old equal to "Var01".
##' @param sep Character separator in \code{value_old} and
##'     \code{value_new} strings in a wide key. Default is are "|".
##'     It is also allowed to use "<" for ordered variables. Use
##'     regular expressions in supplying separator values.
##' @param na.strings Values that should be converted to missing data.
##'     This is relevant in \code{name_new} as well as
##'     \code{value_new}. In spreadsheet cells, we treat "empty" cells
##'     (the string ""), or values like "." or "N/A", as missing with
##'     defaults ".", "", "\\s" (white space), and "N/A". Change that
##'     if those are not to be treated as missings.
##' @param ... additional arguments for read.csv or read.xlsx.
##' @param keynames Don't use this unless you are very careful. In our
##'     current scheme, the column names in a key should be
##'     c("name_old", "name_new", "class_old", "class_new",
##'     "value_old", "value_new", "missings", "recodes"). If your key
##'     does not use those column names, it is necessary to provide
##'     keynames in a format "our_name"="your_name". For example,
##'     keynames = c(name_old = "oldvar", name_new = "newname",
##'     class_old = "vartype", class_new = "class", value_old =
##'     "score", value_new = "val").
##' @param missSymbol Defaults to period "." as missing value
##'     indicator.
##' @export
##' @return key object, should be same "wide" or "long" as the input
##'     Missing symbols in value_old and value_new converted to ".".
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' mydf.key.path <- system.file("extdata", "mydf.key.csv", package = "kutils")
##' mydf.key <-  keyImport(mydf.key.path)
##' ## Create some dupes
##' mydf.key <- rbind(mydf.key, mydf.key[c(1,7), ])
##' mydf.key2 <- keyImport(mydf.key)
##' mydf.key2
##' ## create some empty value_new cells
##' mydf.key[c(3, 5, 7) , "value_new"] <- ""
##' mydf.key3 <- keyImport(mydf.key)
##' mydf.key3
##' mydf.keylong.path <- system.file("extdata", "mydf.key_long.csv", package = "kutils")
##' mydf.keylong <- keyImport(mydf.keylong.path)
##'
##' ## testDF is a slightly more elaborate version created for unit testing:
##' testdf.path <- system.file("extdata", "testDF.csv", package = "kutils")
##' testdf <- read.csv(testdf.path, header = TRUE)
##' keytemp <- keyTemplate(testdf, long = TRUE)
##' ## A "hand edited key file"
##' keyPath <- system.file("extdata", "testDF-key.csv", package="kutils")
##' key <- keyImport(keyPath)
##' keydiff <- keyDiff(keytemp, key)
##' key2 <- rbind(key, keydiff$neworaltered)
##' key2 <- unique(key)
##' if(interactive())View(key2)
##' 
keyImport <- function(key, ignoreCase = TRUE,
                      sep = c(character = "\\|", logical = "\\|",
                              integer = "\\|", factor = "\\|",
                              ordered = "[\\|<]", numeric = "\\|")
                      , na.strings = c("\\.", "", "\\s+",  "N/A")
                      , missSymbol = "."
                      , ...
                      , keynames = NULL)
{
    if (is.character(key)) key <- keyRead(key)

    legalClasses = c("integer", "numeric", "double", "factor",
                     "ordered", "character", "logical")
    keynames.std <- c("name_old",
                      "name_new",
                      "class_old",
                      "class_new",
                      "value_old",
                      "value_new",
                      "missings",
                      "recodes")

    if(!is.null(keynames)){
        ## User supplied keynames, so reverse their keynames
        ## to put corrected names onto the  key
        colnames(key) <- mapvalues(colnames(key), from = keynames,
                                   to = names(keynames), warn_missing=FALSE)
    }
    ## Use partial matching to try to fix column names that editor
    ## may have altered accidentally. 
    ## If key does not include all of the expected
    ## names, this uses partial matching to replace the existing
    ## unrecognized names.
    if (any(!keynames.std %in% colnames(key))) { 
        key.oldnames <- colnames(key)
        ## Filter out oldnames that don't pmatch, rm the NAs
        key.oldnames.pmatches <- na.omit(key.oldnames[pmatch(key.oldnames, keynames.std)])
        keynames.std.pmatches <- keynames.std[pmatch(key.oldnames.pmatches, keynames.std)]
        colnames(key) <- mapvalues(colnames(key),
                                   from = key.oldnames.pmatches,
                                   to = keynames.std.pmatches)
    }
    ## Now, what to do if columns are missing?
    ## if no "name_old" "class_old", "value_old", then quit
    ## if "name_new", "class_new", "value_new" missing, then copy from old.
    ## if "missings" and "recodes" are missing, create new full of ""
    ## TODO: omit blank rows
    cols.required <- c("name_old", "class_old", "value_old")
    cols.copyable <- c("name_new", "class_new", "value_new")
    cols.blankable <- c("missings", "recodes")
    if(any(!cols.required %in% colnames(key))){
        MESSG <- paste0("Key is missing columns: ",
                 paste0(cols.required[!cols.required %in% colnames(key)], collapse = " "))
        stop(MESSG)
    }
    if(any(!cols.copyable %in% colnames(key))){
        cols.missing <- cols.copyable[!cols.copyable %in% colnames(key)]
        for(i in cols.missing) {
            j <- gsub("new$", "old", i)
            key[ , i] <- key[ ,j]
        }
    }
    if(any(!cols.blankable %in% colnames(key))){
        cols.missing <- cols.blankable[!cols.blankable %in% colnames(key)]
        for(i in cols.missing) {
            key[ , i] <- ""
        }
    }

    
   
    uniquifyNameNew <- function(key, long = FALSE){
        ## wide key easy!
        if(!long){
            name_new_orig <- key[ , "name_new"]
            key[ , "name_new"] <- make.names(key[ , "name_new"], unique = TRUE)
            if (sumdupes <- sum(key[ , "name_new"] != name_new_orig)){
                MESSG <- paste("keyImport: Wide key with duplicated name_new values.",
                               sumdupes, " name_new values were altered:\n")
                cat(MESSG)
                print(key[name_new_orig != key[ , "name_new"], c("name_old", "name_new")])
            }
            return(key[ , "name_new"])
        } else {
            ## long key, TODO 20171003: Difficult to get right for
            ## sure, so lets return an error now, think on it later.
            newindex <- paste0(key[ , "name_old"], "|",  key[ , "name_new"], "|",
                               key[ , "value_old"])
            dupes <- duplicated(newindex)
            if(any(dupes)){
                cat("keyImport: These are duplicated rows in the key:\n")
                print(key[dupes, c("name_old", "name_new", "value_old")])
                MESSG <- paste("keyImport terminated: long key format is incorrect.")
                stop(MESSG)
            }
            return(key[ , "name_new"])
        }
     }

    key.orig <- key

    ## Omit key rows in which name_new is a missing or na.string
    key <- key[!isNA(key$name_new), ]
    
    ## Deduce if this is a long key. If separators | < assume it is wide
    if (any(grepl("[\\|<]", key$value_old))) {
        ## There are separators, this is almost surely a wide key
        long <- FALSE
    } else {
        long <- TRUE
    }
    
    if (!long){
        MESSG <- paste("keyImport guessed that is a wide format key.\n")
        cat(MESSG)
        ## Use the unique-ified key now
        key[ , "name_new"] <- uniquifyNameNew(key, long = FALSE)
        ## If wide key and value_new is empty/missing, make it a copy of value_old.
        ## Otherwise, the missing treatment sets all values as missing, which is pointless.
        ismissing <- isNA( key[ , "value_new"])
        key[ismissing, "value_new"] <- key[ismissing , "value_old"]
        key <- wide2long(key, sep)
        keysplit <- split(key, key[ , "name_new"])
        valcheck <- vapply(keysplit, function(keyvar){
            keyclass <- unique(keyvar$class_new)
            isTRUE(checkCoercion(keyvar$value_new,  unique(keyclass),
                                 na.strings = na.strings))
        }, logical(1))
        if(any(!valcheck)){
            MESSG <- "value_new in key cannot be coerced to class_new\n"
            cat(MESSG)
            print(keysplit[!valcheck])
            MESSG <- 
            stop(MESSG)
        }
        
        rownames(key) <- NULL
    } else {
        key[ , "name_new"] <- uniquifyNameNew(key, long = TRUE)
        keyw <- long2wide(key)
        ## Use same trick as in import of wide key to replace empty value_new
        ## with value_old
        ismissing <- grepl(paste0("^", paste0(na.strings, collapse="$|^"), "$"), keyw[ , "value_new"])
        keyw[ismissing, "value_new"] <- keyw[ismissing , "value_old"]
        key <- wide2long(keyw)
    }
    
    ## For Logicals: If value_new is 1 or 0, convert to TRUE/FALSE symbols
    key[key$class_new == "logical" & !is.na(key$value_new) & key$value_new == 1, "value_new"] <- TRUE
    key[key$class_new == "logical" & !is.na(key$value_new) & key$value_new == 0, "value_new"] <- FALSE
    key[key$class_old == "logical" & !is.na(key$value_old) & key$value_old == 1, "value_old"] <- TRUE
    key[key$class_old == "logical" & !is.na(key$value_old) & key$value_old == 0, "value_old"] <- FALSE
    
    key$missings <- gsub("<-", "< -", key$missings, fixed = TRUE)
    
    ## protect against user-inserted spaces (leading or trailing)
    key$name_old <- zapspace(key$name_old)
    key$name_new <- zapspace(key$name_new)
    
    ## if recode or missings are not supplied, we get logicals.
    ## handle empty missing/recode columns
    if (all(is.na(key$missings))) key$missings <- character(length(key$missings))
    if (all(is.na(key$recodes))) key$recodes <- character(length(key$recodes))
    
    ## If name_new is any missing symbol, remove from key
    remove <- key$name_old[key$name_new %in% na.strings]
    key <- key[!key$name_old %in% remove,]

    if (length(unique(remove)) >= 1){
        MESSG <- paste0("keyImport: Delete entries for ",
                        length(remove),
                        "variables because name_new is an na.string.\n") 
        warning(MESSG)
    }

    ## if this is long key, following is safe. How about wide key?
    key$value_old <- trimws(key$value_old)
    key$value_new <- trimws(key$value_new)

    ## pj 20171002: following appears redundant
    ## key$value_old <- ifelse(is.na(key$value_old), missSymbol, key$value_old)
    ## key$value_new <- ifelse(is.na(key$value_new), missSymbol, key$value_new)
    key[key$value_new %in% na.strings, "value_new"] <- missSymbol
    key[key$value_old %in% na.strings, "value_old"] <- missSymbol
    
    ## Delete repeated rows:
    key <- key[!duplicated(key), ]

    if (any(!unique(key$class_new) %in% legalClasses)){
        messg <- paste("Unfamiliar class_new value\n",
                       paste(unique(key$class_new)[!unique(key$class_new) %in% legalClasses], collapse = ", ")
                       )
        warning(messg)
    }

    if (long){
        class(key) <- c("keylong", "data.frame")
        attr(key, "ignoreCase") <- ignoreCase
        attr(key, "na.strings") <- na.strings
        return(key)
    } else {
        keywide <- long2wide(key)
        class(keywide) <- c("key", "data.frame")
        attr(keywide, "ignoreCase") <- ignoreCase
        attr(keywide, "na.strings") <- na.strings
        return(keywide)
    }
    stop("keyImport should not reach this point")
}



##' Convert the variable key into a list of long keys, one for each variable.
##'
##' A keylist element is a long key for one new variable. The new
##' variables are named by "name_old.name_new" for uniqueness.
##'
##' If the key has one old variable being recoded 6 ways, that begets
##' 6 elements in the resulting list. Attributes including the classes
##' of the old and new variables are included.
##'
##' @param key A key object or a file name, csv, xlsx or rds.
##' @param sep Separator regular expressions
##' @param na.strings Strings that will be treated as NA. This will be
##'     used only if the key object does not have an na.strings
##'     attribute.
##' @keywords internal
##' @return A list with one element per variable name, along with some
##'     attributes like class_old and class_new. The class is set as
##'     well, "keylist"
##' @author Paul Johnson <pauljohn@@ku.edu>
## ##' @examples
## ##' mydf.key.path <- system.file("extdata", "mydf.key.csv", package = "kutils")
## ##' mydf.key <-  keyImport(mydf.key.path)
## ##' mydf.key.keylist <- makeKeylist(mydf.key)
## ##' mydf.keylong.path <- system.file("extdata", "mydf.key_long.csv", package = "kutils")
## ##' mydf.keylong <- keyImport(mydf.keylong.path)
## ##' mydf.keylong.keylist
makeKeylist <- function(key,
                        sep = c(character = "\\|", logical = "\\|",
                              integer = "\\|", factor = "\\|",
                              ordered = "[\\|<]", numeric = "\\|"),
                        na.strings = c("\\.", "", "\\s+",  "N/A"))
{
    ## if x is in na.strings, return NA
    ## if "split" is NA or blank space or TAB, return unchanged x
    strsplit2 <- function(x, split, fixed = FALSE, perl = FALSE, useBytes = FALSE){
        if (is.na(n2NA(zapspace(split)))) return(x)
        if (is.na(n2NA(zapspace(x)))) return(NA)
        strsplit(x, split, fixed = fixed, perl = perl, useBytes = useBytes)
    }

    ## if it is already a key list, return with no changes
    if (inherits(key,"keylist")) return(key)
    if (is.character(key)) key <- keyImport(key)
    if (inherits(key, "key")){
        long <- FALSE
    } else if (inherits(key, "keylong")){
        long <- TRUE
    }
    ## Allow arguments to override na.strings from key?
    if (missing(na.strings) && !is.null(attr.na.strings <- attr(key, "na.strings"))){
        na.strings <- attr.na.strings
    }
    if (!long){
        key <- wide2long(key)
    }
    
    ## like unique, but it throws away white space, NA, and
    ## fails if there is more than one unique nonmissing value
    unique.one <- function(x){
        y <- unique(na.omit(n2NA(zapspace(x))))
        if (length(y) != 1){
            messg <- paste("Value of", deparse(substitute(x)), "not unique")
            stop (messg)
        }
        y
    }

    ## Clean up the "name_new" column. Danger that 2 name_new
    ## values are same for different variables, but they must be
    ## unique named columns.
    name_new.unique <- unique(key$name_new)
    name_new.clean <- make.names(name_new.unique, unique = TRUE)
    names(name_new.clean) <- name_new.unique
    key[ , "name_new"] <- name_new.clean[key[, "name_new"]]
    
    ## Create a keylist member for a given key chunk
    makeOneVar <- function(keyds){
        name_old <- unique.one(keyds$name_old)
        name_new <- unique.one(keyds$name_new)
        class_old <- unique.one(keyds$class_old)
        class_new <- unique.one(keyds$class_new)
        recodes <- unique(na.omit(n2NA(zapspace(keyds$recodes))))
        recodes <- if(length(recodes) > 0) unlist(strsplit(recodes, split=";", fixed = TRUE))
        missings <- paste(na.omit(n2NA(zapspace(keyds$missings))), collapse = ";")
        value_new <- keyds$value_new
        value_old <- keyds$value_old
        if (class_old == "logical") {
            value_old[value_old == "TRUE"] <- TRUE
            value_old[value_old == "FALSE"] <- FALSE
        }
        if (class_new == "logical") {
            value_new[value_new == "TRUE"] <- TRUE
            value_new[value_new == "FALSE"] <- FALSE
        }
        value_new[isNA(value_new, na.strings)] <- NA
        value_old[isNA(value_old, na.strings)] <- NA
        ## If not a factor, cast new values with class_new. Don't do this to
        ## factors, though, leave as text
        if (!class_new %in% c("factor", "ordered") && class(value_new) != class_new){
            mytext <- paste0("value_new <- as.", class_new, "(value_new)")
            value_new <- eval(parse(text = mytext))
        }
        values <- data.frame("value_old" = value_old,
                             "value_new" = value_new, stringsAsFactors = FALSE)
        list(name_old = name_old, name_new = name_new,
             class_old = class_old, class_new = class_new,
             values = values,  missings = missings, recodes = recodes)
    }
    
    ## Make this a factor and control the ordering of the levels. Otherwise,
    ## split applies factor() and re-alphabetizes it.
    name_old.new <- paste0(key[ , "name_old"], ".", key[ , "name_new"])
    name_old.new <- factor(name_old.new, levels = unique(name_old.new))
    keysplit <- split(key, name_old.new, drop = FALSE)
    keylist <- lapply(keysplit, makeOneVar)

    attr(keylist, "na.strings") <- na.strings
    attr(keylist, "class_old") <- sapply(keylist, function(keyds) keyds$class_old)
    attr(keylist, "class_new") <- sapply(keylist, function(keyds) keyds$class_new)
    class(keylist) <- "keylist"
    keylist
}
NULL




##' Apply variable key to data frame (generate recoded data frame)
##'
##' This is the main objective of the variable key system.
##' @param dframe An R data frame
##' @param key A variable key object, of class either "key" or
##'     "keylong"
##' @param diagnostic Default TRUE: Compare the old and new data
##'     frames carefully with the keyDiagnostic function.
##' @param drop Default TRUE. True implies drop = c("vars",
##'     "vals"). TRUE applies to both variables ("vars") and values
##'     ("vals"). "vars" means that a column will be omitted from data
##'     if it is not in the key "name_old". Similarly, if anything
##'     except "." appears in value_old, then setting drop="vals"
##'     means omission of a value from key "value_old" causes
##'     observations with those values to become NA.  This is the
##'     original variable key behavior.  The drop argument allows
##'     "partial keys", beginning with kutils version 1.12. drop =
##'     FALSE means that neither values nor variables are omitted.
##'     Rather than TRUE, one can specify either drop = "vars", or
##'     drop = "vals".
##' @param safeNumericToInteger Default TRUE: Should we treat values
##'     which appear to be integers as integers? If a column is
##'     numeric, it might be safe to treat it as an integer.  In many
##'     csv data sets, the values coded c(1, 2, 3) are really
##'     integers, not floats c(1.0, 2.0, 3.0). See \code{safeInteger}.
##' @param trimws Default is "both", can change to "left", "right", or
##'     set as NULL to avoid any trimming.
##' @param ignoreCase Default TRUE. If column name is capitalized
##'     differently than name_old in the key, but the two are
##'     otherwise identical, then the difference in capitalization
##'     will be ignored.
##' @param debug Default FALSE. If TRUE, emit some warnings.
##' @return A new data.frame object, with renamed and recoded variables
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @export
##' @importFrom plyr mapvalues
##' @examples
##' mydf.key.path <- system.file("extdata", "mydf.key.csv", package = "kutils")
##' mydf.key <-  keyImport(mydf.key.path)
##' mydf.path <- system.file("extdata", "mydf.csv", package = "kutils")
##'
##' mydf <- read.csv(mydf.path, stringsAsFactors = FALSE)
##' mydf2 <- keyApply(mydf, mydf.key)
##'
##' nls.keylong.path <- system.file("extdata", "natlongsurv.key_long.csv", package = "kutils")
##' nls.keylong <- keyImport(nls.keylong.path, long = TRUE)
##' data(natlongsurv)
##' nls.dat <- keyApply(natlongsurv, nls.keylong)
##'
keyApply <- function(dframe, key, diagnostic = TRUE,
                     safeNumericToInteger = TRUE, trimws = "both",
                     ignoreCase = TRUE,
                     drop = TRUE, debug = FALSE)
{
    legalClasses <- c("integer", "numeric", "double", "factor",
                      "ordered", "character", "logical")
    
    if(is.na(drop)|| is.null(drop)) stop("keyApply: drop argument is NA.")
    if (is.character(drop)){
        stopifnot(drop %in%  c("vars","vals"))
    }
    if (is.logical(drop)){
        if(drop) drop <- c("vars", "vals")
    }

    if (class(dframe)[1] != "data.frame"){
        MESSG <- paste("Warning: keyApply is intended for an R *data.frame*,",
                       "not subclasses like tibbles or data.tables.",
                       "keyApply coerced your input",
                       "with as.data.frame().\n")
        if (inherits(dframe, "data.frame")){
            ## Coerce without warning, but print message
            cat(MESSG)
        } else {
            ## Elevate the message to a warning
            warning(MESSG)
        }
        dframe <- as.data.frame(dframe)
    }
    dframe <- cleanDataFrame(dframe, safeNumericToInteger = safeNumericToInteger,
                             trimws = trimws)
    if (diagnostic) dforig <- dframe

    ## Need to snapshot class of input variables
    class_old.dframe <- sapply(dframe, function(x) class(x)[1])

    ## implement ignoreCase by keeping vector dfname_old.orig that we
    ## can use later to put old names back onto data frame.
    ## If key has multiple entries that are identcal after tolower(), will use
    ## first one.
    ## Keep vector of original names. If ignoreCase=FALSE, this changes nothing.
    dfname_old.orig <- colnames(dframe)
    if (ignoreCase){
        colnames(dframe) <- tolower(colnames(dframe))
    }
    names(dfname_old.orig) <- colnames(dframe)
    ## Create vector to use for re-assignment later
    names(class_old.dframe) <- colnames(dframe)

    na.strings <- attr(key, "na.strings")
    ## TODO: figure out what to do if class_old does not match input data
    ## coerce existing column to type requested in data frame?

    ## keylist: a list of long keys, one per variable
    keylist <- makeKeylist(key)
    ## list for collecting new variables.
    xlist <- list()

    names_in_key <- unique(key$name_old)
    if (ignoreCase) names_in_key <- tolower(names_in_key)
    names_not_in_key <- setdiff(colnames(dframe), names_in_key)

    ## B/c "vars" in drop, copy over the variables that are
    ## not in the key.
    if(length(names_not_in_key) && (!"vars" %in% drop)){
        for(jj in names_not_in_key){
            mytext <- paste0("xlist[[\"", jj, "\"]] <- ", "dframe[ , jj]")
            eval(parse(text = mytext))
        }
    }

    ## Process variables in keylist, put into "xlist"
    for (v in keylist) {
        if(debug){
            print(paste("\n debug"))
            print(v)
        }
        ## Extract values for convenience
        values <- v$values
        oldVals <- v$values$value_old
        newVals <- v$values$value_new
        ## keep spare copy of original name
        v$name_old.orig <- v$name_old
        if(ignoreCase) v$name_old <- tolower(v$name_old)

        ## TODO: what if class_old does not match class of imported
        ## data?  Need to think through implications of doing something like
        ## xnew <- as(xnew, class_old)

        ## If variable name from key is not in the data frame, go to next variable.
        if (!v$name_old %in% colnames(dframe)){
            messg <- paste("keyApply: ", v$name_old.orig, "not in data.\n")
            cat(messg)
            next()
        }

        ## DELETE VARIABLES from data frame
        ## If name_new is empty, then ignore that variable
        if (length(v$name_new) == 0 || isTRUE(isNA(unique(v$name_new)))){
            messg <- paste("keyApply: ", v$name_old.orig, "dropped.\n")
            cat(messg)
            next()
        }
        ## Extract candidate variable to column, will recode xnew.
        xnew <- dframe[ , v$name_old]

        ## Apply missing codes
        if (length(v$missings) > 0){
            xnew <- assignMissing(xnew, v$missings)
        }

        ## Be simple. If they have "recodes" in key, apply them.
        ## Ignore value_new. next().
        if (length(v$recodes) > 0 && !all(is.na(v$recodes))) {
            for (cmd in v$recodes) xnew <- assignRecode(xnew, cmd)
            if(!inherits(xnew, v$class_new)) {
                messg <- paste("keyApply: the key's recode function for the variable",
                               v$name_old, " did generate correct output class")
                print(v)
                stop(messg)
            }
            mytext <- paste0("xlist[[\"", v$name_new, "\"]] <- ", "xnew")
            eval(parse(text = mytext))
            next()
        }

        ## value_old and value_new are full of only NA, so
        ## only perform direct class conversions
        if (NROW(na.omit(values)) == 0 || length(na.omit(newVals)) == 0) {
            ## if coercion not safe, stop
            if(!isTRUE(checkCoercion(xnew, unique(v$class_new), na.strings))){
                print(v)
                stop("keyApply: coercion failed, ", unique(v$name_old.orig))
            }
            if (v$class_new %in% c("factor", "ordered")){
                # no direct conversion to factors
                xnew1 <- factor(xnew, ordered=(v$class_new == "ordered"))  
            } else {
                eval(parse(text=paste0("xnew1 <- ", "as.", v$class_new, "(xnew)")))
            }
            mytext <- paste0("xlist[[\"", v$name_new, "\"]] <- ", "xnew1")
            eval(parse(text = mytext))
            next()
        }

        ## if vals is drop, following SHOULD set as missing scores in xnew that
        ## are not in the key 
        if("vals" %in% drop){
            xnew[!xnew %in% values$value_old] <- NA
            ## pj 20171006 the factor gotca killed this:
            ## xnew <- ifelse(xnew %in% values$value_old, xnew,  NA)
        }

        ## If output is ordered or factor
        if(length(v$class_new) > 0 && v$class_new %in% c("ordered", "factor")){
            xnew2 <- plyr::mapvalues(xnew, from = values$value_old,
                                     to = values$value_new, warn_missing = FALSE)
            xnew2 <- factor(xnew2, levels = unique(values$value_new), 
                            ordered=(v$class_new == "ordered"))
            mytext <- paste0("xlist[[\"", v$name_new, "\"]] <- xnew2")
            eval(parse(text = mytext))
            next()
        }

        ## if class from data frame is not same as class_old, then MUST cast
        ## as correct type. Could cast as character.
        if ((class(xnew)[1] != v$class_old) ||
           (v$class_old %in% c("ordered", "factor"))) {
            if(!isTRUE(checkCoercion(unique(xnew), unique(v$class_old), na.strings))){
                MESSG <- paste("keyApply:", unique(v$name_old.orig),
                               "input data cannot be coerced to original class.")
                print(v)
                stop(MESSG)
            }
            xnew.orig <- xnew
            if(v$class_old %in% c("ordered", "factor")){
                ## creates factor with levels in value_old
                mytext1 <- paste0("xnew <- ", v$class_old, "(xnew, values$value_old)")
            } else {
                ## coerce to class_old
                mytext1 <- paste0("xnew <- as.", v$class_old, "(xnew)")
            }
            eval(parse(text = mytext1))
        }
        
        ##Class stays same, use mapvalues, only on values that differ:
        if (classsame <- v$class_new == v$class_old)
        {
            ## keep only rows where value_old and value_new differ
            values <- values[!mapply(identical, values$value_old,
                                     values$value_new), ]
            xnew <- plyr::mapvalues(xnew, values[ , "value_old"],
                                    values[ ,"value_new"],
                                    warn_missing = FALSE)
            mytext <- paste0("xlist[[\"", v$name_new, "\"]] <- xnew")
            eval(parse(text = mytext))
            next()
        }
        
        ## Special treatment to convert level values to integer/numeric values
        if (v$class_old %in% c("factor", "ordered") &&
            v$class_new %in% c("integer", "numeric", "double"))
        {
            if(!isTRUE(checkCoercion(values$value_new, unique(v$class_new), na.strings))){
                ## Value new cannot be coerced numeric, should stop
                MESSG <- paste("key error:", v$name_old.orig, "value_new not consistent with class_new:\n")
                print(v)
                stop(MESSG)
            }
            if(!isTRUE(checkCoercion(xnew, unique(v$class_new), na.strings))){
                ## Value new cannot be coerced numeric, should stop
                MESSG <- paste("key error: values of", v$name_old.orig,
                               "don't match class_new:\n")
                print(v)
                stop(MESSG)
            }
            
            ## otherwise continue
            xnew2 <- plyr::mapvalues(xnew, values[ , "value_old"],
                                     values[ , "value_new"],
                                     warn_missing = FALSE)
            if(!isTRUE(checkCoercion(levels(xnew2), unique(v$class_new), na.strings))){
                MESSG <- paste("keyApply value error:", v$name_old, "levels don't match class_new.")
                print(v)
                warning(MESSG)
            }
            xnew3 <- as(levels(xnew2)[xnew2], v$class_new)
            mytext <- paste0("xlist[[\"", v$name_new, "\"]] <- xnew3")
            eval(parse(text = mytext))
            next()
        }
        
        ## convert for variables with new mapping of value_old to value_new
        ## need to run this if values differ or one is NA but other not
        valueDiff <- v$values$value_old != v$values$value_new
        if (any(valueDiff, na.rm = TRUE) || any(is.na(valueDiff))) {
            xnew <- plyr::mapvalues(xnew, values$value_old, values$value_new, warn_missing = FALSE)
            mytext1 <- paste0("xnew2 <- as.", v$class_new, "(xnew)")
            eval(parse(text = mytext1))
            mytext <- paste0("xlist[[\"", v$name_new, "\"]] <- xnew2")
            eval(parse(text = mytext))
            next()
        }
        print(v)
        MESSG <- paste("keyApply:", v$name_old, "Logic error in value mapping.")
        stop(MESSG)
    }
    
    ## How to pass stringsAsFactors=FALSE as argument? Only way is
    ## run through environment?
    stringsAsFactors.orig <-  unname(unlist(options("stringsAsFactors")))
    options(stringsAsFactors = FALSE)
    dframe <- do.call(data.frame, xlist)
    options(stringsAsFactors = stringsAsFactors.orig )
    if(diagnostic) keyDiagnostic(dforig, dframe, keylist)
    dframe
}
NULL

##' Diagnose accuracy of result from applying variable key to data
##'
##' Compare the old and new data frames, checking for accuracy of
##' calculations in various ways.
##' 
##' CAUTION: This can print WIDE matrices.  Because the on-screen
##' output will be WIDE, make the display window WIDE!
##'
##' Crosstabulate new variable versus old variable to see the coding
##' mismatches.  For tables of up to 10 values or so, that will be
##' satisfactory.
##'
##' For numeric variables, it appears there is no good thing to do
##' except possibly to re-apply any transformations.
##' 
##' @param dfold Original data frame
##' @param dfnew The new recoded data frame
##' @param keylist The imported variable key that was used to
##'     transform dfold into dfnew.
##' @param max.values Show up to this number of values for the old
##'     variable
##' @param nametrunc Truncate column and row names. Needed if there
##'     are long factor labels and we want to fit more information on
##'     table. Default = 18 for new name, old name is 10 more
##'     characters (18 + 10 = 28).
##' @param wide Number of characters per row in printed
##'     output. Suggest very wide screen, default = 200.
##' @param confidential Should numbers in table be rounded to nearest
##'     "10" to comply with security standard enforced by some
##'     American research departments.
##' @return NULL
##' @author Paul Johnson <pauljohn@@ku.edu>
keyDiagnostic <- function(dfold, dfnew, keylist, max.values = 20,
                          nametrunc = 18, wide = 200, confidential = FALSE)
{

    ## TODO if class in dfnew does not match keylist specification, fail

    ## TODO think more deeply on warning signs of bad recoding.
    ## other summary of match and mismatch.

    width.orig <- options("width")
    options(width = wide)
    if (confidential) {
        roundAt <- -1
    } else {
        roundAt <- 2
    }
    for (v in keylist){
        if (is.na(v$name_new) ||(length(v$name_new) == 0) || (!v$name_new %in% colnames(dfnew))){
            messg <- paste("Variable", v$name_new, "is not included in the new data frame")
            next()
        }
        if (length(unique(dfnew[ , v$name_new])) <= max.values){
            name_new.trunc <- substr(v$name_new, 1, min(nchar(v$name_new), nametrunc))
            name_old.trunc <- paste0(substr(v$name_old, 1, min(nchar(v$name_old), nametrunc + 10)), " (old var)")
            print(round(table(dfnew[ , v$name_new], dfold[ , v$name_old],
                              exclude = NULL, dnn = c(name_new.trunc, name_old.trunc)), roundAt))
        } else {
            messg <- paste("Variable", v$name_new, "has", max.values,
                           "unique values.", "Too large for a table.")
            print(messg)
        }
    }
    options(width = unlist(width.orig))
    NULL
}


##' Convert a key object from wide to long format
##'
##' This is not flexible, assumes columns are named in our canonical
##' style, which means the columns are named c("name_old", "name_new",
##' "class_old", "class_new", "value_old", "value_new").
##' @param key A variable key in the wide format
##' @param sep Default separator is the pipe, "\\|" for most
##'     variables, while ordered accepts pipe or less than, "\\|<". If
##'     the key did not follow those customs, other sep values may be
##'     specified for each variable class.
##' @return A long format variable key
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' mydf.path <- system.file("extdata", "mydf.csv", package = "kutils")
##' mydf <- read.csv(mydf.path, stringsAsFactors=FALSE)
##' ## Target we are trying to match:
##' mydf.keylong <- keyTemplate(mydf, long = TRUE, sort = FALSE)
##' 
##' mydf.key <- keyTemplate(mydf)
##' mydf.keywide2long <- wide2long(mydf.key)
##'
##' ## rownames not meaningful in long key, so remove in both versions
##' row.names(mydf.keywide2long) <- NULL
##' row.names(mydf.keylong) <- NULL
##' all.equal(mydf.keylong, mydf.keywide2long)
wide2long <- function(key, sep = c(character = "\\|", logical = "\\|",
                              integer = "\\|", factor = "\\|",
                              ordered = "[\\|<]", numeric = "\\|"))
{
    makeOneVar <- function(x){
        value_old <- unlist(strsplit(x$value_old, sep[x$class_old]))
        value_new <- unlist(strsplit(x$value_new, sep[x$class_new]))
        if (length(value_old) == 0) value_old <- as.character("")
        if (length(value_new) == 0) value_new <- as.character("")
        values <- cbind(value_old, value_new)
        values <- unique(values)
        ## missings <- na.omit(unlist(strsplit(x$missings, ";")))
        missings <- c(x$missings, rep("", NROW(values) - length(x$missings)))
        ## recodes <- na.omit(unlist(strsplit(x$recodes, ";")))
        recodes <- c(x$recodes, rep("", NROW(values) - length(x$recodes)))

        zz <- data.frame(name_old = x$name_old,
                         name_new = x$name_new,
                         class_old = x$class_old,
                         class_new = x$class_new,
                         value_old = values[ , "value_old"],
                         value_new = values[ , "value_new"],
                         missings = missings,
                         recodes = recodes, stringsAsFactors = FALSE)
        zz <- sortStanza(zz)
        zz
    }

    ## Unique-ify name_new
    key[ , "name_new"] <- make.names(key[ , "name_new"], unique = TRUE)
    
    ## keysplit
    name_old.new <- paste0(key[ , "name_old"], ".", key[ , "name_new"])
    name_old.new <- factor(name_old.new, levels = unique(name_old.new))
    ks <- split(key, name_old.new, drop = TRUE)
    ## build a "long stanza" for each variable
    ksl <- lapply(ks, makeOneVar)

    keylong <- do.call(rbind, lapply(ksl, as.data.frame, stringsAsFactors = FALSE))
    attr(keylong, "na.strings") <- attr(key, "na.strings")
    attr(keylong, "varlab") <- attr(key, "varlab")
    class(keylong) <- c("keylong", "data.frame")
    keylong
}



##' convert a key object from long to wide format
##'
##' ##' This is not flexible, assumes columns are named in our canonical
##' style, which means the columns are named c("name_old", "name_new",
##' "class_old", "class_new", "value_old", "value_new").
##' @param keylong A variable key in the long format
##' @param na.strings Strings to be treated as missings in value_new
##' @param missSymbol Default is ".", character to insert in value when R NA is found.
##' @return A wide format variable key
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' mydf.path <- system.file("extdata", "mydf.csv", package = "kutils")
##' mydf <- read.csv(mydf.path, stringsAsFactors=FALSE)
##' ## A wide key we are trying to match:
##' mydf.key <- keyTemplate(mydf, long = FALSE, sort = TRUE)
##' mydf.key["x4", "missings"] <- "999"
##' ## A long key we will convert next
##' mydf.keylong <- keyTemplate(mydf, long = TRUE, sort = TRUE)
##' mydf.keylong[mydf.keylong[ , "name_old"] == "x4" &
##'     mydf.keylong[ , "value_old"] == "999", "missings"] <- "999"
##' mydf.long2wide <- long2wide(mydf.keylong)
##' all.equal(mydf.key, mydf.long2wide)
##' 
##' mydf.keylong.path <- system.file("extdata", "mydf.key_long.csv", package = "kutils")
##' mydf.keylong <- keyImport(mydf.keylong.path)
##' mydf.keywide <- long2wide(mydf.keylong)
##' mydf.keylong2 <- wide2long(mydf.keywide)
##' ## Is error if following not TRUE
##' all.equal(mydf.keylong2, mydf.keylong)
##' 
long2wide <- function(keylong, na.strings = c("\\.", "", "\\s+",  "N/A"),
                      missSymbol = "."){
    name_old.new <- paste0(keylong[ , "name_old"], ".", keylong[ , "name_new"])
    name_old.new <- factor(name_old.new, levels = unique(name_old.new))
    ##kls = keylong split.
    ## 20161215: why didn't I use keylist maker here?
    keylong[is.na(keylong[ , "value_new"]), "value_new"] <- missSymbol
    keylong[is.na(keylong[ , "value_old"]), "value_old"] <- missSymbol

    kls <- split(keylong, name_old.new, drop = TRUE)

    makeOneWide <- function(x){
        sep_old <- if("ordered" %in% unique(x$class_old)) "<" else "|"
        sep_new <- if("ordered" %in% unique(x$class_new)) "<" else "|"
        ## Replace "" with NA, then get rid of NAs
        if(is.null(x$missings) || all(is.na(x$missings))){
            missings <- ""
        } else {
            missings <- n2NA(unique(x$missings))
            missings <- if (all(is.na(missings))) "" else na.omit(missings)
        }
        if(is.null(x$recodes) || all(is.na(x$recodes))) {
            recodes <- ""
        } else{
            recodes <- n2NA(unique(x$recodes))
            recodes <- if (all(is.na(recodes))) "" else na.omit(recodes)
        }
        values <- cbind(value_old = x$value_old, value_new = x$value_new)
        values <- unique(values)
        values <- sortStanza(values)
        value_new.ismissing <- isNA(values[ , "value_new"])
                   
        if(any(!na.omit(value_new.ismissing))){
            newvalues <- paste(values[ , "value_new"], collapse = sep_new)
        } else {
            newvalues <- "."
        }
        list(name_old = unique(x$name_old),
             name_new = unique(x$name_new),
             class_old = unique(x$class_old),
             class_new = unique(x$class_new),
             value_old = paste(values[ , "value_old"], collapse = sep_old),
             value_new = newvalues,
             missings = paste(missings, collapse = ";"),
             recodes =  paste(recodes, collapse = ";"))
    }

    keywide <- lapply(kls, makeOneWide)

    key <- do.call("rbind", lapply(keywide, data.frame, stringsAsFactors = FALSE))
    class(key) <- c("key", "data.frame")
    attr(key, "na.strings") <- attr(keylong, "na.strings")
    attr(key, "varlab") <- attr(keylong, "varlab")
    key
}

##' An all.equal method for variable wide keys
##'
##' Disregards attributes by defaults. Before comparing the two keys,
##' the values are sorted by \code{"name_new")}.
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @method all.equal key
##' @param target A wide variable key
##' @param current A wide variable key
##' @param ... Other arguments that are ignored
##' @param check.attributes Default FALSE
##' @export
all.equal.key <- function(target, current, ..., check.attributes = FALSE){
    target <- target[order(target[ , "name_new"]), ]
    current <- current[order(current[ , "name_new"]), ]
    reslt <- base::all.equal.list(target, current, check.attributes = FALSE)
    reslt
}


##' An all.equal method for variable long keys
##'
##' Disregards attributes by defaults. Before comparing the two keys,
##' the values are sorted by \code{"name_new")}.
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @method all.equal keylong
##' @export
##' @param target A long variable key
##' @param current A long variable key
##' @param ... Other arguments that are ignored
##' @param check.attributes Default FALSE
all.equal.keylong <- function(target, current, ..., check.attributes = FALSE){
    target <- target[order(target[ , "name_new"],
                           target[ , "value_old"],
                           target[ , "value_new"]), ]
    current <- current[order(current[ , "name_new"],
                             current[ , "value_old"],
                             current[ , "value_new"]), ]
    reslt <- base::all.equal.list(target, current, check.attributes = FALSE)
    reslt
}


##' Update a key in light of a new data frame (add variables and
##' values)
##'
##' The following chores must be handled.
##' 1. If the data.frame has variables which are not currently
##' listed in the variable key's "name_old" variable, then new
##' variables are added to the key.
##' 2. If the data.frame has new values for the previously
##' existing variables, then those values must be added to the
##' keys.
##' 3. If the old key has "name_new" or "class_new" designated
##' for variables, those MUST be preserved in the new key
##' for all new values of those variables.
##'
##' This function will not alter key values for "class_old",
##' "value_old" or "value_new" for variables that have no new
##' information.
##'
##' This function deduces if the key provided is in the wide or long
##' format from the class of the object.
##' @param key A variable key
##' @param dframe A data.frame object.
##' @param append If long key, should new rows be added to the end of
##'     the updated key? Default is TRUE. If FALSE, new rows will be
##'     sorted with the original values.
##' @param safeNumericToInteger Default TRUE: Should we treat
##'     variables which appear to be integers as integers? In many csv
##'     data sets, the values coded \code{c(1, 2, 3)} are really
##'     integers, not floats \code{c(1.0, 2.0, 3.0)}. See
##'     \code{safeInteger}.
##' ## Need to consider implementing this:
##' ## @param ignoreCase
##' @export
##' @return Updated variable key.
##' @importFrom plyr rbind.fill
##' @author Ben Kite <bakite@ku.edu>
##' @examples
##' ## Original data frame has 2 variables
##' dat1 <- data.frame("Score" = c(1, 2, 3, 42, 4, 2),
##'                    "Gender" = c("M", "M", "M", "F", "F", "F"))
##' ## New data has all of original dat1, plus a new variable "Weight"
##' #and has new values for "Gender" and "Score"
##' dat2 <- plyr::rbind.fill(dat1, data.frame("Score" = 7,
##'            "Gender" = "other", "Weight" = rnorm(3)))
##' ## Create a long key for the original data, specify some
##' ## recodes for Score and Gender in value_new
##' key1.long <- keyTemplate(dat1, long = TRUE, varlab = TRUE)
##' 
##' key1.long$value_new <- gsub("42", "10", key1.long$value_new)
##' key1.long$value_new[key1.long$name_new == "Gender"] <-
##'        mgsub(c("F", "M"), c("female", "male"),
##'        key1.long$value_new[key1.long$name_new == "Gender"])  
##' key1.long[key1.long$name_old == "Score", "name_new"] <- "NewScore"
##' keyUpdate(key1.long, dat2, append = TRUE)
##' ## Throw away one row, make sure key still has Score values
##' dat2 <- dat2[-1,]
##' (key1.long.u <- keyUpdate(key1.long, dat2, append = FALSE))
##' ## Key change Score to character variable
##' key1.longc <- key1.long
##' key1.longc[key1.longc$name_old == "Score", "class_new"] <- "character"
##' keyUpdate(key1.longc, dat2, append = TRUE)
##' str(dat3 <- keyApply(dat2, key1.longc))
##' ## Now try a wide key
##' key1.wide <- keyTemplate(dat1)
##' ## Put in new values, same as in key1.long
##' key1.wide[key1.wide$name_old == "Score", c("name_new", "value_new")] <-
##'                             c("NewScore", "1|2|3|4|10|.")
##' key1.wide[key1.wide$name_old == "Gender", "value_new"] <- "female|male|."
##' ## Make sure key1.wide equivalent to key1.long:
##' ## If this is not true, it is a fail
##' all.equal(long2wide(key1.long), key1.wide, check.attributes = FALSE)
##' (key1.wide.u <- keyUpdate(key1.wide, dat2))
##' key1.long.to.wide <- long2wide(key1.long.u)
##' all.equal(key1.long.to.wide, key1.wide.u, check.attributes = FALSE)
##' str(keyApply(dat2, key1.wide.u))
##' 
##' mydf.key.path <- system.file("extdata", "mydf.key.csv", package = "kutils")
##' mydf.key <-  keyImport(mydf.key.path)
##' ##'
##' set.seed(112233)
##' N <- 20
##' ## The new Jan data arrived!
##' mydf2 <- data.frame(x5 = rnorm(N),
##'                     x4 = rpois(N, lambda = 3),
##'                     x3 = ordered(sample(c("lo", "med", "hi"),
##'                                        size = N, replace=TRUE),
##'                                 levels = c("med", "lo", "hi")),
##'                     x2 = letters[sample(c(1:4,6), N, replace = TRUE)],
##'                     x1 = factor(sample(c("jan"), N, replace = TRUE)),
##'                     x7 = ordered(letters[sample(c(1:4,6), N, replace = TRUE)]),
##'                     x6 = sample(c(1:5), N, replace = TRUE),
##'                     stringsAsFactors = FALSE)
##' mydf.key2 <- keyUpdate(mydf.key, mydf2)
##' mydf.key2
##' mydf.key2["x1", "value_old"] <- "cindy|bobby|jan|peter|marcia|greg|."
##' mydf.key2["x1", "value_new"] <- "Cindy<Bobby<Jan<Peter<Marcia<Greg<."
##' ##'
##' mydf.key.path <- system.file("extdata", "mydf.key.csv", package = "kutils")
##' mydf.path <- system.file("extdata", "mydf.csv", package = "kutils")
##' mydf <- read.csv(mydf.path, stringsAsFactors=FALSE)
##' mydf3 <- rbind(mydf, mydf2)
##' ## Now recode with revised key
##' mydf4 <- keyApply(mydf3, mydf.key2)
##' rockchalk::summarize(mydf4)
keyUpdate <- function(key, dframe, append = TRUE,
                      safeNumericToInteger = TRUE)
{
    ## it is a long key, or convert it into one
    long <- TRUE
    if (class(key)[1] == "key") {
        key <- wide2long(key)
        long <- FALSE ## key in was wide
    }
    if (class(key)[1] != "keylong") {
        messg <- paste("The key object is from the wrong class.")
        stop(messg)
    }

    dframe <- cleanDataFrame(dframe, safeNumericToInteger = safeNumericToInteger)
    keynew <- keyTemplate(dframe, long = TRUE)
    if (isTRUE(all.equal(keynew, key))){
        return(key)
    }
   
    ## pj 20170929 BIG QUESTION. If keyDiff is correct (still dont know),
    ## can't we use it, and then just take the neworaltered object and
    ## append to old key?  We never want to delete rows from key, just
    ## add new ones.  Maybe then need check for contradictions.

    ## CHECK: what does "long2wide" do when rows in a long key are
    ## "shuffled" or if the new values all exist at end of long key.

    nameval.old <- paste0(key$name_old, key$value_old)
    nameval.new <- paste0(keynew$name_old, keynew$value_old)
    ## Throw away previously observed name-value combinations
    ## Creates new rows to insert in original key long form
    keynew2 <- keynew[!nameval.new %in% nameval.old, ]

    ## if name_new and class_new are re-defined in old key,
    ## copy  those into new key
    name.old.new <- unique(key[ , c("name_old", "name_new")])
    rownames(name.old.new) <- name.old.new[ , "name_old"]

    ## Tricky if new variable arrived with data, can't just copy
    ## name_new without checking
    keynew2$name_new <- ifelse(keynew2$name_old %in% name.old.new[ , "name_old"],
                               name.old.new[keynew2$name_old, "name_new"],
                               keynew2$name_new)

    class.old.new <- unique(key[ , c("name_old", "class_old", "class_new")])
    rownames(class.old.new)<- class.old.new[ , "name_old"]
    ## for same-name_old cases, copy in class
    keynew2$class_new <- ifelse(keynew2$name_old %in% name.old.new[ , "name_old"],
                                class.old.new[keynew2$name_old, "class_new"],
                                keynew2$class_new)
    ## 20181127
    ## output <- rbind(key, keynew2)
    output <- keysPool(list(key, keynew2))
    ## User expects key returned in same format, keylong or key
    if(!long) {
        output <- long2wide(output)
        row.names(output) <- output$name_old
        return(output)
    } else {
        if (!append) {
            output <- output[order(output$name_old),]
            output <- naLast(output)
        }
        row.names(output) <- seq(1, nrow(output), 1)
        return(output)
    }
    attr(output, "na.strings") <- attr(key, "na.strings")
    output
}

##' Sort key so that non missing values are first in the
##' value vector.
##'
##' @keywords internal
##' @param key key, long or wide
##' @param byvar Default is "name_new", the column for sorting into blocks.
##' @param valvar The value variable on which sorting is to be done, "value_new".
##' @param na.strings vector of characters to be treated as NA
##' @return sorted key, with blocks that have the missings last
naLast <- function(key, byvar = "name_new", valvar = "value_new", 
                   na.strings = c("\\.", "", "\\s+",  "N/A"))
{
    ##keep attributes not equal to "names" and "row.names"
    attrs <- attributes(key)[!names(attributes(key)) %in% c("names", "row.names")]
    long <- if(inherits(key, "keylong")) TRUE else FALSE
    if(!long) key <- wide2long(key)
    keysplit <- split(key, key[ , byvar])
    for(jj in names(keysplit)) {
        keysplit[[jj]] <- sortStanza(keysplit[[jj]],
                                     na.strings = na.strings)
    }
    key <- do.call(rbind, keysplit)
    if(!long) return(long2wide(key))
    ## else long return
    key
}


##' Move missing values to last row in long key block
##' 
##' Receive key stanza and sort the rows so that the missing
##' values are last in the list. Leaves ordering of other rows
##' unchanged otherwise.
##' 
##' @param keyblock a variable key, or section of rows
##' @param valvar Default is "value_new", the column for sorting.
##' @param na.strings Stings to be treated as missing, along with R's
##'     NA symbol
##' @return keyblock, row re-arranged with missings last
##' @keywords internal
sortStanza <- function(keyblock, valvar = "value_new",
                       na.strings = c("\\.", "", "\\s+",  "N/A"))
{
    ordr <- seq_along(keyblock[ , valvar])
    ismissing <- isNA(keyblock[ , valvar])
    neworder <- c(ordr[!ismissing], ordr[ismissing])
    keyblock[neworder, , drop = FALSE]
}
    

##' Show difference between 2 keys
##'
##' @param oldkey key, original 
##' @param newkey key, possibly created by keyUpdate or by user edits
##' @return NULL, or list with as many as 2 key difference data.frames,
##'  named "deleted" and "neworaltered"
##' @author Ben Kite <bakite@@ku.edu> and Paul Johnson <pauljohn@@ku.edu>
##' @export
##' @examples
##' 
##' dat1 <- data.frame("Score" = c(1, 2, 3, 42, 4, 2),
##'                    "Gender" = c("M", "M", "M", "F", "F", "F"))
##' ## First try with a long key
##' key1 <- keyTemplate(dat1, long = TRUE)
##' key1$value_new <- gsub("42", "10", key1$value_new)
##' key1$value_new[key1$name_new == "Gender"] <-
##'        mgsub(c("F", "M"), c("female", "male"),
##'        key1$value_new[key1$name_new == "Gender"])  
##' key1[key1$name_old == "Score", "name_new"] <- "NewScore"
##' dat2 <- data.frame("Score" = 7, "Gender" = "other", "Weight" = rnorm(3))
##' dat2 <- plyr::rbind.fill(dat1, dat2)
##' dat2 <- dat2[-1,]
##' key2 <- keyUpdate(key1, dat2, append = TRUE)
##' (kdiff <- keyDiff(key1, key2))
keyDiff <- function(oldkey, newkey){
    oldkey.name <- deparse(substitute(oldkey))
    newkey.name <- deparse(substitute(newkey))
    rownames(oldkey) <- paste0(rownames(oldkey), ".old")
    rownames(newkey) <- paste0(rownames(newkey), ".new")
    oldkey$key <- "old"
    newkey$key <- "new"
    xx <- rbind(oldkey, newkey)
    ## xx$top: is a duplicate of one that went before
    xx$top <- duplicated(xx[, -match("key", colnames(xx))])
    ## xx$bot: will be duplicated below
    xx$bot <- duplicated(xx[, -match(c("key", "top"), colnames(xx))], fromLast = TRUE)
    ## lines that are in old key but not in new key:
    deleted <- xx[xx$key == "old" & !xx$bot, -match(c("key", "top", "bot"), colnames(xx)) , drop = FALSE]
    ## in newkey but not old key
    neworaltered <- xx[xx$key == "new" & !xx$top, -match(c("key", "top", "bot"), colnames(xx)), drop = FALSE]

    if(NROW(deleted) == 0  && NROW(neworaltered)  == 0){
        print("There are no differences between these keys!")
        return(NULL)
    }

    if(NROW(deleted) > 0)  {
        messg1 <- paste0("keyDiff: ", NROW(deleted),
                         " rows in ", oldkey.name,
                         " are not in ", newkey.name,
                         "\n")
        attr(deleted, "message") <- messg1
        cat(messg1)
    } else {
        deleted <- NULL
    }
     if(NROW(neworaltered) > 0){
         messg2 <- paste0("keyDiff: ", NROW(neworaltered),
                          " rows in ", newkey.name,
                          " are not in ", oldkey.name, "\n")
         attr(neworaltered, "message") <- messg2
         cat(messg2)
     }
      
    output <- list(deleted = deleted,
                   neworaltered = neworaltered)
    class(output) <- "keyDiff"
    invisible(output)
}

##' Print a keyDiff object
##'
##' @method print keyDiff
##' @export
##' @param x A keyDiff object
##' @param ... Other arguments passed through to print
##' @author Ben Kite <bakite@@ku.edu>
print.keyDiff <- function(x, ...){
    if(!is.null(x[["deleted"]])){
        dat <- x[["deleted"]]
        cat(attr(dat, "message")) 
        print(dat, ...)
    }
    if(!is.null(x[["neworaltered"]])){
        dat <- x[["neworaltered"]]
        cat(attr(dat, "message"))
        print(x[["neworaltered"]], ...)
    }
}





##' Compares keys from different data sets; finds differences classes of variables.

##' This used to check for similarity of keys from various data sets,
##' one precursor to either combining the keys or merging the data
##' sets themselves.
##'
##' When several supposedly "equivalent" data sets are used
##' to generate variable keys, there may be trouble. If variables
##' with same name have different classes, keyApply might fail
##' when applied to one of the data sets.
##'
##' This reports on differences in classes among keys. By default, it
##' looks for differences in "class_old", because that's where we
##' usually see trouble.
##'
##' The output here is diagnostic. The keys can be fixed manually, or the
##' function keysPool can implement an automatic correction.
##' @param keys A list with variable keys.
##' @param col Name of key column to check for equivalence. Default is "class_old", but
##' "class_new" can be checked as well.
##' @param excludere Exclude variables matching a regular expression
##'     (re). Default example shows exclusion of variables that end in
##'     the symbol "TEXT".
##' @return Data.frame summarizing class differences among keys
##' @author Paul Johnson
##' @export
##' @examples
##' set.seed(234)
##' dat1 <- data.frame(x1 = rnorm(100),
##'                    x2 = sample(c("Male", "Female"), 100, replace = TRUE),
##'                    x3_TEXT = "A", x4 = sample(1:10000, 100))
##' dat2 <- data.frame(x1 = rnorm(100), x2 = sample(c("Male", "Female"),
##'                    100, replace = TRUE),
##'                    x3_TEXT = sample(1:100, 100),
##'                    stringsAsFactors = FALSE)
##' key1 <- keyTemplate(dat1)
##' key2 <- keyTemplate(dat2)
##' keys <- list(key1, key2)
##' keysPoolCheck(keys)
##' ## See problem in class_old
##' keysPoolCheck(keys, col = "class_old")
##' ## problems in class_new
##' keysPoolCheck(keys, col = "class_new")
##' keysPoolCheck(keys, excludere = "TEXT$")
keysPoolCheck <- function(keys, col = "class_old", excludere = "TEXT$"){
    ## How spot trouble? class_old changes among keys?
    classnameold <- lapply(keys, function(x) {
        fst <- x[!duplicated(x$name_old), ]
        res <- fst[ , c("name_old", col)]
        rownames(res) <- NULL
        res
    })

    for(i in 2:length(keys)){
        if (i == 2){
            classmerge <- merge(classnameold[[1]], classnameold[[2]],
                                by = "name_old", suffixes = c("1", "2"))
        } else {
            classmerge <- merge(classmerge, classnameold[[i]],
                                by = "name_old", suffixes = c("", i))
        }
    }
    ## ## Suppose all detected logicals should be integer (long story why)
    ## classmerge[classmerge == "logical"] <- "integer"
    ## ## Promote all integers to numeric
    ## classmerge[classmerge == "integer"] <- "numeric"

    classmerge$troublevar <- apply(classmerge, 1, function(x){length(unique(x[grep(col, names(x))])) > 1})
    classProblems <- classmerge[classmerge$troublevar, ]
   
    classProblems <- classProblems[!grepl(excludere, classProblems$name_old), ]
    classProblems
}



##' Check a key for consistency of names, values with classes.
##'
##' Split the key into blocks of rows defined by "name_new". Within
##' these blocks, Perform these checks: 1. name_old must be
##' homogeneous (identical) within a block of rows. class_old and
##' class_new must also be identical.
##' 2. elements in "value_new" must be consistent with "class_new".
##' If values cannot be coerced to match the class specified by
##' class_new, there must be user error.
##' Same for "value_old" and "class_old".
##' @param key A variable key object.
##' @param colname Leave as default to check consistency between classes, values, and names.
##' One can specify a check only on "class_old" or "class_new", for example.  But now that
##' all work correctly, I suggest you leave the default.
##' @param na.strings A regular expression of allowed text strings that represent missings.
##' Now it amounts to any of these: ".", "NA", "N/A", or any white space or tab as signified by \\s+.
##' @return Profuse warnings and a list of failed key blocks.
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu> and Ben Kite <bakite@@ku.edu>
keyCheck <- function(key,
                     colname = c("name_new", "class_old", "class_new"),
                     na.strings = c("\\.", "", "\\s+",  "N/A")){
    if (is.character(key)){
        key <- keyRead(key)
    }
    if (prod(c("name_old", "name_new", "class_old", "class_new",
               "value_old", "value_new") %in% names(key)) != 1L){
        stop ("At a minimum a variable key needs to have the following columns: name_old, name_new, class_old, class_new, value_old, value_new")
    }
    ## Deduce if this is a long key

    if(inherits(key, "keylong")) long = TRUE else long = FALSE

    if (!long){
        xx <- strsplit(key$value_old, split = "[|<]", fixed = FALSE)
        xx.l <- sapply(xx, length)
        yy <- strsplit(key$value_new, "[|<]", fixed = FALSE)
        yy.l <- sapply(yy, length)
        inconsistent <- xx.l != yy.l
        issues <- key[inconsistent, "name_old"]
        if (length(issues) > 0){
            stop (paste0("Value error, columns: ", issues))
        }
    }
    ## Transition from Ben's work to PJ's
    if(!inherits(key, "keylong")) keylong <- kutils::wide2long(key) else keylong <- key

    keysplit <- split(keylong, keylong[ , "name_new"])

    keyfails <- list()
    for(ii in intersect(c("name_old", "class_old", "class_new"), colname)){
            ## check same-value for all of "class_old", or "class_new"
            for(jj in names(keysplit)) {
                keyblock <- keysplit[[jj]]
                ## Stanza 1: check homogeneous colname = class_old(or new) values"
                if (length(unique(keyblock[ , ii])) > 1) {
                    warning(paste("Key Violation:", jj, ii,  "\n"), immediate. = TRUE)
                    keyfails[[jj]] <- keysplit[[jj]]
                }
            }
    }
    for(ii in intersect(c("class_old", "class_new"), colname)){
        ## compare value_old(new) against
        ## class_old(new). If all non-missing cannot be coerced to
        ## indicated class, key should fail.
        for (jj in names(keysplit)) {
            keyblock <- keysplit[[jj]]
            value_col <- paste0("value_", gsub("class_(.*)", "\\1", ii))
            value <- keyblock[ , value_col]
            ## exclude any that have missing marker from na.strings
            value <- na.omit(value[!value %in% na.strings])
            testcol <- NA
            mytext <- paste0("testcol <- as.", keyblock[1, ii], "(value)")
            eval(parse(text = mytext))
            if (sum(is.na(testcol)) > 0L){
                warning(paste("Key value violation:", ii, jj, "\n"), immediate. = TRUE)
                keyfails[[jj]] <- keysplit[[jj]]
            }
            ## did not fail yet, so return NULL for fails
            ##NULL
        }
    }


    if (long){
        if (!identical(key, wide2long(long2wide(key)))){
            stop ("Key error: wide2long(long2wide(key)) fails.")
        }
    } else {
        if (!identical(key, long2wide(wide2long(key)))){
            stop ("Key error: long2wide(wide2long(key)) fails.")
        }
    }
    if (length(keyfails) > 0){
        return(keyfails)
    }else{
        message("No errors were detected")
    }
}


##' Homogenize class values and create a long key by pooling variable
##' keys.
##'
##' For long-format keys, this is one way to correct for errors in
##' "class_old" or "class_new" for common variables. For a long key
##' created by stacking together several long keys, or for a list of
##' long keys, this will try to homogenize the classes by using a
##' "highest common denominator" approach.  If one key has x1 as a
##' floating point, but another block of rows in the key has x1 as
##' integer, then class must be changed to floating point
##' (numeric). If another section of a key has x1 as a character, then
##' character becomes the class.
##'
##' Users might run keyTemplate on several data sets, arriving
##' at keys that need to be combined.  The long versions of the
##' keys can be stacked together by a function like \code{rbind}.
##' If the values class_old and class_new for a single variable are
##' inconsistent, then the "key stack" will fail the tests in keyCheck.
##' This function automates the process of fixing the class variables by
##' "promoting" classes where possible.
##'
##' Begin with a simple example.  In one data set, the value of x is
##' drawn from integers 1L, 2L, 3L, while in another set it is
##' floating values like 1.1, 2.2. After creating long format keys,
##' and stacking them together, the values of class_old will clash.
##' For x, we will observe both "integer" and "numeric" in the
##' class_old column.  In that situation, the class_old for all of the
##' rows under consideration should be set as "numeric".
##'
##' The promotion schemes are described by the variable classes, where
##' we have the most conservative changes first. The most destructive
##' change is when variables are converted from integer to character,
##' for example. The conservative conversion strategies are specified
##' in the classes variable, in which the last element in a vector
##' will be used to replace the preceeding classes.  For example,
##' c("ordered", "factor", "character") means that the class_old
##' values of "ordered" and "factor" will be replaced by "character".
##'
##' The conversions specified by classes are tried, in order.
##' 1. logical -> integer
##' 2. integer -> numeric
##' 3. ordered -> factor
##'
##' If their application fails to homogenize a vector, then class is
##' changed to "character". For example, when the value of class_old
##' observed is c("ordered", "numeric", "character"). In that case,
##' the class is promoted to "character", it is the least common
##' denominator.
##' @param keylong A list of long keys, or just one long key,
##'     presumably a result of rbinding several long keys.
##' @param keysplit Not often needed for user-level code. A list of
##'     key blocks, each of which is to be inspected and
##'     homogenized. Not used if a keylong argument is provided.
##' @param classes A list of vectors specifying legal promotions.
##' @param colnames Either c("class_old","class_new), ""class_old", or
##'     "class_new".  The former is the default.
##' @param textre A regular expression matching a column name to be
##'     treated as character. Default matches any variable name ending
##'     in "TEXT"
##' @return A class-corrected version of the same format as the input,
##'     either a long key or a list of key elements.
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @export
##' @examples
##' dat1 <- data.frame(x1 = as.integer(rnorm(100)), x2 = sample(c("Apple", "Orange"),
##'                    100, replace = TRUE), x3 = ifelse(rnorm(100) < 0, TRUE, FALSE))
##' dat2 <- data.frame(x1 = rnorm(100), x2 = ordered(sample(c("Apple", "Orange"),
##'                    100, replace = TRUE)), x3 = rbinom(100, 1, .5),
##'                    stringsAsFactors = FALSE)
##' key1 <- keyTemplate(dat1, long = TRUE)
##' key2 <- keyTemplate(dat2, long = TRUE)
##' keys2stack <- rbind(key1, key2)
##' keys2stack.fix <- keysPool(keys2stack)
##' keys2stack.fix2 <- keysPool(keys2stack.fix, colname = "class_new")
##' ## Sometimes this will not be able to homogenize
##' dat1 <- data.frame(x1 = as.integer(rnorm(100)),
##'                    x2 = sample(c("Apple", "Orange"), 100, replace = TRUE))
##' dat2 <- data.frame(x1 = rnorm(100),
##'                    x2 = sample(c("Apple", "Orange"), 100, replace = TRUE),
##'                    stringsAsFactors = FALSE)
##' key1 <- keyTemplate(dat1, long = TRUE)
##' key2 <- keyTemplate(dat2, long = TRUE)
##' ## Create a stack of keys for yourself
##' keys2stack <- rbind(key1, key2)
##' keys.fix <- keysPool(keys2stack)
##' ## We will create stack of keys for you
##' keys.fix2 <- keysPool(list(key1, key2)) 
##' ## View(keys.fix)
##' ## View(keys.fix2)
##'
##' 
##' ## If you have wide keys, convert them with wide2long, either by
##' key1 <- keyTemplate(dat1)
##' key2 <- keyTemplate(dat2)
##' keysstack.wide <- rbind(wide2long(key1), wide2long(key2))
##' keys.fix <- keysPool(keysstack.wide)
##' ## or
##' keysPool(list(wide2long(key1), wide2long(key2)))
keysPool <- function(keylong = NULL, keysplit = NULL,
                        classes = list(c("logical", "integer"),
                                       c("integer", "numeric"),
                                       c("ordered", "factor"),
                                       c("factor", "character")),
                        colnames = c("class_old","class_new"),
                        textre = "TEXT$")
{

    if (is.list(keylong) && !is.data.frame(keylong)){
        islongkey <- vapply(keylong, function(x){
            inherits(x, "keylong")
        }, logical(1))
        if (any(!islongkey)){
            MESSG <- "All elements in keylong list must be long keys"
            stop(MESSG)
        }
        keylong <- do.call(rbind, keylong)
    }
        
    if (!is.null(keylong)){
        if (!is.null(keysplit)){
            warning("keysplit is ignored because keylong was not null")
        }
        if (!inherits(keylong, "keylong")) {
            stop("keylong is not recognized as a long key")
        } else {
            NULL
        }
        if (missing(keysplit)) keysplit <- split(keylong, keylong[ , "name_new"])
    } else {
        if (is.null(keysplit)) {
            stop("A key or keysplit argument must be provided")
        }
    }

    ## Change the values of colname (say, class_old) to equal the last value of classes.
    ## keyblock: a row block from a variable key
    ## classes: vector of classes, the last of which is the acceptable one, to replace
    ## the others.
    ## colname: either "class_old" or "class_new"
    classClean <- function (keyblock, colnames = colnames, classes)
    {
        for (col in colnames) {
            keyblock[keyblock[ , col] %in% classes[-length(classes)], col] <- classes[length(classes)]
        }
        keyblock
    }

    ## if mixed, promote all to last named class
    for(i in seq_along(keysplit)){
        keyblock <- keysplit[[i]]
        if (length(unique(keyblock[, colnames[1]])) == 1) next() # use first col if two
        ## Special case. Any variable matching textre
        if (any(grepl(textre, names(keysplit)[i], ignore.case = TRUE))) {
            for (col in colnames) {
                keyblock[ , col] <- "character"
            }
            keysplit[[i]] <- unique(keyblock)
            next()
        }

        for (j in classes) {
            keyblock <- classClean(keyblock, colnames = colnames, classes = j)
            keysplit[[i]] <- unique(keyblock)
            if (length(unique(keyblock[, colnames[1]])) == 1) {
                next()
            }
        }

        if (length(unique(keyblock[, colnames[1]])) > 1) {
            MESSG <- paste("Cannot painlessly reduce key classes to homogeneous class.",
                           names(keysplit)[i], "changing class to character")
            warning(paste0(names(keysplit)[i], " ", paste(unique(keyblock[ , colnames[1]]),
                                                    collapse = " + "), ". ", MESSG), immediate. = TRUE)
            for (col in colnames) {
                keyblock[ , col] <- "character"
            }
            keysplit[[i]] <- unique(keyblock)
            ##warning(paste(names(keysplit)[i], "changing class to character"), immediate. = TRUE)
        }
    }

    ## If a key came in, give back a key
    if(!is.null(keylong)){
        keystack <- do.call(rbind, keysplit)
        class(keystack) <- c("keylong", "data.frame")
        return(keystack)
    }
    ## If a keysplit was passed in, give that back
    return(keysplit)
}


##' Checks a key for dangerous matches of old and new values in a key
##' for different levels.
##'
##' Positions in a long key are referred to as levels. If a value is
##' mismatched at levels 1 and 3, this means that issues are in rows 1
##' and 3 of the section of the given variable in a long key.
##'
##' @title keyCrossRef
##' @param key A variable key, ideally a long key. If a wide key is
##'     provided it is converted to long.
##' @param ignoreClass Classes that should be excluded from
##'     check. Useful when many integer variables are being reverse-
##'     coded. Takes a string or vector.
##' @param verbose Should a statement about the number of issues
##'     detected be returned? Defaults to FALSE.
##' @param lowercase Should old and new values be passed through
##'     tolower function? Defaults to FALSE.
##' @return Presents a warning for potentially problematic key
##'     sections. Return is dependent on verbose argument.
##' @author Ben Kite <bakite@@ku.edu>
##' @examples
##' dat <- data.frame(x1 = sample(c("a", "b", "c", "d"), 100, replace = TRUE),
##'                   x2 = sample(c("Apple", "Orange"), 100, replace = TRUE),
##'                   x3 = ordered(sample(c("low", "medium", "high"), 100, replace = TRUE),
##'                   levels = c("low", "medium", "high")),
##'                   stringsAsFactors = FALSE)
##' key <- keyTemplate(dat, long = TRUE)
##' ## No errors with a fresh key.
##' kutils:::keyCrossRef(key, verbose = TRUE)
##' key[1:2, "value_new"] <- c("b", "a")
##' key[5, "value_new"]
##' key[7:9, "value_new"] <- c("high", "medium", "low")
##' kutils:::keyCrossRef(key)
##' kutils:::keyCrossRef(key, ignoreClass = c("ordered", "character"), verbose = TRUE)
keyCrossRef <- function(key, ignoreClass = NULL, verbose = FALSE, lowercase = FALSE){
    if(!inherits(key, "keylong")){
        key <- kutils::wide2long(key)
    }
    keysplit <- split(key, key[ , "name_new"])
    problems <- 0
    for (k in keysplit){
        if (k[1, "class_new"] %in% ignoreClass){
            next()
        }
        N <- nrow(k)
        for (r in 1:length(k[,"value_new"])){
            i <- k[r, "value_new"]
            others <- k[seq(1, N)[!seq(1,N) %in% r],"value_old"]
            if (isTRUE(lowercase)){
                i <- tolower(i)
                others <- tolower(others)
            }
            if (i %in% others){
                problems <- problems + 1
                warning(k[1, "name_new"], ": The value of \"", i, "\" is at level ",
                        r, " in value_new, but \"", i,
                        "\" is also the value at level ",
                        which(k[,"value_old"] == i), " in value_old.")
            }
        }
    }
    if (isTRUE(verbose)){
        if (problems == 0){
            return("No potentially problematic value matches across levels detected.")
        } else {
            return(paste0("There are ", problems,
                          " potential issues with this key that need to be considered"))
        }
    }

}


##' Look for old (or new) names in variable key
##'
##' Use the key to find the original name of a variable that has been
##' renamed, or find the new name of an original variable.  The
##' \code{get} argument indicates if the \code{name_old} or
##' \code{name_new} is desired.
##'
##' If \code{get = "name_old"}, the return is a character vector, with
##' one element per value of \code{x}.  If there is no match for a
##' value of \code{x}, the value NA is returned for that
##' value. However, if \code{get = "name_new"}, the return might be
##' either a vector (one element per value of \code{x}) or a list with
##' one element for each value of \code{x}.  The list is returned when
##' a value of \code{x} corresponds to more than one element in
##' \code{name_old}.
##' @param x A variable name. If \code{get = "name_old"}, then
##'     \code{x} is a value for \code{name_new}. If \code{get =
##'     "name_new"}, \code{x} should be a value for \code{name_old}.
##' @param key Which key should be used
##' @param get Either "name_old" (to retrieve the original name) or
##'     "name_new" (to get the new name)
##' @return A vector or list of matches between x and either name_new
##'     or name_old elements in the key.
##' @author Paul Johnson
##' @export
##' @examples
##' mydf.key.path <- system.file("extdata", "mydf.key.csv", package = "kutils")
##' mydf.key <-  keyImport(mydf.key.path)
##' mydf.key$name_new <- paste0("new_", mydf.key$name_new)
##' keyLookup("new_x5", mydf.key, get = "name_old")
##' keyLookup(c("new_x6", "new_x1"), mydf.key, get = "name_old")
##' keyLookup(c("x6", "x1"), mydf.key, get = "name_new")
##' keyLookup(c("asdf", "new_x1"), mydf.key, get = "name_old")
##'
##' mydf.key <- rbind(mydf.key,
##'                  c("x3", "x3f",  "ordered", "factor", "","","",""))
##' keyLookup(c("x3"), mydf.key, get = "name_new")
##' keyLookup(c("x1", "x3", "x5"), mydf.key, get = "name_new")
keyLookup <- function(x, key, get = "name_old"){
    if(!length(match.arg(get, c("name_old", "name_new")))){
        MESSG <- "keyLookup: get must be 'name_old' or 'name_new'"
        stop(MESSG)
    }
    
    if(class(key)[1] == "keylong"){
        key <- long2wide(key)
    }
    
    if(get == "name_old"){
        if (any(duplicated(key$name_new))){
            MESSG <- paste0("keyLookup finds duplicates in 'name_new'")
            stop(MESSG)
        }
        target <- key$name_new[match(x, key$name_new, nomatch = NA)]
        return(target)
    }
    ## else get == "name_new"
    target <- sapply(x, function(jj){
        fits <- key[which(key$name_old %in% jj), "name_new"]
        fits
    })

    for(jj in names(target)){
        if(!length(target[[jj]])) {
            MESSG <- paste("No value in name_old matches:", jj)
            warning(MESSG)
            target[jj] <- NULL
        } else if (length(target[[jj]]) > 1){
            MESSG <- paste0("Note: name_old '", jj,
                            "' matches several values in name_new: ",
                            paste(target[[jj]], collapse = ", "))
            print(MESSG)
        }
    }
    if(length(target) == 0 || is.null(target)){
        return(NULL)
    }
    target
}



##' Import an SPSS file, create a key representing the numeric ->
##' factor transition
##'
##' This is a way to keep track of the scores that are used in the SPSS file.
##' It is also an easy way to start a new variable key that makes it convenient
##' to work on the value_new column with R text functions.
##' @param dat A character string path to the SPSS file
##' @param long TRUE returns a long key, otherwise a wide key
##' @return A variable key (long or wide)
##' @importFrom foreign read.spss
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
keyTemplateSPSS <- function(dat, long = TRUE){
    datf <- read.spss(dat, max.value.labels = 15, to.data.frame = TRUE,
                      use.value.labels = TRUE)
    datn <- read.spss(dat, to.data.frame = TRUE, use.value.labels = FALSE)
    key <- statdatKey(datf, datn, long)
    key
}



##' Import a Stata (version 12 or lower) file, create a key
##' representing the numeric -> factor transition
##'
##' This is a way to keep track of the scores that are used in the Stata file.
##' It is also an easy way to start a new variable key that makes it convenient
##' to work on the value_new column with R text functions.
##' @param dat A character string path to the Stata file
##' @param long TRUE returns a long key, otherwise a wide key
##' @return A variable key (long or wide)
##' @importFrom foreign read.dta
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
keyTemplateStata <- function(dat, long = TRUE){
    datf <- read.dta(dat, convert.factors = TRUE)
    datn <- read.dta(dat, convert.factors = FALSE)
    key <- statdatKey(datf, datn, long)
    key
}

##' keyFactors: private function that does work for keyTemplateSPSS and
##' key template Stata
##' @param datf Data frame with factors
##' @param datn Numeric data frame
##' @param long Should the result be a long or wide key
statdatKey <- function(datf, datn, long = TRUE){
    numericTF <- vapply(datf, is.numeric, logical(1)) 
    notnumeric <- colnames(datf)[!numericTF]
    isnumeric <- colnames(datf)[numericTF]
    
    partA <- lapply(isnumeric, function(i) {
        data.frame(name_old = i, name_new = i,
                   class_old = class(datf[ , i])[1], class_new = class(datf[ , i])[1],
                   value_old = ".", value_new = ".", stringsAsFactors = FALSE)
    })
    partB <- lapply(notnumeric, function(i) {
        val <- unique(datn[ , i])
        val <- val[order(val)]
        val.level <- datf[match(val, datn[ , i]),  i]
        data.frame(name_old = i, name_new = paste0(i, "f"),
                                   class_old = class(datn[ , i])[1], class_new = class(datf[ , i])[1],
                                   value_old = val, value_new = val.level, stringsAsFactors = FALSE)
        ## match(datn[match(levels(datf$Q76), datf$Q76),"Q76"])
    })
    keylong <- rbind(do.call(rbind, partA), do.call(rbind, partB))
    ## 20180418: variables did not come out in same order as SPSS.
    keywide <- long2wide(keylong)
    rownames(keywide) <- keywide$name_old
    keywide <- keywide[colnames(datf), ]
    if (!long) return(keywide)
    ## else turn the wide key long
    wide2long(keywide)
}

## unit tests for variable key
## Ben Kite
## Charlie Redmon
## 20170801

## load packages
library(RUnit)
#library(kutils)


## set data file paths (for package build)
dfPath <- system.file("extdata", "testDF.csv", package="kutils")
keyPath <- system.file("extdata", "testDF-key.csv", package="kutils")
widekeyPath <- system.file("extdata", "mydf.key.csv", package="kutils")
longkeyPath <- system.file("extdata", "mydf.key_long.csv", package="kutils")
widekeyXLSPath <- system.file("extdata", "mydf.key.xlsx", package="kutils")

## define precision level for float comparisons
floatPrecision <- 1e-6

## testing safeInteger() function:
##   1. Test safeInteger non-application (digits < tolerance)
##   2. Test safeInteger application (digits > tolerance)
test.safeInteger <- function() {
    checkIdentical(kutils::safeInteger(1.001), NULL)
    checkIdentical(kutils::safeInteger(1.0000000000001), 1L)
}

## testing assignMissing() function:
##   1. Test integers
##   2. Test characters
##   3. Test factors
##   4. Test ordered factors
##   5. Test numerics
test.assignMissing <- function() {

    ## CHECK INTEGERS
    x <- seq.int(-2L, 22L, by = 2L)

    ## specify specific values to drop
    x1 <- ifelse(x %in% c(8, 10, 18), NA, x)
    x2 <- assignMissing(x, "8;10;18")
    checkIdentical(x1, x2)

    ## specify range, inclusive
    x1 <- ifelse(x >= 4 & x <= 12, NA, x)
    x2 <- assignMissing(x, "[4,12]")
    checkIdentical(x1, x2)

    ## specify range, exclusive
    x1 <- ifelse(x > 4 & x < 12, NA, x)
    x2 <- assignMissing(x,  "(4,12)")
    checkIdentical(x1, x2)

    ## specify single inequality, exclusive
    x1 <- ifelse(x < 7, NA, x)
    x2 <- assignMissing(x, "< 7")
    checkIdentical(x1, x2)
    x1 <- ifelse(x > 11, NA, x)
    x2 <- assignMissing(x, "> 11")
    checkIdentical(x1, x2)

    ## specify single inequality, inclusive
    x1 <- ifelse(x <= 8, NA, x)
    x2 <- assignMissing(x, "<= 8")
    checkIdentical(x1, x2)

    ## specify multiple inequalities
    x1 <- ifelse(x < -1 | x==2 | x==4 | (x > 7 & x < 9) | x > 20, NA, x)
    x2 <- assignMissing(x, "< -1;2;4;(7, 9);> 20")
    checkIdentical(x1, x2)

    ## CHECK CHARACTERS
    x <- c("low", "low", "med", "high")

    ## specify multiple values to drop
    x1 <- ifelse(x %in% c("low", "high"), NA, x)
    x2 <- assignMissing(x, "low;high")
    checkIdentical(x1, x2)

    ## handling nonexistent values (possibly due to spelling mistakes)
    x1 <- ifelse(x %in% c("med","lo"), NA, x)
    x2 <- assignMissing(x, "med;lo")
    checkIdentical(x1, x2)
    
    ## test alternate separator
    x1 <- ifelse(x %in% c("low", "med"), NA, x)
    x2 <- assignMissing(x, "low|med", sep = "|")
    checkIdentical(x1, x2)

    ## CHECK FACTORS
    x <- factor(c("low", "low", "med", "high"), levels = c("low", "med", "high"))
    x1 <- x
    x1[x1 %in% c("low", "high")] <- NA
    x1 <- factor(x1)
    x2 <- assignMissing(x, "low;high")
    checkIdentical(x1, x2)

    ## CHECK ORDINAL FACTORS
    x <- ordered(c("low", "low", "med", "high"), levels = c("low", "med", "high"))
    x1 <- x
    x1[x1 %in% c("low", "high")] <- NA
    x1 <- factor(x1)
    x2 <- assignMissing(x, c("low", "high"))
    checkIdentical(x1, x2)

    ## CHECK NUMERICS
    set.seed(234234)
    x <- rnorm(10)

    ## test inequalities
    x1 <- ifelse(x < 0, NA, x)
    x2 <- assignMissing(x, "< 0")
    checkIdentical(x1, x2)
    x1 <- ifelse(x > -0.2, NA, x)
    x2 <- assignMissing(x, "> -0.2")
    checkIdentical(x1, x2)
    
    ## test ranges
    x1 <- ifelse(x > 0.1 & x < 0.7, NA, x)
    x2 <- assignMissing(x, "(0.1,0.7)")
    checkEqualsNumeric(x1, x2, tolerance=floatPrecision)
    x1 <- ifelse(x > -0.487971 & x < 0.143579, NA, x)
    x2 <- assignMissing(x, "(-0.487971, 0.143579)")
    checkEqualsNumeric(x1, x2, tolerance=floatPrecision)
}


## testing assignRecode() function:
##   1. check numeric transformations
test.assignRecode <- function() {

    set.seed(234234)
    x <- rpois(100, lambda = 3)
    x <- x[order(x)]

    ## log transformation
    xlog1 <- log(x + 1)
    xlog2 <- assignRecode(x, "log(x + 1)")
    checkEqualsNumeric(xlog1, xlog2, tolerance=floatPrecision)

    ## quadratic transformation
    xsq1 <- x^2
    xsq2 <- assignRecode(x, "x^2")
    checkEqualsNumeric(xsq1, xsq2, tolerance=floatPrecision)
    
    ## square root transformation
    xsqrt1 <- sqrt(x)
    xsqrt2 <- assignRecode(x, "sqrt(x)")
    checkEqualsNumeric(xsqrt1, xsqrt2, tolerance=floatPrecision)
    
}


## testing keyTemplate() function:
##   1. Test wide key
##   2. Test long key
##   3. Test sorting
test.keyTemplate <- function(){
    set.seed(234234)
    N <- 200
    mydf <- data.frame(x5 = rnorm(N),
                       x4 = rpois(N, lambda = 3),
                       x3 = ordered(sample(c("lo", "med", "hi"),
                                           size = N, replace=TRUE),
                                    levels = c("med", "lo", "hi")),
                       x2 = letters[sample(c(1:4,6), N, replace = TRUE)],
                       x1 = factor(sample(c("cindy", "bobby", "marcia",
                                            "greg", "peter"), N,
                                          replace = TRUE)),
                       x7 = ordered(letters[sample(c(1:4,6), N,
                                                   replace = TRUE)]),
                       x6 = sample(c(1:5), N, replace = TRUE),
                       stringsAsFactors = FALSE)
    mydf$x4[sample(1:N, 10)] <- 999
    mydf$x5[sample(1:N, 10)] <- -999

    ## test basic keyTemplate functionality
    kt <- keyTemplate(mydf)
    checkEquals(kt$class_old, kt$class_new, c("numeric", "integer", "ordered",
                                              "character", "factor", "ordered",
                                              "integer"))
    checkEquals(kt$value_old, kt$value_new, c("", "0|1|2|3|4|5|6|7|11|999",
                                              "med<lo<hi", "a|b|c|d|f",
                                              "bobby|cindy|greg|marcia|peter",
                                              "a<b<c<d<f", "1|2|3|4|5"))

    ## test long key
    ktL <- keyTemplate(mydf, long=TRUE)
    checkEquals(nrow(ktL[ktL$name_old=="x3",]), 4)
    checkEquals(unique(ktL$class_old), unique(kt$class_old))

    ## test key sorting
    ktS <- keyTemplate(mydf, sort=TRUE)
    checkEquals(ktS$name_old, paste0("x", 1:7))
    ktSvals <- unique(ktS$value_old)
    ktVals <- unique(kt$value_old)
    checkEquals(ktSvals[order(ktSvals)], ktVals[order(ktVals)])

}

## testing keyImport() function:
##   1. check that direct import produces same result as data.frame import
##   (all other parameters handled by separate functions)
test.keyImport <- function() {
    ## check wide key direct import and import from read-in data
    widekey1 <- keyImport(widekeyPath, long=FALSE)
    widekeyDF <- read.csv(widekeyPath, stringsAsFactors=FALSE)
    widekey2 <- keyImport(widekeyDF, long=FALSE)
    attr(widekey1, "varlab") <- NULL
    attr(widekey2, "varlab") <- NULL
    checkEquals(widekey1, widekey2)

    ## check long key import
    longkey1 <- keyImport(longkeyPath, long=TRUE)
    longkeyDF <- read.csv(longkeyPath, stringsAsFactors=FALSE)
    longkey2 <- keyImport(longkeyDF, long=TRUE)
    attr(longkey1, "varlab") <- NULL
    attr(longkey2, "varlab") <- NULL
    checkEquals(longkey1, longkey2)

}


## test keyApply function
test.keyApply <- function() {

    df0 <- read.csv(dfPath, stringsAsFactors=TRUE)
    key <- keyImport(keyPath)
    df1 <- keyApply(df0, key, diagnostic = FALSE)

    ## TEST CONVERSIONS FROM LOGICAL
    l <- df0[ ,"varL"]
    
    ## logical --> logical
    ll0 <- ifelse(l == FALSE, TRUE, FALSE)
    checkIdentical(ll0, df1[,"varLL"])  #compare code version with VarKey version

    ## logical --> integer (1)
    ## li0 <- as.integer(l)
    ## checkIdentical(li0, df1[,"varLI1"])

    ## logical --> integer (2)
    li0 <- as.integer(ifelse(l == FALSE, 0, 1))
    checkIdentical(li0, df1[,"varLI2"])

    ## logical --> numeric (1)
    ## ln0 <- as.numeric(l)
    ## checkEqualsNumeric(ln0, df1[,"varLN1"], tolerance=floatPrecision)
    
    ## logical --> numeric (2)
    ln0 <- ifelse(l == FALSE, -0.5, 0.5)
    checkEqualsNumeric(ln0, df1[,"varLN2"], tolerance=floatPrecision)

    ## logical --> factor (1)
    ## lf0 <- factor(l)
    ## checkIdentical(lf0, df1[,"varLF1"])
    
    ## logical --> factor (2)
    lf0 <- factor(ifelse(l == FALSE, "no", "yes"), levels=c("yes","no"))
    checkIdentical(lf0, df1[,"varLF2"])

    ## logical --> ordinal (1)
    ## lo0 <- ordered(l)
    ## checkIdentical(lo0, df1[,"varLO1"])
    
    ## logical --> ordinal (2)
    lo0 <- ordered(ifelse(l == FALSE, "fail", "pass"))
    checkIdentical(lo0, df1[,"varLO2"])

    ## logical --> character (1)
    lc0 <- as.character(l)
    checkIdentical(lc0, df1[,"varLC1"])

    ## logical --> character (2)
    lc0 <- as.character(ifelse(l == FALSE, "A", "B"))
    checkIdentical(lc0, df1[,"varLC2"])

    ## TEST CONVERSIONS FROM INTEGER
    i1 <- df0[,"varI1"]
    i2 <- df0[,"varI2"]
    
    ## integer --> logical (1)
    ## il0 <- as.logical(ifelse(i1 >= 999, NA, i1))
    ## checkIdentical(il0, df1[,"varIL1"])

    ## integer --> logical (2)
    il0 <- as.logical(ifelse(i2 == 1, TRUE, FALSE))
    checkIdentical(il0, df1[,"varIL2"])
    
    ## integer --> integer
    ii0 <- ifelse(i1 >= 999, NA, i1)
    checkIdentical(ii0, df1[,"varII"])

    ## integer --> numeric (1)
    in0 <- as.numeric(ifelse(i1 >= 999, NA, i1))
    checkEqualsNumeric(in0, df1[,"varIN1"], tolerance=floatPrecision)
 
    ## integer --> numeric (2)
    in0 <- as.numeric(plyr::mapvalues(i2, from=1:5, to=seq(1, 3, by=0.5)))
    checkEqualsNumeric(in0, df1[,"varIN2"], tolerance=floatPrecision)

    ## integer --> factor (1)
    if0 <- factor(i2)
    checkIdentical(if0, df1[,"varIF1"])
    
    ## integer --> factor (2)
    if0 <- factor(ifelse(i2 < 3, "nonHOV", "HOV"), levels=c("nonHOV","HOV"))
    checkIdentical(if0, df1[,"varIF2"])

    ## integer --> ordinal (1)
    io0 <- ordered(i2)
    checkIdentical(io0, df1[,"varIO1"])
    
    ## integer --> ordinal (2)
    io0 <- ordered(plyr::mapvalues(i2, from=1:5,
                   to=c("unfavorable", "slightlyUnfavorable", "neutral",
                        "slightlyFavorable", "favorable")),
                   levels=c("unfavorable", "slightlyUnfavorable", "neutral",
                            "slightlyFavorable", "favorable"))
    checkIdentical(io0, df1[,"varIO2"])

    ## integer --> character (1)
    ic0 <- as.character(ifelse(i1 >= 999, NA, i1))
    checkIdentical(ic0, df1[,"varIC1"])
    
    ## integer --> character (2)
    ic0 <- as.character(plyr::mapvalues(i2, from=1:5,
                                        to=c("a", "b", "c", "d", "f")))
    checkIdentical(ic0, df1[,"varIC2"])
    
    ## TEST CONVERSIONS FROM NUMERIC
    n1 <- df0[,"varN1"]
    n2 <- df0[,"varN2"]

    ## numeric --> logical (2)
    ## nl0 <- as.logical(n2)
    ## checkIdentical(nl0, df1[,"varNL1"])
    
    ## numeric --> logical (2)
    nl0 <- ifelse(n2 < 0.5, FALSE, TRUE)
    checkIdentical(nl0, df1[,"varNL2"])

    ## numeric --> integer (1)
    ni0 <- as.integer(n2)
    checkEquals(ni0, df1[,"varNI1"])
    
    ## numeric --> integer (2)
    ni0 <- as.integer(ifelse(n2 < 0.5, 0, 1))
    checkEquals(ni0, df1[,"varNI2"])

    ## numeric --> numeric
    nn0 <- ifelse(n1 <= -999, NA, n1)
    checkEqualsNumeric(nn0, df1[,"varNN"], tolerance=floatPrecision)

    ## numeric --> factor (1)
    nf0 <- factor(n2)
    checkIdentical(nf0, df1[,"varNF1"])
    
    ## numeric --> factor (2)
    nf0 <- factor(plyr::mapvalues(as.character(n2),
                  from=as.character(seq(0, 1, 0.1)),
                  to=c(rep("Q1", 3), rep(paste0("Q",2:5), each=2))))
    checkIdentical(nf0, df1[,"varNF2"])
    
    ## numeric --> ordinal (1)
    no1 <- ordered(n2)
    checkIdentical(no1, df1[,"varNO1"])

    ## numeric --> ordinal (2)
    no1 <- ordered(plyr::mapvalues(as.character(n2),
                   from=as.character(seq(0, 1, 0.1)),
                   to=c(rep("lower",5), rep("middle",4), rep("upper",2))))
    checkIdentical(no1, df1[,"varNO2"])
    
    ## numeric --> character (1)
    nc0 <- as.character(ifelse(n1 <= -999, NA, n1))
    checkIdentical(nc0, df1[,"varNC1"])

    ## numeric --> character (2)
    nc0 <- plyr::mapvalues(as.character(n2), from=as.character(seq(0, 1, 0.1)),
                           to=c("A","B","C","D","E","F","G","H","I","J","K"))
    checkIdentical(nc0, df1[,"varNC2"])

    ## TEST CONVERSIONS FROM FACTOR
    f1 <- df0[,"varF1"]
    f2 <- df0[,"varF2"]
    f3 <- df0[,"varF3"]
    f4 <- df0[,"varF4"]

    ## factor --> logical (1)
    ## fl0 <- as.logical(f1)
    ## checkIdentical(fl0, df1[,"varFL1"])
    
    ## factor --> logical (2)
    ## fl0 <- ifelse(f1 == "yes", TRUE, FALSE)
    ## checkIdentical(fl0, df1[,"varFL2"])

    ## factor --> integer (1)
    ## fi0 <- as.integer(f1)
    ## checkIdentical(fi0, df1[,"varFI1"])
    
    ## factor --> integer (2)
    fi0 <- as.integer(ifelse(f1 == "yes", 1, 0))
    checkIdentical(fi0, df1[,"varFI2"])

    ## factor --> numeric (1)
    ## fn0 <- as.numeric(f2)
    ## checkEqualsNumeric(fn0, df1[,"varFN1"], tolerance=floatPrecision)
    
    ## factor --> numeric (2)
    fn0 <- as.numeric(as.character(plyr::mapvalues(f2,
                                                   from=c("lo", "med", "hi"),
                                                   to=c(-0.5, 0, 0.5))))
    checkEqualsNumeric(fn0, df1[,"varFN2"], tolerance=floatPrecision)

    ## factor --> factor
    ff0 <- plyr::mapvalues(f3, from=levels(f3), to=c(rep("P", 4), "F"))
    checkEquals(ff0, df1[,"varFF"])

    ## factor --> ordinal (2)
    fo0 <- ordered(f2)
    checkIdentical(fo0, df1[,"varFO1"])

    ## factor --> ordinal (2)
    fo0 <- ordered(plyr::mapvalues(f2,
                   from=levels(f2), to=c("high", "low", "mid")),
                   levels=c("low","mid","high"))
    checkIdentical(fo0, df1[,"varFO2"])

    ## factor --> character (1)
    fc0 <- as.character(f4)
    checkIdentical(fc0, df1[,"varFC1"])

    
    ## factor --> character (2)
    fc0 <- as.character(plyr::mapvalues(f4, from=levels(f4),
                        to=c("Bobby","Cindy","Greg","Marcia","Peter")))
    checkIdentical(fc0, df1[,"varFC2"])

    ## TEST CONVERSIONS FROM ORDINAL
    o <- df0[,"varO1"]

    ## ordinal --> logical (1)
    ## ol0 <- as.logical(o)
    ## checkIdentical(ol0, df1[,"varOL1"])
    
    ## ordinal --> logical (2)
    ol0 <- ifelse(o == "1", FALSE, TRUE)
    checkIdentical(ol0, df1[,"varOL2"])

    ## ordinal --> integer (1)
    ## oi0 <- as.integer(o)
    ## checkIdentical(oi0, df1[,"varOI1"])
    
    ## ordinal --> integer (2)
    oi0 <- as.integer(as.character(o))
    checkIdentical(oi0, df1[,"varOI2"])

    ## ordinal --> numeric (1)
    ## on0 <- as.numeric(o)
    ## checkEqualsNumeric(on0, df1[,"varON1"], tolerance=floatPrecision)
    
    ## ordinal --> numeric (2)
    on0 <- as.numeric(as.character(o))
    checkEqualsNumeric(on0, df1[,"varON2"], tolerance=floatPrecision)

    ## ordinal --> factor (1)
    ## of0 <- factor(o, ordered=FALSE)
    ## checkIdentical(of0, df1[,"varOF1"])
    
    ## ordinal --> factor (2)
    of0 <- factor(plyr::mapvalues(o, c(1,3,5), c("A","B","C")), ordered=FALSE)
    checkIdentical(of0, df1[,"varOF2"])

    ## ordinal --> ordinal
    ## of0 <- plyr::mapvalues(o, from=c(1,3,5), to=c("low","mid","high"))
    ## checkIdentical(of0, df1[,"varOO"])

    ## ordinal --> character (1)
    ## oc0 <- as.character(o)
    ## checkIdentical(oc0, df1[,"varOC1"])
    
    ## ordinal --> character (2)
    oc0 <- as.character(plyr::mapvalues(o, from=c("1","3","5"),
                                        to=c("1","5","7")))
    checkIdentical(oc0, df1[,"varOC2"])

    ## TEST CONVERSIONS FROM CHARACTER
    c1 <- df0[,"varC1"]

    ## character --> logical (1)
    ## cl0 <- as.logical(c1)
    ## checkIdentical(cl0, df1[,"varCL1"])
    
    ## character --> logical (2)
    cl0 <- ifelse(c1 %in% c("0","0.1","0.2","0.3","0.4"), FALSE, TRUE)
    checkIdentical(cl0, df1[,"varCL2"])

    ## character --> integer (1)
    ## ci0 <- as.integer(c1)
    ## checkIdentical(ci0, df1[,"varCI1"])
    
    ## character --> integer (2)
    ci0 <- as.integer(ifelse(c1 %in% c("0","0.1","0.2","0.3","0.4"), 0, 1))
    checkIdentical(ci0, df1[,"varCI2"])

    ## character --> numeric (1)
    ## cn0 <- as.numeric(c1)
    ## checkEqualsNumeric(cn0, df1[,"varCN1"], tolerance=floatPrecision)
    
    ## character --> numeric (2)
    ## PJ What is problem with this one? I saved data onto data set
    ## cn0 <- as.numeric(plyr::mapvalues(c1, from=seq(0, 1, 0.1),
    ##                                   to=seq(-0.5, 0.5, 0.1),
    ##                                   warn_missing = TRUE))
    ## checkEqualsNumeric(cn0, df1[,"varCN2"], tolerance=floatPrecision)

    ## character --> factor (1)
    ## cf0 <- factor(c1)
    ## checkIdentical(cf0, df1[,"varCF1"])
    
    ## character --> factor (2)
    cf0 <- factor(plyr::mapvalues(c1, from=as.character(seq(0,1,0.1)),
                  to=c("X","X","X","X","Y","Y","Y","Y","Z","Z","Z")))
    checkIdentical(cf0, df1[,"varCF2"])

    ## character --> ordinal (1)
    ## co0 <- ordered(c1)
    ## checkIdentical(co0, df1[,"varCO1"])
        
    ## character --> ordinal (2) 
    co0 <- ordered(plyr::mapvalues(c1, from=as.character(seq(0,1,0.1)),
                   to=c("F","F","F","F","F","F","D","C","B","A","A")),
                   levels=c("F","D","C","B","A"))
    checkIdentical(co0, df1[,"varCO2"])

    ## character --> character
    cc0 <- plyr::mapvalues(c1, from=as.character(seq(0,1,0.1)),
                           to=letters[seq(1, 11)])
    checkIdentical(cc0, df1[,"varCC"])
}


## test wide2long() function
test.wide2long <- function() {
    ## confirm round-trip
    wkey <- keyImport(widekeyPath, long=FALSE)
    wkeylong <- wide2long(wkey)
    checkEquals(wkey, long2wide(wkeylong))
    ## Import supposedly equivalent long key
    lkey0 <- keyImport(longkeyPath, long=TRUE)
    lkey1 <- wide2long(wkey)
    ## Ignore differences in attributes
    rownames(lkey0) <- NULL
    attr(lkey0, "ignoreCase") <- NULL
    attr(lkey0, "varlab") <- NULL
    rownames(lkey1) <- NULL
    attr(lkey1, "ignoreCase") <- NULL
    attr(lkey1, "varlab") <- NULL
    checkEquals(lkey0, lkey1)
}


## test long2wide() function
test.long2wide <- function() {
    ## confirm round trip
    keylong <- keyImport(longkeyPath, long=TRUE)
    keylong2wide <- long2wide(keylong)
    checkEquals(keylong, wide2long(keylong2wide))
    ## Import supposedly equivalent wide key
    wkey0 <- keyImport(widekeyPath, long=FALSE)
    ## Confirm wkey0 equivalent to keylongwide
    checkEquals(wkey0, keylong2wide)
    ## confirm can convert long2wide output
    row.names(wkey0) <- NULL
    attr(wkey0, "ignoreCase") <- NULL
    row.names(keylong2wide) <- NULL
    attr(wkey0, "varlab") <- NULL
    attr(keylong2wide, "varlab") <- NULL
    checkEquals(wkey0, keylong2wide)
}


## test keyUpdate() function:
##   1. Add new variables to data frame
##   2. Modify existing entries in data frame
##   3. Preserve name_new and class_new in updated key
test.keyUpdate <- function() {
    ## check long key
    set.seed(1233)
    dat1 <- data.frame(Score = c(1, 2, 3, 42, 4, 2),
                       Gender = c("M", "M", NA, "F", "F", "F"),
                       stringsAsFactors = TRUE)
    ##   setting up key
    key1 <- keyTemplate(dat1, long=TRUE)
    key1[5, "value_new"] <- 10
    key1[7, "value_new"] <- "female"
    key1[8, "value_new"] <- "male"
    ##   modifying data
    dat2 <- data.frame(Score = c(7, 13, NA),
                       Gender = c("other", NA, "F"),
                       Weight = rnorm(3), stringsAsFactors = TRUE)
    dat2 <- plyr::rbind.fill(dat1, dat2)
    ##   update key
    key2 <- keyUpdate(key1, dat2, append=FALSE)
    ## difference should be 
    alteredrows <-
        structure(list(name_old = c("Gender", "Score", "Score", "Weight"
), name_new = c("Gender", "Score", "Score", "Weight"), class_old = c("factor", 
"integer", "integer", "numeric"), class_new = c("factor", "integer", 
"integer", "numeric"), value_old = c("other", "7", "13", "."), 
    value_new = c("other", "7", "13", "."), missings = c("", 
    "", "", ""), recodes = c("", "", "", "")), .Names = c("name_old", 
"name_new", "class_old", "class_new", "value_old", "value_new", 
"missings", "recodes"), row.names = c("3.new", "10.new", "11.new", 
"13.new"), class = c("keylong", "data.frame"))
    checkEquals(keyDiff(key1, key2)[[2]], alteredrows)

    ## check wide key
    key1 <- keyTemplate(dat1)
    key2 <- keyUpdate(key1, dat2)
    keydelta <- keyDiff(key1, key2)
    checkEquals(2, NROW(keydelta[[1]]))
    checkEquals(3, NROW(keydelta[[2]]))
    neworaltered <- structure(list(name_old = c("Score", "Gender", "Weight"), name_new = c("Score", 
"Gender", "Weight"), class_old = c("integer", "factor", "numeric"
), class_new = c("integer", "factor", "numeric"), value_old = c("1|2|3|4|42|7|13|.", 
"F|M|other|.", "."), value_new = c("1|2|3|4|42|7|13|.", "F|M|other|.", 
"."), missings = c("", "", ""), recodes = c("", "", "")), .Names = c("name_old", 
"name_new", "class_old", "class_new", "value_old", "value_new", 
"missings", "recodes"), row.names = c("Score.new", "Gender.new", 
"Weight.new"), class = c("key", "data.frame"))
     checkEquals(neworaltered, keydelta[[2]])
        
    ## check name preservation
    key1$name_new <- c("ScoreVar", "GenderVar")
    key2 <- keyUpdate(key1, dat2)
    ## Check that all name_new from key1 are still present in key2
    checkEquals(all(key1[ , "name_new"] %in% key2[ , "name_new"]), TRUE)
}


## ## test keyDiff() function
## ## test.keyDiff <- function() {
## ##     ## check that keyUpdate changes are reflected in keyDiff
## ##     dat1 <- data.frame(Score = c(1, 2, 3, 42, 4, 2),
## ##                        Gender = c("M", "M", "M", "F", "F", "F"))
## ##     key1 <- keyTemplate(dat1, long = TRUE)
## ##     key1[5, "value_new"] <- 10
## ##     key1[6, "value_new"] <- "female"
## ##     key1[7, "value_new"] <- "male"
## ##     key1[key1$name_old=="Score", "name_new"] <- "ScoreVar"
## ##     dat2 <- data.frame(Score = 7, Gender = "other", Weight = rnorm(3))
## ##     dat2 <- plyr::rbind.fill(dat1, dat2)
## ##     key2 <- keyUpdate(key1, dat2, append=FALSE)
## ##     diffOutput1 <- keyDiff2(key1, key2)
## ##     checkEquals(grepl("Gender", diffOutput1), TRUE)
## ##     checkEquals(grepl("ScoreVar", diffOutput1), FALSE)

## ##     ## check identity condition
## ##     diffOutput2 <- keyDiff(key1, key1)
## ##     checkEquals(grepl("no differences", diffOutput2), TRUE)
    
## ## }


## test keyRead() function:
##   1. xlsx and csv imports should be equivalent
test.keyRead <- function() {
    key1 <- kutils:::keyRead(widekeyPath)
    key2 <- kutils:::keyRead(widekeyXLSPath)
    attr(key1, "varlab") <- NULL
    attr(key2, "varlab") <- NULL
    checkEquals(key1, key2)
}


## test keyPool() function:
##   1. Legal conversions (on both or a single class column)
##   2. Non-legal conversions result in warning
test.keysPool <- function() {
    ## setup
    set.seed(1234)
    dat1 <- data.frame(x1 = as.integer(rnorm(100)),
                       x2 = sample(c("Apple", "Orange"), 100, replace = TRUE),
                       x3 = ifelse(rnorm(100) < 0, TRUE, FALSE))
    dat2 <- data.frame(x1 = rnorm(100),
                       x2 = ordered(sample(c("Apple","Orange"),100,replace=TRUE)),
                       x3 = rbinom(100, 1, .5),
                       stringsAsFactors = FALSE)
    key1 <- keyTemplate(dat1, long=TRUE)
    key2 <- keyTemplate(dat2, long=TRUE)
    stackedKeys <- rbind(key1, key2)

    ## check default usage with promotable classes
    stackedKeysFix <- kutils:::keysPool(stackedKeys)
    checkEquals(stackedKeysFix$class_old,
                stackedKeysFix$class_new,
                c(rep("numeric", 4), rep("factor", 3), rep("numeric", 5)))

    ## check case where to homogenization is not possible
    set.seed(123456)

    key3 <- structure(list(
        name_old = c("x1", "x1", "x1", "x1", "x1", "x1",
                     "x2", "x2", "x2"),
        name_new = c("x1", "x1", "x1", "x1", "x1",
                     "x1", "x2", "x2", "x2"),
        class_old = c("integer", "integer",
                      "integer", "integer", "integer", "integer", "factor", "factor",
                      "factor"),
        class_new = c("integer", "integer", "integer", "integer",
                      "integer", "integer", "factor", "factor", "factor"),
        value_old = c("-2", "-1", "0", "1", "2", ".", "Apple", "Orange", "."),
        value_new = c("-2","-1", "0", "1", "2", ".", "Apple", "Orange", "."),
        missings = c("","", "", "", "", "", "", "", ""),
        recodes = c("", "", "", "",
                    "", "", "", "", "")),
        row.names = c(NA, -9L), class = c("keylong",
                                          "data.frame"), missSymbol = ".")

    key4 <- structure(list(
        name_old = c("x1", "x2", "x2", "x2"), name_new = c("x1",
                                                           "x2", "x2", "x2"),
        class_old = c("numeric", "character", "character",
                      "character"),
        class_new = c("numeric", "character", "character",
                      "character"),
        value_old = c(".", "Apple", "Orange", "."),
        value_new = c(".", "Apple", "Orange", "."),
        missings = c("", "", "", ""),
        recodes = c("", "", "", "")),
        row.names = c(NA, -4L), class = c("keylong", "data.frame"),
        missSymbol = ".")
    stackedKeys2 <- rbind(key3, key4)
    stackedKeys2Fix <- keysPool(stackedKeys2)

    stackedKeys2Fix.saved <-
        structure(list(name_old = c("x1", "x1", "x1", "x1", "x1", "x1", 
                                    "x2", "x2", "x2"), name_new = c("x1", "x1", "x1", "x1", "x1", 
                                                                    "x1", "x2", "x2", "x2"),
                       class_old = c("numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "character", "character", 
                                     "character"), class_new = c("numeric", "numeric", "numeric", "numeric", "numeric", "numeric",
                                                                 "character", "character", "character"),
                       value_old = c("-2", "-1", "0", "1", "2", ".", "Apple", "Orange",  "."),
                       value_new = c("-2", "-1", "0", "1", "2", ".", "Apple", "Orange", "."),
                       missings = c("", "", "", "", "", "", "", "", ""),
                       recodes = c("", "", "", "", "", "", "", "", "")),
                  missSymbol = ".", row.names = c("x1.1", "x1.2", "x1.3", "x1.4", "x1.5", "x1.6", "x2.7", "x2.8", "x2.9"),
                  class = c("keylong", "data.frame"))
    stack.diff <-  keyDiff(stackedKeys2Fix, stackedKeys2Fix.saved)
    saveRDS(stack.diff, file = "stack.diff.rds")
    saveRDS(key3, file = "key3.rds")
    saveRDS(key4, file = "key4.rds")
    saveRDS(stackedKeys2Fix, file = "stackedKeys2Fix.rds")
    saveRDS(stackedKeys2Fix.saved, file = "stackedKeys2Fix.saved.rds")
    checkEquals(stack.diff, NULL)
}


## ## test keyCrossRef() function
## test.keyCrossRef <- function() {
##     ## set up key
##     dat <- data.frame(x1 = sample(c("a", "b", "c", "d"), 100, replace = TRUE),
##                      x2 = sample(c("Apple", "Orange"), 100, replace = TRUE),
##                      x3 = ordered(sample(c("low", "medium", "high"), 100, replace = TRUE),
##                                   levels = c("low", "medium", "high")),
##                      stringsAsFactors = FALSE)
##     key1 <- keyTemplate(dat, long = TRUE)
    
##     ## with a fresh key no flags should be produced
##     crossref1 <- tryCatch(kutils:::keyCrossRef(key1, verbose = TRUE),
##                           warning = function(w) w)
##     checkTrue(!("warning" %in% class(crossref1)))

##     ## modify key and check for flags
##     key2 <- key1
##     key2[1:2, "value_new"] <- c("b", "a")
##     key2[7:9, "value_new"] <- c("high", "medium", "low")
##     crossref2 <- tryCatch(kutils:::keyCrossRef(key2, verbose=TRUE),
##                           warning = function(w) w)
##     checkTrue("warning" %in% class(crossref2))
    
## }

##' Show variables, one at a time, QUICKLY and EASILY.
##'
##' This makes it easy to quickly scan through all of the columns in a
##' data frame to spot unexpected patterns or data entry errors.  Numeric variables are depicted as
##' histograms, while factor and character variables are summarized by
##' the R table function and then presented as barplots. This is most
##' useful with a large screen graphic device (try running the function
##' provided with this package, \code{dev.create(height=7, width=7)})
##' or any other method you prefer to create a large device. 
##'
##' @section Try the Defaults: Every effort has been made to make this
##'     simple and easy to use. Please run the examples as they are
##'     before becoming too concerned about customization.  This
##'     function is intended for getting a quick look at each
##'     variable, one-by-one, it is not intended to create publication
##'     quality histograms.  For sake of the fastidious users, a lot
##'     of settings can be adjusted. Users can control the parameters
##'     for presentation of histograms (parameters for \code{hist})
##'     and barplots (parameters for \code{barplot}). The function also
##'     can create frequency tables (which users can control by providing
##'     additional named arguments).
##'
##' @section Style: The histograms are standard, upright histograms.
##'     The barplots are horizontal. I chose to make the bars
##'     horizontal because long value labels are more easily
##'     accomodated on the left axis.  The code measures the length
##'     (in inches) for strings and the margin is increased
##'     accordingly.  The examples have a demonstration of that
##'     effect.
##' 
##' @section Dealing with Dots: additional named arguments,
##'     \code{...}, are inspected and sorted into groups intended to
##'     control use of R functions \code{hist}, \code{barplot},
##'     \code{table} and \code{pdf}.  \cr \cr The parameters
##'     c("exclude", "dnn", "useNA", "deparse.level") and will go to
##'     the \code{table} function, which is used to make barplots for
##'     factor and character variables. These named arguments are
##'     extracted and sent to the pdf function: c("width", "height",
##'     "onefile", "family", "title", "fonts", "version", "paper",
##'     "encoding", "bg", "fg", "pointsize", "pagecentre",
##'     "colormodel", "useDingbats", "useKerning", "fillOddEven",
##'     "compress"). Any other arguments that are unique to
##'     \code{hist} or \code{barplot} are sorted out and sent only to
##'     those functions.  \cr \cr Any other arguments, including
##'     graphical parameters will be sent to both the histogram and
##'     barplot functions, so it is a convenient way to obtain uniform
##'     appearance. Additional arguments that are common to
##'     \code{barplot} and \code{hist} will work, and so will any
##'     graphics parameters (named arguments of \code{par}, for
##'     example). However, if one wants to target some arguments to
##'     \code{hist}, but not \code{barplot}, then the \code{histargs}
##'     list argument should be used. Similarly, \code{barargs} should
##'     be used to send argument to the \code{barplot}
##'     function. Warning: the defaults for \code{histargs} and
##'     \code{barargs} include some settings that are needed for the
##'     existing design.  If new lists for \code{histargs} or
##'     \code{barargs} are supplied, the previously specified defaults
##'     are lost.  Hence, users should include the existing members of
##'     those lists, possibly with revised values.  \cr \cr All of
##'     this argument sorting effort is done in order to reduce a
##'     prolific number of warnings that were observed in previous
##'     editions of this function.
##' 
##' @param dat An R data frame or something that can be coerced to a
##'     data frame by \code{as.data.frame}
##' @param sort Default TRUE. Do you want display of the columns in
##'     alphabetical order?
##' @param file Should output go in file rather than to the screen.
##'     Default is NULL, meaning show on screen. If you supply a file
##'     name, we will write PDF output into it.
##' @param textout If TRUE, counts from histogram bins and tables will
##'     appear in the console.
##' @param ask As in the old style R \code{par(ask = TRUE)}: should
##'     keyboard interaction advance to the next plot.  Will default
##'     to false if the file argument is non-null.  If file is null,
##'     setting ask = FALSE will cause graphs to whir bye without
##'     pausing.
##' @param ... Additional arguments for the pdf, histogram, table, or
##'     barplot functions. Please see Details below.
##' @param xlabstub A text stub that will appear in the x axis
##'     label. Currently it includes advertising for this package.
##' @param freq As in the histogram frequency argument. Should graphs
##'     show counts (freq = TRUE) or proportions (AKA densities) (freq
##'     = FALSE)
##' @param histargs A list of arguments to be passed to the
##'     \code{hist} function.
##' @param barargs A list of arguments to be passed to the
##'     \code{barplot} function.
##' @export
##' @importFrom utils modifyList
##' @importFrom grDevices dev.off devAskNewPage pdf
##' @importFrom graphics strwidth par barplot.default hist.default
##' @import datasets
##' @return A vector of column names that were plotted
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @aliases histOMatic
##' @examples
##' \donttest{
##' set.seed(234234)
##' N <- 200
##' mydf <- data.frame(x5 = rnorm(N), x4 = rnorm(N), x3 = rnorm(N),
##'                    x2 = letters[sample(1:24, 200, replace = TRUE)],
##'                    x1 = factor(sample(c("cindy", "bobby", "marsha",
##'                                         "greg", "chris"), 200, replace = TRUE)),
##'                    stringsAsFactors = FALSE)
##' ## Insert 16 missings
##' mydf$x1[sample(1:150, 16,)] <- NA
##' mydf$adate <- as.Date(c("1jan1960", "2jan1960", "31mar1960", "30jul1960"), format = "%d%b%y")
##' peek(mydf)
##' peek(mydf, sort = FALSE)
##' ## Demonstrate the dot-dot-dot usage to pass in hist params
##' peek(mydf, breaks = 30, ylab = "These are Counts, not Densities", freq = TRUE)
##' ## Not Run: file output
##' ## peek(mydf, sort = FALSE, file = "three_histograms.pdf")
##' ## Use some objects from the datasets package
##' library(datasets)
##' peek(cars, xlabstub = "R cars data: ")
##' peek(EuStockMarkets, xlabstub = "Euro Market Data: ")
##' peek(EuStockMarkets, xlabstub = "Euro Market Data: ", breaks = 50,
##'      freq = TRUE)
##' ## Not run: file output
##' ## peek(EuStockMarkets, breaks = 50, file = "myeuro.pdf",
##' ##      height = 4, width=3, family = "Times")
##' ## peek(EuStockMarkets, breaks = 50, file = "myeuro-%d3.pdf",
##' ##      onefile = FALSE, family = "Times", textout = TRUE)
##' ## xlab goes into "..." and affects both histograms and barplots
##' peek(mydf, breaks = 30, ylab = "These are Counts, not Densities",
##'     freq = TRUE)
##' ## xlab is added in the barargs list.
##' peek(mydf, breaks = 30, ylab = "These are Counts, not Densities",
##'     freq = TRUE, barargs = list(horiz = TRUE, las = 1, xlab = "I'm in barargs"))
##' peek(mydf, breaks = 30, ylab = "These are Counts, not Densities", freq = TRUE,
##'      barargs = list(horiz = TRUE, las = 1, xlim = c(0, 100),
##'      xlab = "I'm in barargs, not in histargs"))
##' levels(mydf$x1) <- c(levels(mydf$x1), "arthur philpot smythe")
##' mydf$x1[4] <- "arthur philpot smythe"
##' mydf$x2[1] <- "I forgot what letter"
##' peek(mydf, breaks = 30,
##'      barargs = list(horiz = TRUE, las = 1))
##' }
  
peek <-
    function(dat, sort = TRUE, file = NULL, textout = FALSE, ask, ...,
             xlabstub = "kutils peek: ", freq = FALSE,
             histargs = list(probability = !freq),
             barargs = list(horiz = TRUE, las = 1))
{
    quickhist <- function(i){
        args <- list(xlab = paste0(xlabstub, i),
                     main = i)
        histargz <- modifyList(args, dots)
        histargz <- modifyList(histargz, histargs)
        qqq <- modifyList(list(x = dat[ , i]), histargz)
        h1 <- do.call("hist", qqq)
        if (textout){
            df1 <- data.frame("midpoints" = h1$mids, "density" = h1$density)
            cat(i, "\n")
            print(df1)
        }
    }
    
    quickbar <- function(i){
        targs <- list(dat[ , i], exclude = NULL,
                      dnn =  if (!freq) "Proportion" else "Frequency")
        targz <- modifyList(targs, dotsForTable)
        ## don't allow hist or barplot arguments to to to table, silences warning
        ## targz[c(names.par, names.barplot.unique, names.hist.unique)] <- NULL
        t1 <- do.call("table", targz)
        if (!freq) t1 <- t1/margin.table(t1)
        names(t1) <- ifelse(is.na(names(t1)), "NA", names(t1))
        ## decision 1: brute force chop
        ## names(t1) <- shorten(names(t1), k = LIMIT, unique = TRUE)
        names(t1) <- sapply(names(t1), stringbreak, k = LIMIT)
        args <- list(t1, main = i,
                     xlab = paste(xlabstub,  i, if (freq)"Frequencies" else "(Proportion)"))
        barargz <- modifyList(args, dots)
        barargz <- modifyList(barargz, barargs)
        ## Remove args that would have gone to hist
        barargz[names.hist.unique] <- NULL
        par.old <- par(no.readonly = TRUE)
        ## if longest name exceeds guess of space in margin,
        ## then pad the margin left side.
        marinch <- par("mai")
        marrequired <- max(strwidth(names(t1), units = "inches"))
        if (marrequired + 0.35 > marinch[2]) {
            marinch[2] <- marrequired + 0.5
            par(xpd = TRUE)
            par("mai" = marinch)
        }
        do.call("barplot", barargz)
                
        par(par.old)
        if (textout) {
            cat(i, "\n")
            print(t1)
        }
    }

    ## limit on character strings for barplots
    LIMIT <- 30
    
    ## We get lots of warnings about inappropriate arguments to functions.
    ## Focus on most likely objections by getting names unique to
    ## hist and removing them from the bar portion, and
    ## names unique to bar and removing from the hist portion
    ## This, however, causes a spurious device, so have to hard code parms
    ## names.par <- names(par())

    names.par <- c("xlog", "ylog", "adj", "ann", "ask", "bg", "bty",
                   "cex", "cex.axis", "cex.lab", "cex.main",
                   "cex.sub", "cin", "col", "col.axis", "col.lab",
                   "col.main", "col.sub", "cra", "crt", "csi", "cxy",
                   "din", "err", "family", "fg", "fig", "fin", "font",
                   "font.axis", "font.lab", "font.main", "font.sub",
                   "lab", "las", "lend", "lheight", "ljoin", "lmitre",
                   "lty", "lwd", "mai", "mar", "mex", "mfcol", "mfg",
                   "mfrow", "mgp", "mkh", "new", "oma", "omd", "omi",
                   "page", "pch", "pin", "plt", "ps", "pty", "smo",
                   "srt", "tck", "tcl", "usr", "xaxp", "xaxs", "xaxt",
                   "xpd", "yaxp", "yaxs", "yaxt", "ylbias")
    names.hist <- removeMatches(names(formals(hist.default)), "...")
    names.barplot <- removeMatches(names(formals(barplot.default)), "...")
    names.barplot.unique <- names.barplot[!names.barplot %in% names.hist]
    names.hist.unique <- names.hist[!names.hist %in% names.barplot]
    
    varType <- function(x){
        ifelse(is.numeric(x), "numeric",
        ifelse(is.character(x) | is.factor(x), "factor", "noneoftheabove"))
    }

    if (is.atomic(dat)){
        xclean <- xname <- deparse(substitute(dat))
        dat <- as.data.frame(dat)
        if (length(grep("\\$", xname) > 0)){
            xsplit <- unlist(strsplit(xname, "\\$"))
            xclean <- xsplit[length(xsplit)]
        }
        colnames(dat) <- xclean
    } else {
        if (!is.data.frame(dat)) dat <- as.data.frame(dat)
    }
    
    namez <- colnames(dat)
    namez <- if(sort) sort(namez) else namez
    
    colTypes <- sapply(dat, varType)
    colTypes <- colTypes[namez]

    if (length(colTypes[colTypes == "noneoftheabove"]) > 0) {
        cat(paste("These variables are being omitted because they are",
                  "neither numbers nor can they be interpreted as factors: \n"))
        cat(paste(names(colTypes[colTypes == "noneoftheabove"]), "\n"))
        ## remove unrecognized types
        colTypes <- colTypes[colTypes != "noneoftheabove"]
    }
    
    dots <- list(...)
      
    ## copy args for table, remove from dots
    tableFormals <- c("exclude", "dnn", "useNA", "deparse.level")
    dotsForTable <- dots[tableFormals[tableFormals %in% names(dots)]]
    dots[names(dotsForTable)] <- NULL

    ## copy args for pdf, remove from dots
    pdfFormals <- c("width", "height", "onefile", "family", "title", "fonts",
                    "version", "paper", "encoding", "bg", "fg", "pointsize",
                    "pagecentre", "colormodel", "useDingbats", "useKerning",
                    "fillOddEven", "compress")
    dotsForPDF <- dots[pdfFormals[pdfFormals %in% names(dots)]]

    deviceFormals <- c("width", "height", "noRStudioGD")
    dotsForDevice <- dots[deviceFormals[deviceFormals %in% names(dots)]]
    dots[names(dotsForPDF)] <- NULL
     
    
    ## Unless user sets ask, we assume TRUE if there is no file argument
    if (missing(ask)) {
        if (is.null(file)) ask <- TRUE else ask <- FALSE
    }
   
    closeFile <- FALSE 
    if (!is.null(file)){
        if (!is.character(file)) stop("file name should be a character string")
        pdfargs <- list(file = file, onefile = TRUE)
        pdfargz <- modifyList(pdfargs, dotsForPDF)
        do.call("pdf", pdfargz)
        closeFile <- TRUE
    }
    
    for (i in names(colTypes)){
        if (colTypes[i] == "numeric" ){
            quickhist(i)
            if (is.null(file) || isTRUE(ask)) devAskNewPage(TRUE)
        } else {
            quickbar(i)
            if (is.null(file) || isTRUE(ask)) devAskNewPage(TRUE)
        }
    }

    if(isTRUE(closeFile)) {
        dev.off()
    }
    namez
}





##' Create a graphics device
##'
##' This is a way to create a graphic device on screen that can
##' display R plots. It is performing the same purpose as R's dev.new,
##' but it overcomes the limitations of RStudio.  It is needed because
##' RStudio does not implement fully the functionality of
##' dev.new. This is suitable for Windows, Linux, and
##' Macintosh operating systems.
##'
##' The argument in dev.new named noRStudioGD seems to be aimed at same
##' purpose. But it does not do what I want and documentation is too
##' sparse.
##' @param ... Currently, height and width parameters that would be
##'     suitable with dev.new
##' @return NULL
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @export
##' @examples
##' \donttest{
##' if(interactive()) dev.create(height = 7, width = 3)
##' dev.off()
##' }
dev.create <- function(...){
    dots <- list(...)
    systype <- tolower(Sys.info()["sysname"])
    devtype = switch(systype,
                     "darwin"  = "quartz",
                     "linux"   = "x11",
                     "windows" = "windows")
    do.call(devtype, dots)
}

##' First draft of function to diagnose problems in merges and key variables
##'
##' This is a first effort. It works with 2 data frames and 1 key
##' variable in each. It does not work if the by parameter includes
##' more than one column name (but may work in future). The return is
##' a list which includes full copies of the rows from the data frames
##' in which trouble is observed.
##' @param x data frame
##' @param y data frame
##' @param by Commonly called the "key" variable. A column name to be
##'     used for merging (common to both \code{x} and \code{y})
##' @param by.x Column name in \code{x} to be used for merging. If not
##'     supplied, then \code{by.x} is assumed to be same as \code{by}.
##' @param by.y Column name in \code{y} to be used for merging. If not
##'     supplied, then \code{by.y} is assumed to be same as \code{by}.
##' @param incomparables values in the key (by) variable that are
##'     ignored for matching. We default to include these values as
##'     incomparables: c(NULL, NA, NaN, Inf, "\\s+", ""). Note this is
##'     a larger list of incomparables than assumed by R merge (which
##'     assumes only NULL). 
##' @return A list of data structures that are displayed for keys and
##'     data sets.  The return is \code{list(keysBad, keysDuped,
##'     unmatched)}. \code{unmatched} is a list with 2 elements, the
##'     unmatched cases from \code{x} and \code{y}.
##' @export
##' @author Paul Johnson
##' @examples
##' df1 <- data.frame(id = 1:7, x = rnorm(7))
##' df2 <- data.frame(id = c(2:6, 9:10), x = rnorm(7))
##' mc1 <- mergeCheck(df1, df2, by = "id")
##' ## Use mc1 objects mc1$keysBad, mc1$keysDuped, mc1$unmatched
##' df1 <- data.frame(id = c(1:3, NA, NaN, "", " "), x = rnorm(7))
##' df2 <- data.frame(id = c(2:6, 5:6), x = rnorm(7))
##' mergeCheck(df1, df2, by = "id")
##' df1 <- data.frame(idx = c(1:5, NA, NaN), x = rnorm(7))
##' df2 <- data.frame(idy = c(2:6, 9:10), x = rnorm(7))
##' mergeCheck(df1, df2, by.x = "idx", by.y = "idy")
mergeCheck <- function(x, y, by, by.x = by, by.y = by,
                       incomparables = c(NULL, NA, NaN, Inf, "\\s+", "")) {

    ## copied from R base merge function
    fix.by <- function(by, df) {
        if (is.null(by)) 
            by <- numeric()
        by <- as.vector(by)
        nc <- ncol(df)
        if (is.character(by)) {
            poss <- c("row.names", names(df))
            if (any(bad <- !charmatch(by, poss, 0L))) 
                stop(ngettext(sum(bad), "'by' must specify a uniquely valid column", 
                              "'by' must specify uniquely valid columns"), 
                  domain = NA)
            by <- match(by, poss) - 1L
        }
        else if (is.numeric(by)) {
            if (any(by < 0L) || any(by > nc)) 
                stop("'by' must match numbers of columns")
        }
        else if (is.logical(by)) {
            if (length(by) != nc) 
                stop("'by' must match number of columns")
            by <- seq_along(by)[by]
        }
        else stop("'by' must specify one or more columns as numbers, names or logical")
        if (any(bad <- is.na(by))) 
            stop(ngettext(sum(bad), "'by' must specify a uniquely valid column", 
                          "'by' must specify uniquely valid columns"), 
                 domain = NA)
        by.integer <- unique(by)
        colnames(df)[by.integer]
    }

    if(missing(by)){
        if(missing(by.x) || missing(by.y)) {
            MESSG <- "Must specify a column name in by or both by.x and by.y"
        }
    }
    
    by.x <- fix.by(by.x, x)
    by.y <- fix.by(by.y, y)
    
    #get data frame name
    label1 <- deparse(substitute(x))
    label2 <- deparse(substitute(y))
    
    dlist <- list(x, y)
    names(dlist) <- c(label1, label2)

    bylist <- list(by.x, by.y)
    names(bylist) <- c(label1, label2)
    
    keyEmpty <- function(df, keyName){
        df2 <- df[ df[ , keyName] %in% incomparables, , drop = FALSE]
        if(NROW(df2) > 0) return(df2) else return(NULL)
    }
    
    keyNotUnique <- function(df, keyName){
        dupes <-  df[df[ , keyName] %in% df[duplicated(df[ , keyName]), keyName], , drop = FALSE]
        if(NROW(dupes) > 0) return(dupes) else return(NULL)
    }

    keysBad <- list()
    for(i in names(dlist)) keysBad[[i]] <- keyEmpty(dlist[[i]], keyName = bylist[[i]])
    keysDuped <- list()
    for(i in names(dlist)) keysDuped[[i]] <- keyNotUnique(dlist[[i]], keyName = bylist[[i]])

    unmatched <- list()
    unmatched[[label1]] <- x[is.na(match(x[ , bylist[[label1]]], y[ , bylist[[label2]]])), , drop = FALSE]
    unmatched[[label2]] <- y[is.na(match(y[ , bylist[[label2]]], x[ , bylist[[label1]]])), , drop = FALSE]
   
    #create a list structure as a returned value
    res <- list(keysBad = keysBad, keysDuped = keysDuped, unmatched = unmatched)
    attr(res, "bylist") <- bylist
    class(res) <- "keycheck"
    ## if any results are not NULL, yell at user
    if(any(unlist(lapply(res, function(yyy) lapply(yyy, function(zz) length(zz) > 0))))){
        MESSG <- "Merge difficulties detected\n\n"
        cat(MESSG)
        print(res)
    }
    invisible(res)
}




##' Print out the result of mergeCheck function.
##'
##' This is a placeholder for a more elaborate print method
##' to be prepared in the future. Please advise us what might
##' be most helpful.
##' @param x keycheck output from mergeCheck 
##' @param ... Other arguments
##' @return None, side effect if print to screen
##' @method print keycheck
##' @export
##' @author Paul Johnson
print.keycheck <- function(x, ...){
    if (length(x$keysBad) > 0){
        MESSG <- paste("Unacceptable key values\n")
        cat(MESSG)
        lapply(names(x$keysBad), function(yy){
            cat(paste(yy, "\n"))
            print(x$keysBad[[yy]])})
    }
    if (length(x$keysDuped) > 0){
        MESSG <- paste("Duplicated key values\n")
        cat(MESSG)
        lapply(names(x$keysDuped), function(yy){
            cat(paste(yy, "\n"))
            print(x$keysDuped[[yy]])})
        }
    MESSG <- paste("Unmatched cases from", paste(names(x$unmatched), collapse = " and "), ":\n")
    cat(MESSG)
    lapply(names(x$unmatched), function(yy) {
        cat(paste(yy, "\n"))
        print(x$unmatched[[yy]])
    })
}

##' Delete trailing slash
##'
##' This function cleans up a path string by removing the trailing
##' slash.  This is necessary on MS Windows, \code{file.exists(fn)} fails
##' if "/" is on end of file name. Deleting the trailing slash is thus
##' required on Windows and it is not harmful on other platforms.
##'
##' All usages of \code{file.exists(fn)} in R should be revised
##' to be multi-platform safe by writing \code{file.exists(dts(fn))}.
##'
##' This version also removes repeated adjacent slashes, so that
##' \code{"/tmp///paul//test/"} becomes \code{"/tmp/paul/test"}.
##' @param name A path
##' @return Same path with trailing "/" removed.
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
dts <- function(name) gsub("/$", "", dms(name))
NULL

##' Delete multiple slashes, replace with one
##'
##' Sometimes paths end up with "/too//many//slashes".
##' While harmless, this is untidy. Clean it up.
##' @param name A character string to clean
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
dms <- function(name) gsub("(/)\\1+", "/", name)
NULL


##' Removes redundant words from beginnings of character strings
##'
##' In Qualtrix data, we sometimes find repeated words in column
##' names. For whatever reason, the variable names have repeated words
##' like "Philadelphia_Philadelphia_3".  This function changes a
##' vector c("Philadelphia_Philadelphia_3", "Denver_Denver_4") to
##' c("Philadelphia_3", "Denver_4"). It is non destructive, so that
##' other values will not be altered.
##' 
##' See \url{https://stackoverflow.com/questions/43711240/r-regular-expression-match-omit-several-repeats}
##' @param x Character vector
##' @param sep Delimiter. A regular expression indicating the point at
##'     which to split the strings before checking for
##'     duplicates. Default will look for repeat separated by comma,
##'     underscore, or one space character.
##' @param  n Limit on number of duplicates to remove. Default, NULL,
##'      means delete all duplicates at the beginning of a string.
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' x <- c("Philadelphia_Philadelphia_3", "Denver_Denver_4",
##'         "Den_Den_Den_Den_Den_Den_Den_5")
##' deduper(x)
##' deduper(x, n = 2)
##' deduper(x, n = 3)
##' deduper(x, n = 4)
##' x <- c("Philadelphia,Philadelphia_3", "Denver Denver_4")
##' ## Shows comma also detected by default
##' deduper(x)
##' ## Works even if delimiter is inside matched string,
##' ## or separators vary
##'  x <- c("Den_5_Den_5_Den_5,Den_5 Den_5")
##' deduper(x)
##' ## generate vector
##' x <- replicate(10, paste(sample(letters, 5), collapse = ""))
##' n <- c(paste0("_", sample(1:10, 5)), rep("", 5))
##' x <- paste0(x, "_", x, n, n)
##' x
##' deduper(x)
##'
##' @return Cleaned up vector.
deduper <- function(x, sep = ",_\\s-", n = NULL) {
    gsub(paste0("^(.*?)(?:[", sep, "]\\1){1,", n, "}"),
         "\\1", x, perl = TRUE)
}
NULL





##' How many stars would we need for this p value?
##'
##' Regression table makers need to know how many stars
##' to attach to parameter estimates. This takes
##' p values and a vector which indicates how many stars
##' are deserved.  It returns a required number of asterixes.
##' Was named "stars" in previous version, but renamed due to
##' conflict with R base function \code{stars}
##'
##' Recently, we have requests for different symbols. Some people want
##' a "+" symbol if the p value is smaller than 0.10 but greater than
##' 0.05, while some want tiny smiley faces if p is smaller than
##' 0.001. We accomodate that by allowing a user specified vector of
##' symbols, which defaults to c("*", "**", "***")
##' @param pval P value
##' @param alpha alpha vector, defaults as c(0.05, 0.01, 0.001).
##' @param symbols The default is c("*", "**", "***"), corresponding
##'     to mean that p values smaller than 0.05 receive one star,
##'     values smaller than 0.01 get two stars, and so forth.  Must be
##'     same number of elements as alpha. These need not be asterixes,
##'     could be any character strings that users desire. See example.
##' @return a character vector of symbols (eg asterixes), same length as pval
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @export
##' @examples
##' starsig(0.06)
##' starsig(0.021)
##' starsig(0.001)
##' alpha.ex <- c(0.10, 0.05, 0.01, 0.001)
##' symb.ex <- c("+", "*", "**", ":)!")
##' starsig(0.07, alpha = alpha.ex, symbols = symb.ex)
##' starsig(0.04, alpha = alpha.ex, symbols = symb.ex)
##' starsig(0.009, alpha = alpha.ex, symbols = symb.ex)
##' starsig(0.0009, alpha = alpha.ex, symbols = symb.ex)
##'
starsig <-
    function(pval, alpha = c(0.05, 0.01, 0.001),
             symbols = c("*", "**", "***"))
{
    if (length(alpha) != length(symbols)) {
        messg <- "alpha vector must have same number of elements as symbols vector"
        stop(messg)
    }
    if(is.vector(pval) && !is.numeric(pval)) pval <- as.numeric(pval)
    nstars <- sapply(pval, function(x) sum(abs(x) < alpha))
    sapply(nstars, function(x) if(!is.na(x) && x > 0) symbols[x] else " ")
}
NULL


##' Remove elements if they are in a target vector, possibly replacing with NA
##'
##' If a vector has c("A", "b", "c") and we want to remove "b" and
##' "c", this function can do the work. It can also replace "b" and
##' "c" with the NA symbol.
##'
##' If elements in y are not members of x, they are silently ignored.
##'
##' The code for this is not complicated, but it is
##' difficult to remember.  Here's the recipe to remove
##' elements y from x: \code{x <- x[!x \%in\% y[y \%in\% x]]}. It is
##' easy to get that wrong when in a hurry, so we use this function
##' instead.  The \code{padNA} was an afterthought, but it helps sometimes.
##'
##' @param x vector from which elements are to be removed
##' @param y shorter vector of elements to be removed
##' @param padNA Default FALSE, Should removed items be replaced with NA values?
##' @return a vector with elements in y removed
##' @author Ben Kite <bakite@@ku.edu> and Paul Johnson <pauljohn@@ku.edu>
##' @export
##' @examples
##' x <- c("a", "b", "c", "d", "e", "f")
##' y <- c("e", "a")
##' removeMatches(x, y)
##' y <- c("q", "r", "s")
##' removeMatches(x, y)
removeMatches <- function(x, y, padNA = FALSE){
    if (padNA) {
        x[x %in% y] <- NA
    } else {
        x <- x[!x %in% y[y %in% x]]
    }
    x
}
NULL

##' Use new information to update a vector. Similar in concept to
##' R's modify list
##'
##' Original purpose was to receive 2 named vectors, x and y, and copy
##' "updated" named values from y into x. If x or y are not named,
##' however, this will do something useful.
##' \itemize{
##' \item Both vectors are named: values in x for which y names match will be
##'     updated with values from y. If \code{augment} is true, then named
##'     values in y that are not present in x will be added to x.
##' \item If neither vector is named: returns a new vector with x as the values
##'     and y as the names. Same as returning \code{names(x) <- y}.
##' \item If x is not named, y is named: replaces elements in x with values of y
##'     where suitable (x matches names(y)). For matches, returns x = y[x]
##'     if names(y) include x.
##' \item If x is named, y is not named: returns y, but with names from x. Lengths
##'     of x and y must be identical.
##' \item If y is NULL or not provided, x is returned unaltered.
##' }
##' @param x vector to be updated, may be named or not.
##' @param y possibly a named vector. If unnamed, must match
##'     length of x. If named, and length is shorter than x, then
##'     name-value pairs in x will be replaced with name-value pairs
##'     with y. If names in y are not in x, the augment argument
##'     determines the result.
##' @param augment If TRUE, add new items in x from y. Otherwise,
##'     ignore named items in y that are not in x.
##' @param warnings Defaults as FALSE. Show warnings about augmentation
##'     of the target vector.
##' @export
##' @return an updated vector
##' @author Paul Johnson
##' @examples
##' x <- c(a = 1, b = 2, c = 3)
##' y <- c(b = 22)
##' modifyVector(x, y)
##' y <- c(c = 7, a = 13, e = 8)
##' ## If augment = TRUE, will add more elements to x
##' modifyVector(x, y, augment = TRUE)
##' modifyVector(x, y)
##' x <- c("a", "b", "c")
##' y <- c("income", "education", "sei")
##' ## Same as names(x) <- y
##' modifyVector(x, y)
##' x <- c("a", "b", "c")
##' y <- c(a = "happy")
##' modifyVector(x, y)
##' y <- c(a = "happy", g = "glum")
##' ## Will give error unless augment = TRUE
##' modifyVector(x, y, augment = TRUE)
modifyVector <- function(x, y, augment = FALSE, warnings = FALSE){
    if (missing(x) || is.null(x)) stop("modifyVector: x should not be null")
    if (missing(y) || is.null(y)) return(x)

    ## local immitation of mapvalues, with no warnings or NA checking
    replace <- function(x, from, to){
        if(is.null(from) || is.null(to)) return(x)
        idx <- match(x, from)
        idxNNA <- !is.na(idx)
        x[idxNNA] <- to[idx[idxNNA]]
        x
    }
    
    ## neither has names, so values of y are names for x
    if (is.null(names(x)) && is.null(names(y))){
        if (length(x) == length(y)){
            names(x) = y
        } else{
            MESSG <- paste("if neither x nor y has names,",
                           "then x and y must be of same length")
            stop(MESSG)
        }
        return(x)
    }
    
    ## x has names, but y does not,  y is new values of x
    ## but x keeps old names
    if (!is.null(names(x)) && is.null(names(y))){
        if (length(x) == length(y)){
            x.names <- names(x)
            x <- y
            names(x) <- x.names
        } else{
            MESSG <- paste("if y has no names,",
                           "x and y must be of same length")
            stop(MESSG)
        }
        return(x)
    }
    
    ## x has no names, but y does.  Assume user meant
    ## to replace x *values* by y *names* and values. 
    if (is.null(names(x)) && !is.null(names(y))){
        ## x has no names, so we will give x its values
        ## as its names. 
        names(x) <- x
        return(modifyVector(x, y, augment))
    }
    
    ## x and y both have names, y may be different in length
    x.names <- names(x)
    y.names <- names(y)
    y.unique <- y[!y.names %in% x.names]
    y.names.unique <- names(y.unique)
    y.names.notunique <- y.names[!y.names %in% y.names.unique]

    ## 20180509: this appears to have been an error
    ##x <- replace(x, y.names, y)
    x <- replace(x, x[y.names.notunique], y[y.names.notunique])
    
    if(augment){
        x <- c(x, y.unique)
    } else {
        if(length(y.unique) > 0 && warnings){
            MESSG <- paste("if augment = FALSE, elements in y",
                           "with names not in names(x) will be discarded")
            warning(MESSG)
        }
    }            
    x
}
NULL


##' apply a vector of replacements, one after the other.
##'
##' This is multi-gsub.  Use it when it is necessary to process
##' many patterns and replacements in a given order on a vector.
##'
##' @param pattern vector of values to be replaced. A vector filled
##'     with patterns as documented in the \code{gsub} pattern
##'     argument
##' @param replacement vector of replacements, otherwise same as
##'     \code{gsub}.  Length of replacement must be either 1 or same
##'     as pattern, otherwise an error results.
##' @param x the vector in which elements are to be replaced, same as
##'     \code{gsub}
##' @param ... Additional arguments to be passed to gsub
##' @return vector with pattern replaced by replacement
##' @author Jared Harpole <jared.harpole@@gmail.com> and Paul Johnson
##'     <pauljohn@@ku.edu>
##' @export
##' @examples
##' x <- c("Tom", "Jerry", "Elmer", "Bugs")
##' pattern <- c("Tom", "Bugs")
##' replacement <- c("Thomas", "Bugs Bunny")
##' (y <- mgsub(pattern, replacement, x))
##' x[1] <- "tom"
##' (y <- mgsub(pattern, replacement, x, ignore.case = TRUE))
##' (y <- mgsub(c("Elmer", "Bugs"), c("Looney Characters"), x, ignore.case = TRUE))
mgsub <- function(pattern, replacement, x, ... ){
    if (length(pattern) != length(replacement)) {
        if (length(replacement) == 1) {
            replacement <- rep(replacement, length(pattern))
        } else {
            messg <- paste("replacement must either be 1 element or the same number of elements as pattern")
            stop(messg)
        }
    }
    for (i in seq_along(pattern)){
        x <- gsub(pattern[i], replacement[i], x, ...)
    }
    x
}



##' Reduce each in a vector of strings to a given length
##'
##' This is a simple "chop" at k characters, no fancy truncation at
##' spaces or such. Optionally, this will make unique the resulting
##' truncated strings. That way, truncation at character 4 of
##' "Washington" and "Wash" and "Washingham" will not result in 3
##' values of "Wash", but rather "Wash", "Wash.1", and "Wash.2"
##' @param x character string
##' @param k integer limit on length of string. Default is 20
##' @param unique Default FALSE
##' @return vector of character variables no longer than k
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @export
##' @examples
##' x <- c("Washington", "Washingham", "Washmylaundry")
##' shorten(x, 4)
##' shorten(x, 4, unique = TRUE)
shorten <- function(x, k = 20, unique = FALSE){
    if(!is.character(x)) stop("shorten: x must be a character variable")
    y <- substr(x, 1, k)
    if (unique) y <- make.unique(y)
    y
}


##' Insert "\\n" after the k'th character in a string. This IS vectorized,
##' so can receive just one or many character strings in a vector.
##'
##' If a string is long, insert linebreak "\\n"
##'
##' If x is not a character string, x is returned without alteration. And
##' without a warning
##' @param x Character string.
##' @param k Number of characters after which to insert "\\n". Default is 20
##' @return Character with "\\n" inserted
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @export
##' @examples
##' x <- "abcdef ghijkl mnopqrs tuvwxyz abc def ghi jkl mno pqr stv"
##' stringbreak(x, 10)
##' stringbreak(x, 20)
##' stringbreak(x, 25)
##' x <- c("asdf asdfjl asfdjkl asdfjklasdfasd", "qrweqwer qwerqwerjklqw erjqwe")
##' stringbreak(x, 5)
stringbreak <- function(x, k = 20){
    if (!is.character(x)) return(x)
    breakOneString <- function(y, k){
        ylength <- nchar(y)
        if (ylength <= k) return (y)

        yseq <- seq(1, ylength, by = k)

        ## iterate on successive pairs of yseq, but exclude last one
        res <- ""
        for(i in seq_along(yseq[-length(yseq)])){
            res <- paste0(res, paste0(substr(y, yseq[i], (yseq[i+1] - 1)), "\n"))
        }
        if (yseq[i] < ylength) res <- paste0(res, substr(y, yseq[i + 1], ylength))
        res
    }
    vapply(x, breakOneString, k = k, character(1))
}



##' Insert 0's in the front of existing digits or characters so that
##' all elements of a vector have the same number of characters.
##'
##' The main purpose was to correct ID numbers in studies that are
##' entered as integers with leading 0's as in 000001 or 034554.  R's
##' default coercion of integers will throw away the preceding 0's,
##' and reduce that to 1 or 34554. The user might want to re-insert
##' the 0's, thereby creating a character vector with values "000001"
##' and "045665".
##'
##' If x is an integer or a character vector, the result is the
##' more-or-less expected outcome (see examples). If x is numeric,
##' but not an integer, then x will be rounded to the lowest integer.
##'
##' The previous versions of this function failed when there were
##' missing values (NA) in the vector x.  This version returns NA for
##' those values.
##'
##' One of the surprises in this development was that sprintf() in R
##' does not have a known consequence when applied to a character
##' variable. It is platform-dependent (unredictable). On Ubuntu Linux
##' 16.04, for example \code{sprintf("\%05s", 2)} gives back
##' \code{" 2"}, rather than (what I expected) \code{"00002"}. The
##' problem is mentioned in the documentation for \code{sprintf}. The
##' examples show this does work now, but please test your results.
##' @param x vector to be converted to a character variable by
##'     inserting 0's at the front. Should be integer or character,
##'     floating point numbers will be rounded down. All other
##'     variable types will return errors.
##' @param n Optional parameter.  The desired final length of
##'     character vector.  This parameter is a guideline to determine
##'     how many 0's must be inserted.  This will be ignored if
##'     \code{n} is smaller than the number of characters in the
##'     longest element of \code{x}.
##' @return A character vector
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' x1 <- c(0001, 0022, 3432, NA)
##' padW0(x1)
##' padW0(x1, n = 10)
##' x2 <- c("1", "22", "3432", NA)
##' padW0(x2)
##' ## There's been a problem with strings, but this works now.
##' ## It even preserves non-leading spaces. Hope that's what you want.
##' x3 <- c("1", "22 4", "34323  42", NA)
##' padW0(x3)
##' x4 <- c(1.1, 334.52, NA)
##' padW0(x4)
padW0 <- function (x, n = 0) {
    if (!is.numeric(x) && !is.character(x))
        stop("padW0 only accepts integer or character values for x")

    xismiss <- is.na(x)
    ## springf trouble with characters, not platform independent
    if(is.numeric(x)) {
        if (is.double(x)) x <- as.integer(x)
        maxlength <- max(nchar(x), n, na.rm = TRUE)
        vtype <- "d"
        res <- sprintf(paste0("%0", maxlength, vtype), x)
    } else {
        maxlength <- max(nchar(x), n, na.rm = TRUE)
        xlength <- vapply(x, nchar, integer(1))
        padcount <- maxlength - xlength
        pads <- vapply(padcount, function(i){paste0(rep("0", max(0, i, na.rm=TRUE)), collapse = "")}, character(1))
        res <- paste0(pads, x)
    }

    res[xismiss] <- NA
    res
}


##' Write CSV files with quotes same as MS Excel 2013 or newer
##'
##' R's write.csv inserts quotes around all elements in a character
##' vector (if quote = TRUE).  In contrast, MS Excel CSV export no
##' longer inserts quotation marks on all elements in character
##' variables, except when the cells include commas or quotation
##' marks.  This function generates CSV files that are, so far as we
##' know, exactly the same "quoted style" as MS Excel CSV export
##' files.
##'
##' This works by manually inserting quotation marks where necessary and
##' turning FALSE R's own method to insert quotation marks.
##' @param x a data frame
##' @param file character string for file name
##' @param row.names Default FALSE for row.names
##' @importFrom utils write.table
##' @return the return from write.table, using revised quotes
##' @export
##' @author Paul Johnson
##' @examples
##' set.seed(234)
##' x1 <- data.frame(x1 = c("a", "b,c", "b", "The \"Washington, DC\""),
##'       x2 = rnorm(4), stringsAsFactors = FALSE)
##' x1
##' fn <- tempfile(pattern = "testcsv", fileext = ".csv")
##' writeCSV(x1, file = fn)
##' readLines(fn)
##' x2 <- read.table(fn, sep = ",", header = TRUE, stringsAsFactors = FALSE)
##' all.equal(x1,x2)
writeCSV <- function(x, file, row.names = FALSE){
    xischar <- colnames(x)[sapply(x, is.character)]
    for(jj in xischar){
        x[ , jj] <- gsub('"', '""', x[ , jj], fixed = TRUE)
        needsquotes <- grep('[\",]', x[ ,jj])
        x[needsquotes, jj] <- paste0("\"", x[needsquotes, jj], "\"")
    }
    write.table(x, file = file, sep = ",", quote = FALSE,
                row.names = row.names)
}
NULL

##' Creates Structural Equation Modeling Tables
##'
##' Creates LaTeX markup for structural equation modeling output
##' tables in the style of the American Psychological
##' Association(APA). Input objects should be created by the
##' "\code{lavaan}" package.
##'
##' The argument paramSets determines the inclusion of estimate sections.
##' One can specify "all", which means that all types of parameters that
##' we can find in the fitted model are presented.  Otherwise, a subset
##' of parameter sets can be chosen by the user.
##' \itemize{
##' \item "loadings" are the factor loadings in the model.
##' \item "slopes" are the regression slopes in the model.
##' \item "intercepts" are the estimated constants in the measurement
##'      models.
##' \item "residualvariances" are the observed variable residual variances.
##' \item "residualcovariances" are the observed covariances among
##' residuals of observed variables.
##' \item "latentvariances" are the variances of unobserved variables.
##' \item "latentcovariances" are the covariances between unobserved variables.
##' \item "latentmeans" are means of unobserved variables
##' \item "thresholds" arise in latent
##' response variates (non-numeric indicator data).
##' \item "constructed" are parameters that are calculated from a formula
##'       in the model specification, such as an indirect path c=a*b.
##' \item "fits" the summary indicators of the mismatch between
##' the theoretical and observed covariance matrices, such as
##' RMSEA, CLI, TFI. While the fits are not technically parameters, they
##' are displayed in the same block style as parameters
##' }
##'
##' The columns parameter is used to specify different columns,
##' while columnLabels will alter the displayed labels for them.
##' 
##' @param object A lavaan object (e.g., returned by cfa() or sem()),
##'     or a named list of lavaan objects, e.g., \code{list("Model A"
##'     = obj1, "Model B" = obj2)}. Results will be displayed side by
##'     side.
##' @param paramSets Parameter sets to be included for each fitted
##'     object.  Valid values of the vector are \code{"all"} or a any
##'     of the following: \code{c("loadings", "slopes", "intercepts",
##'     "residualvariances", "residualcovariances", "latentmeans",
##'     "latentvariances", "latentcovariances", "thresholds",
##'     "constructed", "fits")}. Default is "all", any of the
##'     estimates present in the fitted model that are listed in the
##'     previous sentence will be included in the output. For the sake
##'     of simplicity, we now allow one vector here, which applies to
##'     all models in the object list.
##' @param paramSetLabels Named vector, used to supply alternative
##'     pretty printing labels for parameter sets. The default values
##'     are \code{c("loadings"= "Factor Loadings", "slopes" =
##'     "Regression Slopes", "intercepts" = "Intercepts", "means"=
##'     "Means", "residualvariances" = "Residual Variances",
##'     "residualcovariances" = "Residual Covariances", "variances" =
##'     "Variances", "latentvariances" = "Latent Variances",
##'     "latentcovariances" = "Latent Covariances", "latentmeans" =
##'     "Latent Intercepts", "thresholds" = "Thresholds",
##'     "constructed" = "Constructed", "fits" = "Fit Indices")}.  The
##'     paramSetLabels argument must be a named vector that overrides
##'     some or all of the default names.
##' @param columns A vector naming estimates to appear for each model.
##'     The allowed columns are "est", "se", "z", "p", "rsquare",
##'     "estse", "eststars", "estsestars". The first 5 have the usual
##'     meanings, while "estse" (can also be written \code{"est(se)"})
##'     displays as, for example "1.21(0.23)", and the last 2 are to
##'     include "significance stars".  \code{"eststars"} shows as
##'     "1.21***" and \code{"estsestars"} (or \code{"est(se)stars"})
##'     displays as "1.21(0.23)**". See parameter \code{alpha}. One
##'     may request different columns for each model by providing a
##'     named list of vectors.  Use model names in the list,
##'     \code{list("Model A" = c("est", "se"), "Model B" = c("estse",
##'     "p"))}.
##' @param columnLabels A named vector of "pretty labels" for the
##'     headings in the table. The default labels are \code{c("est" =
##'     "Estimate", se = "Std. Err.", z = "z", p = "p", rsquare =
##'     "R Square", estse = "Estimate(Std.Err."), eststars =
##'     "Estimate", estsestars = "Estimate(Std.Err.)")}.
##' @param fits Summary indicators to be included. May be a list, one
##'     for each model provided, otherwise the same fit indicators
##'     will be presented for each model. Any of the fit indicators
##'     provided by \code{lavaan::fitMeasures(object)} are allowed:
##'     \code{c("npar", "fmin", "chisq", "df", "pvalue",
##'     "baseline.chisq", "baseline.df", "baseline.pvalue", "cfi",
##'     "tli", "nnfi", "rfi", "nfi", "pnfi", "ifi", "rni", "logl",
##'     "unrestricted.logl", "aic", "bic", "ntotal", "bic2", "rmsea",
##'     "rmsea.ci.lower", "rmsea.ci.upper", "rmsea.pvalue", "rmr",
##'     "rmr_nomean", "srmr", "srmr_bentler", "srmr_bentler_nomean",
##'     "srmr_bollen", "srmr_bollen_nomean", "srmr_mplus",
##'     "srmr_mplus_nomean", "cn_05", "cn_01", "gfi", "agfi", "pgfi",
##'     "mfi", "ecvi")}. The return for "chisq" will include markup
##'     for degrees of freedom and p value. If user specifies
##'     \code{NULL}, or if "fits" is excluded from \code{paramSets},
##'     all fit indicators are omitted.
##' @param fitLabels Labels for some or all of the fit measures
##'     requested by the fits parameter, e.g. \code{c(rmsea =
##'     "Root Mean Square Error of Approximation", cli = "CLI")}. The
##'     default labels are the upper-case fits names (except for
##'     "chisq", where a Greek letter is supplied when possible).
##' @param varLabels Named vector of labels to replace variable names
##'     in column 1 of SEM table.
##' @param groups All groups will be printed, unless a subset is
##'     requested here. Estimates for all groups will be displayed
##'     side by side. If ONLY SOME groups should be included, then
##'     specify groups as either names of fit objects or as integers
##'     for elements of the groups vector.
##' @param type Choose "latex", "html", or "csv"
##' @param file Base name for output file. This function will insert
##'     suffix, either "tex", "html" and "csv".
##' @param table.float If TRUE, create a LaTeX floating table object
##'     in which the tabular created here will reside. Default is
##'     FALSE.
##' @param caption Caption for table (if table.float=TRUE) or
##'     longtable output. Ignored otherwise.
##' @param label LaTeX label for this object (for
##'     cross-references). Only used if table.float = TRUE or
##'     longtable = TRUE.
##' @param longtable If TRUE, use longtable for LaTeX
##'     documents. Default is FALSE. If true, \code{table.float}
##'     argument is ignored.
##' @param print.results If TRUE, marked up result will be displayed
##'     within the session. Otherwise, result is returned silently and
##'     user can use \code{cat} to dislay it. Don't use \code{print}
##'     because it inserts unwanted decorations.
##' @param centering Default "siunitx". For method used in previous
##'     editions, replace with "none".
##' @param alpha Thresholds for p-values that determine number of
##'     stars.  Defaults as \code{c(0.05, 0.01, 0.001)} for
##'     \code{c("*", "**", "***")}.
##' @importFrom stats pnorm
##' @importFrom lavaan lavInspect
##' @importFrom plyr mapvalues
##' @return Markup for SEM table. Includes an attribute
##'     "markedResults", which can be converted to other markup
##'     formats by the function markupConvert.
##' @export
##' @author Ben Kite <bakite@@ku.edu> Paul Johnson <pauljohn@@ku.edu>
##' @examples \donttest{
##' ## These run longer than 5 seconds
##' ## CFA model
##' require(lavaan)
##'
##' tempdir <- tempdir()
##' ## The example from lavaan's docs
##' HS.model <- ' visual  =~ x1 + x2 + x3
##'               textual =~ x4 + x5 + x6
##'               speed   =~ x7 + x8 + x9'
##' fit1 <- cfa(HS.model, data = HolzingerSwineford1939,
##'             std.lv = TRUE, meanstructure = TRUE)
##' ## Try a LaTeX file first
##' fit1.t1 <- semTable(fit1, columns = c("estse", "p"),
##'                     fits = c("chisq", "rmsea"), file = file.path(tempdir, "fit1.t1"),
##'                     varLabels = c("x1" = "hello"), type = "latex", print.results = FALSE)
##' ## If you have a working version of pdflatex in your system path, 
##' if (interactive()) testtable("fit1.t1.tex", tempdir)
##' fit1.t1 <- semTable(fit1, columns = c("estse", "p"),
##'                     fits = c("chisq", "rmsea"), file = file.path(tempdir, "fit1.t1.html"),
##'                     varLabels = c("x1" = "hello"), type = "html", print.results = FALSE)
##' ## Use file attribute to get full file name
##' browseURL(attr(fit1.t1, "file"))
##' ## Try CSV output next
##' fit1.t1 <- semTable(fit1, columns = c("estse", "p"),
##'                     fits = c("chisq", "rmsea"), file = file.path(tempdir, "fit1.t1"),
##'                     varLabels = c("x1" = "hello"), type = "csv", print.results = FALSE)
##' ## Go inspect this file with a spread sheet program:
##' attr(fit1.t1, "file")
##' ## Now demonstrate variable labels
##' vl <- c(visual = "Visual", textual = "Textual", speed = "Speed",
##'        x1 = "V1", x2 = "V2", x3 = "V3")
##' fit1.t2 <- semTable(fit1, columns = c("est", "estse"),
##'                     fits = c("chisq", "rmsea"), file = file.path(tempdir, "fit1.t2"),
##'                     varLabels = vl)
##' if (interactive()) testtable("fit1.t2", tempdir)
##' ## floating table
##' fit1.t3 <- semTable(fit1, columns = c("est", "estse"),
##'                     fits = c("chisq", "rmsea"), file = file.path(tempdir, "fit1.t3"),
##'                     varLabels = vl, table.float = TRUE,
##'                     caption = "Holzinger Swineford 1939",
##'                     label = "tab:hs1939")
##' if (interactive()) testtable("fit1.t3", tempdir)
##' fit1.t4 <- semTable(fit1, columns = c("est", "estse"),
##'                     fits = c("chisq", "rmsea"), file = file.path(tempdir, "fit1.t3"),
##'                     varLabels = vl, longtable = TRUE,
##'                     caption = "Holzinger Swineford 1939",
##'                     label = "tab:hs1939")
##' if (interactive()) testtable("fit1.t4", tempdir)
##' fit1.t5 <- semTable(fit1, fits = c("chisq", "rmsea"),
##'                columns = c("est", "se"), columnLabels = c(se = "S.E."),
##'                file = file.path(tempdir, "fit1.t5"))
##' if (interactive()) testtable("fit1.t5", tempdir)
##' fit1.t6 <- semTable(fit1, fits = c("chisq", "rmsea"),
##'                columns = c("estsestars"),
##'                columnLabels = c("estsestars" = "Est(SE)"),
##'                file = file.path(tempdir, "fit1.t6"))
##' if (interactive()) testtable("fit1.t6", tempdir)
##' vl <- c(x1 = "happy 1", x2 = "happy 2", x3 = "happy 3",
##'         visual = "Seeing", textual = "Thumb Texting")
##' fit1.t7 <- semTable(fit1, fits = c("chisq", "rmsea"),
##'                columns = c("eststars", "p"),
##'                columnLabels = c("eststars" = "Est(SE)"),
##'                file = file.path(tempdir, "fit1.t7"),
##'                varLabels = vl, longtable = TRUE, type = "latex")
##' 
##' if (interactive()) testtable("fit1.t7", tempdir)
##' # Change output format to csv
##' cat(markupConvert(attr(fit1.t7, "markedResults"), type = "csv"))
##' 
##' ## 2 groups
##' fit1.g <- cfa(HS.model, data = HolzingerSwineford1939,
##'                         std.lv = TRUE, group = "school", estimator = "MLR")
##' fit1.gt1 <- semTable(fit1.g, columns = c("estsestars", "p"),
##'                columnLabels = c(estsestars = "Est w/stars", p = "p-value"),
##'                file = file.path(tempdir, "fit1.gt1"))
##' if (interactive()) testtable("fit1.gt1", tempdir)
##' ## Now name particular group by name
##' fit1.gt2 <- semTable(fit1.g, columns = c("estsestars", "p"),
##'                columnLabels = c(estsestars = "Est w/stars", p = "p-value"),
##'                file = file.path(tempdir, "fit1.gt2"), groups = "Pasteur")
##' if (interactive()) testtable("fit1.gt2", tempdir)
##' ## Name particular group by number
##' fit1.gt3 <- semTable(fit1.g, columns = c("estsestars", "p"),
##'                columnLabels = c(estsestars = "Est w/stars", p = "p-value"),
##'                file = file.path(tempdir, "fit1.gt3"), groups = 1)
##' if (interactive()) testtable("fit1.gt3", tempdir)
##' 
##' ## Fit same model with standardization
##' fit1.std <- update(fit1, std.lv = TRUE, std.ov = TRUE, meanstructure = TRUE) 
##' ## include 2 models in table request
##' fit1.t2 <- semTable(list("Ordinary" = fit1, "Standardized" = fit1.std),
##'                     file = file.path(tempdir, "fit1.2.1"))
##' semTable(list("Ordinary" = fit1, "Standardized" = fit1.std),
##'     columns = list("Ordinary" = c("est", "se"), "Standardized" = c("est")),
##'     columnLabels = c(est = "Est", se = "SE"), file = file.path(tempdir, "fit1.2.2"))
##' if (interactive()) testtable("fit1.2.2", tempdir)
##'
##' fit1.t2 <- semTable(fit1, fits = c("chisq", "rmsea"), print.results = FALSE)
##' cat(fit1.t2)
##' fit1.t3 <- semTable(fit1, fits = c("chisq", "rmsea", "tli"),
##'                columns = c("est", "se"))
##'
##' ## Can create file with cat
##' cat(fit1.t3, file = file.path(tempdir, "fit1.t3.tex"))
##' 
##' ## Basic SEM
##' regmodel1 <- 'visual  =~ x1 + x2 + x3 
##'              textual =~ x4 + x5 + x6
##'              speed   =~ x7 + x8 + x9
##'              visual ~ textual + speed
##' '
##'
##' fit2 <- sem(regmodel1, data = HolzingerSwineford1939, std.lv = TRUE,
##'             meanstructure = TRUE)
##'
##' fit2.std <- update(fit2, std.lv = TRUE, std.ov = TRUE, meanstructure = TRUE) 
##'
##' fit2.t <- semTable(list("Ordinary" = fit2, "Standardized" = fit2.std), fits = "rmsea",
##'                    columns = list("Ordinary" = c("est", "se", "p"),
##'                                    "Standardized" = c("estsestars")),
##'                    columnLabels = c("est" = "Est", "se" = "Std.Err.", "p" = "p",
##'                                     "estsestars" = "Standardized Est."),
##'                    paramSets = c("loadings", "intercepts", "slopes", "latentcovariances"),
##'                    file = file.path(tempdir, "fit2.t1"), type = "latex")
##'
##' if (interactive()) testtable("fit2.t1", tempdir)
##'
##' # Change output format to csv
##' cat(markupConvert(attr(fit2.t, "markedResults"), type = "csv"))
##' cat(markupConvert(attr(fit2.t, "markedResults"), type = "html"))
##' 
##' fit2.t <- semTable(list("Ordinary" = fit2, "Standardized" = fit2.std),
##'               type = c("html"),
##'               file = file.path(tempdir, "fit2.t"),
##'               varLabels = c(x1 = "happy 1", x2 = "happy 2", x3 = "happy 3"),
##'               print.results = FALSE)
##'
##' if (interactive()) browseURL(file.path(tempdir, "fit2.t.html"))
##'
##' 
##' regmodel2 <- 'visual  =~ x1 + x2 + x3
##'               textual =~ x4 +  x6
##'               speed   =~  x8 + x9
##'               visual ~ textual 
##' '
##' fit3 <- sem(regmodel2, data = HolzingerSwineford1939, std.lv = TRUE,
##'             meanstructure = TRUE)
##'
##' fit3.t1 <-  semTable(fit3, type = c("latex"),
##'                      columns = c("estsestars", "rsquare"), 
##'                      file = file.path(tempdir, "fit3.1"),
##'                      print.results = FALSE)
##' 
##' cat(fit3.t1)
##' if (interactive()) testtable("fit3.1", tempdir)
##'
##' fit3.std <- update(fit2, std.lv = TRUE, std.ov = TRUE)
##'
##' fit3.std.t1 <- semTable(list("Mod 1" = fit2, "Mod 1 std" = fit2.std, "Mod 2" = fit3,
##'               "Mod 3 std" = fit3.std), columns = c("estsestars"), type = "html",
##'                file = file.path(tempdir, "fit3.std.t1"), print.results = FALSE)
##' if(interactive()) browseURL(file.path(tempdir, "fit3.std.t1.html"))
##'
##' fit3 <- sem(regmodel1, data = HolzingerSwineford1939, group = "school")
##' ## if specify 2 types, get a list of them back
##' fit3.t1 <- semTable(fit3, type = "latex")
##' fit3.t2 <- semTable(fit3, columns = c("est", "se"),
##'                      columnLabels = c(est = "Est.", se = "S.E."))
##'
##' fit3.t2 <- semTable(fit3, fits = c("chisq", "rmsea", "cfi"), print.results = FALSE)
##' cat(fit3.t2)
##'
##' fit3.t2 <- semTable(fit3, columns = c("estsestars"),
##'             fits = c("chisq", "rmsea", "cfi"), type = "html",
##'             file = file.path(tempdir, "fit3.t2"))
##' if(interactive()) browseURL(file.path(tempdir, "fit3.t2.html"))
##'  
##' fit3.t2 <- semTable(fit3, fits = c("rmsea", "cfi"))
##' 
##' model <- "factor =~ .7*y1 + .7*y2 + .7*y3 + .7*y4
##'                  y1 | -1*t1 + 1*t2
##'                  y2 | -.5*t1 + 1*t2
##'                  y3 | -.2*t1 + 1*t2
##'                  y4 | -1*t1 + 1*t2"
##' dat <- simulateData(model, sample.nobs = 300)
##' 
##' testmodel <- "ExampleFactor =~ y1 + y2 + y3 + y4"
##' 
##' fit4 <- cfa(testmodel, data = dat, ordered = colnames(dat),
##'             std.lv = FALSE)
##' 
##' fit4.t1 <- semTable(fit4, paramSets = c("loadings", "thresholds",
##'     "residualvariances"), fits = c("tli", "chisq"),
##'     fitLabels = c(tli = "TLI", chisq = "chisq"), type = "html")
##'
##' fit4.t2 <- semTable(fit4, fits = c("rmsea", "cfi", "chisq"),
##'               fitLabels = c(rmsea = "Root M.SQ.E.A", cfi = "CompFitIdx", chisq = "chisq"),
##'               type = "latex")
##'
##' ## Model 5 - Mediation model with equality constraints
##' model5 <-
##'     '
##'     # latent variable definitions
##'     ind60 =~ x1 + x2 + x3
##'     dem60 =~ y1 + e*y2 + d*y3 + y4
##'     dem65 =~ y5 + e*y6 + d*y7 + y8
##'     # regressions
##'     dem60 ~ a*ind60
##'     dem65 ~ c*ind60 + b*dem60
##'     # residual correlations
##'     y1 ~~ y5
##'     y2 ~~ y4 + y6
##'     y3 ~~ y7
##'     y4 ~~ y8
##'     y6 ~~ y8
##' 
##'     # indirect effect (a*b)
##'     ## := operator defines new parameters
##'     ab := a*b
##' 
##'     ## total effect
##'     total := c + (a*b)
##'     '
##' 
##' fit5 <- sem(model5, data=PoliticalDemocracy)
##' fit5boot <- sem(model5, data=PoliticalDemocracy, se = "bootstrap", bootstrap = 100)
##' 
##' fit5.t1 <- semTable(list("Democracy" = fit5), columns = c("estsestars", "rsquare"),
##'            file = file.path(tempdir, "fit5.1t"), type = "html",
##'            print.results = FALSE)
##' if(interactive()) browseURL(file.path(tempdir, "fit5.t1.html"))
##'
##' ## convert same object to LaTeX, save file, then test
##' cat(markupConvert(attr(fit5.t1, "markedResults"), type = "latex"),
##'          file = file.path(tempdir, "fit5.t1.tex"))
##' if (interactive()) testtable("fit5.t1", tempdir)
##' 
##' semTable(list("Democracy" = fit5, "Bootstrapped SE" = fit5boot),
##'          columns = c("estsestars", "rsquare"),
##'          file = file.path(tempdir, "fit5.2"), type = "latex",
##'          longtable = TRUE)
##' 
##' semTable(list("Democracy" = fit5, "Bootstrapped SE" = fit5boot),
##'          columns = c("estsestars", "rsquare"),
##'          paramSets = c("loadings", "slopes", "residualvariances", "constructed"),
##'          file = file.path(tempdir, "fit5.3"), type = "latex",
##'          longtable = TRUE)
##'
##' ## Model 5b - Revision of Model 5s
##' model5b <-
##'     '
##'     # Cut some indicators from the measurement model
##'     ind60 =~ x1 + x2 
##'     dem60 =~ y1 + e*y2 + d*y3 + y4
##'     dem65 =~ y5 + e*y6 + d*y7 
##'     # regressions
##'     dem60 ~ a*ind60
##'     dem65 ~ c*ind60 + b*dem60
##'     # cut out the residual correlations
##'     # indirect effect (a*b)
##'     ## := operator defines new parameters
##'     ab := a*b
##' 
##'     ## total effect
##'     total := c + (a*b)
##'     '
##' 
##' fit5b <- sem(model5b, data=PoliticalDemocracy, se = "bootstrap",
##' bootstrap = 100)
##' semTable(list("Model 5" = fit5boot, "Model 5b" = fit5b),
##'          columns = c("estsestars", "rsquare"),
##'          file = file.path(tempdir, "fit5.5"),
##'           type = "latex",
##'          longtable = TRUE)
##' if(interactive()) testtable("fit5.5", tempdir)
##' 
##' list.files(tempdir)
##' }
semTable <- function(object, file = NULL, paramSets = "all", paramSetLabels,
                     columns = c(est = "Estimate", se = "SE", z = "z", p = "p"),
                     columnLabels, fits = c("chisq", "cfi", "tli", "rmsea"),
                     fitLabels = toupper(fits), varLabels = NULL,
                     groups = NULL, type = "latex", table.float = FALSE,
                     caption = NULL, label = NULL, longtable = FALSE,
                     print.results = TRUE, centering = "siunitx",
                     alpha =  c(0.05, 0.01, 0.001)) {

    messg <- paste("This function is moving to the semTable package",
                   "Please use that package and function in the future.")
    .Deprecated(new = "semTable", package ="semTable", msg = messg)
    
    ## do.call(rbind, alist) unexpectedly converts characters to factors.
    ## it does not accept stringsAsFactors=FALSE,
    ## So set globally to avoid hassle
    options.orig <- options()
    options(stringsAsFactors = FALSE)
    on.exit(options(options.orig))
    
    ## local shorthand to replace the verbose
    ## formatC(round(chimeas$stat, 3), format = 'f', digits = 2)
    ## with frnd(chimeas$stat)
    ## For reasons I don't understand, most prevalent example here
    ## is round to 3, then show 2 digits
    frnd <- function(x, rnd = 2, digits = 2) {
        y <- formatC(round(x, rnd), format = 'f', digits = digits)
        ## regex replace " NA" with "". 
        y[grep("\\s*NA",y )] <- ""
        y
    }

    ## 20171021
    ## The Maker functions all had copies of a standard stanza
    ## that used formatC and round in this way. Aggregate those
    ## actions
    ## TODO: safety check for operations on columns that don't exist
    ## trows must be a row subset of the parameters table
    roundSubtable <- function(trows){
        cols <- intersect(c("est", "se", "stdest", "stdse", "z", "rsquare"), colnames(trows))
        for (i in cols) trows[ , i] <- frnd(trows[ , i])
        trows$p <- frnd(trows$p, 3, 3)
        trows$p <- gsub("0\\.", "\\.", trows$p)
        trows$est <- ifelse(trows$free == 0, paste0(trows$est, "_FIXED_"), trows$est)
        if (any(trows$free == 0)) FIXED <<- TRUE
        trows[trows$free == 0, intersect(colnames(trows), c("se", "z", "p", "stdse"))] <- ""
        ## missings are a problem, maybe in other variables too
        if("rsquare" %in% colnames(trows)){
            trows$rsquare <- ifelse(!is.na(trows$rsquare), trows$rsquare, "")
        }
        trows
    }


    ## If paramSets != "all", follow user request to select paramSets for table
    ## If paramSets == "all", then find out what params are available and
    ## possibly (not now!) organize them in correct order
    ## These don't need to be in order!
    fixParamSets <- function(parameters, paramSets, paramSetLabels) {
        if (length(paramSets) > 1 || paramSets != "all") {
            paramSets <- unique(paramSets)
            if (any(!paramSets %in% names(paramSetLabels))){
                MESSG <- "fixParamSets: invalid paramSets"
                stop(MESSG)
            }
            return(paramSets)
        }

        params <- unique(parameters$paramType)
        params
    }

    ## relabel the "paramType" column in a parTable data frame
    ##
    ## Re-lable the "paramType", the parameterSet indicator for each
    ## row in a parTable. Identifies "residualvariances",
    ## "residualcovariances", "latentvariances" "latentcovariances",
    ## "intercepts" and "latentmeans"
    ##  
    ## @param parameters A parTable data frame, with estimates and some labels. 
    ## @return 
    ## @author Paul Johnson
    insertParamTypes <- function(parameters){
        paramops <- c("=~" = "loadings", "~" = "slopes", "~1" = "means",
                      "~~" = "variances", "|" = "thresholds",
                      ":=" = "constructed", "==" = "constraints")

        variables <- attr(parameters, "variables")
        latents <- attr(parameters, "latents")
        
        parameters[ , "paramType"] <- NA
        parameters[ , "paramType"] <- paramops[parameters$op]
        
        parameters[(parameters$lhs %in% variables &
                    parameters$rhs %in% variables &
                    parameters$lhs == parameters$rhs &
                    parameters$op == "~~"), "paramType"] <- "residualvariances"

        parameters[parameters$lhs %in% variables &
                   parameters$rhs %in% variables &
                   parameters$lhs != parameters$rhs &
                   parameters$op == "~~", "paramType"] <- "residualcovariances"
        parameters[parameters$rhs %in% latents &
                    parameters$lhs %in% latents &
                    parameters$lhs == parameters$rhs &
                    parameters$op == "~~", "paramType"] <- "latentvariances"
        parameters[parameters$rhs %in% latents &
                   parameters$lhs %in% latents &
                   parameters$lhs != parameters$rhs &
                   parameters$op == "~~", "paramType"] <- "latentcovariances"
        parameters[parameters$lhs %in% variables &
                            parameters$op == "~1", "paramType"] <- "intercepts"
        parameters[parameters$lhs %in% latents &
                   parameters$op == "~1", "paramType"] <- "latentmeans"
        ## only keep rows that are not contraint rows
        parameters <- parameters[parameters$op != "==", ]
        ## Remove rows also that have "~*~"
        ## TODO: 20171107, find out what that means, it happens in ordinal output
        parameters <- parameters[parameters$op != "~*~", ]
        parameters
    }
    
    getParamTable <- function(onemodel, paramSets, paramSetLabels){
        createEstSE <- function(dframe, se = TRUE, stars = FALSE){
            dframe <- roundSubtable(dframe)
            separt <- if(se) paste0("{(", dframe[ , "se"], ")}") else ""
            starpart <- if(stars)dframe[ , "starsig"] else ""
            ifelse(dframe[ , "free"] != 0,
                   paste0(dframe[ , "est"], separt,
                          starpart),
                   paste0(dframe[ , "est"]))
        }
        if(class(onemodel)[1] != "lavaan"){
            stop("onemodel is not a lavaan output onemodel.")
        }
        parList <- onemodel@ParTable[
                              intersect(names(onemodel@ParTable),
                                        c("lhs", "op", "rhs", "free", "group",
                                          "est", "se", "label", "plabel"))]
        parameters <- as.data.frame(parList, stringsAsFactors=FALSE)
        ## items previously global are specialized to this model
        attr(parameters, "variables") <- unique(unlist(onemodel@Data@ov.names))
        attr(parameters, "latents") <- unique(unlist(onemodel@pta$vnames$lv))
        parameters <- insertParamTypes(parameters)
        attr(parameters, "params") <- fixParamSets(parameters, paramSets, paramSetLabels)
        ## keep only rows that are in desired params sets
        parameters <- parameters[parameters$paramType %in% attr(parameters, "params"), ]
        
        ## fixed cannot be 0 for constructed variables
        parameters$free[parameters$paramType == "constructed"] <- 99
        
        rsquare <- lavInspect(onemodel, what = "rsquare")
        
        parameters$z <- ifelse(parameters$free != 0 & parameters$se != 0,
                               parameters$est/parameters$se, NA)
        parameters$p <- 2*pnorm(abs(parameters$z), lower.tail = FALSE)
        parameters$starsig <- starsig(parameters$p, alpha = alpha, symbols = starsymbols)
        parameters$starsig <- ifelse(grepl("^\\s*$", parameters$starsig), "_STAR0_", parameters$starsig)
        parameters$estse <- createEstSE(parameters)
        parameters$eststars <- createEstSE(parameters, se = FALSE, stars = TRUE)
        parameters$estsestars <- createEstSE(parameters, stars = TRUE)

        if(length(onemodel@Data@group.label) > 0L) {
            parameters$group.label <- unlist(onemodel@Data@group.label)[parameters$group]
        } else {
            parameters$group.label <- parameters$group
        }

        ## parse rsquare among
        ## "indicators"
        parameters[ , "rsquare"] <- as.numeric("")
        if (is.list(rsquare)){
            ## multi group model
            for(i in names(rsquare)){
                ##rsq is rsquare for group
                rsq <- rsquare[[i]]
                parameters[parameters$paramType == "loadings" &
                            parameters$group.label == i, "rsquare"] <-
                    rsq[parameters$rhs[parameters$paramType == "loadings" &
                                   parameters$group.label == i]]
                latentrsquares <- rsquare[[i]][parameters[parameters$paramType == "latentvariances" &
                                                           parameters$group.label == i, "lhs"]]
                parameters[parameters$paramType == "latentvariances" &
                           parameters$group.label == i , "rsquare"] <-
                    ifelse(!is.na(latentrsquares), latentrsquares, NA)
            }
        } else {
            ## one group model
            ## convoluted retrieval of rsquare by loading names
            parameters[parameters$paramType == "loadings", "rsquare"] <-
              rsquare[parameters$rhs[parameters$paramType == "loadings"]]
            ## "latentvariances"
            ## "latentsquares" are NA if there is no regression fitted,
            ## must be cautious here
            latentrsquares <- rsquare[parameters[parameters$paramType == "latentvariances", "lhs"]]
            parameters[parameters$paramType == "latentvariances", "rsquare"] <-
                ifelse(!is.na(latentrsquares), latentrsquares, NA)
        }
        parameters <- roundSubtable(parameters)
        parameters
    }

    ## The trows objects have an attribute "blocktitle" and this
    ## formats that
    applyTitleMarkup <- function(blocktitle){
        header  <- "_BR_"
        ## Tricky business b/c row markup does not call for _BOC_ in column 1,
        ## that's provided by "_BR_".
        header <- paste0(header, gsub("_CONTENT_", blocktitle$content, blocktitle$markup), "_EOR_")
        header
    }
    
    makeSubtableTitle <- function(blocktitle, colnum = 1, width = 1, center = TRUE, underline = TRUE){
        colalign <- if(center) "_BOMC" else "_BOML"
        ## Need a separator if colnum > 1. This is ripple effect of removing & from _BOMC_ definition.
        ## Because we assume it is always BOMC, now must add _BOC_ before _BOMC_
        prefix <- if (colnum > 1) paste0("_EOC_", rep("_BOC__EOC_", colnum - 2), "_BOC_")
        markup <- if(underline) {
                      paste0(prefix, colalign, width, "__UL__CONTENT__EOUL__EOMC_")
                  } else {
                      paste0(prefix, colalign, width, "__CONTENT__EOMC_")
                  }
        blocktitle <- list(content = blocktitle, markup = markup,
                           colnum = colnum, width = width)
        blocktitle
    }

    ##works for paramSets = "loadings" or "slopes"
    loadingMaker <- function(parameters, paramType = "loadings", colLabels, modelName) {
        report <- names(colLabels[[modelName]])
        totalNcolumns <- min(9,  length(unname(unlist(colLabels))))
        trows <- parameters[parameters$paramType == paramType,  , drop = TRUE]
        ## The split gotcha
        ## prevent accidental reordering of rows when the factor is created
        ## automatially
        lhsfactor <- factor(trows$lhs, levels = unique(trows$lhs))
        trowsplit <- split(trows, f = lhsfactor, drop = FALSE)
        info <- lapply(trowsplit, function(x){
            ## drop gotcha, if x has one row from df now it is a list.
            if(class(x)[1] == "list") x <- as.data.frame(x)
            vname <- unique(x$lhs)
            rownames(x) <- paste(paramType, vname, x[ , "rhs"], sep = ".")
            xlabels <- replace(x$rhs, names(varLabels), varLabels)
            ## 20180509: Get rid of underscores in var names, no matter what
            xlabels <- gsub("_", ".", xlabels, fixed = TRUE)
            x <- data.frame(col1 = xlabels,  x[ , report, drop = FALSE])
            ## don't put "_BOC_" at beginning if in colnum 1
            ## 20180327: use varLabels for factor names too
            attr(x, "blocktitle") <- makeSubtableTitle(replace(vname, names(varLabels), varLabels),
                                                       colnum = 1,
                                                       width = 1,
                                                       center = FALSE,
                                                       underline = TRUE)
            class(x) <- c("trows", class(x))
            x
        })
        info
    }

    ## local immitation of mapvalues, with no warnings or NA checking
    replace <- function(x, from, to){
        if(is.null(from) || is.null(to)) return(x)
        idx <- match(x, from)
        idxNNA <- !is.na(idx)
        x[idxNNA] <- to[idx[idxNNA]]
        x
    }
    ## works for other paramSets, ones that are simple tables, no lists.
    ## replace the "maker" functions with one function.
    ## Only difference between maker functions was in calculation of col1 and
    ## the rownames, which we can customize here
    ## paramType in c("intercepts", "means" "latentmeans", "latentvariances","residualcovariances"
    ## Check for complications: "latentcovariances" "residualvariances"
    ## "thresholds" "covariances"
    parTableMaker <- function(trows, paramType, colLabels, modelName){
        report <- names(colLabels[[modelName]])
        totalNcolumns <- min(9,  length(unname(unlist(colLabels))))
        ## trows <- parameters[parameters$paramType == paramType,, drop = FALSE]
        if(dim(trows)[1] == 0) return (NULL)
        varlabslhs <- replace(trows$lhs, names(varLabels), varLabels)
        varlabsrhs <- replace(trows$rhs, names(varLabels), varLabels)
        varlabslhs <- gsub("_", ".",  varlabslhs, fixed = TRUE)
        varlabsrhs <- gsub("_", ".",  varlabsrhs, fixed = TRUE)
        if (paramType == "thresholds"){
            thresnum <- substring(trows$rhs, 2, nchar(trows$rhs))
            trows$col1 <- paste0(varlabslhs, "(", thresnum, ")")
            rownames(trows) <- paste(paramType, trows[ , "lhs"], thresnum, sep = ".")
        } else if (paramType == "residualvariances"){
            rownames(trows) <- paste(paramType, trows[ , "lhs"], sep = ".")
            trows$col1 <- varlabslhs
        } else if (paramType == "residualcovariances"){
            trows$col1 <- paste0(varlabslhs, " w/", varlabsrhs)
            rownames(trows) <- paste(paramType, trows[ , "lhs"], trows[ , "rhs"],  sep = ".")
        } else if (paramType == "latentcovariances"){
            trows$col1 <- paste0(varlabslhs, " w/", varlabsrhs)
            rownames(trows) <- paste0(paramType, ".", trows[ , "lhs"],
                           ".", trows[ , "rhs"])
        } else {
            trows$col1 <- varlabslhs
            rownames(trows) <- paste(paramType, trows[ , "lhs"], sep = ".")
        }
        trows <- data.frame(col1 = trows$col1, trows[ , report, drop = FALSE])
        attr(trows, "blocktitle") <- makeSubtableTitle(paramSetLabels[paramType],
                                                  colnum = 2,
                                                  width = totalNcolumns,
                                                  underline = TRUE)
        class(trows) <- c("trows", class(trows))
        trows
    }
    

    ## Create elaborate markup for Chi-Square value
    ## uses "report" = columns to decide where to put p.
    getChiSq <- function(object, report){
        newdf <- data.frame(col1 = "_CHI2_", row.names = "chisq", stringsAsFactors = FALSE)
        newdf[ , report] <- ""
        ## If there is more than 1 test, use last one, and change label to
        ## "Scaled chisq"
        nOfTests <- length(object@Fit@test)
        chimeas <- object@Fit@test[[nOfTests]]
        if (nOfTests > 1) newdf[1 , "col1"] <- "Scaled _CHI2_"
        chimeas$stat <- frnd(chimeas$stat)
        chimeas$pvalue <- frnd(chimeas$pvalue, 3,  3)
        chimeas$pvalue <- gsub("0\\.", "\\.", chimeas$pvalue)
        chipstars <- if (any(c("eststars", "estsestars") %in% colnames(newdf))){
                         starsig(chimeas$pvalue, alpha = alpha, symbols = starsymbols)
                     } else {
                         ""
                     } 
        newdf[ , 2] <- paste0(chimeas$stat, "(", chimeas$df, ")", chipstars)
        if ("p" %in% colnames(newdf)) newdf[ , "p"] <- chimeas$pvalue
        newdf
    }


    ## 20180513: handle markup of paraset lists for loading and slopes
    markupList <- function(alist) {
        header <- applyTitleMarkup(attr(alist, "blocktitle"))
        #trowsf <- trows[ , c("col1", report)]
        res <- vapply(alist, markupTable, character(1))
        res2 <- paste(header, paste(res, collapse = " "))
        return(res2)
    } 
          
    ## A trows object is a matrix, this inserts formatting markup.
    ## trows also has attribute "blocktitle", which is used to create
    ## row 1 in the markup result
    markupTable <- function(trows) {
        trowsf <- trows
        for(i in 1:(NCOL(trowsf) -1)){
            trowsf[ , i] <- paste0(trowsf[ , i], "_EOC__BOC_")
        }
        trowsf[ , 1] <- paste0("_BR_", trowsf[, 1])
        trowsf[ , NCOL(trowsf)] <- paste0(trowsf[ , NCOL(trowsf)], "_EOC__EOR_")
        res <- paste0(apply(trowsf, 1, paste, collapse = ""), collapse = "\n")
        ## TODO 20171028 CAUTION: workaround here b/c blocktitle info sometimes missing, must
        ## go back find why it is missing
        if (!is.null(attr(trows, "blocktitle"))){
            header <- applyTitleMarkup(attr(trows, "blocktitle"))
            res <- paste0(header, "\n", res, "\n")
        }
        return(res)
    } 


    ## Work on one fitted model.  Cycle through process of
    ## 1. Retrieve parameters,
    ## 2. insert title attributes to be used when formatting
    ## Creates a list of tables, one for each parameter type.
    ## These are found by splitting on the "paramType" variable in
    ## the paramTable.
    extractParameters <- function(paramTable, colLabels, modelName){
        params <- attr(paramTable, "params")
        ## report was name used for varnames of columns for
        ## keeing, ex: c("est" "se" "z" "p")
        report <- names(colLabels[[modelName]])

        ## explore following as alternative concept to manage calculations
        ## creates a list of tables for each parameter type, so no need to do
        ## data selection within the table maker function
        paramTableSplit <- split(paramTable, f = paramTable$paramType, drop = FALSE)
        ## Still must treat loadings and slopes differently
        reslt <- list()
        for(jj in names(paramTableSplit)){
            if (jj %in% c("loadings", "slopes")){
                
                info <- loadingMaker(paramTableSplit[[jj]], paramType = jj,
                                     colLabels, modelName)
                if (!is.null(info)){
                    attr(info, "blocktitle") <- makeSubtableTitle(paramSetLabels["loadings"],
                                                                  colnum = 2,
                                                                  width = length(report))
                    class(info) <- c("trowsList", class(info))
                }
                reslt[[jj]] <- info
            } else {
                reslt[[jj]] <- parTableMaker(paramTableSplit[[jj]], paramType = jj, colLabels,
                                            modelName = modelName)
            }
        }

        reslt
    }

    ## Gather summary fit indicators
    fitMaker <- function(object, fitsLabeled, colLabels, modelName) {
        report <- names(colLabels[[modelName]])
        fits <- names(fitsLabeled[[modelName]])
        fitLabels <- fitsLabeled[[modelName]]
        totalNcolumns <- min(9,  length(unname(unlist(colLabels))))
        ## retrieve fit values, will be named vector
        fitmeas <- lavaan::fitMeasures(object)[]
        fitmeas <- sapply(fitmeas, frnd)
        fitmeas <- fitmeas[fits]
        fitmeas[is.na(fitmeas)] <- ""
        
        ## re-label the fit indices
        
        info <- data.frame(col1 = fitLabels[names(fitmeas)], row.names = names(fitmeas))
        info[ , report] <- ""
        info[ , report[1]] <- fitmeas
        
        ## chisq is a special presentation, comes back as a one row data frame
        if ("chisq" %in% names(fitmeas)){
            yy <- getChiSq(object, report)["chisq", ]
            info["chisq", ] <- yy
        }
        
        attr(info, "blocktitle") <- makeSubtableTitle(paramSetLabels["fits"],
                                                      colnum = 2,
                                                      width = totalNcolumns,
                                                      underline = TRUE)
        
        class(info) <- c("trowsList", class(info))
        info
    }

    

    ## Given a list of tables (say, 3 variances tables) this puts them
    ## together side by side, with markup.
    ## If one table is NULL, replace with empty DF
    ## for others, expand rows to same size
    cbindParamSet <- function(tablList, colLabels){
        ## xxx table unique rownames and col1 values, to find all possible vars
        xxx <- lapply(tablList, function(x) cbind(rownames = rownames(x),
                                                  col1 = x[ , "col1"]))
        paramnames <- unique(do.call(rbind, xxx))
        rownames(paramnames) <- paramnames[ , "rownames"]
        for(jj in names(tablList)){
            ## if paramSet lacks desired table, make empty table for it
            if (is.null(tablList[[jj]])) {
                y <- data.frame(col1 = paramnames[ , "col1"],
                                matrix("", ncol = length(colLabels[[jj]]),
                                       nrow = NROW(paramnames),
                                       dimnames = list(paramnames[ , "rownames"],
                                                       names(colLabels[[jj]]))))
                y[ , match("col1", colnames(y))] <- NULL
                tablList[[jj]] <- y
            } else {
                y <- tablList[[jj]][rownames(paramnames), ]
                ## y[ , "col1"] <- paramnames[ , "col1"]
                rownames(y) <- rownames(paramnames)
                y[ , match("col1", colnames(y))] <- NULL
                tablList[[jj]] <- y
                if (is.null(attr(tablList, "blocktitle"))){
                    attr(tablList, "blocktitle") <- attr(tablList[[jj]], "blocktitle")
                }
            }
        }
        ## that has a blocktitle attribute
        tablMatrix <- do.call(cbind,  tablList)
        tablMatrix <- cbind("col1" = paramnames[ , "col1"], tablMatrix)
        tablMatrix[is.na(tablMatrix)] <- ""
        attr(tablMatrix, "blocktitle") <- attr(tablList, "blocktitle")
        tablMatrix
    }

    ###########
    ## 20180513
    ## Creates a list of tables, not applying markup yet
    ## trying to postpone markup application. 
    gatherParamSetsList <- function(paramSetNames, colLabels){
        colNameCounts <- lapply(colLabels, length)
        colHeaderRow <- unname(unlist(colLabels))
        totalNcolumns <- min(9,  length(colHeaderRow))## If more than 9, give up at centering

        buildSubtableForParamSet <- function(jj) {
            ## retrieve param jj from each model
            tablList <- lapply(paramList, function(x) x[[jj]])
            ## treat loadings and slopes differently because they are
            ## lists of tables
            if(jj %in% c("loadings", "slopes")){
                ## get the varnames
                varnames <- unique(unlist(lapply(tablList, names)))
                hh <- list()
                for(i in varnames) {
                    subList <- lapply(tablList, function(x) x[[i]])
                    hh[[i]] <- cbindParamSet(subList, colLabels)
                }
                ##res <- paste0(hh, collapse = " ")
                ## Improvise a section heading for loadings and slopes
                blocktitle <- makeSubtableTitle(paramSetLabels[jj], colnum = 2,
                                           width = totalNcolumns, underline = TRUE)
                ## qqq header <- applyTitleMarkup(blocktitle)
                ## qqq res <- paste0(header, res)
                attr(hh, "blocktitle") <- blocktitle
                return(hh)
            } else {
                hh <- cbindParamSet(tablList, colLabels)
                return(hh)
            }
            stop("semTable: bug in buildSubtableForParamSet. Please report")
        }
        
        results <- lapply(paramSetNames, buildSubtableForParamSet)
        results
    }
    
    ############
    ## take paramList from environment, then iterate through the sections
    ## and impose markup needed.
    finalizeMarkup <- function(paramSetsList, colLabels) {
        colNameCounts <- lapply(colLabels, length)
        colHeaderRow <- unname(unlist(colLabels))
        totalNcolumns <- min(9,  length(colHeaderRow))## If more than 9, give up at centering
          
        results2 <- lapply(paramSetsList, function(x) if(class(x)[1] == "data.frame"){
                                                          markupTable(x)}
                                                      else {
                                                          markupList(x)
                                                          })
        results2 <- paste(results2, collapse = "")
        
        colHeaderRow <- paste0("_BR__EOC_",
                              paste0("_BOC__BOMC1_", paste0(colHeaderRow, collapse = "_EOMC__BOC__BOMC1_")), "_EOMC__EOR__HL_\n")
        
        modelHeaderRow <- paste0("_BR__EOC__BOC_",
                                 paste0("_BOMC", colNameCounts, "_", names(colNameCounts), "_EOMC_", collapse = "_BOC_"),
                                 "_EOR_ _HL_\n", collapse = " ")
        starnote <- if (any(c("eststars", "estsestars") %in% sapply(colLabels, names))) {
                        paste0(starsymbols, "p<", alpha, collapse = ", ")
                    } else ""
        fixnote <-  if(TRUE) paste0("_FIXED_", "Fixed parameter") else ""
        tablesuffix <-  paste0(if(fixnote != "") paste0("_BR__BOML", totalNcolumns + 1, "_", fixnote, "_EOMC__EOR_"),
                               if(starnote != "") paste0("_BR__BOML", totalNcolumns + 1, "_", starnote, "_EOMC__EOR_"))
                               
        resmark <- paste0("_BTABULAR_", modelHeaderRow, colHeaderRow, results2, "_HL_",
                         tablesuffix, "\n_ETABULAR__LB_\n")
        attr(resmark, "colLabels") <- colLabels
        resmark
    }

    ## Now actual work begins.
    ##    
    starsymbols <- c("_STAR1_", "_STAR2_", "_STAR3_")
    if(!is.null(varLabels)) varLabels <- gsub("_", ".", varLabels, fixed = TRUE)
    ## Import and clarify the "object" argument.  If names are
    ## missing, will create.  If object is NOT a list, it is just one
    ## lavaan object, create a list, put that inside a list, so we can
    ## iterate over it.  A generic name for models. If list is not
    ## provided, there's no name, use mname
    mname <- "Model"

    ## Is list, but no names, so make some up
    ## if (is.list(object) && is.null(names(object))){
    ##     names(object) <- paste0(mname, " ", 1:length(object))
    ## }
    
    ## If one model is supplied, create list holder
    if (!is.list(object)) {
        object <- list(object)
        names(object)[1] <- mname
    } else if (is.null(names(object))){
        ## no names, so create names
        names(object) <- paste0(mname, " ", 1:length(object))
    }

    ## apply columnLabels, which may be a partial list of new labels.
    ## columnLabels has a defult, which is used unless
    ## augmented by the columnLabels argument
    columnx <- c(est = "Estimate", se = "Std. Err.", z = "z", p = "p",
                 rsquare = "R Square", estse = "Estimate(Std.Err.)",
                 eststars = "Estimate",
                 estsestars = "Estimate(Std.Err.)")
    if (missing(columnLabels)){
        columnLabels <- columnx
    } else {
        columnLabels <- modifyVector(columnx, columnLabels)
    }
    
    ## paramSetLabels default.
    paramx <- c("loadings" = "Factor Loadings",
                "slopes" = "Regression Slopes",
                "intercepts" = "Intercepts",
                "means" = "Means",
                "residualvariances" = "Residual Variances",
                "residualcovariances" = "Residual Covariances",
                "variances" = "Variances",
                "latentmeans" = "Latent Intercepts",
                "latentvariances" = "Latent Variances",
                "latentcovariances" = "Latent Covariances", 
                "thresholds" = "Thresholds",
                "constructed" = "Constructed",
                "fits" = "Fit Indices")
    if (missing(paramSetLabels)){
        paramSetLabels <- paramx
    } else {
        paramSetLabels <- modifyVector(paramx, paramSetLabels)
    }

    ## Fill in fitLabels against default "toupper"
    if (missing(fitLabels)){
        fitLabels <- toupper(fits)
        names(fitLabels) <- fits
        if("chisq" %in% fits){
            fitLabels["chisq"] <- "_CHI2_"
        }
    } else {
        fitLabelsX <- toupper(fits)
        names(fitLabelsX) <- fits
        if("chisq" %in% fits){
            fitLabelsX["chisq"] <- "_CHI2_"
        }
        ## blended labels, defaults with user labels on top
        fitLabels <-  modifyVector(fitLabelsX, fitLabels)
    }

    
    if(is.null(fits)) paramSets["fits"] <- NULL
    
    if(!is.null(fits) && !is.list(fits)) {
        names(fits) <- fits
        fits <- toupper(fits)
        fitsX <- modifyVector(fits, fitLabels)
        ## copy fitsX for each object
        fitsLabeled <- lapply(object, function(x) fitsX)
    } else {
        ## fits is a list
        if (!is.null(fits) && length(fits) != length(object) ||
            any(!names(fits) %in% names(object))){
            MESSG <- "object list and fits list must match"
            stop(MESSG)
        }
        fitsLabeled <- lapply(names(object), function(xname){
            fitsone <- fits[[xname]]
            names(fitsone) <- fitsone
            fitsone <- toupper(fitsone)
            fitsX <- modifyVector(fits, fitLabels)
        })
    }
    
    ## Next fixes the colLabels
    ## convert columns to a list, one per object, like
    ## list("Model A" = c(est = "Estimates", se = "Std.Err.")
    if (!is.list(columns)) {
        columns.orig <- columns
        columns <- gsub("est(se)", "estse", columns)
        if (is.list(object)){
            colLabels <- lapply(object, function(x){
                cols <- modifyVector(columns, columnLabels)
            })
        } else {
            colLabels <- list(modifyVector(columns, columnLabels))
            names(colLabels) <- mname
        }
    } else {
        if (length(columns) != length(object) || names(columns) != names(object)){
            MESSG <- "object list and columns list must match"
        }
        columns <- lapply(columns, function(x){
            names(x) <- gsub("est(se)", "estse", names(x))
            x
        })
        colLabels <- lapply(columns, function(x) {
            zz <- columnLabels[x]
            names(zz) <- x
            zz})
    }

    paramList <- list()
    ## if one fitted model and 2 or more groups found in there
    if ((length(object) == 1) && ((G <- (object[[1]])@Data@ngroups) > 1)){
       
        onemodel <- object[[1]] 
        ## has both group and group.label variables from @Data@group.label.
        parTable <- getParamTable(onemodel, paramSets, paramSetLabels)
        groupnumbers <- unique(parTable$group)
        grouplabels <- unique(parTable$group.label)
       
        if (!missing(groups)){
            ## If groups provided, remove ones not in groups parameter
            ## if they give a character vector
            if(is.character(groups)){
                if(any(!groups %in% grouplabels)){
                    MESSG <- "invalid group names"
                    stop(MESSG)
                }
                parTable <- parTable[parTable$group.label %in% groups, ]
            } else if (isTRUE(checkCoercion(groups, "integer"))){
                groups <- as.integer(groups)
                if(any(!groups %in% groupnumbers)){
                    MESSG <- "invalid group numbers"
                    stop(MESSG)
                }
                parTable <- parTable[parTable$group %in% groups, ]  
            } else {
                MESSG <- "invalid group numbers"
                stop(MESSG)
            }
        }
        ## Process remaining groups
        parTableSplit <- split(parTable,
                               f = factor(parTable$group.label,
                                          levels = unique(parTable$group.label)))
        colLabels <- lapply(parTableSplit, function(x) colLabels[[1]])
        if (!is.null(fits)){
            fitsLabeled <- lapply(parTableSplit, function(x) fitsLabeled[[1]])
            }
        for(ii in names(parTableSplit)){
            paramList[[ii]] <- extractParameters(parTableSplit[[ii]],
                                                colLabels = colLabels,
                                                modelName = ii)
            if (!is.null(fits) && ii == names(parTableSplit)[1]){
                paramList[[ii]][["fits"]] <- fitMaker(onemodel, fitsLabeled, colLabels, ii)
            }
        }
        attr(paramList, "G") <- G
    } else {
        ## more models, abort if any are multigroup
        G <- lapply(object, function(x) x@Data@ngroups)
        if((length(object) > 1) && any(G > 1)) {
            MESSG <- "Several Multi Group models are not understandable"
            stop(MESSG)
        }
        ## each model object's parameters are pulled
        ## paramList <- lapply(names(object, extractParameters, columns)
        for(ii in names(object)){
            paramTable <- getParamTable(object[[ii]], paramSets, paramSetLabels)
            paramList[[ii]] <- extractParameters(paramTable, colLabels, modelName = ii)
            if(!is.null(fits)){
                paramList[[ii]][["fits"]] <- fitMaker(object[[ii]], fitsLabeled, colLabels, ii)
            }
        }
    }

    ## This is a marker that will turn TRUE if the table needs FIXED param label
    FIXED <- FALSE
    paramSetsFound <- unique(unlist(lapply(paramList, function(x) names(x))))
    ## re-order paramSetsFound according to standard list
    paramSetNames <- intersect(names(paramSetLabels), paramSetsFound)
   
    paramSetsList <- gatherParamSetsList(paramSetNames, colLabels)
    markedResults <- finalizeMarkup(paramSetsList, colLabels)
  
    result <- markupConvert(markedResults, type = type, longtable = longtable,
                            table.float = table.float, caption = caption, label = label, file = file,
                            columns = colLabels, centering = centering)
    attr(result, "markedResults") <- markedResults
    attr(result, "unmarkedResults") <- paramSetsList
    class(result) <- "kutable"    
    if (print.results) cat(result, "\n") else cat("\n")
    
    invisible(result)
}
NULL



##' Convert marked-up semTable structures to latex, html, or csv
##'
##' The conversion key tables are included in the code of the function.
##'
##' The semTable uses a customized markup framework that uses
##' character sequences that begin and end with underscores, such as
##' "_BOMC2_ for "begin of multi-column entity that will use 2
##' columns". These special markups need to be converted into "tex", "html", or "csv" formats.
##' @param marked A character string
##' @param type Output type, latex", "html", or "csv".
##' @param table.float TRUE if you want insertion of '\\begin{table}'
##' @param longtable should a tabular or a longtable object be
##'     created?
##' @param caption A caption to use if either longtable or table is
##'     TRUE
##' @param label A LaTeX label for cross-references
##' @param file A file stub, to which ".tex", ".html", or ".csv" can
##'     be added
##' @param columns For SEM table, the list of columns objects
##' @param centering Default "siunitx". Specify "none" to return to
##'     behavior of semTable before 1.50.
##' @return a list of marked up character objects
##' @export
##' @author Paul Johnson
markupConvert <- function(marked, type = c("latex", "html", "csv"),
                          table.float = FALSE, longtable = FALSE,
                          caption = NULL, label = NULL, 
                          file = NULL, columns, centering = "siunitx")
{
    if(missing(columns)) columns <- attr(marked, "colLabels")
    ##num of columns, except for col1
    Ncolumns <- length(unname(unlist(columns)))

    ## a tabular
    tabularmarkup <-   paste0("{@{}r",
                         if(centering == "siunitx")paste0("*{", Ncolumns, "}{S[
                         input-symbols = ( ) +,
                         group-digits = false,
                         table-number-alignment = center,
                         %table-space-text-pre = (,
                         table-align-text-pre = false,
                         table-align-text-post = false,
                         table-space-text-post = {***},
                         parse-units = false]}") else paste0(rep("c", Ncolumns), collapse = ""), "@{}}\n")
    ## longtable
    if(!longtable && !table.float){
        tcode <- paste0("\\\\begin{tabular}", tabularmarkup)
    } else {
        if(longtable){
            tcode <-  paste0("\\\\begin{longtable}", tabularmarkup)
            if (!is.null(caption)) tcode <- paste0(tcode, "\n\\\\caption{", caption, "}")
            if (!is.null(label)) tcode <- paste0(tcode, "\n\\\\label{", label, "}")
            tcode <- paste0(tcode, "\n\\\\endfirsthead\n\\\\endhead\n")
        } else if (table.float){
            tcode <- "\\\\begin{table}"
            if (!is.null(caption)) tcode <- paste0(tcode, "\n\\\\caption{", caption, "}")
            if (!is.null(label)) tcode <- paste0(tcode, "\n\\\\label{", label, "}")
            tcode <- paste0(tcode, "\\\\begin{tabular}")
            tcode <- paste0(tcode, tabularmarkup)
        } else {
            MESSG <- "Logic error in markupConvert"
            stop(MESSG)
        }
    } 
    ## Replacement strings for LaTeX output
    latexreplace <- c(
        "_LB_" = "\\\n",
        "_EOC_" =  "",
        "_BOC_" = "& ", 
        "_EOMC_" = "}",
        "_EOR_" = "\\\\tabularnewline",
        "_BRU_" = "",
        "_BRT_" = "", 
        "_BOCU_" = "& ",
        "_BR_" = "",
        "_BTABULAR_" = tcode,  
        "_EOL_" = "\n",
        "_HL_" = "\\\\hline", 
        "_UL_" = "\\\\underline{",
        "_EOUL_" = "}",
        "_SEPU_" = " &", 
        "_SEP_" = " &", 
        "_ETABULAR_" = if (longtable) {
                      "\\\\end{longtable}"
                  } else if (table.float) {
                      "\\\\end{tabular}
                       \\\\end{table}"
                  } else {
                      "\\\\end{tabular}"
                  },
        "_BOMR1_" = "& \\\\multirow{1}{c}{",
        "_BOMR2_" = "& \\\\multirow{2}{c}{",
        "_BOMC1_" = "\\\\multicolumn{1}{c}{",
        "_BOMC2_" = "\\\\multicolumn{2}{c}{",
        "_BOMC3_" = "\\\\multicolumn{3}{c}{",
        "_BOMC4_" = "\\\\multicolumn{4}{c}{",
        "_BOMC5_" = "\\\\multicolumn{5}{c}{",
        "_BOMC6_" = "\\\\multicolumn{6}{c}{",
        "_BOMC7_" = "\\\\multicolumn{7}{c}{",
        "_BOMC8_" = "\\\\multicolumn{8}{c}{",
        "_BOMC9_" = "\\\\multicolumn{9}{c}{",
        "_BOML1_" = "\\\\multicolumn{1}{l}{",
        "_BOML2_" = "\\\\multicolumn{2}{l}{",
        "_BOML3_" = "\\\\multicolumn{3}{l}{",
        "_BOML4_" = "\\\\multicolumn{4}{l}{",
        "_BOML5_" = "\\\\multicolumn{5}{l}{",
        "_BOML6_" = "\\\\multicolumn{6}{l}{",
        "_BOML7_" = "\\\\multicolumn{7}{l}{",
        "_BOML8_" = "\\\\multicolumn{8}{l}{",
        "_BOML9_" = "\\\\multicolumn{9}{l}{",
        "_BOMCT1_" = "\\\\multicolumn{1}{c}{",
        "_BOMCT2_" = "\\\\multicolumn{2}{c}{",
        "_BOMCT3_" = "\\\\multicolumn{3}{c}{",
        "_BOMCT4_" = "\\\\multicolumn{4}{c}{",
        "_HTMLHL_" = "",
        "_CHI2_" = "$\\\\chi^{2}(\\\\mathrm{df})$",
        "_R2_" = "$R^2$",
        "_LEQ_" = "$\\\\leq$",
        "_SIGMA_" = "$\\\\sigma$",
        "_NBSP_" = " ",
        "_FIXED_" = "$^+$",
        "_STAR0_" = "$\\\\phantom{{^{***}}}$",
        "_STAR1_" = "$^{*}\\\\phantom{{^{**}}}$",
        "_STAR2_" = "$^{**}\\\\phantom{{^{*}}}$",
        "_STAR3_" = "$^{***}$"
    )

    ## Replacement strings for HTML output
    ## TODO: 20171102: refactor abbreviations
    ## Problem in output is duplicate <td><td ..>, workaround in last item"
    htmlreplace <- c(
        "_LB_" = "<br>",
        "_EOC_" = "</td>",
        "_BOC_" = "<td>",
        "_EOMC_" = "</td>",
        "_EOR_" = "</tr>",
        "_BRU_" = paste("<tr><td style=\"border-bottom: solid thin black; border-collapse:collapse;\">&nbsp;"),
        "_BRT_" = paste("<tr><td style=\"border-top: solid thin black; border-collapse:collapse;\">&nbsp;"),
        "_BOCU_" = paste("<td style=\"border-bottom: solid thin black; border-collapse:collapse;\">&nbsp;"),
        "_BR_" = "<tr><td>",
        "_BTABULAR_" =  "<table style=\"padding-right:20px;padding-left:20px;\">\n",
        "_EOL_" = "\n",
        "_HL_" =  "",
        "_UL_" =  "<span style=\"text-decoration: underline;\">",
        "_EOUL_" = "</span>",
        "_SEPU_" = "</td><td style=\"border-bottom: solid thin black; border-collapse:collapse;\">&nbsp;",
        "_SEP_" = "</td><td>",
        "_ETABULAR_" = "</table>",
        "_BOMR1_" = "<td rowspan = '1'>",
        "_BOMR2_" = "<td rowspan = '2'>",
        "_BOMC1_" = "<td colspan = '1'; align = 'center'>",
        "_BOMC2_" = "<td colspan = '2'; align = 'center'>",
        "_BOMC3_" = "<td colspan = '3'; align = 'center'>",
        "_BOMC4_" = "<td colspan = '4'; align = 'center'>",
        "_BOMC5_" = "<td colspan = '5'; align = 'center'>",
        "_BOMC6_" = "<td colspan = '6'; align = 'center'>",
        "_BOMC7_" = "<td colspan = '7'; align = 'center'>",
        "_BOMC8_" = "<td colspan = '8'; align = 'center'>",
        "_BOMC9_" = "<td colspan = '9'; align = 'center'>",
        "_BOML1_" = "<td colspan = '1'; align = 'left'>",
        "_BOML2_" = "<td colspan = '2'; align = 'left'>",
        "_BOML3_" = "<td colspan = '3'; align = 'left'>",
        "_BOML4_" = "<td colspan = '4'; align = 'left'>",
        "_BOML5_" = "<td colspan = '5'; align = 'left'>",
        "_BOML6_" = "<td colspan = '6'; align = 'left'>",
        "_BOML7_" = "<td colspan = '7'; align = 'left'>",
        "_BOML8_" = "<td colspan = '8'; align = 'left'>",
        "_BOML9_" = "<td colspan = '9'; align = 'left'>",
        "_BOMCT1_" = "<td colspan = '1'; style=\"border-top: solid thin black; border-collapse:collapse;\">&nbsp;",
        "_BOMCT2_" = "<td colspan = '2'; style=\"border-top: solid thin black; border-collapse:collapse;\">&nbsp;",
        "_BOMCT3_" = "<td colspan = '3'; style=\"border-top: solid thin black; border-collapse:collapse;\">&nbsp;",
        "_BOMCT4_" = "<td colspan = '4'; align = 'center'; ; style=\"border-top: solid thin black; border-collapse:collapse;\">&nbsp;",
        "_HTMLHL_" = "<tr><td colspan = '5'; align = 'center'; ; style=\"border-top: solid thin black; border-collapse:collapse;\">&nbsp;</tr>",
        "_CHI2_" = "&chi;<sup>2</sup>",
        "_R2_" = "R<sup>2</sup>",
        "_LEQ_" = "&le;",
        "_SIGMA_" = "&sigma;",
        "_NBSP_" = "&nbsp;",
        "_FIXED_" = "<sup>+</sup>",
        "_STAR0_" = "&nbsp;",
        "_STAR1_" = "<sup>*</sup>",
        "_STAR2_" = "<sup>**</sup>",
        "_STAR3_" = "<sup>***</sup>",
        "<td><td" = "<td"
    )
    
    ## Replacement strings for CSV output
    csvreplace <- c(
        "_LB_" = "\n",
        "_EOC_" =  ",",
        "_BOC_" = "", 
        "_EOMC_" = ",",
        "_EOR_" = "\n",
        "_BRU_" = "",
        "_BRT_" = "", 
        "_BOCU_" = ",",
        "_BR_" = "",
        "_BTABULAR_" = "",
        "_EOL_" = "\n",
        "_HL_" = "", 
        "_UL_" = "",
        "_EOUL_" = "",
        "_SEPU_" = "", 
        "_SEP_" = ",", 
        "_ETABULAR_" = "",
        "_BOMR1_" = "",
        "_BOMR2_" = "",
        "_BOMC1_" = "",
        "_BOMC2_" = "",
        "_BOMC3_" = "",
        "_BOMC4_" = "",
        "_BOMC5_" = "",
        "_BOMC6_" = "",
        "_BOMC7_" = "",
        "_BOMC8_" = "",
        "_BOMC9_" = "",
        "_BOML1_" = "",
        "_BOML2_" = "",
        "_BOML3_" = "",
        "_BOML4_" = "",
        "_BOML5_" = "",
        "_BOML6_" = "",
        "_BOML7_" = "",
        "_BOML8_" = "",
        "_BOML9_" = "",
        "_BOMCT1_" = "",
        "_BOMCT2_" = "",
        "_BOMCT3_" = "",
        "_BOMCT4_" = "",
        "_HTMLHL_" = "",
        "_CHI2_" = "chi^2",
        "_R2_" = "R^2",
        "_LEQ_" = "<=",
        "_SIGMA_" = "sigma",
        "_NBSP_" = " ",
        "_FIXED_" = "+",
        "_STAR0_" = "", 
        "_STAR1_" = "*",
        "_STAR2_" = "**",
        "_STAR3_" = "**"
    )

    if (tolower(type) %in% c("latex", "tex")) {
        result <- mgsub(names(latexreplace), latexreplace, marked)
        if (!is.null(file)){
            if (!isTRUE(grepl(".tex$", file))) file <- paste0(file, ".tex")
            cat(result, file = file)
        }
    } else if (tolower(type) %in% c("html")){
        result <- mgsub(names(htmlreplace), htmlreplace, marked)
        result <- gsub("\\{\\(", "(", result)
        result <- gsub("\\)\\}", ")", result)
        if (!is.null(file)){
            if (!isTRUE(grepl(".html$", file))) file <- paste0(file, ".html")
            cat(result, file = file)
        }
    } else if (!is.na(match("csv", type))) {
        result <- mgsub(names(csvreplace), csvreplace, marked)
        if (!is.null(file)){
            if (!isTRUE(grepl(".csv$", file))) file <- paste0(file, ".csv")
            cat(result, file = file)
        }
    } else {
        MESSG <- "convertMarkup type argument incorrect"
        stop(MESSG)
    }
    attr(result, "file") <- file
    invisible(result)
}
NULL

##' Test viewer for tex tables
##'
##' Creates a small latex template file that includes a table
##' file. Compiles it, then displays in viewer if system has
##' \code{xdg-open} settings.
##' 
##' @param tablefile The base name of the table file
##' @param dir Directory where table is saved, same will be used for build.
##' @param tmpfn File name to be used by example document
##' @return LaTeX log, returned from shell function.
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' \donttest{
##' require(lavaan)
##' tempdir <- tempdir()
##' HS.model <- ' visual  =~ x1 + x2 + x3
##'               textual =~ x4 + x5 + x6
##'               speed   =~ x7 + x8 + x9'
##' fit1 <- cfa(HS.model, data = HolzingerSwineford1939,
##'             std.lv = TRUE, meanstructure = TRUE)
##' fit1.t <- semTable(fit1, fits = c("chisq", "rmsea"),
##'                columns = c("estsestars", "rsquare"),
##'                columnLabels = c("estsestars" = "Est(SE)"),
##'                file = file.path(tempdir, "fit1.t"))
##' if (interactive()) testtable("fit1.t", tempdir)
##' }
testtable <- function(tablefile, dir, tmpfn = "tmp.tex"){
    wd.orig <- getwd()
    isWindoze <- if(Sys.info()[['sysname']] == "Windows") TRUE else FALSE
    type <- if(isWindoze) "cmd" else "sh"
    mynull <-  if(isWindoze) "> nul" else " > /dev/null"

    ## If user forgets to remove .tex on end of filename, do it
    ## for them
    tablefile <- gsub("\\.tex$", "", tablefile)
    
    x1 <- 
"\\documentclass[english]{article}
\\usepackage[T1]{fontenc}
\\usepackage[utf8]{inputenc}
\\usepackage{geometry}
\\geometry{verbose,tmargin=1in,bmargin=1in,lmargin=1in,rmargin=1in}
\\usepackage{babel}
\\usepackage{longtable}
\\usepackage[normalem]{ulem}
\\usepackage{siunitx}
\\begin{document}
"
    x2 <- paste0("\\include{", tablefile, "}\n")

    x3 <- "\\end{document}\n"

    cat(x1, x2, x3, file = file.path(dir, tmpfn))
    cmd <- paste("pdflatex -interaction=batchmode ",
                 shQuote(tmpfn, type),
                 " && pdflatex -interaction=batchmode ",
                 shQuote(tmpfn, type))
    wd.orig <- getwd()
    setwd(dir)
    on.exit(setwd(wd.orig))
    resltfn <- file.path(dir, gsub(".tex", ".pdf", tmpfn))
    if (isWindoze){
        out1 <- tryCatch(shell(cmd, intern = TRUE))
        if(file.exists(resltfn)){
            MESSG <- paste("Please open this file:", resltfn)
            print(MESSG)
        } else {
            MESSG <- paste("pdflatex failed to compile a test document including", tablefile)
            stop(MESSG)
        }
    } else {
        out1 <- tryCatch(system(cmd, intern = TRUE))
        if(file.exists(resltfn)){
            cmd2 <- paste("xdg-open", resltfn)
            out2 <- tryCatch(system(cmd2, intern = TRUE))
        } else {
            MESSG <- paste("pdflatex failed to compile a test document including", tablefile)
            stop(MESSG)
        }
    }
  
    invisible(resltfn)
}

## Run unit tests
## Charlie Redmon
## 20170802

if (!require("RUnit", quietly = TRUE)) {
  warning("Cannot run unit tests. Package RUnit is not available.")
  q('no')
}

library(kutils)

path <- system.file(package="kutils", "unitTests")
stopifnot(file.exists(path), file.info(path.expand(path))$isdir)

testsuite.variableKey <- defineTestSuite("VariableKey",
                                         dirs=path,
                                         testFileRegexp="^runit.+\\.R",
                                         testFuncRegexp="^test.+",
                                         rngKind="Marsaglia-Multicarry",
                                         rngNormalKind="Kinderman-Ramage")

testResult <- runTestSuite(testsuite.variableKey)
printTextProtocol(testResult, showDetails=TRUE)

tmp <- getErrors(testResult)
if (tmp$nFail > 0 | tmp$nErr > 0) {
  stop(paste0("\n\nUnit testing failed (", tmp$nFail, " test failures), ",
              tmp$nErr, " R errors)\n\n"))
}

## natlongsurv <- read.csv2("../extdata/natlongsurv.csv")
## save(natlongsurv, file = "../data/natlongsurv.RData", compress = TRUE)


##' Smoking, Happiness, and other survey responses
##'
##' An idiosyncratic selection of 29 variables from the Original
##' Cohort-Young Women 1968-2003 edition of the US National
##' Longitudinal Survey.  This originally included 5159 rows, but
##' subset includes only 2867 rows, so sample frequencies will not
##' match the values listed in the codebook.  A snapshot of the
##' codebook, "natlongsurv.cdb.txt", which we have trimmed down, is
##' included in the package.
##' 
##' All variables are for the 2003 year, except where otherwise noted.
##'
##' @name natlongsurv
##' @docType data
##' @usage data(natlongsurv)
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @source National Longitudinal Surveys public-use data set (Bureau of Labor Statistics, 2018).
##' @references Bureau of Labor Statistics. 2018.  NLS Original Cohort: Mature and Young Women, US National Longitudinal Surveys Public Use Data Sets (www.bls.gov/nls/home.htm).
##' @format A data frame with 2867 rows and 29 variables:
##'
##'\itemize{
##'   \item	R0000100	IDENTIFICATION CODE
##'   \item	R0003300	MARITAL STATUS, 1968
##'   \item	R0005700	AGE WHEN STOPPED ATTENDING SCHOOL, 1968
##'   \item	R0060300	IQ SCORE, 1968
##'   \item	R1051600	HIGHEST GRADE COMPLETED
##'   \item	R1302000	SMOKING - DOES R SMOKE, 1991
##'   \item	R1302100	SMOKING - NUMBER OF CIGARETTES R SMOKES PER DAY, 91 (PRESENT SMOKER)
##'   \item	R6235600	HIGHEST GRADE COMPLETED
##'   \item	R6502300	IS RESIDENCE/LIVING QUARTERS HOME/APARTMENT/OTHER?
##'   \item	R6513700	HOUSEHOLD RECORD - HOUSEHOLD MEMBER - AGE CALCULATED FROM BIRTH DATE
##'   \item	R6516200	CURRENT MARITAL STATUS
##'   \item	R6520300	HIGHEST GRADE COMPLETED OF HUSBAND
##'   \item	R6553600	HIGHEST GRADE COMPLETED OF PARTNER
##'   \item	R7289200	SMOKING - CURRENTLY SMOKE CIGARETTES
##'   \item	R7289400	ALCOHOL USE - HAS R CONSUMED ANY ALCOHOLIC BEVERAGES IN PAST MONTH?
##'   \item	R7293430	YOUNG WOMEN 20-ITEM CES-D ITEM RESPONSE SCORE
##'   \item	R7312300	INCOME FROM WAGES/SALARY IN PAST YEAR
##'   \item	R7329900	INCOME ADEQUACY: R OPINION OF HER HAPPINESS WITH HER/FAMILY INCOME
##'   \item	R7330000	INCOME ADEQUACY: R OPINION OF AMOUNT NEEDED TO MAKE ENDS MEET $ AMOUNT
##'   \item	R7337600	R HAS ATTENDED/COMPLETED TWO/MORE YEARS OF COLLEGE
##'   \item	R7344600	ATTITUDE TOWARD FEELINGS OVERALL
##'   \item	R7344700	DID R DO ANY UNPAID VOLUNTEER WORK IN PAST YEAR?
##'   \item	R7347500	ATTITUDE TOWARD SOCIAL SECURITY - PERCENT WOULD INVEST IN STOCKS? 2004
##'   \item	R7347600	ATTITUDE TOWARD SOCIAL SECURITY - PERCENT WOULD INVEST IN BONDS OF PRIVATE COMPANIES? 2004
##'   \item	R7347700	ATTITUDE TOWARD SOCIAL SECURITY - PERCENT WOULD INVEST IN U.S. GOVERNMENT BONDS? 2004
##'   \item	R7477700	TOTAL CHILDREN IN ROSTER
##'   \item	R7477800	COUNT ELIGIBLE HOUSEHOLD CHILDREN
##'   \item	R7610300	REGION OF RESIDENCE
##'}
##'@examples
##' data(natlongsurv)
##' peek(natlongsurv, ask = FALSE, file = paste0(tempdir(), "/","peek.pdf"))
NULL

##' Create unique anonymous id values
##'
##' Obscure participant id values by replacing them with "anon-1" and
##' so forth.
##'
##' Caution: the true "confidential" names are used as names in the
##' output vector
##' @param x A column of "confidential" names, possibly with repeats
##' @param prefix Character string to use as prefix in result.
##'     Default is "anon"
##' @return Named character vector of anonymized id names.
##' @author Paul Johnson <pauljohn@@ku.edu>
##' x <- c("bill", "bob", "fred", "bill")
##' (anonomize(x, prefix = "id"))
anonomize <- function(x, prefix = "anon"){
    xunique <- unique(x)
    nameunique <- paste0(prefix, "-", seq_along(xunique))
    names(nameunique) <- xunique
    nameunique[x]
}

##' Update packages, spot new dependencies, and install them
##'
##' Addresses the problem that updates for previously installed R
##' packages may insert new dependencies. R's update.packages does not
##' trigger the installation of packages that are added as new
##' requirements in existing packages.
##'
##' This function checks for existence of updates, ascertains whether
##' those packages impose new requirements, and installs any new
##' requirements. Then it conducts the update.
##' 
##' This function is valuable in system maintenance because sometimes
##' existing packages adopt new requirements and the update.packages
##' function does not notice.  Another possible case would be that a user
##' accidentally deletes some packages without realizing other packages
##' depend on them.
##'
##' If this is run as the root/administrator privileged, then base R
##' packages may be updated, but if user is not root/administrator,
##' there will be a warning that packages were not updated because
##' permissions were lacking. For example
##'
##' "Warning: package 'boot' in library '/usr/lib/R/library' will not
##' be updated.
##' 
##' This warning does not interfere with rest of purpose of this
##' function, since the new dependencies can be installed in a place
##' where the user has privileges, either by specifying libnew as a
##' full directory name or by setting it to NULL, which puts new
##' packages in $R_LIBS_USER
##' @param ask If TRUE, asks user to select packages to update
##' @param checkBuilt If TRUE, packages built under earlier versions
##'     of R are to be considered 'old'
##' @param dependencies A vector specifying which type of dependencies
##'     need to be taken into account. We default to c("Depends",
##'     "Imports", "LinkingTo").
##' @param libnew The R library folder into which the new packages
##'     must be installed.  Defaults to "/usr/share/R/library", which
##'     is where EL7 likes those things.  To install packages in
##'     personal user directory, put libnew = NULL.
##' @param repos A vector of repositories on which to search for
##'     packages. Same definition as in R's install.packages or
##'     install.packages.
##' @param ... additional arguments passed to update.packages and
##'     install.packages
##' @return A vector of new packages being installed
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @importFrom tools package_dependencies
##' @export
##' @importFrom utils available.packages install.packages
##'     installed.packages packageDescription update.packages
##' @examples
##' \dontrun{
##' myrepos <- c("http://rweb.crmda.ku.edu/cran",
##'              "http://www.bioconductor.org/packages/3.3/bioc")
##' updatePackages(repos = myrepos)
##' ## libnew defaults to "/usr/share/R/library". Specify NULL
##' ## so that new packages will go to user's directory
##' updatePackages(libnew = NULL)
##' }
updatePackages <- function(ask = FALSE, checkBuilt = TRUE,
                           dependencies = c("Depends", "Imports", "LinkingTo"),
                           libnew = "/usr/share/R/library/",
                           repos = options("repos"),
                           ...)
{
    ## See if package asks for dependencies not currently installed
    installedPackages <- rownames(installed.packages())
    update.packages(ask = ask, checkBuilt = checkBuilt, repos = repos, ...)
    avail <- available.packages(repos = repos)
    alldeps <- unique(unlist(tools::package_dependencies(installedPackages,
                                                         which = dependencies,
                                                         db = avail,
                                                         recursive = TRUE)))
    
    targets <- alldeps[!alldeps %in% installedPackages]
    if (length(targets) > 0) {
        install.packages(targets, lib = libnew, repos = repos, dependencies = dependencies, ...)
        print(paste("These new packages were needed", targets))
    }
    if (length(targets) == 0){
        print("No new packages were installed to fulfill dependencies")
        return(NULL)
    } else {
        targets
    }
}

##' Text that is to be included as content in documents is cleaned
##' (escaped) to prevent errors
##'
##' This is for fixing up "untrusted text" that is to be passed into a
##' file as content. It protects against "bad" text strings in 3
##' contexts, 1) LaTeX documents, 2) HTML documents, or 3) text in a
##' file name. It converts content text to an improved string that
##' will not cause failures in the eventual document.
##'
##' The special in-document LaTeX symbols like percent sign or dollar sign
##' are "%" and "$". *Warning*: In the R
##' session, these will appear as double-backslashed symbols, while in
##' a saved text file, there will only be the one desired slash.
##'
##' If type = "html", we only clean up <, >, / and &, and quote
##' characters.  If document is in unicode, we don't need to do the
##' gigantic set anymore.
##'
##' If type = "filename", then symbols that are not allowed in file
##' names, such as "\", "*", are replaced. Do not use this on a
##' full path, since it will obliterate path separators.
##'
##' @param x a string, or vector of strings (each of which is
##'     processed separately)
##' @param type "tex" is default, could be "filename" or "html"
##' @return corrected character vector
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' x1 <- c("_asdf&_&$", "asd adf asd_", "^ % & $asdf_")
##' escape(x1)
##' x2 <- c("a>b", "a<b", "a < c", 'Paul "pj" Johnson')
##' escape(x2, type = "tex")
##' escape(x2, type = "html")
##' escape(x2, type = "filename")
escape <- function (x, type = "tex"){
    if (tolower(type) == "tex"){
        x <- gsub("\\\\", "SANITIZE.BACKSLASH", x)
        for(i in c("&", "$", "_", "^", "%", "#", "{", "{")) {
            x <- gsub(paste0("\\", i), paste0("\\\\",i), x)
        }
        x <- gsub("\\~", "$\\sim$", x)
        x <- gsub(">", "$>$", x, fixed = TRUE)
        x <- gsub("<", "$<$", x, fixed = TRUE)
        x <- gsub("|", "$|$", x, fixed = TRUE)
        x <- gsub("^", "\\verb|^|", x, fixed = TRUE)
        x <- gsub("SANITIZE.BACKSLASH", "$\\backslash$",
                  x, fixed = TRUE)
        return(x)
    }

    if (tolower(type) == "filename"){
        ## remove non-alpha numeric characters, any unfamiliar symbols
        xn <- alphaOnly(x)
        xn <- gsub("\\* $", "", xn)
        xn <- gsub("\\", "", x, fixed = TRUE)
        xn <- gsub("\\ ", "_", xn)
        xn <- gsub("_-_", "-", xn)
        xn <- gsub("_-", "-", xn)
        xn <- gsub("-_", "-", xn)
        xn <- gsub("&", "and", xn)
        xn <- gsub("\\\'","", xn)
        xn <- gsub("\\\"","", xn)
        xn <- gsub("#", "_", xn, fixed = TRUE)
        xn <- gsub("c/o", "co", xn)
        xn <- gsub("/", "-", xn)
        xn <- gsub("\\*", "", xn)
        xn <- gsub("\\$", "", xn)
        xn <- gsub("-$", "",  xn)
        xn <- gsub("_$", "", xn)
        xn <- gsub("<", "", xn, fixed = TRUE)
        xn <- gsub(">", "", xn, fixed = TRUE)
        return(xn)
    }

    ## See https://www.owasp.org/index.php/XSS_%28Cross_Site_Scripting%29_Prevention_Cheat_Sheet#RULE_.231_-_HTML_Escape_Before_Inserting_Untrusted_Data_into_HTML_Element_Content
    ## Could have used kutils::mgsub, if it had existed.
    if (tolower(type) == "html"){
        old <- c( "&", "<", ">", "\"", "\'", "/")
        new <- c("_AMPERSAND_", "&lt;", "&gt;", "&quot;", "&#27;", "&#x2F")
        xn <- kutils::mgsub(old, new, x)
        xn <- gsub("_AMPERSAND_", "&amp;", xn)
        return(xn)
    }
    messg <- paste("escape function can only handle \"tex\", \"html\" and \"filename")
    stop(messg)
}




##' Keep only alpha-numeric symbols
##'
##' From a text string, keep ASCII letters, numbers, as well as "'",
##' " ", "_" "(", ")", "-", and "+". For maximum compatability with
##' the cross-platform file-naming standard. Obliterates all
##' characters that migth be mistaken for shell symbols, like "^",
##' "$", "@" and so forth.
##'
##' Removes trailing spaces.
##'
##' This version allows internal spaces in the string, by default. The
##' also argument can be used to eliminate spaces or other hated symbols.
##' @param x text string, or vector of strings (each of which is
##'     processed separately)
##' @param also A named vector of other symbols that the user wants to
##'     remove, along with replacements. For example, c(" " = "_",
##'     "-" = "", "+" = "") to replace space with underscore and minus
##'     and plus signs with nothing.
##' @export
##' @return cleaned text string
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' x <- c("[]kansas(city) Missouri", "percent%slash/",
##'        "\back{squiggle}_under(paren)", "*star-minus+plus")
##' alphaOnly(x)
##' alphaOnly(x, also = c(" " = "_", "+" = "_"))
##' alphaOnly(x, also = c("(" = "[", ")" = "]"))
alphaOnly <- function(x, also){
    y <- gsub("[^a-zA-Z0-9\\\\' _()+-]", "", x)
    y <- gsub("\\ *$", "", y) # Trailing spaces
    if(!missing(also) && !is.null(also)){
        y <-  mgsub(names(also), also, y, fixed = TRUE)
    }
    y
}

##' Percentage tables for Likert Scale variables
##'
##' Creates a table with columns for allowed values and rows for
##' variables.
##' @param data A data frame. Function will try to include all
##'     variables in data, unless vlist is provided.
##' @param vlist A vector of column names in data that should be
##'     displayed
##' @param columnlabels Column labels, optional to beautify variable
##'     names. If not supplied, column names will be used as column
##'     labels. Provide either 1) A named vector that replaces one or
##'     more columns, \code{c(oldname1 = "newlabel1", oldname2 =
##'     "newlabel2")} where oldnames are in colnames(data), or 2) a
##'     vector of same length as vlist (or data if vlist is not
##'     supplied) that will replace them one for one.
##' @param valuelabels A vector of values to beautify existing
##'     levels. If not supplied, factor levels will be used as row
##'     labels
##' @param rows Should output be transposed. This may help if there
##'     are many variables that need to fit on the page.  Percentages
##'     will appear on the rows, rather than columns.
##' @param digits Number of decimals to display in percentages
##' @param ... Arguments to pass to R's table function. We suggest
##'     \code{useNA = "always"} to add missing value information and
##'     \code{exclude = original.value.label} to exclude values
##'     observed. Currently, \code{useNA = "ifany"} does not work as
##'     expected, the number of missings will be displayed, even if
##'     there are none.
##' @return A list, including a frequency table (called "freqTab"),
##'     column counts ("counts"), column sums ("sums"), and
##'     column percents ("pcts").
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @importFrom xtable xtable print.xtable
##' @export
##' @examples
##' vvector <- c("Strongly Disagree", "Disagree", "Neutral",
##'               "Agree", "Strongly Agree")
##' set.seed(2342234)
##' N <- 28
##' scales <-
##'     data.frame(Vegas = factor(sample(1:5, N, replace = TRUE),
##'         levels = 1:5, labels = vvector),
##'                NewYork = factor(sample(1:5, N, replace = TRUE),
##'         levels = 1:5, labels = vvector),
##'                Paris = factor(sample(1:5, N, replace = TRUE),
##'         levels = 1:5, labels = vvector),
##'                Berlin = factor(sample(1:5, N, replace = TRUE),
##'         levels = 1:5, labels = vvector))
##'
##' likert(scales)
##'
##' likert(scales, exclude = "Disagree")
##'
##' likert(scales, exclude = "Strongly Disagree", useNA = "ifany")
##' 
##' (mySummary1 <- likert(data = scales, vlist = c("Vegas", "NewYork", "Paris")))
##' mySummary1[["pcts"]]
##' 
##' (mySummary2 <- likert(scales, vlist = c("Vegas", "NewYork", "Paris"),
##'                     valuelabels = c("SD", "D", "N", "A", "SA")))
##' (mySummary3 <- likert(scales, vlist = c("Vegas", "NewYork", "Paris"),
##'                     valuelabels = c("Strongly Disagree" = "Strong Disagreement")))
##'
##' (mySummary5 <- likert(scales, vlist = c("Vegas", "NewYork", "Paris"),
##'       valuelabels = c("SD", "D", "N", "A", "SA"),
##'       columnlabels = c("Vegas" = "Sin City"), rows = TRUE))
##' 
##'  ## Example of how one might write this in a file. 
##'  ## print(xtable::xtable(mySummary1[[1]], digits = 1),
##'  ##       type = "html", file = "varCount-1.html")       
##'   
likert <-  function(data, vlist, columnlabels, valuelabels,
                    rows = FALSE, digits = 2, ...){
    ## xxx <- lapply(data[ , vlist], table)
    ## t(sapply(data[, vlist], table))
    ## TODO: Insert check that variables have same levels
    
    if (!missing(vlist) && !is.null(vlist)){
        data <- data[ , vlist, drop = FALSE]
    } else {
        vlist <- colnames(data)
    }

    ## If variable is not factor, coerce as factor
    if(any(!sapply(data, is.factor))){
        for (i in colnames(data)){
            data[ , i] <- as.factor(data[ , i])
        }
    }
    
    dots <- list(...)
    
    ## add error checking
    ## All columns must have same factor levels, else stop
    factorlevels <- unique(lapply(data, levels))
    if(length(factorlevels) > 1) {
        warning("All columns do not have same factor levels, will append")
        factorlevels <- unique(unlist(factorlevels))
    } else {
        factorlevels <- unlist(factorlevels)
    }
    if (length(dots$exclude) > 0) {
        factorlevels <- factorlevels[-match(dots$exclude, factorlevels)]
    }

    
    if(!missing(valuelabels) && !is.null(valuelabels)){
        if(length(valuelabels) < length(factorlevels)){
            if(is.null(names(valuelabels))){
                MESSG <- "valuelabels is incomplete or unnamed"
                stop(MESSG)
            } else {
                ## valuelabels is named, so replace
                factorlevels <- modifyVector(factorlevels, valuelabels)
            }
        } else if (length(valuelabels) == length(factorlevels)){
            if (is.null(names(valuelabels))) factorlevels <- valuelabels
            else factorlevels <- valuelabels[factorlevels]
        } else if (length(valuelabels) > length(factorlevels)){
            stop("valuelabels vector too long")
        }
        for (i in seq_along(data)) levels(data[ , i]) <- factorlevels
    }
    
    if(!missing(columnlabels) && !is.null(columnlabels)){
        if(length(columnlabels) < length(vlist)){
            if(is.null(names(columnlabels))){
                stop("columnlabels is incomplete or unnamed")
            } else {
                columnlabels <- modifyVector(vlist, columnlabels)
            }
        } else if (length(columnlabels) == length(vlist)){
            if (is.null(names(columnlabels))) names(columnlabels) <- vlist
        } else if (length(columnlabels) > length(vlist)){
            stop("columnlabels vector too long")
        } 
    } else {
        columnlabels <- colnames(data)
    }
    
    ## User wants missings:
    if ("exclude" %in% names(dots) && is.null(dots$exclude) || (length(dots$useNA) > 0 && dots$useNA != "no")){
        data[is.na(data)] <- "NA"
        factorlevels <- c(factorlevels, "NA")
    }
    
    varCount <- matrix(0, nrow = length(factorlevels), ncol = NCOL(data),
                       dimnames = list(factorlevels, colnames(data)))

    for (i in colnames(data)){
        xxx <- table(data[ , i], ...)
        varCount[rownames(xxx), i ] <- xxx 
    }

    colnames(varCount) = columnlabels
    varSums <- colSums(varCount)

    varColPct <- 100.0 * sweep(varCount, 2, varSums, "/")
    varColPct.char <-  formatC(varColPct, digits = digits, format = "f")
    
    freqTab <- matrix(NA, nrow = NROW(varColPct.char), ncol = NCOL(varColPct.char))
    dimnames(freqTab) <- list(rownames(varColPct.char), colnames(varColPct.char))
    
    for(i in 1:NROW(varColPct.char)) {
        freqTab[i, ] <- paste0(varColPct.char[i, ], "% (", varCount[i, ], ")")
    }
    
    ## freqTab <- rbind( rep(paste0("Pct (Count)")), freqTab)
    ## colnames(freqTab) <- colnames(varColPct.char)
    freqTab <- rbind(freqTab, varSums)
    rownames(freqTab)[NROW(freqTab)] <- "Total"

    if (!rows) {
        res <- list(table = freqTab, count = varCount, sums = varSums, pcts = varColPct)
    } else {
        res <- list(table = t(freqTab), counts = t(varCount), sums = varSums, pcts = t(varColPct))
    }
    class(res) <- c("likert", class(res))
    res
}


##' print method for likert tables
##'
##' Nothing fancy here. \code{cat} is called on first item in list
##' @param x likert object, 1st item will be printed
##' @param ... Arguments passed to print method
##' @return Nothing
##' @method print likert
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
print.likert <- function(x, ...){
    print(x[[1]], ...)
}

##' Cuts a string at a specified linewidth, trying to align cut with a
##' separator
##'
##' Some strings are simply too long. We don't want to chop them
##' exactly at, say, 40 characters, if we could allow 42 and chop on a
##' space or other separator. We'd rather chop at 37 if there is a
##' separator, rather than terminate a word exactly at 40. This function
##' shortens them and attempt to cut at a separator, allowing for a
##' user specified fudge-factor (the tol parameter).
##'
##' The default capwidth value is 1, so the calculations treat all
##' letters equally. In practice, we notice trouble when some strings
##' are written in ALL CAPS and they are longer than the same
##' information in lower case letters.  We have decided to allow a
##' user-specified penalty for capital letters. If each capital
##' counts for, say 1.2 ordinary letters, then we may end up truncating
##' the string on an earlier separator.
##'
##' There's some approximation here. The capital-penalized widths are
##' calculated for all characters and then we left-shift the target
##' value so that it is equal to the last penalized value that is
##' under the target length. Then the "look to the left" and "look
##' to the right" logic begins.  That looking logic ignores the
##' capital letter penalty, it is treating all letters the same. 
##' 
##' @param x character or vector of characters
##' @param target Goal for result length, in characters
##' @param tol number of characters forward/back to check; if single
##'     value then only backwards checking
##' @param separators characters at which truncation is preferred,
##'     such as space or underscore.
##' @param capwidth penalty for capital characters
##' @return shorter string truncated at acceptable separators when
##'     found
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @export
##' @examples
##' x <- "Aasdf asdIasdf fW_asd asd aasjdf_as fasdasdfasdfasd"
##' truncsmart(x, target = 10, tol = c(5, 2))
##' truncsmart(x, target = 10, tol = c(1, 4))
##' truncsmart(x, target = 10, tol = c(5, 2), capwidth = 1.2)
##' truncsmart(x, target = 20, tol = c(5, 2))
##' truncsmart(x, target = 20, tol = c(10,10), capwidth = 2)
##' truncsmart(x, target = 20, tol = c(10,10), capwidth = 3)
##' truncsmart(x, target = 20, tol = c(10,10), capwidth = 4)
##' truncsmart(x, target = 20, tol = c(10,10), capwidth = 6)
truncsmart <- function(x, target = 20, tol = c(5, 3),
                       separators = c(" ", "_", ";", ","), capwidth = 1.0){
    ## How many steps forward does the first TRUE occur?
    ## Subtract 1 so that return is the position of the one before the
    ## first separator.
    nextTrue <- function(y) {
        which(cumsum(y) > 0)[1] #- 1 Removing this made the function
                                #find the limit correctly
    }
    ## Start at the end and count backwards till there is a TRUE.
    ## Then go one more so we omit the separator itself. Returns the
    ## "steps backward" to land on the last value we want to keep in
    ## the truncated string.
    previousTrue <- function(y) {
        which(cumsum(rev(y)) > 0)[1] #+ 1 Removing this made the function
                                #find the limit correctly
    }
    
    xchars <- unlist(strsplit(x, ""))
    ## cumulative length after applying penalty for capital width.
    xcharwidth <- cumsum(ifelse(sapply(xchars,
                                       function(x) x %in% LETTERS), capwidth, 1))

    ## Recalibrate target value. new target that is in vicinity of
    ## capital adjusted length
    target.orig <- target
    target <- max(which(xcharwidth <= target))

    ## Input is under the target, so return the original thing
    if (xcharwidth[length(xcharwidth)] <= target) return(x)

    xseps <- xchars %in% separators
    ## If target is separator, we are finished
    if(xseps[target]) return(paste0(xchars[1:(target -1)], collapse = ""))

    ## Otherwise have to look left and right, so check tol input
    if (length(tol) > 2){
        stop("tol must not have more than 2 values")
    } else {
        if (length(tol) < 2){
            tol <- c(tol, 0)
        }
    }

    ## How many to left and right do I need to go?
    ## previousTrue should return number of steps to left to find last character
    ## we keep.  nextTrue should return number of steps to right to find last one to keep.
    ## Have to be sure ranges stay in bounds
    ## for xseps. A little tricky if target is low or very near end of sequence
    steps <- c(previousTrue(xseps[max(1, (target - tol[1])): (target - 1)]),
               nextTrue(xseps[(target + 1): min(length(xseps), (target + tol[2] + 1))]))
    
    ## Get the smallest number of steps and use that.
    ## which.min breaks ties how? Chooses the first, the left side, which we like.
    direction <- which.min(steps)

    ## no safe trunc exists, just chop at target.
    if (all(is.na(direction))) return (paste0(xchars[1:target], collapse = ""))
    
    ## steps[1] is backwards, so multiply by -1
    if (direction == 1) steps[1] <- -1 * steps[1]
    ## Calculate result at proper truncation.
    paste0(xchars[1:(target + steps[direction] - 1)], collapse = "")
}

##' Discern nesting pattern of SEM coefficients
##'
##' Receives a list of models and orders them by best guess at intended nesting
##' @param models A List of lavaan-fitted SEM models
##' @export
##' @return matrix indicating nesting relationships
##' @author Ben Kite <bakite@@ku.edu>
detectNested <- function(models){
    dfval <- ifelse("df.scaled" %in% lavaan::fitMeasures(models[[1]]), "df.scaled", "df")
    dfs <- lapply(models, lavaan::fitMeasures, dfval)
    dfsrank <- rank(unlist(dfs))
    names(dfsrank) <- names(models)
    sortedmods <- sort(dfsrank)
    nested <- matrix(NA, length(sortedmods), length(sortedmods))
    diag(nested) <- 0
    colnames(nested) <- names(sortedmods)
    rownames(nested) <- names(sortedmods)
    ## Columns represent the possible larger model, rows inform about
    ## whether or not the model is nested in the column
    for (i in rownames(nested)){
        for (j in colnames(nested)){
            if(i != j){
                dfnest <- dfsrank[i] > dfsrank[j]
                iparams <- data.frame(models[[i]]@ParTable)[,c("lhs", "op", "rhs")]
                jparams <- data.frame(models[[j]]@ParTable)[,c("lhs", "op", "rhs")]
                iparams <- apply(iparams, 1, paste, collapse = "")
                jparams <- apply(jparams, 1, paste, collapse = "")
                iparams <- iparams[!iparams %in% iparams[grep("==", iparams)]]
                jparams <- jparams[!jparams %in% jparams[grep("==", jparams)]]
                parnest <- prod(jparams %in% iparams)
                nested[i,j] <- ifelse(dfnest*parnest == 1, 1, 0)
            }
        }
    }
    pairs <- matrix(NA, nrow(nested), 2)
    rownames(pairs) <- rownames(nested)
    colnames(pairs) <- c("nested", "parent")
    ## Make nested pairs
    for(i in rownames(nested)){
        pairs[i,"nested"] <- i
        nestedin <- which(nested[i,] == 1)
        if (length(nestedin) > 0){
            pairs[i,"parent"] <- names(nestedin[max(nestedin)])
        }
    }
    pairs
}

##' Prepare a table to compare fit measures of confirmatory factor analyses (CFA)
##'
##' If the parameter \code{nesting} is not specified, then this
##' function attempts to discern which models are nested and they are
##' ordered accordingly. The user can override that by specifing a
##' nesting structure. This uses a new notation to represent nesting
##' of models.  See the parameter \code{nesting}.  The syntax uses the
##' symbols ">" and "+" in an obvious way to indicate that one model
##' is the superset or on the same level as another. If the
##' 
##' In May 2018, the output approach was changed. The functions
##' \code{xtable} and \code{print.xtable} are used to render the final
##' result and any of the arguments allowed by \code{print.xtable} can
##' be used here (via the \code{...} argument). We have some default
##' settings for some \code{print.xtable}, such as \code{type = NULL},
##' \code{file = NULL}, \code{print.results = TRUE}, and
##' \code{math.style.exponents = TRUE}. There are some other specific
##' defaults for type = "latex" and type = "html", but they can all be
##' overridden by the user. We include a model legend at the bottom of
##' the table, indicating which models are compared by the Chi-squared
##' statistic.
##' 
##' If the type argument is not specified, then the output will be a
##' simple text display of the model table.  If type is either "latex"
##' or "html", then a marked up table will be displayed and the file
##' argument can be used to ask for a saved version. If the user wants
##' to simply save the result in a file, and not display on screen,
##' insert the argument \code{print.results = FALSE}.
##' 
##' @param models list of lavaan cfa or sem models. Model names can be
##'     supplied. See examples.
##' @param fitmeas A vector of fit measures. One or more of these
##'     \code{c("chisq", "df", "pvalue", "rmsea", "cfi", "tli",
##'     "srmr", "aic", "bic", "srmr", "aic", "bic",
##'     "srmr_mplus")}. Other fit measures present in the lavaan
##'     objects will be allowed; fit measures that are requested but
##'     not found are ignored.
##' @param nesting character string indicating the nesting structure
##'     of the models.  Must only contain model names, ">", and "+"
##'     separated by spaces.  The model to the left of a ">" is the
##'     parent model for all models to the right of the same ">", up
##'     until another ">" is reached. When multiple models are nested
##'     in the same parent, they are separated by a "+".
##' @param scaled should scaled versions of the fit measures requested
##'     be used if available?  The scaled statistic is determined by
##'     the model estimation method.  The defaul value is TRUE.
##' @param chidif should the nested models be compared by using the
##'     anova function? The anova function may pass the model
##'     comparison on to another lavaan function.  The results are
##'     added to the last three columns of the comparison table. The
##'     default value is TRUE.
##' @param digits The digits argument that will be passed to xtable.
##' @param ... Arguments that will be passed to print.xtable. These
##'     arguments can be used to control table caption, label, and so
##'     forth. See \code{?print.xtable}.  If \code{type = "latex"} or
##'     \code{"html"}, this function sets additional default values
##'     for print.xtable that can be overridden by specifying
##'     arguments here. Default type is an R data.frame, which is printed on
##'     screen. Note the print.xtable parameter \code{print.results} determines
##'     whether the markup is displayed before it is returned. The \code{file}
##'     parameter can specify a file into which the markup is to be saved.
##' @author Ben Kite <bakite@@ku.edu> and Paul Johnson
##'     <pauljohn@@ku.edu>
##' @export
##' @importFrom stats anova update
##' @importFrom utils modifyList
##' @importFrom xtable xtable print.xtable
##' @return If type = NULL, a data.frame object includes an attribute
##'     called "noteinfo". If type = "tex", return is a character
##'     vector created by xtable. If type = "html", a vector of HTML
##'     markup created by xtable.
##' @examples
##' \donttest{
##' ## These run longer than 5 seconds
##' library(lavaan)
##' library(xtable)
##' set.seed(123)
##' genmodel <- "f1 =~ .7*v1 + .7*v2 + .7*v3 + .7*v4 + .7*v5 + .7*v6
##' f1 ~~ 1*f1"
##' genmodel2 <- "f1 =~ .7*v1 + .7*v2 + .7*v3 + .7*v4 + .7*v5 + .2*v6
##' f1 ~~ 1*f1"
##'
##' dat1 <- simulateData(genmodel, sample.nobs = 300)
##' dat2 <- simulateData(genmodel2, sample.nobs = 300)
##' dat1$group <- 0
##' dat2$group <- 1
##' dat <- rbind(dat1, dat2)
##'
##' congModel <- "
##'               f1 =~ 1*v1 + v2 + v3 + v4 + v5 + v6
##'     		  f1 ~~ f1
##'     		  f1 ~0*1
##'     		 "
##' weakModel <- "
##'               f1 =~ 1*v1 + c(L2,L2)*v2 + c(L3,L3)*v3 + c(L4,L4)*v4 + c(L5,L5)*v5 + c(L6,L6)*v6
##'     		  f1 ~~ f1
##'     		  f1 ~0*1
##'     		"
##' partialweakModel <- "
##'               f1 =~ 1*v1 + c(L2,L2)*v2 + c(L3,L3)*v3 + c(L4,L4)*v4 + c(L5,L5)*v5 + v6
##'     		  f1 ~~ f1
##'     		  f1 ~0*1
##'     		"
##' partialweakModel2 <- "
##'               f1 =~ 1*v1 + c(L2,L2)*v2 + c(L3,L3)*v3 + c(L4,L4)*v4 + v5 + v6
##'     		  f1 ~~ f1
##'     		  f1 ~0*1
##'     		"
##' partialstrongModel1 <- "
##'               f1 =~ 1*v1 + c(L2,L2)*v2 + c(L3,L3)*v3 + c(L4,L4)*v4 + c(L5,L5)*v5 + v6
##'     		  f1 ~~ f1
##'     		  f1 ~ c(0,NA)*1
##'     		  v1 ~ c(I1,I1)*1
##'     		  v2 ~ c(I2,I2)*1
##'     		  v3 ~ c(I3,I3)*1
##'     		  v4 ~ c(I4,I4)*1
##'     		  v5 ~ c(I5,I5)*1
##'     		  v6 ~ c(I6,I6)*1
##'     		"
##' cc1 <- cfa(congModel, data=dat, group="group", meanstructure=TRUE, estimator = "MLR")
##' cc2 <- cfa(weakModel, data=dat, group="group", meanstructure=TRUE, estimator = "MLR")
##' cc21 <- cfa(partialweakModel, data=dat, group="group", meanstructure=TRUE, estimator = "MLR")
##' cc3 <- cfa(partialstrongModel1, data=dat, group="group", meanstructure=TRUE, estimator = "MLR")
##'
##' models <- list(cc1, cc2, cc21, cc3)
##' ## Note, nesting is not specified, so built-in nesting detection applies
##' compareLavaan(models)
##' compareLavaan(models, type = "latex")
##' compareLavaan(models, type = "html")
##'
##' ## Now we specify model labels in the list
##' models <- list("Configural" = cc1, "Metric" = cc2, "PartialMetric" = cc21, "Scalar" = cc3)
##' ## The model labels are used in the nesting parameter
##' compareLavaan(models, nesting = "Configural > Metric + PartialMetric > Scalar")
##' 
##' compareLavaan(models, fitmeas = c("chisq", "df", "cfi", "rmsea", "tli"),
##'             nesting = "Configural > Metric + PartialMetric > Scalar")
##'
##' ## Creates output file
##' ## compareLavaan(models, fitmeas = c("chisq", "df", "cfi", "rmsea", "tli"),
##' ## nesting = "Configural > Metric + PartialMetric > Scalar", type = "tex",
##' ## file = "/tmp/table.tex")
##' }
compareLavaan <- function(models,
                       fitmeas = c("chisq", "df",  "pvalue", "rmsea", "cfi", "tli", "srmr", "aic", "bic"),
                       nesting = NULL, scaled = TRUE, chidif = TRUE, digits = 3,  ...)
{
    messg <- paste("This function is moving to the semTable package",
                   "Please use that package and function in the future.")
    .Deprecated(new = "compareLavaan", package ="semTable", msg = messg)
    dots <- list(...)
    if (!is.null(dots$type)){
        type <- dots$type
        if(type %in% c("latex", "tex")) type <- "latex"
        dots$type <- NULL
    } else {
        type <- NULL
    }
    print.results <- if (!is.null(dots$print.results)) dots$print.results else TRUE
    
    if (is.null(names(models))){
        names(models) <- paste0("Model", seq(1, length(models)))
    }
    estimators <- lapply(models, function(x) x@Options$estimator)
    if (length(unlist(unique(estimators))) > 1L){
        stop(paste("The models provided do not have the same estimator.",
                   "This function cannot handle models with different estimators."))
    }
    if(is.null(nesting)){
        nestedPairs <- detectNested(models)
    }else{
        xx <- unlist(strsplit(nesting, " "))
        ops <- c("+", ">")
        mods <- names(models)
        xx %in% c(ops, mods)
        if (prod(xx %in% c(ops, mods)) != 1){
            MESSG <- paste("nesting parameter has illegal symbols.",
                       "Only \">\", \"+\" are allowed and model names must",
                       "match the names in the model list")
            stop(MESSG)
        }
        yy <- grep(">", xx)
        parents <- xx[c(yy-1)]
        nested <- list()
        for (i in parents){
            nested[[i]] <-  strsplit(unlist(strsplit(paste0(xx[which(xx == i):length(xx)], collapse = ""), ">"))[2], "\\+")
        }
        pairs <- cbind("nested" = names(nested[1]), "parent" = NA)
        for (i in 1:length(nested)){
            pairs <- rbind(pairs, cbind("nested" = nested[[i]][[1]], "parent" = names(nested)[i]))
        }
        nestedPairs <- pairs
    }
    fitmeas_f <- if(scaled) c(fitmeas, paste0(fitmeas, ".scaled")) else  fitmeas
    observed <- names(lavaan::fitMeasures(models[[1]], fitmeas_f[fitmeas_f %in% names(lavaan::fitMeasures(models[[1]]))]))
    for (i in fitmeas){
        if(paste0(i, ".scaled") %in% observed){
            observed[which(observed == i)] <- paste0(i, ".scaled")
        }
    }
    observed <- observed[!duplicated(observed)]
    orderedMods <- list()
    for (i in nestedPairs[,"nested"]){
        orderedMods[[i]] <- models[[i]]
    }
    modelsums <- lapply(orderedMods, function(x) lavaan::fitMeasures(x, observed))
    sumtable <- do.call(rbind, modelsums)
    sumtable <- apply(sumtable, 2, round, 3)
    sumtable <- data.frame(sumtable)
    if(chidif){
        sumtable[,c("dchi", "ddf", "npval")] <- "-"
        letter <- 1
        noteinfo <- list()
        for (i in rownames(sumtable)){
            comparison <- nestedPairs[which(nestedPairs[,"nested"] == i),]
            if (!is.na(comparison["parent"])){
                tmp <- round(lavaan::anova(models[[comparison["nested"]]],
                       models[[comparison["parent"]]])[2, c("Chisq diff", "Df diff", "Pr(>Chisq)")], 3)
                sumtable[i, c("dchi", "ddf", "npval")] <- tmp
                sumtable[i, "dchi"] <- paste0(sumtable[i, "dchi"], letters[letter])
                noteinfo[[letter]] <- paste0(letters[letter], " = ", comparison["nested"], " vs ", comparison["parent"])
                letter <- letter + 1
            }
        }
        dframe <- sumtable
        attr(dframe, "noteinfo") <- unlist(noteinfo)
    } else {
        dframe <- sumtable
        noteinfo <- NULL
    }

    sanitize.colnames <- function(z){
        name_old <- c("^chisq$", "^pvalue$", "^dchi$", "^ddf$", "^npval$")
        name_new <- c("$\\\\chi^{2}$", "\\\\textit{p}-value",
                      "$\\\\Delta\\\\chi^{2}$", "$\\\\Delta df$", "\\\\textit{p}")
        texcode <- mgsub(name_old, name_new, z)
    }

    if(!is.null(type)){
        tableinfo <- dframe
        names(tableinfo) <- gsub(".scaled", "", names(tableinfo))
        xtab1 <- xtable(tableinfo)
        xtab1.legend <- if(!is.null(noteinfo)) noteinfo else NULL
        if(tolower(type) %in% c("tex", "latex")){
            print.xtable.argz <- list(type = NULL,
                                      file = NULL,
                                      print.results = TRUE,
                                      sanitize.colnames.function = sanitize.colnames,
                                      sanitize.rownames.function = escape,
                                      math.style.exponents = TRUE)
            if(!is.null(xtab1.legend)){
                print.xtable.argz[["add.to.row"]] =
                    list(list(NROW(xtab1)),
                         paste0("\\hline\n", paste0("\\multicolumn{4}{l}{", xtab1.legend,
                                                    "} \\\\\n", collapse="")))
            }
                                  
            print.xtable.args <- modifyList(print.xtable.argz, dots)
            print.xtable.args <- modifyList(list(x = xtab1), print.xtable.args)
            
            texcode <- do.call(print.xtable, print.xtable.args)

            if(!is.null(print.xtable.args$file)) {
                cat(texcode, file = print.xtable.args$file)
            }
            class(texcode) <- "kutable"
            return(invisible(texcode))
        } else {
            print.xtable.argz <- list(type = "html",
                                      file = NULL,
                                      print.results = TRUE,
                                      math.style.exponents = TRUE,
                                      html.table.attributes = list(border="0"))
            if(!is.null(xtab1.legend)){
                print.xtable.argz[["add.to.row"]] =
                    list(list(NROW(xtab1)), paste0("\n",
                       paste0("<tr><td colspan=\"4\">", xtab1.legend,
                              "</td></tr>\n", collapse="")))
            }
            print.xtable.args <- modifyList(print.xtable.argz, dots)
            print.xtable.args <- modifyList(list(x = xtab1), print.xtable.args)
            htmlcode <- do.call("print.xtable", print.xtable.args)
            if(!is.null(print.xtable.args$file)) {
                cat(htmlcode, file = print.xtable.args$file)
            }
            class(htmlcode) <- "kutable"
            return(invisible(htmlcode))
        }
    }
    if(print.results){
        print(dframe)
        cat(attr(dframe, "noteinfo"), fill = TRUE)
    }
    invisible(dframe)
}


##' A print method for kutable objects
##' 
##' @method print kutable
##' @export
##' @param x   object to be printed
##' @param ... optional arguments, corrently ignored
##' @return x unchanged
print.kutable <- function(x, ...){
    messg <- paste("This function is moving to the semTable package",
                   "Please use that package and function in the future.")
    .Deprecated(new = "print.kutable", package ="semTable", msg = messg)
    if(is.character(x)) cat(x)
    else print(x)
    x
}

%% LyX 2.3.7 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass[english,american,noae]{scrartcl}
\usepackage{lmodern}
\renewcommand{\sfdefault}{lmss}
\renewcommand{\ttdefault}{lmtt}
\usepackage[T1]{fontenc}
\usepackage[utf8]{inputenc}
\usepackage{geometry}
\geometry{verbose,tmargin=1in,bmargin=1in,lmargin=1in,rmargin=1in}
\usepackage{float}
\usepackage{setspace}
\usepackage[authoryear]{natbib}

\makeatletter

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% LyX specific LaTeX commands.
%% Because html converters don't know tabularnewline
\providecommand{\tabularnewline}{\\}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Textclass specific LaTeX commands.
<<echo=F>>=
  if(exists(".orig.enc")) options(encoding = .orig.enc)
@
\providecommand*{\code}[1]{\texttt{#1}}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
%\VignetteIndexEntry{variablekey}

\usepackage{booktabs}
\usepackage{Sweavel}
\usepackage{graphicx}
\usepackage{color}

\usepackage[samesize]{cancel}



\usepackage{ifthen}

\makeatletter

\renewenvironment{figure}[1][]{%

 \ifthenelse{\equal{#1}{}}{%

   \@float{figure}

 }{%

   \@float{figure}[#1]%

 }%

 \centering

}{%

 \end@float

}

\renewenvironment{table}[1][]{%

 \ifthenelse{\equal{#1}{}}{%

   \@float{table}

 }{%

   \@float{table}[#1]%

 }%

 \centering

%  \setlength{\@tempdima}{\abovecaptionskip}%

%  \setlength{\abovecaptionskip}{\belowcaptionskip}%

% \setlength{\belowcaptionskip}{\@tempdima}%

}{%

 \end@float

}


%\usepackage{listings}
% Make ordinary listings look as if they come from Sweave
\lstset{tabsize=2, breaklines=true, %,style=Rstyle}
                        fancyvrb=false,escapechar=`,language=R,%
                        %%basicstyle={\Rcolor\Sweavesize},%
                        backgroundcolor=\Rbackground,%
                        showstringspaces=false,%
                        keywordstyle=\Rcolor,%
                        commentstyle={\Rcommentcolor\ttfamily\itshape},%
                        literate={<<-}{{$\twoheadleftarrow$}}2{~}{{$\sim$}}1{<=}{{$\leq$}}2{>=}{{$\geq$}}2{^}{{$^{\scriptstyle\wedge}$}}1{==}{{=\,=}}1,%
                        alsoother={$},%
                        alsoletter={.<-},%
                        otherkeywords={!,!=,~,$,*,\&,\%/\%,\%*\%,\%\%,<-,<<-,/},%
                        escapeinside={(*}{*)}}%


% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\def\Sweavesize{\scriptsize} 
\def\Rcolor{\color{black}} 
\def\Rbackground{\color[gray]{0.95}}

% for sideways table
\usepackage{rotating}

\makeatother

\usepackage{babel}
\begin{document}
\title{The Variable Key Data Management Framework}
\author{\selectlanguage{english}%
Paul E. Johnson<pauljohn@ku.edu>\\
Benjamin A. Kite<bakite@ku.edu> \\
Center for Research Methods and Data Analysis\\
University of Kansas}
\date{\selectlanguage{english}%
\singlespacing{}\noindent \textbf{\today}}
\maketitle
\selectlanguage{american}%
\begin{abstract}
This essay describes the ``variable key'' approach to importing
and recoding data. This method has been developed in the Center for
Research Methods and Data Analysis at the University of Kansas to
deal with the importation of large, complicated data sets. This approach
improves teamwork, keeps better records, and reduces slippage between
the intentions of principal investigators the implementation by code
writers. The framework is implemented in the R \citep{RCore} package
\code{kutils}. 
\end{abstract}
<<echo=F>>=
if(!dir.exists("plots")) dir.create("plots")
@

\selectlanguage{english}%
% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\SweaveOpts{prefix.string=plots/t,split=F,ae=F,height=4,width=5.5}

<<Roptions, echo=F>>=
options(device = pdf)
options(width=100, prompt=" ", continue="  ")
options(useFancyQuotes = FALSE) 
options(SweaveHooks=list(fig=function() par(ps=10)))
pdf.options(onefile=F,family="Times",pointsize=10)
@
\selectlanguage{american}%

\section{Introduction}

The staff of the Center for Research Methods and Data Analysis has
been asked to help with data importation and recoding from time to
time. In one very large project, we were asked to combine, recode,
and integrate variables from 21 different files. The various files
used different variable names and had different, unique coding schemes.
A skeptic might have thought that the firm which created the data
sets intentionally obfuscated the records to prevent the comparison
of variables across a series of surveys.

In projects like that, the challenge of importing and fixing the data
seems overwhelming. The graduate research assistants are asked to
cobble together thousands of lines of ``recodes'' as they rename,
regroup, and otherwise harmonize the information. From a managerial
point of view, that is not the main problem. We expect to spend the
time of research assistants. While it may be tedious to read a codebook
and write recodes, social scientists have been doing that since the
1960s. It is not all that difficult. The truly difficult part is mustering
up the confidence in the resulting recoded data. How can a supervisor
check thousands of recode statements for accuracy? The very extensibility
of R itself–its openness to new functions and language elements–makes
proof-reading more difficult. We might shift some of the proof reading
duty to the principle investigators, but they sometimes are not interested
in details. In the end, the responsibility for verifying the recodes
falls on the project supervisors. While most supervisors with whom
we are personally acquainted have nearly super-human reading skills
and almost-perfect comprehension, we have documented a case in which
one of them was unable to catch an error on line 827 within an R file
with 2119 lines. 

To reduce the risk of misunderstanding and error, we propose the \emph{variable
key procedure}. It is a systematic way to separate code writing from
the process of renaming variables and re-designating their values.
The characteristics of the data are summarized in a table, a simple-looking
structure that might be edited in a text editor or a spread sheet
program. This simple structure, which we call the variable key, can
be used by principal investigators and supervisors to designate the
desired results. Once the key is created, then the data set can be
imported and recoded by the application of the key's information.
This does not eliminate the need to proof-read the renaming and recoding
of the variables, it simply shifts that chore into a simpler, more
workable setting.

This essay proceeds in 3 parts. First, the general concepts behind
the variable key system are explored. Second, the four stages in the
variable key procedure are outlined and illustrated with examples.
Third, we offer some examples of ways to double-check the results.

\section{Enter the Variable Key}

The variable key process was first developed for a very large project
for which we were hired by a commercial consulting company. As it
happened, the project manager who hired us was an Excel user who did
not know about R. He was given several SPSS datasets. After going
through the usual R process of importing and recoding data from 6
files, the aggregate of which included more than 40,000 observations
on 150 variables, we arrived at a renamed set of columns. Unfortunately,
the research assistant who had done most of the work resigned in order
to pursue a career as a magician.\footnote{Or graduated, we are not sure which.} 

With the unavailability of our key asset, it was difficult to know
for sure what was in which column. There was nobody to quickly answer
questions like ``which column is the respondent's sexual identity?''
and ``if sex is V23418, did we change 1 to male or female''. The
only way to find out is by hunting and pecking through a giant R file. 

In order to better communicate about that project, we developed a
table that looked like Table \ref{tab:A-Small-Variable-key}.

\begin{table}[H]
\caption{A Small Variable Key\label{tab:A-Small-Variable-key}}

\begin{tabular}{|c|c|c|c|}
\hline 
name\_old & name\_new & values\_old & values\_new\tabularnewline
\hline 
\hline 
V23419 & sex & 1|2|3 & ``male''|''female''|''neither''\tabularnewline
\hline 
V32422 & education & 1|2|3|4|5 & ``elem''<''hs''<''somecoll''<''ba''<''post''\tabularnewline
\hline 
V54532 & income & . & numeric\tabularnewline
\hline 
\end{tabular}
\end{table}

It was tedious to assemble that table, but it helped quite a bit in
our discussions. The vertical bars were used to indicate that the
original data had discrete values. When a variable has a natural ordering,
the new values were placed in order with the symbol (``<''). That
table grew quite large, since it had one row per variable, but it
was otherwise workable. It was popular with the client. 

In the middle of preparing that summary table of recoded values, we
realized that it was possible to write an R program to import the
key table and use its information to recode and rename the variables.
The recodes would \emph{just happen}. If we prepared the functions
properly, we had not just a table masquerading as a codebook, we had
a \emph{programmable codebook}. We wrote some functions that could
import variables (as named in column 1), apply the new values (from
columns 3 and 4), then apply the new name from column 2. The functions
to do that are somewhat difficult to prepare, but they are very appealing
from a supervisor's point of view. There will be less proof-reading
to do, at least in the R code itself. Once we can validate the functions,
then we never have to proof-read them again. These functions can be
applied, row by row, to create a new data frame. Instead, we need
to concentrate our attention on the substance of the problem, the
specification of the new names and values in the table.

In the projects where we have employed this system, we adjusted the
key and the R functions to suit the particular demands of the project
and the client. That was unfortunate, because we had very little accumulation
of code from one project to another. However, we did accumulate experience;
there were concepts and vocabulary which allowed us to understand
the various challenges that might be faced. The effort to develop
a standardized framework for the variable key began in 2016 with the
creation of the \code{kutils} package for R.

The variable key process allows project supervisors to create a table
that instructs the research assistants in the importation, renaming,
and recoding of data. There is still a daunting problem, however,
because the supervisors must create that variable key table. In a
large data set, it might be arduous to simply type the old names of
the variables and their observed values. In 2015 one of the graduate
assistants in our lab was asked to type up a variable key and he couldn't
quite believe that was a good use of his time. After some discussion,
we realized that it was not necessary to type the variable key at
all. We would write a function to do so. If R can import the candidate
data set, then R can certainly output its column names and a roster
of observed values. This lightened the workload considerably. By tabulating
all of the observed variables and their values, the most tedious part
of the process was done mechanically. 

In the remainder of this essay, we discuss the process of creating
a variable key template, revising it, and putting it to use. 

\section{Four Simple Steps}

The variable key process has four steps. First, inspect an R data.frame
object and create a key template file. The key template summarizes
the existing state of the variables and creates ``placeholders''
where we might like to specify revisions. Second, edit the key template
file in a spreadsheet or other program that can work with comma separate
variables. Change the names, values, and designate other recodes (which
we will describe next). Third, import the revised key into R. Fourth,
apply the key to the data to generate a new, improved data frame.
Then run some diagnostic routines.

If all goes well, we should end up with a new data frame in which
\begin{enumerate}
\item The columns are renamed in accordance with the instructions of the
principal investigator (or supervisor).
\item The values of all variables have been recoded according to the instructions
of the principal investigator (or supervisor).
\end{enumerate}
Diagnostic tables are reported to clearly demonstrate the effect of
each coding change, mapping out the difference between the input and
the output variables. 

For purposes of illustration, we have create an example data frame
with various types of variables. This data frame, \code{mydf}, has
most of the challenges that we see in actual projects. It has integer
variables that need to be reorganized and turned into character or
factor variables. It has character variables that might become integers
or factors. 

<<eval=T,include=T>>=
set.seed(234234)
N <- 200
mydf <- data.frame(
    x5 = rnorm(N),
    x4 = rpois(N, lambda = 3),
    x3 = ordered(sample(c("lo", "med", "hi"), size = N, replace=TRUE),
            levels = c("med", "lo", "hi")),
    x2 = letters[sample(c(1:4,6), 200, replace = TRUE)],
    x1 = factor(sample(c("cindy", "jan", "marcia"), 200,
            replace = TRUE)),
    x7 = ordered(letters[sample(c(1:4,6), 200, replace = TRUE)]),
    x6 = sample(c(1:5), 200, replace = TRUE),
            stringsAsFactors = FALSE)
mydf$x4[sample(1:N, 10)] <- 999
mydf$x5[sample(1:N, 10)] <- -999
@

\subsection{Step 1. Create a Key Template}

The function \code{keyTemplate} scans a data frame and generates
a new key template. The key has 8 pieces of information about each
variable. The rows of the key are named by the variables of the data
frame. The 8 columns in the key are \code{name\_old}, \code{name\_new},
\code{class\_old}, \code{class\_new}, \code{value\_old}, \code{value\_new},
\code{missings}, and \code{recodes}. \code{keyTemplate} will fill
\code{name\_old}, \code{class\_old}, and \code{value\_old}, in
the with values based on the data input, while the \code{new} columns
will be copies of those old values. The last 2, missings and recodes,
will be empty. 

There are two formats for the key template, \code{long} and \code{wide}
(determined by the parameter \code{long}). These names are drawn
from terminology in R's reshape function. The long format has one
row per value of each variable, while the wide format has all of the
information in one row. The two key formats are intended to be interchangeable
in functionality; they differ solely for convenience. Some users may
prefer to edit variable information in one style. The re-importation
of the key should deal gracefully with either type of variable key. 

A wide format key can be produced with a call to the \code{keyTemplate}
function like so:

<<eval=F>>=
key_wide <- keyTemplate(mydf, file = "key_wide.csv", max.levels = 5)
@

\noindent If the long argument is not specified, a wide key is the
default. One can ask for a long format:

<<eval=F>>=
key_long <- keyTemplate(mydf, long = TRUE, file = "key_long.csv", max.levels = 5)
@

\noindent The key object is a data.frame. 

Apart from the long argument, the \code{keyTemplate} function has
two especially noteworthy arguments, \code{file} and \code{max.levels}.
If the file argument is supplied, \code{keyTemplate} uses the suffix
to determine storage format. Legal suffixes are \code{.csv}, \code{.xlsx},
and \code{.rds} (for creating comma separated variables, Excel spreadsheets,
and R serialization data structures). 

The \code{max.levels} argument is also important. This is used in
the same sense that functions like \code{read.spss} in the foreign
package use that term. There is guessing involved in deciding if we
should enumerate a character or integer variable. We do want to enumerate
the ``Strongly Disagree'' to ``Strongly Agree'' values of a 7
point scale, but we do not want to enumerate the first names of all
study participants. If the number of discrete values exceeds \code{max.levels},
for which the default is 15, then the key will not enumerate them. 

Table \ref{tab:The-Wide-Key} demonstrates a wide key template as
it is produced by \code{keyTemplate}. We see now why it is called
a wide key; the recoding information is tightly packed into \code{value\_old}
and \code{value\_new}. The key includes more or less obvious columns
for the old and new variable names, their classes, and values of the
variables. Note that the values of x5 and x4 are not enumerated because
we set max.levels at 5. The \code{max.levels} parameter defaults
to 15, so that an integer variable with less than 15 values will have
each value displayed. For this example, the display of that variable
key was too wide for the page, so we reduced the number of values.
When the observed number of scores is above \code{max.levels}, the
key does not try to list the individual values (compare the treatment
of variables \code{x4} and \code{x6}). 

<<key0, include=F>>=
library(kutils)
library(xtable)
@

<<key10, echo=F>>=
key_wide <- keyTemplate(mydf, max.levels = 5)
@

A long key template is displayed in Table \ref{tab:Long-Key}. The
benefit of the long key is that the cells \code{value\_old} and \code{value\_new}
are easier to navigate.

<<key30,echo=F>>=
key_long <- keyTemplate(mydf, long = TRUE, max.levels = 5)
@

\begin{table}[h]
\caption{The Wide Key Template\label{tab:The-Wide-Key}}

\def\Sweavesize{\tiny}
<<key20,echo=FALSE,results=tex>>=
print(xtable(key_wide), include.rownames = FALSE, size = "small", floating = FALSE )
@
\end{table}

\begin{table}
\caption{The Long Key Template\label{tab:Long-Key}}

\def\Sweavesize{\tiny}
<<key40,echo=FALSE,results=tex>>=
print(xtable(key_long), include.rownames = FALSE, size = "small", floating = FALSE)
@
\end{table}

The value of \code{class\_old} in the key is the first element in
the return from the function class for a variable. There is one exception,
where we have tried to differentiate integer variables from numeric
variables. This is a confusing isssue in the history of R, as discussed
in the R help page for the function \code{as.double}. ,In the \emph{Note
on names} section, that page explains an ``anomaly'' in the usage
of term \code{numeric}. The R function \code{as.numeric} creates
a double precision floating point value, not an integer. However,
the is.numeric function responds TRUE for both integers and floating
point values. For purposes of editing the key, it is useful to differentiate
integers from floating point numbers. \code{kutils} includes a function
named \code{safeInteger}. It checks the observed values of a variable
to find out if any floating point values are present. If the aggregate
deviations from integer values are miniscule, then a variable is classified
as an integer. As a result, the keyTemplate function's column \code{class\_old}
should be ``integer'' or ``numeric'', and by the latter we mean
a floating point number. 

In some of our early projects, the variable key was in the wide format.
Difficulty in editing that caused us to shift some projects to the
long key. The idea that we would glide back and forth between keys
created in the wide and long formats dawned on us only recently. To
ease the conversion back-and-forth between the formats, we developed
the functions named \code{wide2long} and \code{long2wide}. We believe
these functions work effectively, but we have experienced some troubles
related to the way spreadsheets store character strings. If the key
in long format has a column of values ``Yes'',''No'', and ``'',
the wide representation should be ``Yes|No|'', but there is some
inclination to say we should have nested quotation marks, as in ``''Yes''|''No''|''{}''{}''.
That kind of string will not generally survive importation to and
export from a spread sheet at the current time.

\subsection{Step 2. Edit the variable key}

If the file argument was specified in \code{keyTemplate}, the work
is laid out for us. One can edit a csv file in any text editor or
in a spreadsheet. An xlsx file can be edited by Libre Office or Microsoft
Office. 

It is not necessary to change all of the values in name\_new, class\_new,
and value\_new. In fact, one might change just a few elements and
the un-altered variables will remain as they were when the data is
re-imported. We suggest users start small by making a few edits in
the key. A principal investigator might change just a few variable
names or values. In a large project, there may be quite a bit of work
involved. 

The \code{name\_old} column must never be edited. Generally, \code{class\_old}
and \code{value\_old} will not be edited either (the only exception
might arise if \code{class\_new} is either ``\code{factor}'' or
``\code{ordered}''). The \code{name\_new} column should include
legal R variable names (do not begin name\_new with a numeral or include
mathematical symbols like ``+'' or ``-''). We use R's \code{make.names}
function to clean up errant entries, so incorrect names are not fatal.

The difficult user decisions will concern the contents of class\_new
and value\_new. The desired variable type, of course, influences the
values that are meaningful. To convert a character variable to integer,
for example, it should go without saying that the value\_new element
should include integer values, not text strings.

The conversion of information from one type of variable into another
may be more complicated than it seems. It is a bit more tricky to
convert a factor into a numeric variable that uses the factor's levels
as numeric values. 

After experimenting with a number of cases, we believe that if \code{class\_old}
and \code{class\_new} are elements of this the \emph{safe class set}:
\code{character}, \code{logical}, \code{integer}, \code{numeric}
(same as \code{double}), \code{factor}, or \code{ordered}, then
the re-importation and recoding of data will be more-or-less automatic.
If \code{class\_new} differs from \code{class\_old}, and \code{class\_new}
is not an element in that 6 element set, then the user must supply
a recode function that creates a variable of the requested class.
Most commonly, we expect that will be used to incorporate \code{Date}
variables.

The enumerated values in the \code{value\_new} column should be specified
in the more or less obvious way. If \code{class\_new} is equal to
\code{character}, \code{factor}, or \code{ordered}, then the new
values can be arbitrary strings. 

The \code{missings} and \code{recodes} columns are empty in the
key template. The user will need to fill in those values if they are
to be used. When the key is later put to use, the order of processing
will be as follows. First, the values declared as missings will be
applied (convert observed value to R's NA symbol). Second, if there
is a recode function in the key, it is applied to a variable. Third,
if there was no recode function supplied, then the conversion of discrete
values by recalculation from \code{value\_old} into \code{value\_new}
will be applied. Note that the discrete values are applied only if
the recode cell is empty in the key.

The decision of whether to approach a given variable via value enumeration
or a recode function is, to an extent, simply a matter of taste. Some
chores that might be handled in either way. If a variable includes
floating point numbers (temperatures, dollar values, etc), then we
would not rely on new assignments in \code{value\_old} and \code{value\_new}.
Truly numeric variables of that sort almost certainly call for assignment
of missings and recodes by the last two cells in the variable key.
However, if a column includes integers or characters (1 = male, 2
= female), one might use the enumerated values (\code{value\_old}
and \code{value\_new}) or one could design a recode function to produce
the same result. It is important to remember that if a recode function
is applied, the enumerated value recoding is not. If one decides to
use a recode statement, then the elements in value\_old and value\_new
are ignored entirely, they could be manually deleted to simplify the
key. (That is to say, the \code{max.levels} parameter is just a way
of guessing how many unique levels is ``too many'' for an enumeration.
Users are free to delete values if recodes are used.)

Despite the possibility that a factor (or ordered) variable may have
many values, we believe that all of the levels of those variables
should be usually be included in the key. If a variable is declared
as a factor, it means the researcher has assigned meaning to the various
observed values and we are reluctant to ignore them. 

There is a more important reason to enumerate all of the legal values
for factor variables. If a value is omitted from the key, that value
will be omitted from the dataset. 

Among our users, we find opinion is roughly balanced between the long
and the wide key formats. One might simply try both. If the number
of observed values is more than 5 or 10, editing the key in a program
like Microsoft Excel is less error prone in the long key. This is
simply a matter of taste, however. The disadvantage of the long format
is that it is somewhat verbose, with repeated values in the name and
class values. If an editor makes an error in the assignment of a block,
then hard to find errors may result.

Because editing the key can be a rather involved process, we will
wait to discuss the details until section \ref{sec:Editing-the-key}.

\subsection{Step 3. keyImport}

Once any desired changes are entered in the variable key, the file
needs to be imported back into R. For that purpose, we supply the
\code{keyImport} function. As in \code{keyTemplate}, the file argument's
suffix is used to discern whether the input should be read as .csv,
.xlsx, or .rds. It is not necessary to specify that the key being
imported is in the long or wide format. \code{keyImport} includes
heuristics that have classified user-edited keys very accurately. 

The returned value is an R data frame that should be very similar
to the template, except that the new values of \code{name\_new},
\code{class\_new}, and \code{value\_new} will be visible. 

In order to test this function with the \code{kutils} package, we
include some variable keys. The usage of those keys is demonstrated
in the help page for \code{keyImport}. In addition to the mydf toy
data frame created above, we also include a subset of the US National
Longitudinal Survey in a data frame named \code{natlongsurv}.

\subsection{Step 4. Apply the imported key to the data}

The final step is to apply the key to the data frame (or some other
data frame that may have arrived in the interim). The syntax is simple

<<eval=F>>=
mydf.cleaned <- keyApply(mydf, mydf.keylist)
@

Because the default value of the argument \code{diagnostic} is TRUE,
the output from \code{keyApply} is somewhat verbose. After we have
more feedback from test users, we will be able to quiet some of that
output. 

The diagnostic output will include information about mismatch between
the key and the data itself. If variables that are included in the
key that are not included in the new data set, there will not be an
error, but a gentle warning will appear. Similarly, if the observed
values of an enumerated variable are not included in the variable
key, there will be a warning. 

The diagnostic will also create a cross tabulation of each new variable
against its older counterpart. This works very well with discrete
variables with 10 or so values, but for variables with more values
it is rather unmanageable. 

\section{Editing the variable key}

\label{sec:Editing-the-key}

The work of revising the variable key can be driven by the separation
of variables into two type. The variables with enumerated values–the
ones for which we intend to rename or re-assign values one-by-one–are
treated in a very different way than the other ones. The enumerated
value strategy works well with variables for which we simply need
to rename categories (e.g, ``cons'' to ``Conservative''). Variables
for which we do not do so (e.g., convert Fahrenheit to Celsius) are
treated differently.

As we will see in this section, the revision of variables of the enumerated
value type emphasizes the revision of the value\_old and value\_new
columns in the key. On the other hand, the other types will depend
on writing correctly formatted statements in the recode column of
the variable key.

\subsection{Enumerated variables}

All of the values observed for \code{logical}, \code{factor}, and
\code{ordered} variables will appear in the key template. Do not
delete them unless the exclusion of those values from the analysis
intended. For character and integer variables with fewer than \code{max.levels}
discrete values, the observed scores will be included in \code{value\_old}.
If one wishes to convert a variable from being treated as an enumerated
to a numeric type, then one can delete all values from \code{value\_old}
and \code{value\_new}. 

The recoding of discrete variables is a fairly obvious chore. For
each old value, a new value must be specified. We first consider the
case of a variable that enters as a character variable but we might
like to recode it and also create factor and integer variants of it.
In the \code{mydf} variable key (Table \ref{tab:The-Wide-Key}),
we have variable \code{x2} which is coded \code{a} through \code{f}.
We demonstrate ways to spawn new character, factor, or integer variable
in Table \ref{tab:Change-Type-Example1}. As long as name\_old is
preserved, as many lines as desired can be used to create variables
of different types. Here we show the middle section of the revised
key in which we have spawned 3 new variants of x2, each with its own
name.

\begin{table}[H]
\caption{Change Class, Example 1\label{tab:Change-Type-Example1}}

\begin{tabular}{cccccc}
\hline 
name\_old & name\_new & class\_old & class\_new & value\_old & value\_new\tabularnewline
\hline 
x2 & x2.char & character & character & a|b|c|d|f & Excellent|Proficient|Good|Fair|Poor\tabularnewline
x2 & x2.fac & character & factor & a|b|c|d|f & Excellent|Proficient|Good|Fair|Poor\tabularnewline
x2 & x2.gpa & character & integer & a|b|c|d|f & 4|3|2|1|0\tabularnewline
\hline 
\end{tabular}
\end{table}

In line one of Table \ref{tab:Change-Type-Example1}, the class \code{character}
remains the same. That line will produce a new character variable
with embellished values. Line two demonstrates how to create an R
factor variable, \code{x2.fac}, and line three converts the character
to an integer variable. Remember that it is important to match the
value of \code{class\_new} with the content proposed for \code{value\_new}.
Do not include character values in a variable for which the new class
will be numeric or integer. 

Similarly, it is obvious to see how an integer input can be converted
into either an integer, character, or factor variable by employing
any of the rows seen in Table \ref{tab:Change-Type-Example2}.

\begin{table}[H]
\caption{Change Class Example 2\label{tab:Change-Type-Example2}}

\begin{tabular}{cccccc}
\hline 
name\_old & name\_new & class\_old & class\_new & value\_old & value\_new\tabularnewline
\hline 
x6 & x6.i100 & integer & integer & 1|2|3|4|5 & 100|200|300|400|500\tabularnewline
x6 & x6.c & integer & character & 1|2|3|4|5 & Austin|Denver|Nashville|Provo|Miami\tabularnewline
x6 & x6.f & integer & factor & 1|2|3|4|5 & F|D|C|B|A\tabularnewline
\hline 
\end{tabular}
\end{table}

If a variable's \code{class\_old} is ordered, and we simply want
to relabel the existing levels, the work is also easy (see Table \ref{tab:Change-Type-Example3}).
The second row in Table \ref{tab:Change-Type-Example3} shows that
factor levels can be ``combined'' by assigning the same character
string to several ``<'' separated values. 

\begin{table}[H]
\caption{Change Class, Example 3\label{tab:Change-Type-Example3}}

\begin{tabular}{cccccc}
\hline 
name\_old & name\_new & class\_old & class\_new & value\_old & value\_new\tabularnewline
\hline 
x7 & x7.grades & ordered & ordered & f<d<c<b<a & F<D<C<B<A\tabularnewline
x7 & x7.passfail & ordered & ordered & f<d<c<b<a & Fail<Fail<Pass<Pass<Pass\tabularnewline
x7 & x7.gpa & ordered & integer & f<d<c<b<a & 0|1|2|3|4\tabularnewline
\hline 
\end{tabular}
\end{table}

Working with ordered variables, whether as input or output, becomes
more complicated if the existing data are not ordered in the way we
want. In the \code{mydf} example, the variable \code{mydf\$x3} was
coded as an ordered variable with levels (``med'', ``lo'', ``high'').
That might have been one person's idea of a joke, so we need to rearrange
these as (``lo'', ``med'', ``high''). If the original ordering
of the values is not consistent with the desired ordering of the new
ordered factor, then we need notation that allows researchers to achieve
two purposes. First, the values must be re-leveled. Second, allow
for the possibility that the new values must be relabeled as well.
We'd rather not proliferate new columns in the variable key or create
some confusing new notation.

Reordering variable levels requires us to do something that seems
dangerous. We need to edit the \code{value\_old} to correct the ordering
of the levels \emph{as they are currently labeled}. This is the only
time where we suggest that users edit the \code{value\_old} column.
In \code{value\_new}, supply new labels in the in the correct order
to parallel the newly edited \code{value\_old} column (see Table
\ref{tab:Reorder-Values}).

\begin{table}[H]
\caption{Reorder Values, Example 1\label{tab:Reorder-Values}}

\begin{tabular}{cccccc}
\hline 
name\_old & name\_new & class\_old & class\_new & value\_old & value\_new\tabularnewline
\hline 
x3 & x3.lo2hi & ordered & ordered & low<med<hi & low<medium<high\tabularnewline
x3 & x3.passfail & ordered & ordered & low<med<hi & low<pass<pass\tabularnewline
\hline 
\end{tabular}
\end{table}

The key importer will check that all ``duplicated levels'' are adjacent
with one another, so that the values above low are grouped together.

In the long key format, the equivalent information would be conveyed
by altering the ordering of the rows. For example, it is necessary
to re-order the rows to indicate the lo is lower than med, and then
for the new values we put in the desired names (see Table \ref{tab:Reorder-Values-Example2}).

\begin{table}[H]
\caption{Reorder Values, Example 2\label{tab:Reorder-Values-Example2}}

\begin{tabular}{llllllll}
\toprule
name\_old & name\_new & class\_old & class\_new & value\_old & value\_new\\
\midrule
x3 & x3lo2hi & ordered & ordered & lo & low\\
x3 & x3lo2hi & ordered & ordered & med & medium\\
x3 & x3lo2hi & ordered & ordered & hi & high\\
x3 & x3.passfail & ordered & ordered & low & low\\
x3 & x3.passfail & ordered & ordered & medium & pass\\
x3 & x3.passfail & ordered & ordered & high & pass\\
\bottomrule
\end{tabular}
\end{table}

What to do about missing values. Even within our small group, there
is some disagreement about this. The SAS tradition would have us enter
a period, ``.'', in value\_new for a level that is to be treated
as missing. Others are tempted to use special purpose character strings
like ``NA'' or ``N/A''. As long as neither ``NA'' nor ``N/A''
are legal values, that seems safe. There is another school of thought
to argue that if a user wants nothing, the clearest, safest thing
to do in the key is to enter nothing at all. 

As a result of this diversity, the keyImport function includes an
argument named na.strings (again, an argument drawn from R core funcitons).
The default setting has an ``all of the above'' flavor, treating
any of the values that any of us think might be missing as NA when
the key is later put to use. If one is fastidious in editing the key
and represents all value\_new that should be missing as a period (hooray
for SAS!), then the value of na.strings can be tightened up in the
obvious way. 

It is also possible to delete a level from value\_old and value\_new
in order to indicate that it should be treated as missing, but we
do not recommend this approach. This destroys an element of ``book
keeping'' because future users of the data, and the key, might like
to know that some levels were obliterated.

The missing value cell in the key can be used with these enumerated
variables. Usually, writing the values to be omitted is not as easy
as putting the desired missing symbol in the value\_new column, but
either method should work.

In a long format key, there will be many repeated rows for each variable.
The entries in the \code{missings} and \code{recodes} columns are
harvested from the cells corresponding to each combination of name\_old
and name\_new, it is not necessary to retype the \code{missings}
and \code{recodes} entries for each element.

The work of assigning missing values is carried out by a \code{kutils}
function named \code{assignMissing}. The help page for that function
has verbose commentary to explain the format of the cells that might
work well in the missings column.


\subsection{About non-enumerated variables}

In the process that creates the variable key template, we think differently
about the enumerated variables–ones that have meaningfully discrete
values that can be reassigned one-by-one–and numeric variables. Numeric
variables, whether we literally mean integers like age or the number
of pennies in a jar, or floating point numbers (weight, temperature,
volume, etc) are different. If we guessed right with \code{max.levels}
when the key was created, those non-discrete variables are included
as just one row, whether the key is wide or long. In those variables,
the main elements of interest for these variables are the columns
labeled \code{missings} and \code{recodes}. 

The instructions for specifying missing values are detailed in the
help page of the \code{assignMissing} function in \code{kutils}.
If the class\_old is \code{numeric} or \code{integer}, there are
only three types of statements allowed in the \code{missings} column.
Legal values must must begin with the characters ``<'', ``>'',
or ``c''. These are illustrated in Table \ref{tab:Recode-Examples}.
The symbols ``<='' and ``>='' are accepted in the obvious way. 

\begin{table}[H]
\caption{Missings Examples\label{tab:Recode-Examples}}

\begin{tabular}{|c|c|c|}
\hline 
missings & interpretation: NA will be assigned to  & example\tabularnewline
\hline 
> t & values greater than t & > 99\tabularnewline
>= t & values greater than or equal to t  & >=99\tabularnewline
<t & values less than t  & <0\tabularnewline
<=t & values less than or equal to t  & <=0\tabularnewline
c(a,b) & values equal to or greater than a and less than or equal to b  & c(-999,-1)\tabularnewline
\hline 
\end{tabular}
\end{table}

The key specification for \code{recodes} is discussed in the help
page of the \code{assignRecode} function in \code{kutils}. The \code{recodes}
column takes R code and applies it to the desired variable. For example,
if one wanted to transform a variable by taking its square root, this
could be done by providing ``sqrt(x)'' in the \code{recodes} column.
Here ``x'' is simply a placeholder where the name of the variable
indicated in the \code{name\_new} column will be inserted when the
variable key is applied. 

\begin{table}[H]
\caption{Recoding Integer and Numeric Variables}

{\footnotesize

\begin{tabular}{lllllllll}
  \toprule
name\_old & name\_new & class\_old & class\_new & value\_old & value\_new & missings & recodes \\
  \midrule
x5 & x5 & numeric & numeric &  &  & <0 & log(x) \\
x4 & x4 & numeric & numeric &  &  & c(-999) & abs(x) \\
x6 & x6 & integer & integer & 1$|$2$|$3$|$4$|$5 & 1$|$2$|$3$|$4$|$5 & c(-9) & \\
   \bottomrule
\end{tabular}
}
\end{table}


\subsection{Class conversions}

The problem of recoding a variable, but leaving its class the same,
is mostly solved. 

The conversion of variables from one class to another requires special
care. R's built in functions for coercion of variables from one type
to another succeed meaningfully in many cases, but not all. The coercion
of an integer to a floating point number produces understandable effects.
The coercion of a character to an integer is not always understandable,
and the conversion of factors to integers or numeric is, well, almost
always a source of concern.

We are intending to support, at minimum, the 6 safe classes, which
can be used in any combination of \code{value\_old} and \code{value\_new}.
We need to be sure that conversion from each one into the other types
is handled accurately. 

In the \code{keyApply} function, we have implemented special purpose
code to handle a class change from factor to numeric, for example,
in a way that preserves the levels as the new values. This is needed
when the input data has values like 10, 12, 14, and ``unknown''.
If an R function like read.table is used to import a column with those
values, the default behavior will convert that to a factor variable
with values ``10'', ``12, ``14'' and ``unknown''. What the
user probably wants–and what we do now–is open the possibility that
``unknown'' should be treated as NA and the values that appear to
be numbers (but are actually text strings) are converted into integer
format.

\subsection{Unanticipated \protect\code{class\_new} values}

By far, the most troublesome variable type is date and time information.
This is difficult for a number of reasons. In order from less troublesome
to more troublesome:
\begin{enumerate}
\item No two people seem to represent dates with the same style. Virtuous
people (the authors) write 2016-12-01, while others write ambiguous
strings like 12/01/2016, 01/12/2016 or 01-12-2016.
\item Computer programs absorb dates and convert them to internal numeric
schemes. However, programs differ in the way they record and export
date information. Most software converts dates as integers, for example.
The value is the number of days since time began. 

Unfortunately, it appears that the day on which time began differs
among programs. Microsoft Excel records dates as integers, and (shockingly!)
the origin date differs between Excel files created in Windows and
Macintosh. Mac designers were perplexed by the problem of leap years,
which caused them to use the origin 1904-01-01, while in MS Windows
Excel dates begin at 1900-01-01. The program SPSS uses integers for
dates as well, but the origin for SPSS is 1582-10-14 (we guess most
readers already knew that one, we apologize for being surprised). 

On the other hand, dates on Unix/Linux systems tend to follow the
POSIX standard, so date information is interpreted as the number of
days since 1970-01-01. That is a local favorite because it reminds
us of Nixon and Vietnam.
\item Dates are, to a certain extent, arbitrary without time zone information.
Its Tuesday, December 8 in Lawrence, Kansas, but it is Wednesday,
December 9 in Moscow, Russia. If time zones are not specified, than
all date information is, well, approximate. Furthermore, when scholars
try to put time zone information onto existing data that was collected
without it, they sometimes accidentally create impossible dates and
times (February 29, 1999, for example). When R functions like \code{as.Date}
encounter impossible dates, they return NA.
\item Even when programs seem to have absorbed dates and saved the information,
even in casual testing we have found that the saved information is
not retrieved in a consistent way. Some spreadsheet programs, on some
computers, return values dates as integers, while the same program
with a different user returns a character string. 
\end{enumerate}
All of this complaining is just our way of saying, ``we left the
recode box open, why not use it to recode your date information?''
If the variable key template says that a column is ``integer,''
but you expected a date, then all is likely to be fine if we set the
origin. One needs to find out what the origin should be and tell R
about it. The right recode statement is likely to be \code{as.Date(x, origin = \textquotedbl 1970-01-01\textquotedbl )}.
On the other hand, if the date information is in text format, with
values like ``2016-11-19'', then it is necessary to learn the shorthand
notation for date symbols. The right recode will be something like
\code{as.Date(x, format = \textquotedbl\%Y-\%m-\%d\textquotedbl )}.

If one sets \code{class\_new} = \code{Date}, the \code{keyApply}
function will succeed if the return value from the recode function
is an R Date variable. Otherwise, an error will be raised.

\subsection{Where more work should be done}

For the most part, our work now lies in development of better diagnostics
and error-checking routines. 

First, we need better filtering of user-created key entries. User
typographical errors occur in key preparation and more comprehensive
filtering should be done. At the current time, the \code{keyImport}
function does not include many subroutines for the validation of the
key. We'd like to filter the values requested in \code{class\_new},
for example, but it is difficult to see how that can be done. We want
to leave open the possibility that users may specify values of \code{class\_new}
that are unfamiliar to us. Another problem may arise if the number
of elements in \code{value\_old} and \code{value\_new} are not aligned
one-to-one. There will be an error when the key is put to use by \code{keyApply},
but we should have a stop message in \code{keyImport}.

Second, we need to better understand the problems that result when
users enter variable keys in programs like MS Excel or Libre Office.
Between programs, or versions of programs, or programs on various
platforms, we notice disappointing inconsistencies. If keys are kept
in ``flat text'' csv files, there is less danger, but there can
still be some trouble because spread sheets change the way they export
to csv from time to time. During the testing process, we have found
that version updates Excel cause unexpected changes in the way character
vectors and dates are stored. Until now, we have relied on the very
popular package \code{openxlsx} (\citealp{Walker2015}). This does
well most of the time, but we still find some inexplicable variations
in imported Excel files. Sometimes, empty cells in the variable key
appear as imported empty strings ``'', while sometimes they are
imported as R NA symbols. While it is probably safe to treat an entirely
empty value as a missing value in most projects, we'd rather not do
so. To deal with this problem, we need to develop a standard framework
for testing the quality of imported variable keys. 

Third, we need more comprehensive error-checking for the accuracy
of the imported data and compliance with the variable key. The variable
key system works well with the data sets for which it was created,
but we need more formal criteria for verifying that claim. 

\section{Discussion}

When a project has a small budget, we invite the principal investigator
to economize on the expenses by filling out the variable key's \code{name\_new},
\code{class\_new}, and \code{value\_new} columns. There are several
benefits in inviting the clients (or PIs) to be directly involved
in filling in the variable key. Most importantly, they are allowed
to name the variables in any way that is meaningful to them. When
statistical results are obtained, it is never necessary for them to
ask, ``what did you mean by this variable \code{occupation}?''
There are other benefits, however. By making the principal investigator
aware of the values that are actually observed, and by offering the
opportunity to specify how they ought to be recoded, a substantial
element of administrative slippage is ameliorated. The variable key
will specify exactly how categories are to be re-mapped, there is
much less danger of an accident buried in thousands of lines of recodes.

It often happens that the raw data to be imported are provided by
one of the national data centers. The variables are given exciting,
meaningful column names like V34342a. It appears to be almost certain
that research assistants will conclude that these names are not meaningful,
so they create names that are more meaningful to them, such as \emph{gender},
\emph{sex}, \emph{male}, \emph{female}, or whatnot. The research assistants
disappear into a haze of code and come out talking about the effect
of income, gender, and education on educational achievement, and the
principal investigator has to say, ``which of those variables is
income, again?'' and ``what's the coding on education?'' A very
exciting conversation then follows as one of the research assistant
realizes that V34342b is the one that should have been used for gender,
while V34342a indicates if the respondent ever visited Eastern Europe.

The variable key is intended to create neutral territory between principal
investigators and project supervisors on one side and the research
assistants on the other. The numbers are more likely to come out correctly
if the names, values, and classes can be specified in one document
that is accessible in numerous formats on which many eyes can be laid.
The distance between a re-design of the imported values is minimized
and the production cycle of the projects is accelerated. As we are
frequently reminded, there are many some bright people who don't know
R, it appears they are all fluent in spread sheet.

\bibliographystyle{chicago}
\bibliography{kutils}

\end{document}

#LyX 2.2 created this file. For more info see http://www.lyx.org/
\lyxformat 508
\begin_document
\begin_header
\save_transient_properties true
\origin unavailable
\textclass scrartcl
\begin_preamble
%\VignetteIndexEntry{variablekey}

\usepackage{booktabs}
\usepackage{Sweavel}
\usepackage{graphicx}
\usepackage{color}

\usepackage[samesize]{cancel}



\usepackage{ifthen}

\makeatletter

\renewenvironment{figure}[1][]{%

 \ifthenelse{\equal{#1}{}}{%

   \@float{figure}

 }{%

   \@float{figure}[#1]%

 }%

 \centering

}{%

 \end@float

}

\renewenvironment{table}[1][]{%

 \ifthenelse{\equal{#1}{}}{%

   \@float{table}

 }{%

   \@float{table}[#1]%

 }%

 \centering

%  \setlength{\@tempdima}{\abovecaptionskip}%

%  \setlength{\abovecaptionskip}{\belowcaptionskip}%

% \setlength{\belowcaptionskip}{\@tempdima}%

}{%

 \end@float

}


%\usepackage{listings}
% Make ordinary listings look as if they come from Sweave
\lstset{tabsize=2, breaklines=true, %,style=Rstyle}
                        fancyvrb=false,escapechar=`,language=R,%
                        %%basicstyle={\Rcolor\Sweavesize},%
                        backgroundcolor=\Rbackground,%
                        showstringspaces=false,%
                        keywordstyle=\Rcolor,%
                        commentstyle={\Rcommentcolor\ttfamily\itshape},%
                        literate={<<-}{{$\twoheadleftarrow$}}2{~}{{$\sim$}}1{<=}{{$\leq$}}2{>=}{{$\geq$}}2{^}{{$^{\scriptstyle\wedge}$}}1{==}{{=\,=}}1,%
                        alsoother={$},%
                        alsoletter={.<-},%
                        otherkeywords={!,!=,~,$,*,\&,\%/\%,\%*\%,\%\%,<-,<<-,/},%
                        escapeinside={(*}{*)}}%


% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\def\Sweavesize{\scriptsize} 
\def\Rcolor{\color{black}} 
\def\Rbackground{\color[gray]{0.95}}

% for sideways table
\usepackage{rotating}
\end_preamble
\use_default_options true
\begin_modules
sweave
logicalmkup
\end_modules
\maintain_unincluded_children false
\language american
\language_package default
\inputencoding utf8
\fontencoding global
\font_roman "lmodern" "default"
\font_sans "lmss" "default"
\font_typewriter "lmtt" "default"
\font_math "auto" "auto"
\font_default_family default
\use_non_tex_fonts false
\font_sc false
\font_osf false
\font_sf_scale 100 100
\font_tt_scale 100 100
\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref false
\papersize default
\use_geometry true
\use_package amsmath 1
\use_package amssymb 1
\use_package cancel 1
\use_package esint 1
\use_package mathdots 1
\use_package mathtools 1
\use_package mhchem 1
\use_package stackrel 1
\use_package stmaryrd 1
\use_package undertilde 1
\cite_engine natbib
\cite_engine_type authoryear
\biblio_style plainnat
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\justification true
\use_refstyle 1
\index Index
\shortcut idx
\color #008000
\end_index
\leftmargin 1in
\topmargin 1in
\rightmargin 1in
\bottommargin 1in
\secnumdepth 3
\tocdepth 3
\paragraph_separation indent
\paragraph_indentation default
\quotes_language english
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Title
The Variable Key Data Management Framework
\end_layout

\begin_layout Author

\lang english
Paul E.
 Johnson<pauljohn@ku.edu>
\begin_inset Newline newline
\end_inset

Benjamin A.
 Kite<bakite@ku.edu> 
\begin_inset Newline newline
\end_inset

Center for Research Methods and Data Analysis
\begin_inset Newline newline
\end_inset

University of Kansas
\end_layout

\begin_layout Date
\paragraph_spacing single
\noindent

\series bold
\lang english
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
today
\end_layout

\end_inset


\end_layout

\begin_layout Abstract
This essay describes the 
\begin_inset Quotes eld
\end_inset

variable key
\begin_inset Quotes erd
\end_inset

 approach to importing and recoding data.
 This method has been developed in the Center for Research Methods and Data
 Analysis at the University of Kansas to deal with the importation of large,
 complicated data sets.
 This approach improves teamwork, keeps better records, and reduces slippage
 between the intentions of principal investigators the implementation by
 code writers.
 The framework is implemented in the R 
\begin_inset CommandInset citation
LatexCommand citep
key "RCore"

\end_inset

 package 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
kutils
\end_layout

\end_inset

.
 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=F>>=
\end_layout

\begin_layout Plain Layout

if(!dir.exists("plots")) dir.create("plots")
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard

\lang english
\begin_inset ERT
status open

\begin_layout Plain Layout

% In document Latex options:
\end_layout

\begin_layout Plain Layout


\backslash
fvset{listparameters={
\backslash
setlength{
\backslash
topsep}{0em}}}
\end_layout

\begin_layout Plain Layout


\backslash
SweaveOpts{prefix.string=plots/t,split=F,ae=F,height=4,width=5.5}
\end_layout

\end_inset


\end_layout

\begin_layout Standard

\lang english
\begin_inset ERT
status open

\begin_layout Plain Layout

<<Roptions, echo=F>>=
\end_layout

\begin_layout Plain Layout

options(device = pdf)
\end_layout

\begin_layout Plain Layout

options(width=100, prompt=" ", continue="  ")
\end_layout

\begin_layout Plain Layout

options(useFancyQuotes = FALSE) 
\end_layout

\begin_layout Plain Layout

options(SweaveHooks=list(fig=function() par(ps=10)))
\end_layout

\begin_layout Plain Layout

pdf.options(onefile=F,family="Times",pointsize=10)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Section
Introduction
\end_layout

\begin_layout Standard
The staff of the Center for Research Methods and Data Analysis has been
 asked to help with data importation and recoding from time to time.
 In one very large project, we were asked to combine, recode, and integrate
 variables from 21 different files.
 The various files used different variable names and had different, unique
 coding schemes.
 A skeptic might have thought that the firm which created the data sets
 intentionally obfuscated the records to prevent the comparison of variables
 across a series of surveys.
\end_layout

\begin_layout Standard
In projects like that, the challenge of importing and fixing the data seems
 overwhelming.
 The graduate research assistants are asked to cobble together thousands
 of lines of 
\begin_inset Quotes eld
\end_inset

recodes
\begin_inset Quotes erd
\end_inset

 as they rename, regroup, and otherwise harmonize the information.
 From a managerial point of view, that is not the main problem.
 We expect to spend the time of research assistants.
 While it may be tedious to read a codebook and write recodes, social scientists
 have been doing that since the 1960s.
 It is not all that difficult.
 The truly difficult part is mustering up the confidence in the resulting
 recoded data.
 How can a supervisor check thousands of recode statements for accuracy?
 The very extensibility of R itself–its openness to new functions and language
 elements–makes proof-reading more difficult.
 We might shift some of the proof reading duty to the principle investigators,
 but they sometimes are not interested in details.
 In the end, the responsibility for verifying the recodes falls on the project
 supervisors.
 While most supervisors with whom we are personally acquainted have nearly
 super-human reading skills and almost-perfect comprehension, we have documented
 a case in which one of them was unable to catch an error on line 827 within
 an R file with 2119 lines.
 
\end_layout

\begin_layout Standard
To reduce the risk of misunderstanding and error, we propose the 
\emph on
variable key procedure
\emph default
.
 It is a systematic way to separate code writing from the process of renaming
 variables and re-designating their values.
 The characteristics of the data are summarized in a table, a simple-looking
 structure that might be edited in a text editor or a spread sheet program.
 This simple structure, which we call the variable key, can be used by principal
 investigators and supervisors to designate the desired results.
 Once the key is created, then the data set can be imported and recoded
 by the application of the key's information.
 This does not eliminate the need to proof-read the renaming and recoding
 of the variables, it simply shifts that chore into a simpler, more workable
 setting.
\end_layout

\begin_layout Standard
This essay proceeds in 3 parts.
 First, the general concepts behind the variable key system are explored.
 Second, the four stages in the variable key procedure are outlined and
 illustrated with examples.
 Third, we offer some examples of ways to double-check the results.
\end_layout

\begin_layout Section
Enter the Variable Key
\end_layout

\begin_layout Standard
The variable key process was first developed for a very large project for
 which we were hired by a commercial consulting company.
 As it happened, the project manager who hired us was an Excel user who
 did not know about R.
 He was given several SPSS datasets.
 After going through the usual R process of importing and recoding data
 from 6 files, the aggregate of which included more than 40,000 observations
 on 150 variables, we arrived at a renamed set of columns.
 Unfortunately, the research assistant who had done most of the work resigned
 in order to pursue a career as a magician.
\begin_inset Foot
status collapsed

\begin_layout Plain Layout
Or graduated, we are not sure which.
\end_layout

\end_inset

 
\end_layout

\begin_layout Standard
With the unavailability of our key asset, it was difficult to know for sure
 what was in which column.
 There was nobody to quickly answer questions like 
\begin_inset Quotes eld
\end_inset

which column is the respondent's sexual identity?
\begin_inset Quotes erd
\end_inset

 and 
\begin_inset Quotes eld
\end_inset

if sex is V23418, did we change 1 to male or female
\begin_inset Quotes erd
\end_inset

.
 The only way to find out is by hunting and pecking through a giant R file.
 
\end_layout

\begin_layout Standard
In order to better communicate about that project, we developed a table
 that looked like Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:A-Small-Variable-key"

\end_inset

.
\end_layout

\begin_layout Standard
\begin_inset Float table
placement H
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
A Small Variable Key
\begin_inset CommandInset label
LatexCommand label
name "tab:A-Small-Variable-key"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Tabular
<lyxtabular version="3" rows="4" columns="4">
<features tabularvalignment="middle">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<row>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
name_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
name_new
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
values_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
values_new
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
V23419
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
sex
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
1|2|3
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset Quotes eld
\end_inset

male
\begin_inset Quotes erd
\end_inset

|
\begin_inset Quotes erd
\end_inset

female
\begin_inset Quotes erd
\end_inset

|
\begin_inset Quotes erd
\end_inset

neither
\begin_inset Quotes erd
\end_inset


\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
V32422
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
education
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
1|2|3|4|5
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset Quotes eld
\end_inset

elem
\begin_inset Quotes erd
\end_inset

<
\begin_inset Quotes erd
\end_inset

hs
\begin_inset Quotes erd
\end_inset

<
\begin_inset Quotes erd
\end_inset

somecoll
\begin_inset Quotes erd
\end_inset

<
\begin_inset Quotes erd
\end_inset

ba
\begin_inset Quotes erd
\end_inset

<
\begin_inset Quotes erd
\end_inset

post
\begin_inset Quotes erd
\end_inset


\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
V54532
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
income
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
.
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
numeric
\end_layout

\end_inset
</cell>
</row>
</lyxtabular>

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
It was tedious to assemble that table, but it helped quite a bit in our
 discussions.
 The vertical bars were used to indicate that the original data had discrete
 values.
 When a variable has a natural ordering, the new values were placed in order
 with the symbol (
\begin_inset Quotes eld
\end_inset

<
\begin_inset Quotes erd
\end_inset

).
 That table grew quite large, since it had one row per variable, but it
 was otherwise workable.
 It was popular with the client.
 
\end_layout

\begin_layout Standard
In the middle of preparing that summary table of recoded values, we realized
 that it was possible to write an R program to import the key table and
 use its information to recode and rename the variables.
 The recodes would 
\emph on
just happen
\emph default
.
 If we prepared the functions properly, we had not just a table masquerading
 as a codebook, we had a 
\emph on
programmable codebook
\emph default
.
 We wrote some functions that could import variables (as named in column
 1), apply the new values (from columns 3 and 4), then apply the new name
 from column 2.
 The functions to do that are somewhat difficult to prepare, but they are
 very appealing from a supervisor's point of view.
 There will be less proof-reading to do, at least in the R code itself.
 Once we can validate the functions, then we never have to proof-read them
 again.
 These functions can be applied, row by row, to create a new data frame.
 Instead, we need to concentrate our attention on the substance of the problem,
 the specification of the new names and values in the table.
\end_layout

\begin_layout Standard
In the projects where we have employed this system, we adjusted the key
 and the R functions to suit the particular demands of the project and the
 client.
 That was unfortunate, because we had very little accumulation of code from
 one project to another.
 However, we did accumulate experience; there were concepts and vocabulary
 which allowed us to understand the various challenges that might be faced.
 The effort to develop a standardized framework for the variable key began
 in 2016 with the creation of the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
kutils
\end_layout

\end_inset

 package for R.
\end_layout

\begin_layout Standard
The variable key process allows project supervisors to create a table that
 instructs the research assistants in the importation, renaming, and recoding
 of data.
 There is still a daunting problem, however, because the supervisors must
 create that variable key table.
 In a large data set, it might be arduous to simply type the old names of
 the variables and their observed values.
 In 2015 one of the graduate assistants in our lab was asked to type up
 a variable key and he couldn't quite believe that was a good use of his
 time.
 After some discussion, we realized that it was not necessary to type the
 variable key at all.
 We would write a function to do so.
 If R can import the candidate data set, then R can certainly output its
 column names and a roster of observed values.
 This lightened the workload considerably.
 By tabulating all of the observed variables and their values, the most
 tedious part of the process was done mechanically.
 
\end_layout

\begin_layout Standard
In the remainder of this essay, we discuss the process of creating a variable
 key template, revising it, and putting it to use.
 
\end_layout

\begin_layout Section
Four Simple Steps
\end_layout

\begin_layout Standard
The variable key process has four steps.
 First, inspect an R data.frame object and create a key template file.
 The key template summarizes the existing state of the variables and creates
 
\begin_inset Quotes eld
\end_inset

placeholders
\begin_inset Quotes erd
\end_inset

 where we might like to specify revisions.
 Second, edit the key template file in a spreadsheet or other program that
 can work with comma separate variables.
 Change the names, values, and designate other recodes (which we will describe
 next).
 Third, import the revised key into R.
 Fourth, apply the key to the data to generate a new, improved data frame.
 Then run some diagnostic routines.
\end_layout

\begin_layout Standard
If all goes well, we should end up with a new data frame in which
\end_layout

\begin_layout Enumerate
The columns are renamed in accordance with the instructions of the principal
 investigator (or supervisor).
\end_layout

\begin_layout Enumerate
The values of all variables have been recoded according to the instructions
 of the principal investigator (or supervisor).
\end_layout

\begin_layout Standard
Diagnostic tables are reported to clearly demonstrate the effect of each
 coding change, mapping out the difference between the input and the output
 variables.
 
\end_layout

\begin_layout Standard
For purposes of illustration, we have create an example data frame with
 various types of variables.
 This data frame, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
mydf
\end_layout

\end_inset

, has most of the challenges that we see in actual projects.
 It has integer variables that need to be reorganized and turned into character
 or factor variables.
 It has character variables that might become integers or factors.
 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=T,include=T>>=
\end_layout

\begin_layout Plain Layout

set.seed(234234)
\end_layout

\begin_layout Plain Layout

N <- 200
\end_layout

\begin_layout Plain Layout

mydf <- data.frame(
\end_layout

\begin_layout Plain Layout

    x5 = rnorm(N),
\end_layout

\begin_layout Plain Layout

    x4 = rpois(N, lambda = 3),
\end_layout

\begin_layout Plain Layout

    x3 = ordered(sample(c("lo", "med", "hi"), size = N, replace=TRUE),
\end_layout

\begin_layout Plain Layout

            levels = c("med", "lo", "hi")),
\end_layout

\begin_layout Plain Layout

    x2 = letters[sample(c(1:4,6), 200, replace = TRUE)],
\end_layout

\begin_layout Plain Layout

    x1 = factor(sample(c("cindy", "jan", "marcia"), 200,
\end_layout

\begin_layout Plain Layout

            replace = TRUE)),
\end_layout

\begin_layout Plain Layout

    x7 = ordered(letters[sample(c(1:4,6), 200, replace = TRUE)]),
\end_layout

\begin_layout Plain Layout

    x6 = sample(c(1:5), 200, replace = TRUE),
\end_layout

\begin_layout Plain Layout

            stringsAsFactors = FALSE)
\end_layout

\begin_layout Plain Layout

mydf$x4[sample(1:N, 10)] <- 999
\end_layout

\begin_layout Plain Layout

mydf$x5[sample(1:N, 10)] <- -999
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Subsection
Step 1.
 Create a Key Template
\end_layout

\begin_layout Standard
The function 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyTemplate
\end_layout

\end_inset

 scans a data frame and generates a new key template.
 The key has 8 pieces of information about each variable.
 The rows of the key are named by the variables of the data frame.
 The 8 columns in the key are 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
name_old
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
name_new
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_old
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
missings
\end_layout

\end_inset

, and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
recodes
\end_layout

\end_inset

.
 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyTemplate
\end_layout

\end_inset

 will fill 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
name_old
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_old
\end_layout

\end_inset

, and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

, in the with values based on the data input, while the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
new
\end_layout

\end_inset

 columns will be copies of those old values.
 The last 2, missings and recodes, will be empty.
 
\end_layout

\begin_layout Standard
There are two formats for the key template, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
long
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
wide
\end_layout

\end_inset

 (determined by the parameter 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
long
\end_layout

\end_inset

).
 These names are drawn from terminology in R's reshape function.
 The long format has one row per value of each variable, while the wide
 format has all of the information in one row.
 The two key formats are intended to be interchangeable in functionality;
 they differ solely for convenience.
 Some users may prefer to edit variable information in one style.
 The re-importation of the key should deal gracefully with either type of
 variable key.
 
\end_layout

\begin_layout Standard
A wide format key can be produced with a call to the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyTemplate
\end_layout

\end_inset

 function like so:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=F>>=
\end_layout

\begin_layout Plain Layout

key_wide <- keyTemplate(mydf, file = "key_wide.csv", max.levels = 5)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\noindent
If the long argument is not specified, a wide key is the default.
 One can ask for a long format:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=F>>=
\end_layout

\begin_layout Plain Layout

key_long <- keyTemplate(mydf, long = TRUE, file = "key_long.csv", max.levels
 = 5)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\noindent
The key object is a data.frame.
 
\end_layout

\begin_layout Standard
Apart from the long argument, the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyTemplate
\end_layout

\end_inset

 function has two especially noteworthy arguments, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
file
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
max.levels
\end_layout

\end_inset

.
 If the file argument is supplied, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
keyTemplate
\end_layout

\end_inset

 uses the suffix to determine storage format.
 Legal suffixes are 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
.csv
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
.xlsx
\end_layout

\end_inset

, and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
.rds
\end_layout

\end_inset

 (for creating comma separated variables, Excel spreadsheets, and R serializatio
n data structures).
 
\end_layout

\begin_layout Standard
The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
max.levels
\end_layout

\end_inset

 argument is also important.
 This is used in the same sense that functions like 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
read.spss
\end_layout

\end_inset

 in the foreign package use that term.
 There is guessing involved in deciding if we should enumerate a character
 or integer variable.
 We do want to enumerate the 
\begin_inset Quotes eld
\end_inset

Strongly Disagree
\begin_inset Quotes erd
\end_inset

 to 
\begin_inset Quotes eld
\end_inset

Strongly Agree
\begin_inset Quotes erd
\end_inset

 values of a 7 point scale, but we do not want to enumerate the first names
 of all study participants.
 If the number of discrete values exceeds 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
max.levels
\end_layout

\end_inset

, for which the default is 15, then the key will not enumerate them.
 
\end_layout

\begin_layout Standard
Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:The-Wide-Key"

\end_inset

 demonstrates a wide key template as it is produced by 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyTemplate
\end_layout

\end_inset

.
 We see now why it is called a wide key; the recoding information is tightly
 packed into 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

.
 The key includes more or less obvious columns for the old and new variable
 names, their classes, and values of the variables.
 Note that the values of x5 and x4 are not enumerated because we set max.levels
 at 5.
 The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
max.levels
\end_layout

\end_inset

 parameter defaults to 15, so that an integer variable with less than 15
 values will have each value displayed.
 For this example, the display of that variable key was too wide for the
 page, so we reduced the number of values.
 When the observed number of scores is above 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
max.levels
\end_layout

\end_inset

, the key does not try to list the individual values (compare the treatment
 of variables 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
x4
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
x6
\end_layout

\end_inset

).
 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<key0, include=F>>=
\end_layout

\begin_layout Plain Layout

library(kutils)
\end_layout

\begin_layout Plain Layout

library(xtable)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<key10, echo=F>>=
\end_layout

\begin_layout Plain Layout

key_wide <- keyTemplate(mydf, max.levels = 5)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
A long key template is displayed in Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Long-Key"

\end_inset

.
 The benefit of the long key is that the cells 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

 are easier to navigate.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<key30,echo=F>>=
\end_layout

\begin_layout Plain Layout

key_long <- keyTemplate(mydf, long = TRUE, max.levels = 5)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float table
placement h
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
The Wide Key Template
\begin_inset CommandInset label
LatexCommand label
name "tab:The-Wide-Key"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
def
\backslash
Sweavesize{
\backslash
tiny}
\end_layout

\begin_layout Plain Layout

<<key20,echo=FALSE,results=tex>>=
\end_layout

\begin_layout Plain Layout

print(xtable(key_wide), include.rownames = FALSE, size = "small", floating
 = FALSE )
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float table
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
The Long Key Template
\begin_inset CommandInset label
LatexCommand label
name "tab:Long-Key"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
def
\backslash
Sweavesize{
\backslash
tiny}
\end_layout

\begin_layout Plain Layout

<<key40,echo=FALSE,results=tex>>=
\end_layout

\begin_layout Plain Layout

print(xtable(key_long), include.rownames = FALSE, size = "small", floating
 = FALSE)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
The value of 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_old
\end_layout

\end_inset

 in the key is the first element in the return from the function class for
 a variable.
 There is one exception, where we have tried to differentiate integer variables
 from numeric variables.
 This is a confusing isssue in the history of R, as discussed in the R help
 page for the function 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
as.double
\end_layout

\end_inset

.
 ,In the 
\emph on
Note on names
\emph default
 section, that page explains an 
\begin_inset Quotes eld
\end_inset

anomaly
\begin_inset Quotes erd
\end_inset

 in the usage of term 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
numeric
\end_layout

\end_inset

.
 The R function 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
as.numeric
\end_layout

\end_inset

 creates a double precision floating point value, not an integer.
 However, the is.numeric function responds TRUE for both integers and floating
 point values.
 For purposes of editing the key, it is useful to differentiate integers
 from floating point numbers.
 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
kutils
\end_layout

\end_inset

 includes a function named 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
safeInteger
\end_layout

\end_inset

.
 It checks the observed values of a variable to find out if any floating
 point values are present.
 If the aggregate deviations from integer values are miniscule, then a variable
 is classified as an integer.
 As a result, the keyTemplate function's column 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_old
\end_layout

\end_inset

 should be 
\begin_inset Quotes eld
\end_inset

integer
\begin_inset Quotes erd
\end_inset

 or 
\begin_inset Quotes eld
\end_inset

numeric
\begin_inset Quotes erd
\end_inset

, and by the latter we mean a floating point number.
 
\end_layout

\begin_layout Standard
In some of our early projects, the variable key was in the wide format.
 Difficulty in editing that caused us to shift some projects to the long
 key.
 The idea that we would glide back and forth between keys created in the
 wide and long formats dawned on us only recently.
 To ease the conversion back-and-forth between the formats, we developed
 the functions named 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
wide2long
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
long2wide
\end_layout

\end_inset

.
 We believe these functions work effectively, but we have experienced some
 troubles related to the way spreadsheets store character strings.
 If the key in long format has a column of values 
\begin_inset Quotes eld
\end_inset

Yes
\begin_inset Quotes erd
\end_inset

,
\begin_inset Quotes erd
\end_inset

No
\begin_inset Quotes erd
\end_inset

, and 
\begin_inset Quotes eld
\end_inset


\begin_inset Quotes erd
\end_inset

, the wide representation should be 
\begin_inset Quotes eld
\end_inset

Yes|No|
\begin_inset Quotes erd
\end_inset

, but there is some inclination to say we should have nested quotation marks,
 as in 
\begin_inset Quotes eld
\end_inset


\begin_inset Quotes erd
\end_inset

Yes
\begin_inset Quotes erd
\end_inset

|
\begin_inset Quotes erd
\end_inset

No
\begin_inset Quotes erd
\end_inset

|
\begin_inset Quotes erd
\end_inset


\begin_inset Quotes erd
\end_inset


\begin_inset Quotes erd
\end_inset

.
 That kind of string will not generally survive importation to and export
 from a spread sheet at the current time.
\end_layout

\begin_layout Subsection
Step 2.
 Edit the variable key
\end_layout

\begin_layout Standard
If the file argument was specified in 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyTemplate
\end_layout

\end_inset

, the work is laid out for us.
 One can edit a csv file in any text editor or in a spreadsheet.
 An xlsx file can be edited by Libre Office or Microsoft Office.
 
\end_layout

\begin_layout Standard
It is not necessary to change all of the values in name_new, class_new,
 and value_new.
 In fact, one might change just a few elements and the un-altered variables
 will remain as they were when the data is re-imported.
 We suggest users start small by making a few edits in the key.
 A principal investigator might change just a few variable names or values.
 In a large project, there may be quite a bit of work involved.
 
\end_layout

\begin_layout Standard
The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
name_old
\end_layout

\end_inset

 column must never be edited.
 Generally, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_old
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 will not be edited either (the only exception might arise if 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

 is either 
\begin_inset Quotes eld
\end_inset


\begin_inset Flex Code
status open

\begin_layout Plain Layout
factor
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

 or 
\begin_inset Quotes eld
\end_inset


\begin_inset Flex Code
status open

\begin_layout Plain Layout
ordered
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

).
 The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
name_new
\end_layout

\end_inset

 column should include legal R variable names (do not begin name_new with
 a numeral or include mathematical symbols like 
\begin_inset Quotes eld
\end_inset

+
\begin_inset Quotes erd
\end_inset

 or 
\begin_inset Quotes eld
\end_inset

-
\begin_inset Quotes erd
\end_inset

).
 We use R's 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
make.names
\end_layout

\end_inset

 function to clean up errant entries, so incorrect names are not fatal.
\end_layout

\begin_layout Standard
The difficult user decisions will concern the contents of class_new and
 value_new.
 The desired variable type, of course, influences the values that are meaningful.
 To convert a character variable to integer, for example, it should go without
 saying that the value_new element should include integer values, not text
 strings.
\end_layout

\begin_layout Standard
The conversion of information from one type of variable into another may
 be more complicated than it seems.
 It is a bit more tricky to convert a factor into a numeric variable that
 uses the factor's levels as numeric values.
 
\end_layout

\begin_layout Standard
After experimenting with a number of cases, we believe that if 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_old
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

 are elements of this the 
\begin_inset Flex Emph
status open

\begin_layout Plain Layout
safe class set
\end_layout

\end_inset

: 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
character
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
logical
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
integer
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
numeric
\end_layout

\end_inset

 (same as 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
double
\end_layout

\end_inset

), 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
factor
\end_layout

\end_inset

, or 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
ordered
\end_layout

\end_inset

, then the re-importation and recoding of data will be more-or-less automatic.
 If 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

 differs from 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_old
\end_layout

\end_inset

, and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

 is not an element in that 6 element set, then the user must supply a recode
 function that creates a variable of the requested class.
 Most commonly, we expect that will be used to incorporate 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
Date
\end_layout

\end_inset

 variables.
\end_layout

\begin_layout Standard
The enumerated values in the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

 column should be specified in the more or less obvious way.
 If 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

 is equal to 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
character
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
factor
\end_layout

\end_inset

, or 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
ordered
\end_layout

\end_inset

, then the new values can be arbitrary strings.
 
\end_layout

\begin_layout Standard
The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
missings
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
recodes
\end_layout

\end_inset

 columns are empty in the key template.
 The user will need to fill in those values if they are to be used.
 When the key is later put to use, the order of processing will be as follows.
 First, the values declared as missings will be applied (convert observed
 value to R's NA symbol).
 Second, if there is a recode function in the key, it is applied to a variable.
 Third, if there was no recode function supplied, then the conversion of
 discrete values by recalculation from 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 into 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

 will be applied.
 Note that the discrete values are applied only if the recode cell is empty
 in the key.
\end_layout

\begin_layout Standard
The decision of whether to approach a given variable via value enumeration
 or a recode function is, to an extent, simply a matter of taste.
 Some chores that might be handled in either way.
 If a variable includes floating point numbers (temperatures, dollar values,
 etc), then we would not rely on new assignments in 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

.
 Truly numeric variables of that sort almost certainly call for assignment
 of missings and recodes by the last two cells in the variable key.
 However, if a column includes integers or characters (1 = male, 2 = female),
 one might use the enumerated values (
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

) or one could design a recode function to produce the same result.
 It is important to remember that if a recode function is applied, the enumerate
d value recoding is not.
 If one decides to use a recode statement, then the elements in value_old
 and value_new are ignored entirely, they could be manually deleted to simplify
 the key.
 (That is to say, the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
max.levels
\end_layout

\end_inset

 parameter is just a way of guessing how many unique levels is 
\begin_inset Quotes eld
\end_inset

too many
\begin_inset Quotes erd
\end_inset

 for an enumeration.
 Users are free to delete values if recodes are used.)
\end_layout

\begin_layout Standard
Despite the possibility that a factor (or ordered) variable may have many
 values, we believe that all of the levels of those variables should be
 usually be included in the key.
 If a variable is declared as a factor, it means the researcher has assigned
 meaning to the various observed values and we are reluctant to ignore them.
 
\end_layout

\begin_layout Standard
There is a more important reason to enumerate all of the legal values for
 factor variables.
 If a value is omitted from the key, that value will be omitted from the
 dataset.
 
\end_layout

\begin_layout Standard
Among our users, we find opinion is roughly balanced between the long and
 the wide key formats.
 One might simply try both.
 If the number of observed values is more than 5 or 10, editing the key
 in a program like Microsoft Excel is less error prone in the long key.
 This is simply a matter of taste, however.
 The disadvantage of the long format is that it is somewhat verbose, with
 repeated values in the name and class values.
 If an editor makes an error in the assignment of a block, then hard to
 find errors may result.
\end_layout

\begin_layout Standard
Because editing the key can be a rather involved process, we will wait to
 discuss the details until section 
\begin_inset CommandInset ref
LatexCommand ref
reference "sec:Editing-the-key"

\end_inset

.
\end_layout

\begin_layout Subsection
Step 3.
 keyImport
\end_layout

\begin_layout Standard
Once any desired changes are entered in the variable key, the file needs
 to be imported back into R.
 For that purpose, we supply the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyImport
\end_layout

\end_inset

 function.
 As in 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyTemplate
\end_layout

\end_inset

, the file argument's suffix is used to discern whether the input should
 be read as .csv, .xlsx, or .rds.
 It is not necessary to specify that the key being imported is in the long
 or wide format.
 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyImport
\end_layout

\end_inset

 includes heuristics that have classified user-edited keys very accurately.
 
\end_layout

\begin_layout Standard
The returned value is an R data frame that should be very similar to the
 template, except that the new values of 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
name_new
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

, and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

 will be visible.
 
\end_layout

\begin_layout Standard
In order to test this function with the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
kutils
\end_layout

\end_inset

 package, we include some variable keys.
 The usage of those keys is demonstrated in the help page for 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyImport
\end_layout

\end_inset

.
 In addition to the mydf toy data frame created above, we also include a
 subset of the US National Longitudinal Survey in a data frame named 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
natlongsurv
\end_layout

\end_inset

.
\end_layout

\begin_layout Subsection
Step 4.
 Apply the imported key to the data
\end_layout

\begin_layout Standard
The final step is to apply the key to the data frame (or some other data
 frame that may have arrived in the interim).
 The syntax is simple
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=F>>=
\end_layout

\begin_layout Plain Layout

mydf.cleaned <- keyApply(mydf, mydf.keylist)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Because the default value of the argument 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
diagnostic
\end_layout

\end_inset

 is TRUE, the output from 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyApply
\end_layout

\end_inset

 is somewhat verbose.
 After we have more feedback from test users, we will be able to quiet some
 of that output.
 
\end_layout

\begin_layout Standard
The diagnostic output will include information about mismatch between the
 key and the data itself.
 If variables that are included in the key that are not included in the
 new data set, there will not be an error, but a gentle warning will appear.
 Similarly, if the observed values of an enumerated variable are not included
 in the variable key, there will be a warning.
 
\end_layout

\begin_layout Standard
The diagnostic will also create a cross tabulation of each new variable
 against its older counterpart.
 This works very well with discrete variables with 10 or so values, but
 for variables with more values it is rather unmanageable.
 
\end_layout

\begin_layout Section
Editing the variable key
\begin_inset CommandInset label
LatexCommand label
name "sec:Editing-the-key"

\end_inset


\end_layout

\begin_layout Standard
The work of revising the variable key can be driven by the separation of
 variables into two type.
 The variables with enumerated values–the ones for which we intend to rename
 or re-assign values one-by-one–are treated in a very different way than
 the other ones.
 The enumerated value strategy works well with variables for which we simply
 need to rename categories (e.g, 
\begin_inset Quotes eld
\end_inset

cons
\begin_inset Quotes erd
\end_inset

 to 
\begin_inset Quotes eld
\end_inset

Conservative
\begin_inset Quotes erd
\end_inset

).
 Variables for which we do not do so (e.g., convert Fahrenheit to Celsius)
 are treated differently.
\end_layout

\begin_layout Standard
As we will see in this section, the revision of variables of the enumerated
 value type emphasizes the revision of the value_old and value_new columns
 in the key.
 On the other hand, the other types will depend on writing correctly formatted
 statements in the recode column of the variable key.
\end_layout

\begin_layout Subsection
Enumerated variables
\end_layout

\begin_layout Standard
All of the values observed for 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
logical
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
factor
\end_layout

\end_inset

, and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
ordered
\end_layout

\end_inset

 variables will appear in the key template.
 Do not delete them unless the exclusion of those values from the analysis
 intended.
 For character and integer variables with fewer than 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
max.levels
\end_layout

\end_inset

 discrete values, the observed scores will be included in 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

.
 If one wishes to convert a variable from being treated as an enumerated
 to a numeric type, then one can delete all values from 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

.
 
\end_layout

\begin_layout Standard
The recoding of discrete variables is a fairly obvious chore.
 For each old value, a new value must be specified.
 We first consider the case of a variable that enters as a character variable
 but we might like to recode it and also create factor and integer variants
 of it.
 In the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
mydf
\end_layout

\end_inset

 variable key (Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:The-Wide-Key"

\end_inset

), we have variable 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
x2
\end_layout

\end_inset

 which is coded 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
a
\end_layout

\end_inset

 through 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
f
\end_layout

\end_inset

.
 We demonstrate ways to spawn new character, factor, or integer variable
 in Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Change-Type-Example1"

\end_inset

.
 As long as name_old is preserved, as many lines as desired can be used
 to create variables of different types.
 Here we show the middle section of the revised key in which we have spawned
 3 new variants of x2, each with its own name.
\end_layout

\begin_layout Standard
\begin_inset Float table
placement H
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Change Class, Example 1
\begin_inset CommandInset label
LatexCommand label
name "tab:Change-Type-Example1"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Tabular
<lyxtabular version="3" rows="4" columns="6">
<features tabularvalignment="middle">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<row>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
name_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
name_new
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
class_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
class_new
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
value_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
value_new
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x2
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x2.char
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
character
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
character
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
a|b|c|d|f
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Excellent|Proficient|Good|Fair|Poor
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x2
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x2.fac
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
character
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
factor
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
a|b|c|d|f
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Excellent|Proficient|Good|Fair|Poor
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x2
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x2.gpa
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
character
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
integer
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
a|b|c|d|f
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
4|3|2|1|0
\end_layout

\end_inset
</cell>
</row>
</lyxtabular>

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
In line one of Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Change-Type-Example1"

\end_inset

, the class 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
character
\end_layout

\end_inset

 remains the same.
 That line will produce a new character variable with embellished values.
 Line two demonstrates how to create an R factor variable, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
x2.fac
\end_layout

\end_inset

, and line three converts the character to an integer variable.
 Remember that it is important to match the value of 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

 with the content proposed for 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

.
 Do not include character values in a variable for which the new class will
 be numeric or integer.
 
\end_layout

\begin_layout Standard
Similarly, it is obvious to see how an integer input can be converted into
 either an integer, character, or factor variable by employing any of the
 rows seen in Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Change-Type-Example2"

\end_inset

.
\end_layout

\begin_layout Standard
\begin_inset Float table
placement H
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Change Class Example 2
\begin_inset CommandInset label
LatexCommand label
name "tab:Change-Type-Example2"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Tabular
<lyxtabular version="3" rows="4" columns="6">
<features tabularvalignment="middle">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<row>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
name_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
name_new
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
class_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
class_new
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
value_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
value_new
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x6
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x6.i100
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
integer
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
integer
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
1|2|3|4|5
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
100|200|300|400|500
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x6
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x6.c
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
integer
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
character
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
1|2|3|4|5
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Austin|Denver|Nashville|Provo|Miami
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x6
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x6.f
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
integer
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
factor
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
1|2|3|4|5
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
F|D|C|B|A
\end_layout

\end_inset
</cell>
</row>
</lyxtabular>

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
If a variable's 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_old
\end_layout

\end_inset

 is ordered, and we simply want to relabel the existing levels, the work
 is also easy (see Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Change-Type-Example3"

\end_inset

).
 The second row in Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Change-Type-Example3"

\end_inset

 shows that factor levels can be 
\begin_inset Quotes eld
\end_inset

combined
\begin_inset Quotes erd
\end_inset

 by assigning the same character string to several 
\begin_inset Quotes eld
\end_inset

<
\begin_inset Quotes erd
\end_inset

 separated values.
 
\end_layout

\begin_layout Standard
\begin_inset Float table
placement H
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Change Class, Example 3
\begin_inset CommandInset label
LatexCommand label
name "tab:Change-Type-Example3"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Tabular
<lyxtabular version="3" rows="4" columns="6">
<features tabularvalignment="middle">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<row>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
name_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
name_new
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
class_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
class_new
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
value_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
value_new
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x7
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x7.grades
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
ordered
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
ordered
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
f<d<c<b<a
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
F<D<C<B<A
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x7
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x7.passfail
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
ordered
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
ordered
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
f<d<c<b<a
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Fail<Fail<Pass<Pass<Pass
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x7
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x7.gpa
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
ordered
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
integer
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
f<d<c<b<a
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
0|1|2|3|4
\end_layout

\end_inset
</cell>
</row>
</lyxtabular>

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
Working with ordered variables, whether as input or output, becomes more
 complicated if the existing data are not ordered in the way we want.
 In the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
mydf
\end_layout

\end_inset

 example, the variable 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
mydf$x3
\end_layout

\end_inset

 was coded as an ordered variable with levels (
\begin_inset Quotes eld
\end_inset

med
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset

lo
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset

high
\begin_inset Quotes erd
\end_inset

).
 That might have been one person's idea of a joke, so we need to rearrange
 these as (
\begin_inset Quotes eld
\end_inset

lo
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset

med
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset

high
\begin_inset Quotes erd
\end_inset

).
 If the original ordering of the values is not consistent with the desired
 ordering of the new ordered factor, then we need notation that allows researche
rs to achieve two purposes.
 First, the values must be re-leveled.
 Second, allow for the possibility that the new values must be relabeled
 as well.
 We'd rather not proliferate new columns in the variable key or create some
 confusing new notation.
\end_layout

\begin_layout Standard
Reordering variable levels requires us to do something that seems dangerous.
 We need to edit the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 to correct the ordering of the levels 
\emph on
as they are currently labeled
\emph default
.
 This is the only time where we suggest that users edit the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 column.
 In 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

, supply new labels in the in the correct order to parallel the newly edited
 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 column (see Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Reorder-Values"

\end_inset

).
\end_layout

\begin_layout Standard
\begin_inset Float table
placement H
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Reorder Values, Example 1
\begin_inset CommandInset label
LatexCommand label
name "tab:Reorder-Values"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Tabular
<lyxtabular version="3" rows="3" columns="6">
<features tabularvalignment="middle">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<row>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
name_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
name_new
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
class_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
class_new
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
value_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
value_new
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x3
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x3.lo2hi
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
ordered
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
ordered
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
low<med<hi
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
low<medium<high
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x3
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x3.passfail
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
ordered
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
ordered
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
low<med<hi
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
low<pass<pass
\end_layout

\end_inset
</cell>
</row>
</lyxtabular>

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
The key importer will check that all 
\begin_inset Quotes eld
\end_inset

duplicated levels
\begin_inset Quotes erd
\end_inset

 are adjacent with one another, so that the values above low are grouped
 together.
\end_layout

\begin_layout Standard
In the long key format, the equivalent information would be conveyed by
 altering the ordering of the rows.
 For example, it is necessary to re-order the rows to indicate the lo is
 lower than med, and then for the new values we put in the desired names
 (see Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Reorder-Values-Example2"

\end_inset

).
\end_layout

\begin_layout Standard
\begin_inset Float table
placement H
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Reorder Values, Example 2
\begin_inset CommandInset label
LatexCommand label
name "tab:Reorder-Values-Example2"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
begin{tabular}{llllllll}
\end_layout

\begin_layout Plain Layout


\backslash
toprule
\end_layout

\begin_layout Plain Layout

name
\backslash
_old & name
\backslash
_new & class
\backslash
_old & class
\backslash
_new & value
\backslash
_old & value
\backslash
_new
\backslash

\backslash

\end_layout

\begin_layout Plain Layout


\backslash
midrule
\end_layout

\begin_layout Plain Layout

x3 & x3lo2hi & ordered & ordered & lo & low
\backslash

\backslash

\end_layout

\begin_layout Plain Layout

x3 & x3lo2hi & ordered & ordered & med & medium
\backslash

\backslash

\end_layout

\begin_layout Plain Layout

x3 & x3lo2hi & ordered & ordered & hi & high
\backslash

\backslash

\end_layout

\begin_layout Plain Layout

x3 & x3.passfail & ordered & ordered & low & low
\backslash

\backslash

\end_layout

\begin_layout Plain Layout

x3 & x3.passfail & ordered & ordered & medium & pass
\backslash

\backslash

\end_layout

\begin_layout Plain Layout

x3 & x3.passfail & ordered & ordered & high & pass
\backslash

\backslash

\end_layout

\begin_layout Plain Layout


\backslash
bottomrule
\end_layout

\begin_layout Plain Layout


\backslash
end{tabular}
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
What to do about missing values.
 Even within our small group, there is some disagreement about this.
 The SAS tradition would have us enter a period, 
\begin_inset Quotes eld
\end_inset

.
\begin_inset Quotes erd
\end_inset

, in value_new for a level that is to be treated as missing.
 Others are tempted to use special purpose character strings like 
\begin_inset Quotes eld
\end_inset

NA
\begin_inset Quotes erd
\end_inset

 or 
\begin_inset Quotes eld
\end_inset

N/A
\begin_inset Quotes erd
\end_inset

.
 As long as neither 
\begin_inset Quotes eld
\end_inset

NA
\begin_inset Quotes erd
\end_inset

 nor 
\begin_inset Quotes eld
\end_inset

N/A
\begin_inset Quotes erd
\end_inset

 are legal values, that seems safe.
 There is another school of thought to argue that if a user wants nothing,
 the clearest, safest thing to do in the key is to enter nothing at all.
 
\end_layout

\begin_layout Standard
As a result of this diversity, the keyImport function includes an argument
 named na.strings (again, an argument drawn from R core funcitons).
 The default setting has an 
\begin_inset Quotes eld
\end_inset

all of the above
\begin_inset Quotes erd
\end_inset

 flavor, treating any of the values that any of us think might be missing
 as NA when the key is later put to use.
 If one is fastidious in editing the key and represents all value_new that
 should be missing as a period (hooray for SAS!), then the value of na.strings
 can be tightened up in the obvious way.
 
\end_layout

\begin_layout Standard
It is also possible to delete a level from value_old and value_new in order
 to indicate that it should be treated as missing, but we do not recommend
 this approach.
 This destroys an element of 
\begin_inset Quotes eld
\end_inset

book keeping
\begin_inset Quotes erd
\end_inset

 because future users of the data, and the key, might like to know that
 some levels were obliterated.
\end_layout

\begin_layout Standard
The missing value cell in the key can be used with these enumerated variables.
 Usually, writing the values to be omitted is not as easy as putting the
 desired missing symbol in the value_new column, but either method should
 work.
\end_layout

\begin_layout Standard
In a long format key, there will be many repeated rows for each variable.
 The entries in the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
missings
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
recodes
\end_layout

\end_inset

 columns are harvested from the cells corresponding to each combination
 of name_old and name_new, it is not necessary to retype the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
missings
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
recodes
\end_layout

\end_inset

 entries for each element.
\end_layout

\begin_layout Standard
The work of assigning missing values is carried out by a 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
kutils
\end_layout

\end_inset

 function named 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
assignMissing
\end_layout

\end_inset

.
 The help page for that function has verbose commentary to explain the format
 of the cells that might work well in the missings column.
\end_layout

\begin_layout Standard
\begin_inset Note Note
status open

\begin_layout Plain Layout
TODO: fix this, describe the output key 
\begin_inset Quotes eld
\end_inset

mydf.key.csv
\begin_inset Quotes erd
\end_inset

 and get EXCEL discussion on how to add quoted elements inside strings.
 Need to write out what Excel does to a string like 
\begin_inset Quotes eld
\end_inset

cindy|bobby|
\begin_inset Quotes erd
\end_inset

NA
\begin_inset Quotes erd
\end_inset

|
\begin_inset Quotes erd
\end_inset

, figure out why the key currently exports one big string 
\begin_inset Quotes eld
\end_inset

bobby|cindy|marcia
\begin_inset Quotes erd
\end_inset

 when perhaps instead it should export 
\begin_inset Quotes eld
\end_inset


\begin_inset Quotes erd
\end_inset

bobby
\begin_inset Quotes erd
\end_inset

|
\begin_inset Quotes erd
\end_inset

cindy
\begin_inset Quotes erd
\end_inset

|
\begin_inset Quotes erd
\end_inset

marcia
\begin_inset Quotes erd
\end_inset

).
\end_layout

\end_inset


\end_layout

\begin_layout Subsection
About non-enumerated variables
\end_layout

\begin_layout Standard
In the process that creates the variable key template, we think differently
 about the enumerated variables–ones that have meaningfully discrete values
 that can be reassigned one-by-one–and numeric variables.
 Numeric variables, whether we literally mean integers like age or the number
 of pennies in a jar, or floating point numbers (weight, temperature, volume,
 etc) are different.
 If we guessed right with 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
max.levels
\end_layout

\end_inset

 when the key was created, those non-discrete variables are included as
 just one row, whether the key is wide or long.
 In those variables, the main elements of interest for these variables are
 the columns labeled 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
missings
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
recodes
\end_layout

\end_inset

.
 
\end_layout

\begin_layout Standard
The instructions for specifying missing values are detailed in the help
 page of the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
assignMissing
\end_layout

\end_inset

 function in 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
kutils
\end_layout

\end_inset

.
 If the class_old is 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
numeric
\end_layout

\end_inset

 or 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
integer
\end_layout

\end_inset

, there are only three types of statements allowed in the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
missings
\end_layout

\end_inset

 column.
 Legal values must must begin with the characters 
\begin_inset Quotes eld
\end_inset

<
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset

>
\begin_inset Quotes erd
\end_inset

, or 
\begin_inset Quotes eld
\end_inset

c
\begin_inset Quotes erd
\end_inset

.
 These are illustrated in Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Recode-Examples"

\end_inset

.
 The symbols 
\begin_inset Quotes eld
\end_inset

<=
\begin_inset Quotes erd
\end_inset

 and 
\begin_inset Quotes eld
\end_inset

>=
\begin_inset Quotes erd
\end_inset

 are accepted in the obvious way.
 
\end_layout

\begin_layout Standard
\begin_inset Float table
placement H
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Missings Examples
\begin_inset CommandInset label
LatexCommand label
name "tab:Recode-Examples"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Tabular
<lyxtabular version="3" rows="6" columns="3">
<features tabularvalignment="middle">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<row>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
missings
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
interpretation: NA will be assigned to 
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
example
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
> t
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
values greater than t
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
> 99
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
>= t
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
values greater than or equal to t 
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
>=99
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
<t
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
values less than t 
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
<0
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
<=t
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
values less than or equal to t 
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
<=0
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
c(a,b)
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
values equal to or greater than a and less than or equal to b 
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
c(-999,-1)
\end_layout

\end_inset
</cell>
</row>
</lyxtabular>

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
The key specification for 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
recodes
\end_layout

\end_inset

 is discussed in the help page of the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
assignRecode
\end_layout

\end_inset

 function in 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
kutils
\end_layout

\end_inset

.
 The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
recodes
\end_layout

\end_inset

 column takes R code and applies it to the desired variable.
 For example, if one wanted to transform a variable by taking its square
 root, this could be done by providing 
\begin_inset Quotes eld
\end_inset

sqrt(x)
\begin_inset Quotes erd
\end_inset

 in the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
recodes
\end_layout

\end_inset

 column.
 Here 
\begin_inset Quotes eld
\end_inset

x
\begin_inset Quotes erd
\end_inset

 is simply a placeholder where the name of the variable indicated in the
 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
name_new
\end_layout

\end_inset

 column will be inserted when the variable key is applied.
 
\end_layout

\begin_layout Standard
\begin_inset Float table
placement H
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Recoding Integer and Numeric Variables
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

{
\backslash
footnotesize
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout


\backslash
begin{tabular}{lllllllll}
\end_layout

\begin_layout Plain Layout

  
\backslash
toprule
\end_layout

\begin_layout Plain Layout

name
\backslash
_old & name
\backslash
_new & class
\backslash
_old & class
\backslash
_new & value
\backslash
_old & value
\backslash
_new & missings & recodes 
\backslash

\backslash

\end_layout

\begin_layout Plain Layout

  
\backslash
midrule
\end_layout

\begin_layout Plain Layout

x5 & x5 & numeric & numeric &  &  & <0 & log(x) 
\backslash

\backslash

\end_layout

\begin_layout Plain Layout

x4 & x4 & numeric & numeric &  &  & c(-999) & abs(x) 
\backslash

\backslash

\end_layout

\begin_layout Plain Layout

x6 & x6 & integer & integer & 1$|$2$|$3$|$4$|$5 & 1$|$2$|$3$|$4$|$5 & c(-9)
 & 
\backslash

\backslash

\end_layout

\begin_layout Plain Layout

   
\backslash
bottomrule
\end_layout

\begin_layout Plain Layout


\backslash
end{tabular}
\end_layout

\begin_layout Plain Layout

}
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Subsection
Class conversions
\end_layout

\begin_layout Standard
The problem of recoding a variable, but leaving its class the same, is mostly
 solved.
 
\end_layout

\begin_layout Standard
The conversion of variables from one class to another requires special care.
 R's built in functions for coercion of variables from one type to another
 succeed meaningfully in many cases, but not all.
 The coercion of an integer to a floating point number produces understandable
 effects.
 The coercion of a character to an integer is not always understandable,
 and the conversion of factors to integers or numeric is, well, almost always
 a source of concern.
\end_layout

\begin_layout Standard
We are intending to support, at minimum, the 6 safe classes, which can be
 used in any combination of 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

.
 We need to be sure that conversion from each one into the other types is
 handled accurately.
 
\end_layout

\begin_layout Standard
In the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyApply
\end_layout

\end_inset

 function, we have implemented special purpose code to handle a class change
 from factor to numeric, for example, in a way that preserves the levels
 as the new values.
 This is needed when the input data has values like 10, 12, 14, and 
\begin_inset Quotes eld
\end_inset

unknown
\begin_inset Quotes erd
\end_inset

.
 If an R function like read.table is used to import a column with those values,
 the default behavior will convert that to a factor variable with values
 
\begin_inset Quotes eld
\end_inset

10
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset

12, 
\begin_inset Quotes eld
\end_inset

14
\begin_inset Quotes erd
\end_inset

 and 
\begin_inset Quotes eld
\end_inset

unknown
\begin_inset Quotes erd
\end_inset

.
 What the user probably wants–and what we do now–is open the possibility
 that 
\begin_inset Quotes eld
\end_inset

unknown
\begin_inset Quotes erd
\end_inset

 should be treated as NA and the values that appear to be numbers (but are
 actually text strings) are converted into integer format.
\end_layout

\begin_layout Subsection
Unanticipated 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

 values
\end_layout

\begin_layout Standard
By far, the most troublesome variable type is date and time information.
 This is difficult for a number of reasons.
 In order from less troublesome to more troublesome:
\end_layout

\begin_layout Enumerate
No two people seem to represent dates with the same style.
 Virtuous people (the authors) write 2016-12-01, while others write ambiguous
 strings like 12/01/2016, 01/12/2016 or 01-12-2016.
\end_layout

\begin_layout Enumerate
Computer programs absorb dates and convert them to internal numeric schemes.
 However, programs differ in the way they record and export date information.
 Most software converts dates as integers, for example.
 The value is the number of days since time began.
 
\end_layout

\begin_deeper
\begin_layout Standard
Unfortunately, it appears that the day on which time began differs among
 programs.
 Microsoft Excel records dates as integers, and (shockingly!) the origin
 date differs between Excel files created in Windows and Macintosh.
 Mac designers were perplexed by the problem of leap years, which caused
 them to use the origin 1904-01-01, while in MS Windows Excel dates begin
 at 1900-01-01.
 The program SPSS uses integers for dates as well, but the origin for SPSS
 is 1582-10-14 (we guess most readers already knew that one, we apologize
 for being surprised).
 
\end_layout

\begin_layout Standard
On the other hand, dates on Unix/Linux systems tend to follow the POSIX
 standard, so date information is interpreted as the number of days since
 1970-01-01.
 That is a local favorite because it reminds us of Nixon and Vietnam.
\end_layout

\end_deeper
\begin_layout Enumerate
Dates are, to a certain extent, arbitrary without time zone information.
 Its Tuesday, December 8 in Lawrence, Kansas, but it is Wednesday, December
 9 in Moscow, Russia.
 If time zones are not specified, than all date information is, well, approximat
e.
 Furthermore, when scholars try to put time zone information onto existing
 data that was collected without it, they sometimes accidentally create
 impossible dates and times (February 29, 1999, for example).
 When R functions like 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
as.Date
\end_layout

\end_inset

 encounter impossible dates, they return NA.
\end_layout

\begin_layout Enumerate
Even when programs seem to have absorbed dates and saved the information,
 even in casual testing we have found that the saved information is not
 retrieved in a consistent way.
 Some spreadsheet programs, on some computers, return values dates as integers,
 while the same program with a different user returns a character string.
 
\end_layout

\begin_layout Standard
All of this complaining is just our way of saying, 
\begin_inset Quotes eld
\end_inset

we left the recode box open, why not use it to recode your date information?
\begin_inset Quotes erd
\end_inset

 If the variable key template says that a column is 
\begin_inset Quotes eld
\end_inset

integer,
\begin_inset Quotes erd
\end_inset

 but you expected a date, then all is likely to be fine if we set the origin.
 One needs to find out what the origin should be and tell R about it.
 The right recode statement is likely to be 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
as.Date(x, origin = "1970-01-01")
\end_layout

\end_inset

.
 On the other hand, if the date information is in text format, with values
 like 
\begin_inset Quotes eld
\end_inset

2016-11-19
\begin_inset Quotes erd
\end_inset

, then it is necessary to learn the shorthand notation for date symbols.
 The right recode will be something like 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
as.Date(x, format = "%Y-%m-%d")
\end_layout

\end_inset

.
\end_layout

\begin_layout Standard
If one sets 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

 = 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
Date
\end_layout

\end_inset

, the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyApply
\end_layout

\end_inset

 function will succeed if the return value from the recode function is an
 R Date variable.
 Otherwise, an error will be raised.
\end_layout

\begin_layout Subsection
Where more work should be done
\end_layout

\begin_layout Standard
For the most part, our work now lies in development of better diagnostics
 and error-checking routines.
 
\end_layout

\begin_layout Standard
First, we need better filtering of user-created key entries.
 User typographical errors occur in key preparation and more comprehensive
 filtering should be done.
 At the current time, the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyImport
\end_layout

\end_inset

 function does not include many subroutines for the validation of the key.
 We'd like to filter the values requested in 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

, for example, but it is difficult to see how that can be done.
 We want to leave open the possibility that users may specify values of
 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

 that are unfamiliar to us.
 Another problem may arise if the number of elements in 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

 are not aligned one-to-one.
 There will be an error when the key is put to use by 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyApply
\end_layout

\end_inset

, but we should have a stop message in 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyImport
\end_layout

\end_inset

.
\end_layout

\begin_layout Standard
Second, we need to better understand the problems that result when users
 enter variable keys in programs like MS Excel or Libre Office.
 Between programs, or versions of programs, or programs on various platforms,
 we notice disappointing inconsistencies.
 If keys are kept in 
\begin_inset Quotes eld
\end_inset

flat text
\begin_inset Quotes erd
\end_inset

 csv files, there is less danger, but there can still be some trouble because
 spread sheets change the way they export to csv from time to time.
 During the testing process, we have found that version updates Excel cause
 unexpected changes in the way character vectors and dates are stored.
 Until now, we have relied on the very popular package 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
openxlsx
\end_layout

\end_inset

 (
\begin_inset CommandInset citation
LatexCommand citealp
key "Walker2015"

\end_inset

).
 This does well most of the time, but we still find some inexplicable variations
 in imported Excel files.
 Sometimes, empty cells in the variable key appear as imported empty strings
 
\begin_inset Quotes eld
\end_inset


\begin_inset Quotes erd
\end_inset

, while sometimes they are imported as R NA symbols.
 While it is probably safe to treat an entirely empty value as a missing
 value in most projects, we'd rather not do so.
 To deal with this problem, we need to develop a standard framework for
 testing the quality of imported variable keys.
 
\end_layout

\begin_layout Standard
Third, we need more comprehensive error-checking for the accuracy of the
 imported data and compliance with the variable key.
 The variable key system works well with the data sets for which it was
 created, but we need more formal criteria for verifying that claim.
 
\end_layout

\begin_layout Section
Discussion
\end_layout

\begin_layout Standard
When a project has a small budget, we invite the principal investigator
 to economize on the expenses by filling out the variable key's 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
name_new
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

, and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

 columns.
 There are several benefits in inviting the clients (or PIs) to be directly
 involved in filling in the variable key.
 Most importantly, they are allowed to name the variables in any way that
 is meaningful to them.
 When statistical results are obtained, it is never necessary for them to
 ask, 
\begin_inset Quotes eld
\end_inset

what did you mean by this variable 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
occupation
\end_layout

\end_inset

?
\begin_inset Quotes erd
\end_inset

 There are other benefits, however.
 By making the principal investigator aware of the values that are actually
 observed, and by offering the opportunity to specify how they ought to
 be recoded, a substantial element of administrative slippage is ameliorated.
 The variable key will specify exactly how categories are to be re-mapped,
 there is much less danger of an accident buried in thousands of lines of
 recodes.
\end_layout

\begin_layout Standard
It often happens that the raw data to be imported are provided by one of
 the national data centers.
 The variables are given exciting, meaningful column names like V34342a.
 It appears to be almost certain that research assistants will conclude
 that these names are not meaningful, so they create names that are more
 meaningful to them, such as 
\emph on
gender
\emph default
, 
\emph on
sex
\emph default
, 
\emph on
male
\emph default
, 
\emph on
female
\emph default
, or whatnot.
 The research assistants disappear into a haze of code and come out talking
 about the effect of income, gender, and education on educational achievement,
 and the principal investigator has to say, 
\begin_inset Quotes eld
\end_inset

which of those variables is income, again?
\begin_inset Quotes erd
\end_inset

 and 
\begin_inset Quotes eld
\end_inset

what's the coding on education?
\begin_inset Quotes erd
\end_inset

 A very exciting conversation then follows as one of the research assistant
 realizes that V34342b is the one that should have been used for gender,
 while V34342a indicates if the respondent ever visited Eastern Europe.
\end_layout

\begin_layout Standard
The variable key is intended to create neutral territory between principal
 investigators and project supervisors on one side and the research assistants
 on the other.
 The numbers are more likely to come out correctly if the names, values,
 and classes can be specified in one document that is accessible in numerous
 formats on which many eyes can be laid.
 The distance between a re-design of the imported values is minimized and
 the production cycle of the projects is accelerated.
 As we are frequently reminded, there are many some bright people who don't
 know R, it appears they are all fluent in spread sheet.
\end_layout

\begin_layout Standard
\begin_inset Note Note
status open

\begin_layout Plain Layout
\begin_inset Box Frameless
position "t"
hor_pos "c"
has_inner_box 1
inner_pos "t"
use_parbox 0
use_makebox 0
width "100col%"
special "none"
height "1in"
height_special "totalheight"
thickness "0.4pt"
separation "3pt"
shadowsize "4pt"
framecolor "black"
backgroundcolor "none"
status open

\begin_layout Plain Layout
Variable Key Editing Guidelines
\end_layout

\begin_layout Enumerate
Omit a variable: delete its row(s) from the (long)key.
\end_layout

\begin_layout Enumerate
The 
\begin_inset Quotes eld
\end_inset

class_new
\begin_inset Quotes erd
\end_inset

 value
\end_layout

\begin_deeper
\begin_layout Enumerate
discrete variable types:
\begin_inset Separator latexpar
\end_inset


\end_layout

\begin_deeper
\begin_layout Description
logical
\end_layout

\begin_layout Description
integer
\end_layout

\begin_layout Description
factor
\end_layout

\begin_layout Description
ordered
\end_layout

\begin_layout Description
logical
\end_layout

\begin_layout Description
character
\end_layout

\begin_layout Description
Date
\end_layout

\end_deeper
\begin_layout Enumerate
floating-point numeric variable types:
\begin_inset Separator latexpar
\end_inset


\end_layout

\begin_deeper
\begin_layout Description
numeric
\end_layout

\end_deeper
\end_deeper
\begin_layout Enumerate
Creating new variables.
 Each input variable may be recoded in several different ways, assigning
 a name_new value unique for each.
 
\end_layout

\begin_layout Enumerate
Assigning new values.
 
\begin_inset Separator latexpar
\end_inset


\end_layout

\begin_deeper
\begin_layout Enumerate
discrete variable types.
\begin_inset Separator latexpar
\end_inset


\end_layout

\begin_deeper
\begin_layout Enumerate
New 
\begin_inset Quotes eld
\end_inset

value_old
\begin_inset Quotes erd
\end_inset

 elements can be inserted in anticipation of new data sets with previously
 unobserved values.
\end_layout

\begin_layout Enumerate
To assign new values to any of the discrete variable types, set values in
 
\begin_inset Quotes eld
\end_inset

value_new
\begin_inset Quotes erd
\end_inset

.
 It is important that the values can be coerced by R to match the value
 of 
\begin_inset Quotes eld
\end_inset

class_new
\begin_inset Quotes erd
\end_inset

.
 That is to say, if 
\begin_inset Quotes eld
\end_inset

class_new
\begin_inset Quotes erd
\end_inset

 is a 
\begin_inset Quotes eld
\end_inset

integer
\begin_inset Quotes erd
\end_inset

, then 
\begin_inset Quotes eld
\end_inset

value_new
\begin_inset Quotes erd
\end_inset

 should be an integer.
\end_layout

\end_deeper
\begin_layout Enumerate
numeric variable types.
 
\begin_inset Separator latexpar
\end_inset


\end_layout

\begin_deeper
\begin_layout Enumerate
Because floating point numbers do not allow comparison with 
\begin_inset Quotes eld
\end_inset

==
\begin_inset Quotes erd
\end_inset

, it is not allowed to use 
\begin_inset Quotes eld
\end_inset

value_old
\begin_inset Quotes erd
\end_inset

 and 
\begin_inset Quotes eld
\end_inset

value_new
\begin_inset Quotes erd
\end_inset

 pairs for these variables.
 It is necessary to write statements in the recode column of the variable
 key.
\end_layout

\end_deeper
\end_deeper
\begin_layout Enumerate
Assign missings:
\begin_inset Separator latexpar
\end_inset


\end_layout

\begin_deeper
\begin_layout Enumerate
discrete variables: several options, we should settle on one suggestion
\begin_inset Separator latexpar
\end_inset


\end_layout

\begin_deeper
\begin_layout Enumerate
insert 
\begin_inset Quotes eld
\end_inset

NA
\begin_inset Quotes erd
\end_inset

 in value_new column
\end_layout

\begin_layout Enumerate
delete the value_old entry (not the best way, maybe don't mention it)
\end_layout

\begin_layout Enumerate
list a vector of missing values in the recode column.
 This vector applies to data as it was imported, before applying the value_old
 to value_new transition.
 PROBLEM: if input data's class does not match class_old, there may be a
 problem.
\end_layout

\end_deeper
\begin_layout Enumerate
numeric variables
\end_layout

\end_deeper
\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset CommandInset bibtex
LatexCommand bibtex
bibfiles "kutils"
options "chicago"

\end_inset


\end_layout

\end_body
\end_document

#LyX 2.3 created this file. For more info see http://www.lyx.org/
\lyxformat 544
\begin_document
\begin_header
\save_transient_properties true
\origin unavailable
\textclass scrartcl
\begin_preamble
%\VignetteIndexEntry{variablekey}

\usepackage{booktabs}
\usepackage{Sweavel}
\usepackage{graphicx}
\usepackage{color}

\usepackage[samesize]{cancel}



\usepackage{ifthen}

\makeatletter

\renewenvironment{figure}[1][]{%

 \ifthenelse{\equal{#1}{}}{%

   \@float{figure}

 }{%

   \@float{figure}[#1]%

 }%

 \centering

}{%

 \end@float

}

\renewenvironment{table}[1][]{%

 \ifthenelse{\equal{#1}{}}{%

   \@float{table}

 }{%

   \@float{table}[#1]%

 }%

 \centering

%  \setlength{\@tempdima}{\abovecaptionskip}%

%  \setlength{\abovecaptionskip}{\belowcaptionskip}%

% \setlength{\belowcaptionskip}{\@tempdima}%

}{%

 \end@float

}


%\usepackage{listings}
% Make ordinary listings look as if they come from Sweave
\lstset{tabsize=2, breaklines=true, %,style=Rstyle}
                        fancyvrb=false,escapechar=`,language=R,%
                        %%basicstyle={\Rcolor\Sweavesize},%
                        backgroundcolor=\Rbackground,%
                        showstringspaces=false,%
                        keywordstyle=\Rcolor,%
                        commentstyle={\Rcommentcolor\ttfamily\itshape},%
                        literate={<<-}{{$\twoheadleftarrow$}}2{~}{{$\sim$}}1{<=}{{$\leq$}}2{>=}{{$\geq$}}2{^}{{$^{\scriptstyle\wedge}$}}1{==}{{=\,=}}1,%
                        alsoother={$},%
                        alsoletter={.<-},%
                        otherkeywords={!,!=,~,$,*,\&,\%/\%,\%*\%,\%\%,<-,<<-,/},%
                        escapeinside={(*}{*)}}%


% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\def\Sweavesize{\scriptsize} 
\def\Rcolor{\color{black}} 
\def\Rbackground{\color[gray]{0.95}}

% for sideways table
\usepackage{rotating}
\end_preamble
\use_default_options true
\begin_modules
sweave
logicalmkup
\end_modules
\maintain_unincluded_children false
\language american
\language_package default
\inputencoding utf8
\fontencoding global
\font_roman "lmodern" "default"
\font_sans "lmss" "default"
\font_typewriter "lmtt" "default"
\font_math "auto" "auto"
\font_default_family default
\use_non_tex_fonts false
\font_sc false
\font_osf false
\font_sf_scale 100 100
\font_tt_scale 100 100
\use_microtype false
\use_dash_ligatures false
\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref false
\papersize default
\use_geometry true
\use_package amsmath 1
\use_package amssymb 1
\use_package cancel 1
\use_package esint 1
\use_package mathdots 1
\use_package mathtools 1
\use_package mhchem 1
\use_package stackrel 1
\use_package stmaryrd 1
\use_package undertilde 1
\cite_engine natbib
\cite_engine_type authoryear
\biblio_style plainnat
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\justification true
\use_refstyle 1
\use_minted 0
\index Index
\shortcut idx
\color #008000
\end_index
\leftmargin 1in
\topmargin 1in
\rightmargin 1in
\bottommargin 1in
\secnumdepth 3
\tocdepth 3
\paragraph_separation indent
\paragraph_indentation default
\is_math_indent 0
\math_numbering_side default
\quotes_style english
\dynamic_quotes 0
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Title
The Variable Key Data Management Framework
\end_layout

\begin_layout Author

\lang english
Paul E.
 Johnson<pauljohn@ku.edu>
\begin_inset Newline newline
\end_inset

Benjamin A.
 Kite<bakite@ku.edu> 
\begin_inset Newline newline
\end_inset

Center for Research Methods and Data Analysis
\begin_inset Newline newline
\end_inset

University of Kansas
\end_layout

\begin_layout Date
\paragraph_spacing single
\noindent

\series bold
\lang english
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
today
\end_layout

\end_inset


\end_layout

\begin_layout Abstract
This essay describes the 
\begin_inset Quotes eld
\end_inset

variable key
\begin_inset Quotes erd
\end_inset

 approach to importing and recoding data.
 This method has been developed in the Center for Research Methods and Data
 Analysis at the University of Kansas to deal with the importation of large,
 complicated data sets.
 This approach improves teamwork, keeps better records, and reduces slippage
 between the intentions of principal investigators the implementation by
 code writers.
 The framework is implemented in the R 
\begin_inset CommandInset citation
LatexCommand citep
key "RCore"
literal "true"

\end_inset

 package 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
kutils
\end_layout

\end_inset

.
 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=F>>=
\end_layout

\begin_layout Plain Layout

if(!dir.exists("plots")) dir.create("plots")
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard

\lang english
\begin_inset ERT
status open

\begin_layout Plain Layout

% In document Latex options:
\end_layout

\begin_layout Plain Layout


\backslash
fvset{listparameters={
\backslash
setlength{
\backslash
topsep}{0em}}}
\end_layout

\begin_layout Plain Layout


\backslash
SweaveOpts{prefix.string=plots/t,split=F,ae=F,height=4,width=5.5}
\end_layout

\end_inset


\end_layout

\begin_layout Standard

\lang english
\begin_inset ERT
status open

\begin_layout Plain Layout

<<Roptions, echo=F>>=
\end_layout

\begin_layout Plain Layout

options(device = pdf)
\end_layout

\begin_layout Plain Layout

options(width=100, prompt=" ", continue="  ")
\end_layout

\begin_layout Plain Layout

options(useFancyQuotes = FALSE) 
\end_layout

\begin_layout Plain Layout

options(SweaveHooks=list(fig=function() par(ps=10)))
\end_layout

\begin_layout Plain Layout

pdf.options(onefile=F,family="Times",pointsize=10)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Section
Introduction
\end_layout

\begin_layout Standard
The staff of the Center for Research Methods and Data Analysis has been
 asked to help with data importation and recoding from time to time.
 In one very large project, we were asked to combine, recode, and integrate
 variables from 21 different files.
 The various files used different variable names and had different, unique
 coding schemes.
 A skeptic might have thought that the firm which created the data sets
 intentionally obfuscated the records to prevent the comparison of variables
 across a series of surveys.
\end_layout

\begin_layout Standard
In projects like that, the challenge of importing and fixing the data seems
 overwhelming.
 The graduate research assistants are asked to cobble together thousands
 of lines of 
\begin_inset Quotes eld
\end_inset

recodes
\begin_inset Quotes erd
\end_inset

 as they rename, regroup, and otherwise harmonize the information.
 From a managerial point of view, that is not the main problem.
 We expect to spend the time of research assistants.
 While it may be tedious to read a codebook and write recodes, social scientists
 have been doing that since the 1960s.
 It is not all that difficult.
 The truly difficult part is mustering up the confidence in the resulting
 recoded data.
 How can a supervisor check thousands of recode statements for accuracy?
 The very extensibility of R itself–its openness to new functions and language
 elements–makes proof-reading more difficult.
 We might shift some of the proof reading duty to the principle investigators,
 but they sometimes are not interested in details.
 In the end, the responsibility for verifying the recodes falls on the project
 supervisors.
 While most supervisors with whom we are personally acquainted have nearly
 super-human reading skills and almost-perfect comprehension, we have documented
 a case in which one of them was unable to catch an error on line 827 within
 an R file with 2119 lines.
 
\end_layout

\begin_layout Standard
To reduce the risk of misunderstanding and error, we propose the 
\emph on
variable key procedure
\emph default
.
 It is a systematic way to separate code writing from the process of renaming
 variables and re-designating their values.
 The characteristics of the data are summarized in a table, a simple-looking
 structure that might be edited in a text editor or a spread sheet program.
 This simple structure, which we call the variable key, can be used by principal
 investigators and supervisors to designate the desired results.
 Once the key is created, then the data set can be imported and recoded
 by the application of the key's information.
 This does not eliminate the need to proof-read the renaming and recoding
 of the variables, it simply shifts that chore into a simpler, more workable
 setting.
\end_layout

\begin_layout Standard
This essay proceeds in 3 parts.
 First, the general concepts behind the variable key system are explored.
 Second, the four stages in the variable key procedure are outlined and
 illustrated with examples.
 Third, we offer some examples of ways to double-check the results.
\end_layout

\begin_layout Section
Enter the Variable Key
\end_layout

\begin_layout Standard
The variable key process was first developed for a very large project for
 which we were hired by a commercial consulting company.
 As it happened, the project manager who hired us was an Excel user who
 did not know about R.
 He was given several SPSS datasets.
 After going through the usual R process of importing and recoding data
 from 6 files, the aggregate of which included more than 40,000 observations
 on 150 variables, we arrived at a renamed set of columns.
 Unfortunately, the research assistant who had done most of the work resigned
 in order to pursue a career as a magician.
\begin_inset Foot
status collapsed

\begin_layout Plain Layout
Or graduated, we are not sure which.
\end_layout

\end_inset

 
\end_layout

\begin_layout Standard
With the unavailability of our key asset, it was difficult to know for sure
 what was in which column.
 There was nobody to quickly answer questions like 
\begin_inset Quotes eld
\end_inset

which column is the respondent's sexual identity?
\begin_inset Quotes erd
\end_inset

 and 
\begin_inset Quotes eld
\end_inset

if sex is V23418, did we change 1 to male or female
\begin_inset Quotes erd
\end_inset

.
 The only way to find out is by hunting and pecking through a giant R file.
 
\end_layout

\begin_layout Standard
In order to better communicate about that project, we developed a table
 that looked like Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:A-Small-Variable-key"

\end_inset

.
\end_layout

\begin_layout Standard
\begin_inset Float table
placement H
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
A Small Variable Key
\begin_inset CommandInset label
LatexCommand label
name "tab:A-Small-Variable-key"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Tabular
<lyxtabular version="3" rows="4" columns="4">
<features tabularvalignment="middle">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<row>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
name_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
name_new
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
values_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
values_new
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
V23419
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
sex
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
1|2|3
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset Quotes eld
\end_inset

male
\begin_inset Quotes erd
\end_inset

|
\begin_inset Quotes erd
\end_inset

female
\begin_inset Quotes erd
\end_inset

|
\begin_inset Quotes erd
\end_inset

neither
\begin_inset Quotes erd
\end_inset


\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
V32422
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
education
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
1|2|3|4|5
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
\begin_inset Quotes eld
\end_inset

elem
\begin_inset Quotes erd
\end_inset

<
\begin_inset Quotes erd
\end_inset

hs
\begin_inset Quotes erd
\end_inset

<
\begin_inset Quotes erd
\end_inset

somecoll
\begin_inset Quotes erd
\end_inset

<
\begin_inset Quotes erd
\end_inset

ba
\begin_inset Quotes erd
\end_inset

<
\begin_inset Quotes erd
\end_inset

post
\begin_inset Quotes erd
\end_inset


\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
V54532
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
income
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
.
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
numeric
\end_layout

\end_inset
</cell>
</row>
</lyxtabular>

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
It was tedious to assemble that table, but it helped quite a bit in our
 discussions.
 The vertical bars were used to indicate that the original data had discrete
 values.
 When a variable has a natural ordering, the new values were placed in order
 with the symbol (
\begin_inset Quotes eld
\end_inset

<
\begin_inset Quotes erd
\end_inset

).
 That table grew quite large, since it had one row per variable, but it
 was otherwise workable.
 It was popular with the client.
 
\end_layout

\begin_layout Standard
In the middle of preparing that summary table of recoded values, we realized
 that it was possible to write an R program to import the key table and
 use its information to recode and rename the variables.
 The recodes would 
\emph on
just happen
\emph default
.
 If we prepared the functions properly, we had not just a table masquerading
 as a codebook, we had a 
\emph on
programmable codebook
\emph default
.
 We wrote some functions that could import variables (as named in column
 1), apply the new values (from columns 3 and 4), then apply the new name
 from column 2.
 The functions to do that are somewhat difficult to prepare, but they are
 very appealing from a supervisor's point of view.
 There will be less proof-reading to do, at least in the R code itself.
 Once we can validate the functions, then we never have to proof-read them
 again.
 These functions can be applied, row by row, to create a new data frame.
 Instead, we need to concentrate our attention on the substance of the problem,
 the specification of the new names and values in the table.
\end_layout

\begin_layout Standard
In the projects where we have employed this system, we adjusted the key
 and the R functions to suit the particular demands of the project and the
 client.
 That was unfortunate, because we had very little accumulation of code from
 one project to another.
 However, we did accumulate experience; there were concepts and vocabulary
 which allowed us to understand the various challenges that might be faced.
 The effort to develop a standardized framework for the variable key began
 in 2016 with the creation of the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
kutils
\end_layout

\end_inset

 package for R.
\end_layout

\begin_layout Standard
The variable key process allows project supervisors to create a table that
 instructs the research assistants in the importation, renaming, and recoding
 of data.
 There is still a daunting problem, however, because the supervisors must
 create that variable key table.
 In a large data set, it might be arduous to simply type the old names of
 the variables and their observed values.
 In 2015 one of the graduate assistants in our lab was asked to type up
 a variable key and he couldn't quite believe that was a good use of his
 time.
 After some discussion, we realized that it was not necessary to type the
 variable key at all.
 We would write a function to do so.
 If R can import the candidate data set, then R can certainly output its
 column names and a roster of observed values.
 This lightened the workload considerably.
 By tabulating all of the observed variables and their values, the most
 tedious part of the process was done mechanically.
 
\end_layout

\begin_layout Standard
In the remainder of this essay, we discuss the process of creating a variable
 key template, revising it, and putting it to use.
 
\end_layout

\begin_layout Section
Four Simple Steps
\end_layout

\begin_layout Standard
The variable key process has four steps.
 First, inspect an R data.frame object and create a key template file.
 The key template summarizes the existing state of the variables and creates
 
\begin_inset Quotes eld
\end_inset

placeholders
\begin_inset Quotes erd
\end_inset

 where we might like to specify revisions.
 Second, edit the key template file in a spreadsheet or other program that
 can work with comma separate variables.
 Change the names, values, and designate other recodes (which we will describe
 next).
 Third, import the revised key into R.
 Fourth, apply the key to the data to generate a new, improved data frame.
 Then run some diagnostic routines.
\end_layout

\begin_layout Standard
If all goes well, we should end up with a new data frame in which
\end_layout

\begin_layout Enumerate
The columns are renamed in accordance with the instructions of the principal
 investigator (or supervisor).
\end_layout

\begin_layout Enumerate
The values of all variables have been recoded according to the instructions
 of the principal investigator (or supervisor).
\end_layout

\begin_layout Standard
Diagnostic tables are reported to clearly demonstrate the effect of each
 coding change, mapping out the difference between the input and the output
 variables.
 
\end_layout

\begin_layout Standard
For purposes of illustration, we have create an example data frame with
 various types of variables.
 This data frame, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
mydf
\end_layout

\end_inset

, has most of the challenges that we see in actual projects.
 It has integer variables that need to be reorganized and turned into character
 or factor variables.
 It has character variables that might become integers or factors.
 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=T,include=T>>=
\end_layout

\begin_layout Plain Layout

set.seed(234234)
\end_layout

\begin_layout Plain Layout

N <- 200
\end_layout

\begin_layout Plain Layout

mydf <- data.frame(
\end_layout

\begin_layout Plain Layout

    x5 = rnorm(N),
\end_layout

\begin_layout Plain Layout

    x4 = rpois(N, lambda = 3),
\end_layout

\begin_layout Plain Layout

    x3 = ordered(sample(c("lo", "med", "hi"), size = N, replace=TRUE),
\end_layout

\begin_layout Plain Layout

            levels = c("med", "lo", "hi")),
\end_layout

\begin_layout Plain Layout

    x2 = letters[sample(c(1:4,6), 200, replace = TRUE)],
\end_layout

\begin_layout Plain Layout

    x1 = factor(sample(c("cindy", "jan", "marcia"), 200,
\end_layout

\begin_layout Plain Layout

            replace = TRUE)),
\end_layout

\begin_layout Plain Layout

    x7 = ordered(letters[sample(c(1:4,6), 200, replace = TRUE)]),
\end_layout

\begin_layout Plain Layout

    x6 = sample(c(1:5), 200, replace = TRUE),
\end_layout

\begin_layout Plain Layout

            stringsAsFactors = FALSE)
\end_layout

\begin_layout Plain Layout

mydf$x4[sample(1:N, 10)] <- 999
\end_layout

\begin_layout Plain Layout

mydf$x5[sample(1:N, 10)] <- -999
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Subsection
Step 1.
 Create a Key Template
\end_layout

\begin_layout Standard
The function 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyTemplate
\end_layout

\end_inset

 scans a data frame and generates a new key template.
 The key has 8 pieces of information about each variable.
 The rows of the key are named by the variables of the data frame.
 The 8 columns in the key are 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
name_old
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
name_new
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_old
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
missings
\end_layout

\end_inset

, and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
recodes
\end_layout

\end_inset

.
 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyTemplate
\end_layout

\end_inset

 will fill 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
name_old
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_old
\end_layout

\end_inset

, and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

, in the with values based on the data input, while the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
new
\end_layout

\end_inset

 columns will be copies of those old values.
 The last 2, missings and recodes, will be empty.
 
\end_layout

\begin_layout Standard
There are two formats for the key template, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
long
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
wide
\end_layout

\end_inset

 (determined by the parameter 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
long
\end_layout

\end_inset

).
 These names are drawn from terminology in R's reshape function.
 The long format has one row per value of each variable, while the wide
 format has all of the information in one row.
 The two key formats are intended to be interchangeable in functionality;
 they differ solely for convenience.
 Some users may prefer to edit variable information in one style.
 The re-importation of the key should deal gracefully with either type of
 variable key.
 
\end_layout

\begin_layout Standard
A wide format key can be produced with a call to the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyTemplate
\end_layout

\end_inset

 function like so:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=F>>=
\end_layout

\begin_layout Plain Layout

key_wide <- keyTemplate(mydf, file = "key_wide.csv", max.levels = 5)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\noindent
If the long argument is not specified, a wide key is the default.
 One can ask for a long format:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=F>>=
\end_layout

\begin_layout Plain Layout

key_long <- keyTemplate(mydf, long = TRUE, file = "key_long.csv", max.levels
 = 5)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\noindent
The key object is a data.frame.
 
\end_layout

\begin_layout Standard
Apart from the long argument, the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyTemplate
\end_layout

\end_inset

 function has two especially noteworthy arguments, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
file
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
max.levels
\end_layout

\end_inset

.
 If the file argument is supplied, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyTemplate
\end_layout

\end_inset

 uses the suffix to determine storage format.
 Legal suffixes are 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
.csv
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
.xlsx
\end_layout

\end_inset

, and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
.rds
\end_layout

\end_inset

 (for creating comma separated variables, Excel spreadsheets, and R serializatio
n data structures).
 
\end_layout

\begin_layout Standard
The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
max.levels
\end_layout

\end_inset

 argument is also important.
 This is used in the same sense that functions like 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
read.spss
\end_layout

\end_inset

 in the foreign package use that term.
 There is guessing involved in deciding if we should enumerate a character
 or integer variable.
 We do want to enumerate the 
\begin_inset Quotes eld
\end_inset

Strongly Disagree
\begin_inset Quotes erd
\end_inset

 to 
\begin_inset Quotes eld
\end_inset

Strongly Agree
\begin_inset Quotes erd
\end_inset

 values of a 7 point scale, but we do not want to enumerate the first names
 of all study participants.
 If the number of discrete values exceeds 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
max.levels
\end_layout

\end_inset

, for which the default is 15, then the key will not enumerate them.
 
\end_layout

\begin_layout Standard
Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:The-Wide-Key"

\end_inset

 demonstrates a wide key template as it is produced by 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyTemplate
\end_layout

\end_inset

.
 We see now why it is called a wide key; the recoding information is tightly
 packed into 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

.
 The key includes more or less obvious columns for the old and new variable
 names, their classes, and values of the variables.
 Note that the values of x5 and x4 are not enumerated because we set max.levels
 at 5.
 The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
max.levels
\end_layout

\end_inset

 parameter defaults to 15, so that an integer variable with less than 15
 values will have each value displayed.
 For this example, the display of that variable key was too wide for the
 page, so we reduced the number of values.
 When the observed number of scores is above 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
max.levels
\end_layout

\end_inset

, the key does not try to list the individual values (compare the treatment
 of variables 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
x4
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
x6
\end_layout

\end_inset

).
 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<key0, include=F>>=
\end_layout

\begin_layout Plain Layout

library(kutils)
\end_layout

\begin_layout Plain Layout

library(xtable)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<key10, echo=F>>=
\end_layout

\begin_layout Plain Layout

key_wide <- keyTemplate(mydf, max.levels = 5)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
A long key template is displayed in Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Long-Key"

\end_inset

.
 The benefit of the long key is that the cells 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

 are easier to navigate.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<key30,echo=F>>=
\end_layout

\begin_layout Plain Layout

key_long <- keyTemplate(mydf, long = TRUE, max.levels = 5)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float table
placement h
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
The Wide Key Template
\begin_inset CommandInset label
LatexCommand label
name "tab:The-Wide-Key"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
def
\backslash
Sweavesize{
\backslash
tiny}
\end_layout

\begin_layout Plain Layout

<<key20,echo=FALSE,results=tex>>=
\end_layout

\begin_layout Plain Layout

print(xtable(key_wide), include.rownames = FALSE, size = "small", floating
 = FALSE )
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float table
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
The Long Key Template
\begin_inset CommandInset label
LatexCommand label
name "tab:Long-Key"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
def
\backslash
Sweavesize{
\backslash
tiny}
\end_layout

\begin_layout Plain Layout

<<key40,echo=FALSE,results=tex>>=
\end_layout

\begin_layout Plain Layout

print(xtable(key_long), include.rownames = FALSE, size = "small", floating
 = FALSE)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
The value of 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_old
\end_layout

\end_inset

 in the key is the first element in the return from the function class for
 a variable.
 There is one exception, where we have tried to differentiate integer variables
 from numeric variables.
 This is a confusing isssue in the history of R, as discussed in the R help
 page for the function 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
as.double
\end_layout

\end_inset

.
 ,In the 
\emph on
Note on names
\emph default
 section, that page explains an 
\begin_inset Quotes eld
\end_inset

anomaly
\begin_inset Quotes erd
\end_inset

 in the usage of term 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
numeric
\end_layout

\end_inset

.
 The R function 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
as.numeric
\end_layout

\end_inset

 creates a double precision floating point value, not an integer.
 However, the is.numeric function responds TRUE for both integers and floating
 point values.
 For purposes of editing the key, it is useful to differentiate integers
 from floating point numbers.
 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
kutils
\end_layout

\end_inset

 includes a function named 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
safeInteger
\end_layout

\end_inset

.
 It checks the observed values of a variable to find out if any floating
 point values are present.
 If the aggregate deviations from integer values are miniscule, then a variable
 is classified as an integer.
 As a result, the keyTemplate function's column 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_old
\end_layout

\end_inset

 should be 
\begin_inset Quotes eld
\end_inset

integer
\begin_inset Quotes erd
\end_inset

 or 
\begin_inset Quotes eld
\end_inset

numeric
\begin_inset Quotes erd
\end_inset

, and by the latter we mean a floating point number.
 
\end_layout

\begin_layout Standard
In some of our early projects, the variable key was in the wide format.
 Difficulty in editing that caused us to shift some projects to the long
 key.
 The idea that we would glide back and forth between keys created in the
 wide and long formats dawned on us only recently.
 To ease the conversion back-and-forth between the formats, we developed
 the functions named 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
wide2long
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
long2wide
\end_layout

\end_inset

.
 We believe these functions work effectively, but we have experienced some
 troubles related to the way spreadsheets store character strings.
 If the key in long format has a column of values 
\begin_inset Quotes eld
\end_inset

Yes
\begin_inset Quotes erd
\end_inset

,
\begin_inset Quotes erd
\end_inset

No
\begin_inset Quotes erd
\end_inset

, and 
\begin_inset Quotes eld
\end_inset


\begin_inset Quotes erd
\end_inset

, the wide representation should be 
\begin_inset Quotes eld
\end_inset

Yes|No|
\begin_inset Quotes erd
\end_inset

, but there is some inclination to say we should have nested quotation marks,
 as in 
\begin_inset Quotes eld
\end_inset


\begin_inset Quotes erd
\end_inset

Yes
\begin_inset Quotes erd
\end_inset

|
\begin_inset Quotes erd
\end_inset

No
\begin_inset Quotes erd
\end_inset

|
\begin_inset Quotes erd
\end_inset


\begin_inset Quotes erd
\end_inset


\begin_inset Quotes erd
\end_inset

.
 That kind of string will not generally survive importation to and export
 from a spread sheet at the current time.
\end_layout

\begin_layout Subsection
Step 2.
 Edit the variable key
\end_layout

\begin_layout Standard
If the file argument was specified in 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyTemplate
\end_layout

\end_inset

, the work is laid out for us.
 One can edit a csv file in any text editor or in a spreadsheet.
 An xlsx file can be edited by Libre Office or Microsoft Office.
 
\end_layout

\begin_layout Standard
It is not necessary to change all of the values in name_new, class_new,
 and value_new.
 In fact, one might change just a few elements and the un-altered variables
 will remain as they were when the data is re-imported.
 We suggest users start small by making a few edits in the key.
 A principal investigator might change just a few variable names or values.
 In a large project, there may be quite a bit of work involved.
 
\end_layout

\begin_layout Standard
The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
name_old
\end_layout

\end_inset

 column must never be edited.
 Generally, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_old
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 will not be edited either (the only exception might arise if 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

 is either 
\begin_inset Quotes eld
\end_inset


\begin_inset Flex Code
status open

\begin_layout Plain Layout
factor
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

 or 
\begin_inset Quotes eld
\end_inset


\begin_inset Flex Code
status open

\begin_layout Plain Layout
ordered
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

).
 The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
name_new
\end_layout

\end_inset

 column should include legal R variable names (do not begin name_new with
 a numeral or include mathematical symbols like 
\begin_inset Quotes eld
\end_inset

+
\begin_inset Quotes erd
\end_inset

 or 
\begin_inset Quotes eld
\end_inset

-
\begin_inset Quotes erd
\end_inset

).
 We use R's 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
make.names
\end_layout

\end_inset

 function to clean up errant entries, so incorrect names are not fatal.
\end_layout

\begin_layout Standard
The difficult user decisions will concern the contents of class_new and
 value_new.
 The desired variable type, of course, influences the values that are meaningful.
 To convert a character variable to integer, for example, it should go without
 saying that the value_new element should include integer values, not text
 strings.
\end_layout

\begin_layout Standard
The conversion of information from one type of variable into another may
 be more complicated than it seems.
 It is a bit more tricky to convert a factor into a numeric variable that
 uses the factor's levels as numeric values.
 
\end_layout

\begin_layout Standard
After experimenting with a number of cases, we believe that if 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_old
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

 are elements of this the 
\begin_inset Flex Emph
status open

\begin_layout Plain Layout
safe class set
\end_layout

\end_inset

: 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
character
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
logical
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
integer
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
numeric
\end_layout

\end_inset

 (same as 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
double
\end_layout

\end_inset

), 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
factor
\end_layout

\end_inset

, or 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
ordered
\end_layout

\end_inset

, then the re-importation and recoding of data will be more-or-less automatic.
 If 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

 differs from 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_old
\end_layout

\end_inset

, and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

 is not an element in that 6 element set, then the user must supply a recode
 function that creates a variable of the requested class.
 Most commonly, we expect that will be used to incorporate 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
Date
\end_layout

\end_inset

 variables.
\end_layout

\begin_layout Standard
The enumerated values in the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

 column should be specified in the more or less obvious way.
 If 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

 is equal to 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
character
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
factor
\end_layout

\end_inset

, or 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
ordered
\end_layout

\end_inset

, then the new values can be arbitrary strings.
 
\end_layout

\begin_layout Standard
The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
missings
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
recodes
\end_layout

\end_inset

 columns are empty in the key template.
 The user will need to fill in those values if they are to be used.
 When the key is later put to use, the order of processing will be as follows.
 First, the values declared as missings will be applied (convert observed
 value to R's NA symbol).
 Second, if there is a recode function in the key, it is applied to a variable.
 Third, if there was no recode function supplied, then the conversion of
 discrete values by recalculation from 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 into 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

 will be applied.
 Note that the discrete values are applied only if the recode cell is empty
 in the key.
\end_layout

\begin_layout Standard
The decision of whether to approach a given variable via value enumeration
 or a recode function is, to an extent, simply a matter of taste.
 Some chores that might be handled in either way.
 If a variable includes floating point numbers (temperatures, dollar values,
 etc), then we would not rely on new assignments in 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

.
 Truly numeric variables of that sort almost certainly call for assignment
 of missings and recodes by the last two cells in the variable key.
 However, if a column includes integers or characters (1 = male, 2 = female),
 one might use the enumerated values (
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

) or one could design a recode function to produce the same result.
 It is important to remember that if a recode function is applied, the enumerate
d value recoding is not.
 If one decides to use a recode statement, then the elements in value_old
 and value_new are ignored entirely, they could be manually deleted to simplify
 the key.
 (That is to say, the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
max.levels
\end_layout

\end_inset

 parameter is just a way of guessing how many unique levels is 
\begin_inset Quotes eld
\end_inset

too many
\begin_inset Quotes erd
\end_inset

 for an enumeration.
 Users are free to delete values if recodes are used.)
\end_layout

\begin_layout Standard
Despite the possibility that a factor (or ordered) variable may have many
 values, we believe that all of the levels of those variables should be
 usually be included in the key.
 If a variable is declared as a factor, it means the researcher has assigned
 meaning to the various observed values and we are reluctant to ignore them.
 
\end_layout

\begin_layout Standard
There is a more important reason to enumerate all of the legal values for
 factor variables.
 If a value is omitted from the key, that value will be omitted from the
 dataset.
 
\end_layout

\begin_layout Standard
Among our users, we find opinion is roughly balanced between the long and
 the wide key formats.
 One might simply try both.
 If the number of observed values is more than 5 or 10, editing the key
 in a program like Microsoft Excel is less error prone in the long key.
 This is simply a matter of taste, however.
 The disadvantage of the long format is that it is somewhat verbose, with
 repeated values in the name and class values.
 If an editor makes an error in the assignment of a block, then hard to
 find errors may result.
\end_layout

\begin_layout Standard
Because editing the key can be a rather involved process, we will wait to
 discuss the details until section 
\begin_inset CommandInset ref
LatexCommand ref
reference "sec:Editing-the-key"

\end_inset

.
\end_layout

\begin_layout Subsection
Step 3.
 keyImport
\end_layout

\begin_layout Standard
Once any desired changes are entered in the variable key, the file needs
 to be imported back into R.
 For that purpose, we supply the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyImport
\end_layout

\end_inset

 function.
 As in 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyTemplate
\end_layout

\end_inset

, the file argument's suffix is used to discern whether the input should
 be read as .csv, .xlsx, or .rds.
 It is not necessary to specify that the key being imported is in the long
 or wide format.
 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyImport
\end_layout

\end_inset

 includes heuristics that have classified user-edited keys very accurately.
 
\end_layout

\begin_layout Standard
The returned value is an R data frame that should be very similar to the
 template, except that the new values of 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
name_new
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

, and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

 will be visible.
 
\end_layout

\begin_layout Standard
In order to test this function with the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
kutils
\end_layout

\end_inset

 package, we include some variable keys.
 The usage of those keys is demonstrated in the help page for 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyImport
\end_layout

\end_inset

.
 In addition to the mydf toy data frame created above, we also include a
 subset of the US National Longitudinal Survey in a data frame named 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
natlongsurv
\end_layout

\end_inset

.
\end_layout

\begin_layout Subsection
Step 4.
 Apply the imported key to the data
\end_layout

\begin_layout Standard
The final step is to apply the key to the data frame (or some other data
 frame that may have arrived in the interim).
 The syntax is simple
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=F>>=
\end_layout

\begin_layout Plain Layout

mydf.cleaned <- keyApply(mydf, mydf.keylist)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Because the default value of the argument 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
diagnostic
\end_layout

\end_inset

 is TRUE, the output from 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyApply
\end_layout

\end_inset

 is somewhat verbose.
 After we have more feedback from test users, we will be able to quiet some
 of that output.
 
\end_layout

\begin_layout Standard
The diagnostic output will include information about mismatch between the
 key and the data itself.
 If variables that are included in the key that are not included in the
 new data set, there will not be an error, but a gentle warning will appear.
 Similarly, if the observed values of an enumerated variable are not included
 in the variable key, there will be a warning.
 
\end_layout

\begin_layout Standard
The diagnostic will also create a cross tabulation of each new variable
 against its older counterpart.
 This works very well with discrete variables with 10 or so values, but
 for variables with more values it is rather unmanageable.
 
\end_layout

\begin_layout Section
Editing the variable key
\end_layout

\begin_layout Standard
\begin_inset CommandInset label
LatexCommand label
name "sec:Editing-the-key"

\end_inset


\end_layout

\begin_layout Standard
The work of revising the variable key can be driven by the separation of
 variables into two type.
 The variables with enumerated values–the ones for which we intend to rename
 or re-assign values one-by-one–are treated in a very different way than
 the other ones.
 The enumerated value strategy works well with variables for which we simply
 need to rename categories (e.g, 
\begin_inset Quotes eld
\end_inset

cons
\begin_inset Quotes erd
\end_inset

 to 
\begin_inset Quotes eld
\end_inset

Conservative
\begin_inset Quotes erd
\end_inset

).
 Variables for which we do not do so (e.g., convert Fahrenheit to Celsius)
 are treated differently.
\end_layout

\begin_layout Standard
As we will see in this section, the revision of variables of the enumerated
 value type emphasizes the revision of the value_old and value_new columns
 in the key.
 On the other hand, the other types will depend on writing correctly formatted
 statements in the recode column of the variable key.
\end_layout

\begin_layout Subsection
Enumerated variables
\end_layout

\begin_layout Standard
All of the values observed for 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
logical
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
factor
\end_layout

\end_inset

, and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
ordered
\end_layout

\end_inset

 variables will appear in the key template.
 Do not delete them unless the exclusion of those values from the analysis
 intended.
 For character and integer variables with fewer than 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
max.levels
\end_layout

\end_inset

 discrete values, the observed scores will be included in 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

.
 If one wishes to convert a variable from being treated as an enumerated
 to a numeric type, then one can delete all values from 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

.
 
\end_layout

\begin_layout Standard
The recoding of discrete variables is a fairly obvious chore.
 For each old value, a new value must be specified.
 We first consider the case of a variable that enters as a character variable
 but we might like to recode it and also create factor and integer variants
 of it.
 In the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
mydf
\end_layout

\end_inset

 variable key (Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:The-Wide-Key"

\end_inset

), we have variable 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
x2
\end_layout

\end_inset

 which is coded 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
a
\end_layout

\end_inset

 through 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
f
\end_layout

\end_inset

.
 We demonstrate ways to spawn new character, factor, or integer variable
 in Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Change-Type-Example1"

\end_inset

.
 As long as name_old is preserved, as many lines as desired can be used
 to create variables of different types.
 Here we show the middle section of the revised key in which we have spawned
 3 new variants of x2, each with its own name.
\end_layout

\begin_layout Standard
\begin_inset Float table
placement H
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Change Class, Example 1
\begin_inset CommandInset label
LatexCommand label
name "tab:Change-Type-Example1"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Tabular
<lyxtabular version="3" rows="4" columns="6">
<features tabularvalignment="middle">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<row>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
name_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
name_new
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
class_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
class_new
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
value_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
value_new
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x2
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x2.char
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
character
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
character
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
a|b|c|d|f
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Excellent|Proficient|Good|Fair|Poor
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x2
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x2.fac
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
character
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
factor
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
a|b|c|d|f
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Excellent|Proficient|Good|Fair|Poor
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x2
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x2.gpa
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
character
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
integer
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
a|b|c|d|f
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
4|3|2|1|0
\end_layout

\end_inset
</cell>
</row>
</lyxtabular>

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
In line one of Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Change-Type-Example1"

\end_inset

, the class 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
character
\end_layout

\end_inset

 remains the same.
 That line will produce a new character variable with embellished values.
 Line two demonstrates how to create an R factor variable, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
x2.fac
\end_layout

\end_inset

, and line three converts the character to an integer variable.
 Remember that it is important to match the value of 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

 with the content proposed for 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

.
 Do not include character values in a variable for which the new class will
 be numeric or integer.
 
\end_layout

\begin_layout Standard
Similarly, it is obvious to see how an integer input can be converted into
 either an integer, character, or factor variable by employing any of the
 rows seen in Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Change-Type-Example2"

\end_inset

.
\end_layout

\begin_layout Standard
\begin_inset Float table
placement H
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Change Class Example 2
\begin_inset CommandInset label
LatexCommand label
name "tab:Change-Type-Example2"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Tabular
<lyxtabular version="3" rows="4" columns="6">
<features tabularvalignment="middle">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<row>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
name_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
name_new
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
class_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
class_new
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
value_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
value_new
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x6
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x6.i100
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
integer
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
integer
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
1|2|3|4|5
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
100|200|300|400|500
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x6
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x6.c
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
integer
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
character
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
1|2|3|4|5
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Austin|Denver|Nashville|Provo|Miami
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x6
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x6.f
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
integer
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
factor
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
1|2|3|4|5
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
F|D|C|B|A
\end_layout

\end_inset
</cell>
</row>
</lyxtabular>

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
If a variable's 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_old
\end_layout

\end_inset

 is ordered, and we simply want to relabel the existing levels, the work
 is also easy (see Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Change-Type-Example3"

\end_inset

).
 The second row in Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Change-Type-Example3"

\end_inset

 shows that factor levels can be 
\begin_inset Quotes eld
\end_inset

combined
\begin_inset Quotes erd
\end_inset

 by assigning the same character string to several 
\begin_inset Quotes eld
\end_inset

<
\begin_inset Quotes erd
\end_inset

 separated values.
 
\end_layout

\begin_layout Standard
\begin_inset Float table
placement H
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Change Class, Example 3
\begin_inset CommandInset label
LatexCommand label
name "tab:Change-Type-Example3"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Tabular
<lyxtabular version="3" rows="4" columns="6">
<features tabularvalignment="middle">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<row>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
name_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
name_new
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
class_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
class_new
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
value_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
value_new
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x7
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x7.grades
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
ordered
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
ordered
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
f<d<c<b<a
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
F<D<C<B<A
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x7
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x7.passfail
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
ordered
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
ordered
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
f<d<c<b<a
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
Fail<Fail<Pass<Pass<Pass
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x7
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x7.gpa
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
ordered
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
integer
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
f<d<c<b<a
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
0|1|2|3|4
\end_layout

\end_inset
</cell>
</row>
</lyxtabular>

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
Working with ordered variables, whether as input or output, becomes more
 complicated if the existing data are not ordered in the way we want.
 In the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
mydf
\end_layout

\end_inset

 example, the variable 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
mydf$x3
\end_layout

\end_inset

 was coded as an ordered variable with levels (
\begin_inset Quotes eld
\end_inset

med
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset

lo
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset

high
\begin_inset Quotes erd
\end_inset

).
 That might have been one person's idea of a joke, so we need to rearrange
 these as (
\begin_inset Quotes eld
\end_inset

lo
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset

med
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset

high
\begin_inset Quotes erd
\end_inset

).
 If the original ordering of the values is not consistent with the desired
 ordering of the new ordered factor, then we need notation that allows researche
rs to achieve two purposes.
 First, the values must be re-leveled.
 Second, allow for the possibility that the new values must be relabeled
 as well.
 We'd rather not proliferate new columns in the variable key or create some
 confusing new notation.
\end_layout

\begin_layout Standard
Reordering variable levels requires us to do something that seems dangerous.
 We need to edit the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 to correct the ordering of the levels 
\emph on
as they are currently labeled
\emph default
.
 This is the only time where we suggest that users edit the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 column.
 In 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

, supply new labels in the in the correct order to parallel the newly edited
 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 column (see Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Reorder-Values"

\end_inset

).
\end_layout

\begin_layout Standard
\begin_inset Float table
placement H
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Reorder Values, Example 1
\begin_inset CommandInset label
LatexCommand label
name "tab:Reorder-Values"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Tabular
<lyxtabular version="3" rows="3" columns="6">
<features tabularvalignment="middle">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<row>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
name_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
name_new
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
class_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
class_new
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
value_old
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
value_new
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x3
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x3.lo2hi
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
ordered
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
ordered
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
low<med<hi
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
low<medium<high
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x3
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
x3.passfail
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
ordered
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
ordered
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
low<med<hi
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
low<pass<pass
\end_layout

\end_inset
</cell>
</row>
</lyxtabular>

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
The key importer will check that all 
\begin_inset Quotes eld
\end_inset

duplicated levels
\begin_inset Quotes erd
\end_inset

 are adjacent with one another, so that the values above low are grouped
 together.
\end_layout

\begin_layout Standard
In the long key format, the equivalent information would be conveyed by
 altering the ordering of the rows.
 For example, it is necessary to re-order the rows to indicate the lo is
 lower than med, and then for the new values we put in the desired names
 (see Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Reorder-Values-Example2"

\end_inset

).
\end_layout

\begin_layout Standard
\begin_inset Float table
placement H
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Reorder Values, Example 2
\begin_inset CommandInset label
LatexCommand label
name "tab:Reorder-Values-Example2"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
begin{tabular}{llllllll}
\end_layout

\begin_layout Plain Layout


\backslash
toprule
\end_layout

\begin_layout Plain Layout

name
\backslash
_old & name
\backslash
_new & class
\backslash
_old & class
\backslash
_new & value
\backslash
_old & value
\backslash
_new
\backslash

\backslash

\end_layout

\begin_layout Plain Layout


\backslash
midrule
\end_layout

\begin_layout Plain Layout

x3 & x3lo2hi & ordered & ordered & lo & low
\backslash

\backslash

\end_layout

\begin_layout Plain Layout

x3 & x3lo2hi & ordered & ordered & med & medium
\backslash

\backslash

\end_layout

\begin_layout Plain Layout

x3 & x3lo2hi & ordered & ordered & hi & high
\backslash

\backslash

\end_layout

\begin_layout Plain Layout

x3 & x3.passfail & ordered & ordered & low & low
\backslash

\backslash

\end_layout

\begin_layout Plain Layout

x3 & x3.passfail & ordered & ordered & medium & pass
\backslash

\backslash

\end_layout

\begin_layout Plain Layout

x3 & x3.passfail & ordered & ordered & high & pass
\backslash

\backslash

\end_layout

\begin_layout Plain Layout


\backslash
bottomrule
\end_layout

\begin_layout Plain Layout


\backslash
end{tabular}
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
What to do about missing values.
 Even within our small group, there is some disagreement about this.
 The SAS tradition would have us enter a period, 
\begin_inset Quotes eld
\end_inset

.
\begin_inset Quotes erd
\end_inset

, in value_new for a level that is to be treated as missing.
 Others are tempted to use special purpose character strings like 
\begin_inset Quotes eld
\end_inset

NA
\begin_inset Quotes erd
\end_inset

 or 
\begin_inset Quotes eld
\end_inset

N/A
\begin_inset Quotes erd
\end_inset

.
 As long as neither 
\begin_inset Quotes eld
\end_inset

NA
\begin_inset Quotes erd
\end_inset

 nor 
\begin_inset Quotes eld
\end_inset

N/A
\begin_inset Quotes erd
\end_inset

 are legal values, that seems safe.
 There is another school of thought to argue that if a user wants nothing,
 the clearest, safest thing to do in the key is to enter nothing at all.
 
\end_layout

\begin_layout Standard
As a result of this diversity, the keyImport function includes an argument
 named na.strings (again, an argument drawn from R core funcitons).
 The default setting has an 
\begin_inset Quotes eld
\end_inset

all of the above
\begin_inset Quotes erd
\end_inset

 flavor, treating any of the values that any of us think might be missing
 as NA when the key is later put to use.
 If one is fastidious in editing the key and represents all value_new that
 should be missing as a period (hooray for SAS!), then the value of na.strings
 can be tightened up in the obvious way.
 
\end_layout

\begin_layout Standard
It is also possible to delete a level from value_old and value_new in order
 to indicate that it should be treated as missing, but we do not recommend
 this approach.
 This destroys an element of 
\begin_inset Quotes eld
\end_inset

book keeping
\begin_inset Quotes erd
\end_inset

 because future users of the data, and the key, might like to know that
 some levels were obliterated.
\end_layout

\begin_layout Standard
The missing value cell in the key can be used with these enumerated variables.
 Usually, writing the values to be omitted is not as easy as putting the
 desired missing symbol in the value_new column, but either method should
 work.
\end_layout

\begin_layout Standard
In a long format key, there will be many repeated rows for each variable.
 The entries in the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
missings
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
recodes
\end_layout

\end_inset

 columns are harvested from the cells corresponding to each combination
 of name_old and name_new, it is not necessary to retype the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
missings
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
recodes
\end_layout

\end_inset

 entries for each element.
\end_layout

\begin_layout Standard
The work of assigning missing values is carried out by a 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
kutils
\end_layout

\end_inset

 function named 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
assignMissing
\end_layout

\end_inset

.
 The help page for that function has verbose commentary to explain the format
 of the cells that might work well in the missings column.
\end_layout

\begin_layout Standard
\begin_inset Note Note
status open

\begin_layout Plain Layout
TODO: fix this, describe the output key 
\begin_inset Quotes eld
\end_inset

mydf.key.csv
\begin_inset Quotes erd
\end_inset

 and get EXCEL discussion on how to add quoted elements inside strings.
 Need to write out what Excel does to a string like 
\begin_inset Quotes eld
\end_inset

cindy|bobby|
\begin_inset Quotes erd
\end_inset

NA
\begin_inset Quotes erd
\end_inset

|
\begin_inset Quotes erd
\end_inset

, figure out why the key currently exports one big string 
\begin_inset Quotes eld
\end_inset

bobby|cindy|marcia
\begin_inset Quotes erd
\end_inset

 when perhaps instead it should export 
\begin_inset Quotes eld
\end_inset


\begin_inset Quotes erd
\end_inset

bobby
\begin_inset Quotes erd
\end_inset

|
\begin_inset Quotes erd
\end_inset

cindy
\begin_inset Quotes erd
\end_inset

|
\begin_inset Quotes erd
\end_inset

marcia
\begin_inset Quotes erd
\end_inset

).
\end_layout

\end_inset


\end_layout

\begin_layout Subsection
About non-enumerated variables
\end_layout

\begin_layout Standard
In the process that creates the variable key template, we think differently
 about the enumerated variables–ones that have meaningfully discrete values
 that can be reassigned one-by-one–and numeric variables.
 Numeric variables, whether we literally mean integers like age or the number
 of pennies in a jar, or floating point numbers (weight, temperature, volume,
 etc) are different.
 If we guessed right with 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
max.levels
\end_layout

\end_inset

 when the key was created, those non-discrete variables are included as
 just one row, whether the key is wide or long.
 In those variables, the main elements of interest for these variables are
 the columns labeled 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
missings
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
recodes
\end_layout

\end_inset

.
 
\end_layout

\begin_layout Standard
The instructions for specifying missing values are detailed in the help
 page of the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
assignMissing
\end_layout

\end_inset

 function in 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
kutils
\end_layout

\end_inset

.
 If the class_old is 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
numeric
\end_layout

\end_inset

 or 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
integer
\end_layout

\end_inset

, there are only three types of statements allowed in the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
missings
\end_layout

\end_inset

 column.
 Legal values must must begin with the characters 
\begin_inset Quotes eld
\end_inset

<
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset

>
\begin_inset Quotes erd
\end_inset

, or 
\begin_inset Quotes eld
\end_inset

c
\begin_inset Quotes erd
\end_inset

.
 These are illustrated in Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Recode-Examples"

\end_inset

.
 The symbols 
\begin_inset Quotes eld
\end_inset

<=
\begin_inset Quotes erd
\end_inset

 and 
\begin_inset Quotes eld
\end_inset

>=
\begin_inset Quotes erd
\end_inset

 are accepted in the obvious way.
 
\end_layout

\begin_layout Standard
\begin_inset Float table
placement H
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Missings Examples
\begin_inset CommandInset label
LatexCommand label
name "tab:Recode-Examples"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Tabular
<lyxtabular version="3" rows="6" columns="3">
<features tabularvalignment="middle">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<column alignment="center" valignment="top">
<row>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
missings
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
interpretation: NA will be assigned to 
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" topline="true" bottomline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
example
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
> t
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
values greater than t
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
> 99
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
>= t
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
values greater than or equal to t 
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
>=99
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
<t
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
values less than t 
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
<0
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
<=t
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
values less than or equal to t 
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
<=0
\end_layout

\end_inset
</cell>
</row>
<row>
<cell alignment="center" valignment="top" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
c(a,b)
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" leftline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
values equal to or greater than a and less than or equal to b 
\end_layout

\end_inset
</cell>
<cell alignment="center" valignment="top" bottomline="true" leftline="true" rightline="true" usebox="none">
\begin_inset Text

\begin_layout Plain Layout
c(-999,-1)
\end_layout

\end_inset
</cell>
</row>
</lyxtabular>

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
The key specification for 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
recodes
\end_layout

\end_inset

 is discussed in the help page of the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
assignRecode
\end_layout

\end_inset

 function in 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
kutils
\end_layout

\end_inset

.
 The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
recodes
\end_layout

\end_inset

 column takes R code and applies it to the desired variable.
 For example, if one wanted to transform a variable by taking its square
 root, this could be done by providing 
\begin_inset Quotes eld
\end_inset

sqrt(x)
\begin_inset Quotes erd
\end_inset

 in the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
recodes
\end_layout

\end_inset

 column.
 Here 
\begin_inset Quotes eld
\end_inset

x
\begin_inset Quotes erd
\end_inset

 is simply a placeholder where the name of the variable indicated in the
 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
name_new
\end_layout

\end_inset

 column will be inserted when the variable key is applied.
 
\end_layout

\begin_layout Standard
\begin_inset Float table
placement H
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Recoding Integer and Numeric Variables
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

{
\backslash
footnotesize
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout


\backslash
begin{tabular}{lllllllll}
\end_layout

\begin_layout Plain Layout

  
\backslash
toprule
\end_layout

\begin_layout Plain Layout

name
\backslash
_old & name
\backslash
_new & class
\backslash
_old & class
\backslash
_new & value
\backslash
_old & value
\backslash
_new & missings & recodes 
\backslash

\backslash

\end_layout

\begin_layout Plain Layout

  
\backslash
midrule
\end_layout

\begin_layout Plain Layout

x5 & x5 & numeric & numeric &  &  & <0 & log(x) 
\backslash

\backslash

\end_layout

\begin_layout Plain Layout

x4 & x4 & numeric & numeric &  &  & c(-999) & abs(x) 
\backslash

\backslash

\end_layout

\begin_layout Plain Layout

x6 & x6 & integer & integer & 1$|$2$|$3$|$4$|$5 & 1$|$2$|$3$|$4$|$5 & c(-9)
 & 
\backslash

\backslash

\end_layout

\begin_layout Plain Layout

   
\backslash
bottomrule
\end_layout

\begin_layout Plain Layout


\backslash
end{tabular}
\end_layout

\begin_layout Plain Layout

}
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Subsection
Class conversions
\end_layout

\begin_layout Standard
The problem of recoding a variable, but leaving its class the same, is mostly
 solved.
 
\end_layout

\begin_layout Standard
The conversion of variables from one class to another requires special care.
 R's built in functions for coercion of variables from one type to another
 succeed meaningfully in many cases, but not all.
 The coercion of an integer to a floating point number produces understandable
 effects.
 The coercion of a character to an integer is not always understandable,
 and the conversion of factors to integers or numeric is, well, almost always
 a source of concern.
\end_layout

\begin_layout Standard
We are intending to support, at minimum, the 6 safe classes, which can be
 used in any combination of 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

.
 We need to be sure that conversion from each one into the other types is
 handled accurately.
 
\end_layout

\begin_layout Standard
In the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyApply
\end_layout

\end_inset

 function, we have implemented special purpose code to handle a class change
 from factor to numeric, for example, in a way that preserves the levels
 as the new values.
 This is needed when the input data has values like 10, 12, 14, and 
\begin_inset Quotes eld
\end_inset

unknown
\begin_inset Quotes erd
\end_inset

.
 If an R function like read.table is used to import a column with those values,
 the default behavior will convert that to a factor variable with values
 
\begin_inset Quotes eld
\end_inset

10
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset

12, 
\begin_inset Quotes eld
\end_inset

14
\begin_inset Quotes erd
\end_inset

 and 
\begin_inset Quotes eld
\end_inset

unknown
\begin_inset Quotes erd
\end_inset

.
 What the user probably wants–and what we do now–is open the possibility
 that 
\begin_inset Quotes eld
\end_inset

unknown
\begin_inset Quotes erd
\end_inset

 should be treated as NA and the values that appear to be numbers (but are
 actually text strings) are converted into integer format.
\end_layout

\begin_layout Subsection
Unanticipated 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

 values
\end_layout

\begin_layout Standard
By far, the most troublesome variable type is date and time information.
 This is difficult for a number of reasons.
 In order from less troublesome to more troublesome:
\end_layout

\begin_layout Enumerate
No two people seem to represent dates with the same style.
 Virtuous people (the authors) write 2016-12-01, while others write ambiguous
 strings like 12/01/2016, 01/12/2016 or 01-12-2016.
\end_layout

\begin_layout Enumerate
Computer programs absorb dates and convert them to internal numeric schemes.
 However, programs differ in the way they record and export date information.
 Most software converts dates as integers, for example.
 The value is the number of days since time began.
 
\end_layout

\begin_deeper
\begin_layout Standard
Unfortunately, it appears that the day on which time began differs among
 programs.
 Microsoft Excel records dates as integers, and (shockingly!) the origin
 date differs between Excel files created in Windows and Macintosh.
 Mac designers were perplexed by the problem of leap years, which caused
 them to use the origin 1904-01-01, while in MS Windows Excel dates begin
 at 1900-01-01.
 The program SPSS uses integers for dates as well, but the origin for SPSS
 is 1582-10-14 (we guess most readers already knew that one, we apologize
 for being surprised).
 
\end_layout

\begin_layout Standard
On the other hand, dates on Unix/Linux systems tend to follow the POSIX
 standard, so date information is interpreted as the number of days since
 1970-01-01.
 That is a local favorite because it reminds us of Nixon and Vietnam.
\end_layout

\end_deeper
\begin_layout Enumerate
Dates are, to a certain extent, arbitrary without time zone information.
 Its Tuesday, December 8 in Lawrence, Kansas, but it is Wednesday, December
 9 in Moscow, Russia.
 If time zones are not specified, than all date information is, well, approximat
e.
 Furthermore, when scholars try to put time zone information onto existing
 data that was collected without it, they sometimes accidentally create
 impossible dates and times (February 29, 1999, for example).
 When R functions like 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
as.Date
\end_layout

\end_inset

 encounter impossible dates, they return NA.
\end_layout

\begin_layout Enumerate
Even when programs seem to have absorbed dates and saved the information,
 even in casual testing we have found that the saved information is not
 retrieved in a consistent way.
 Some spreadsheet programs, on some computers, return values dates as integers,
 while the same program with a different user returns a character string.
 
\end_layout

\begin_layout Standard
All of this complaining is just our way of saying, 
\begin_inset Quotes eld
\end_inset

we left the recode box open, why not use it to recode your date information?
\begin_inset Quotes erd
\end_inset

 If the variable key template says that a column is 
\begin_inset Quotes eld
\end_inset

integer,
\begin_inset Quotes erd
\end_inset

 but you expected a date, then all is likely to be fine if we set the origin.
 One needs to find out what the origin should be and tell R about it.
 The right recode statement is likely to be 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
as.Date(x, origin = "1970-01-01")
\end_layout

\end_inset

.
 On the other hand, if the date information is in text format, with values
 like 
\begin_inset Quotes eld
\end_inset

2016-11-19
\begin_inset Quotes erd
\end_inset

, then it is necessary to learn the shorthand notation for date symbols.
 The right recode will be something like 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
as.Date(x, format = "%Y-%m-%d")
\end_layout

\end_inset

.
\end_layout

\begin_layout Standard
If one sets 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

 = 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
Date
\end_layout

\end_inset

, the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyApply
\end_layout

\end_inset

 function will succeed if the return value from the recode function is an
 R Date variable.
 Otherwise, an error will be raised.
\end_layout

\begin_layout Subsection
Where more work should be done
\end_layout

\begin_layout Standard
For the most part, our work now lies in development of better diagnostics
 and error-checking routines.
 
\end_layout

\begin_layout Standard
First, we need better filtering of user-created key entries.
 User typographical errors occur in key preparation and more comprehensive
 filtering should be done.
 At the current time, the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyImport
\end_layout

\end_inset

 function does not include many subroutines for the validation of the key.
 We'd like to filter the values requested in 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

, for example, but it is difficult to see how that can be done.
 We want to leave open the possibility that users may specify values of
 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

 that are unfamiliar to us.
 Another problem may arise if the number of elements in 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_old
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

 are not aligned one-to-one.
 There will be an error when the key is put to use by 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyApply
\end_layout

\end_inset

, but we should have a stop message in 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
keyImport
\end_layout

\end_inset

.
\end_layout

\begin_layout Standard
Second, we need to better understand the problems that result when users
 enter variable keys in programs like MS Excel or Libre Office.
 Between programs, or versions of programs, or programs on various platforms,
 we notice disappointing inconsistencies.
 If keys are kept in 
\begin_inset Quotes eld
\end_inset

flat text
\begin_inset Quotes erd
\end_inset

 csv files, there is less danger, but there can still be some trouble because
 spread sheets change the way they export to csv from time to time.
 During the testing process, we have found that version updates Excel cause
 unexpected changes in the way character vectors and dates are stored.
 Until now, we have relied on the very popular package 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
openxlsx
\end_layout

\end_inset

 (
\begin_inset CommandInset citation
LatexCommand citealp
key "Walker2015"
literal "true"

\end_inset

).
 This does well most of the time, but we still find some inexplicable variations
 in imported Excel files.
 Sometimes, empty cells in the variable key appear as imported empty strings
 
\begin_inset Quotes eld
\end_inset


\begin_inset Quotes erd
\end_inset

, while sometimes they are imported as R NA symbols.
 While it is probably safe to treat an entirely empty value as a missing
 value in most projects, we'd rather not do so.
 To deal with this problem, we need to develop a standard framework for
 testing the quality of imported variable keys.
 
\end_layout

\begin_layout Standard
Third, we need more comprehensive error-checking for the accuracy of the
 imported data and compliance with the variable key.
 The variable key system works well with the data sets for which it was
 created, but we need more formal criteria for verifying that claim.
 
\end_layout

\begin_layout Section
Discussion
\end_layout

\begin_layout Standard
When a project has a small budget, we invite the principal investigator
 to economize on the expenses by filling out the variable key's 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
name_new
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
class_new
\end_layout

\end_inset

, and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
value_new
\end_layout

\end_inset

 columns.
 There are several benefits in inviting the clients (or PIs) to be directly
 involved in filling in the variable key.
 Most importantly, they are allowed to name the variables in any way that
 is meaningful to them.
 When statistical results are obtained, it is never necessary for them to
 ask, 
\begin_inset Quotes eld
\end_inset

what did you mean by this variable 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
occupation
\end_layout

\end_inset

?
\begin_inset Quotes erd
\end_inset

 There are other benefits, however.
 By making the principal investigator aware of the values that are actually
 observed, and by offering the opportunity to specify how they ought to
 be recoded, a substantial element of administrative slippage is ameliorated.
 The variable key will specify exactly how categories are to be re-mapped,
 there is much less danger of an accident buried in thousands of lines of
 recodes.
\end_layout

\begin_layout Standard
It often happens that the raw data to be imported are provided by one of
 the national data centers.
 The variables are given exciting, meaningful column names like V34342a.
 It appears to be almost certain that research assistants will conclude
 that these names are not meaningful, so they create names that are more
 meaningful to them, such as 
\emph on
gender
\emph default
, 
\emph on
sex
\emph default
, 
\emph on
male
\emph default
, 
\emph on
female
\emph default
, or whatnot.
 The research assistants disappear into a haze of code and come out talking
 about the effect of income, gender, and education on educational achievement,
 and the principal investigator has to say, 
\begin_inset Quotes eld
\end_inset

which of those variables is income, again?
\begin_inset Quotes erd
\end_inset

 and 
\begin_inset Quotes eld
\end_inset

what's the coding on education?
\begin_inset Quotes erd
\end_inset

 A very exciting conversation then follows as one of the research assistant
 realizes that V34342b is the one that should have been used for gender,
 while V34342a indicates if the respondent ever visited Eastern Europe.
\end_layout

\begin_layout Standard
The variable key is intended to create neutral territory between principal
 investigators and project supervisors on one side and the research assistants
 on the other.
 The numbers are more likely to come out correctly if the names, values,
 and classes can be specified in one document that is accessible in numerous
 formats on which many eyes can be laid.
 The distance between a re-design of the imported values is minimized and
 the production cycle of the projects is accelerated.
 As we are frequently reminded, there are many some bright people who don't
 know R, it appears they are all fluent in spread sheet.
\end_layout

\begin_layout Standard
\begin_inset Note Note
status open

\begin_layout Plain Layout
\begin_inset Box Frameless
position "t"
hor_pos "c"
has_inner_box 1
inner_pos "t"
use_parbox 0
use_makebox 0
width "100col%"
special "none"
height "1in"
height_special "totalheight"
thickness "0.4pt"
separation "3pt"
shadowsize "4pt"
framecolor "black"
backgroundcolor "none"
status open

\begin_layout Plain Layout
Variable Key Editing Guidelines
\end_layout

\begin_layout Enumerate
Omit a variable: delete its row(s) from the (long)key.
\end_layout

\begin_layout Enumerate
The 
\begin_inset Quotes eld
\end_inset

class_new
\begin_inset Quotes erd
\end_inset

 value
\end_layout

\begin_deeper
\begin_layout Enumerate
discrete variable types:
\begin_inset Separator latexpar
\end_inset


\end_layout

\begin_deeper
\begin_layout Description
logical
\end_layout

\begin_layout Description
integer
\end_layout

\begin_layout Description
factor
\end_layout

\begin_layout Description
ordered
\end_layout

\begin_layout Description
logical
\end_layout

\begin_layout Description
character
\end_layout

\begin_layout Description
Date
\end_layout

\end_deeper
\begin_layout Enumerate
floating-point numeric variable types:
\begin_inset Separator latexpar
\end_inset


\end_layout

\begin_deeper
\begin_layout Description
numeric
\end_layout

\end_deeper
\end_deeper
\begin_layout Enumerate
Creating new variables.
 Each input variable may be recoded in several different ways, assigning
 a name_new value unique for each.
 
\end_layout

\begin_layout Enumerate
Assigning new values.
 
\begin_inset Separator latexpar
\end_inset


\end_layout

\begin_deeper
\begin_layout Enumerate
discrete variable types.
\begin_inset Separator latexpar
\end_inset


\end_layout

\begin_deeper
\begin_layout Enumerate
New 
\begin_inset Quotes eld
\end_inset

value_old
\begin_inset Quotes erd
\end_inset

 elements can be inserted in anticipation of new data sets with previously
 unobserved values.
\end_layout

\begin_layout Enumerate
To assign new values to any of the discrete variable types, set values in
 
\begin_inset Quotes eld
\end_inset

value_new
\begin_inset Quotes erd
\end_inset

.
 It is important that the values can be coerced by R to match the value
 of 
\begin_inset Quotes eld
\end_inset

class_new
\begin_inset Quotes erd
\end_inset

.
 That is to say, if 
\begin_inset Quotes eld
\end_inset

class_new
\begin_inset Quotes erd
\end_inset

 is a 
\begin_inset Quotes eld
\end_inset

integer
\begin_inset Quotes erd
\end_inset

, then 
\begin_inset Quotes eld
\end_inset

value_new
\begin_inset Quotes erd
\end_inset

 should be an integer.
\end_layout

\end_deeper
\begin_layout Enumerate
numeric variable types.
 
\begin_inset Separator latexpar
\end_inset


\end_layout

\begin_deeper
\begin_layout Enumerate
Because floating point numbers do not allow comparison with 
\begin_inset Quotes eld
\end_inset

==
\begin_inset Quotes erd
\end_inset

, it is not allowed to use 
\begin_inset Quotes eld
\end_inset

value_old
\begin_inset Quotes erd
\end_inset

 and 
\begin_inset Quotes eld
\end_inset

value_new
\begin_inset Quotes erd
\end_inset

 pairs for these variables.
 It is necessary to write statements in the recode column of the variable
 key.
\end_layout

\end_deeper
\end_deeper
\begin_layout Enumerate
Assign missings:
\begin_inset Separator latexpar
\end_inset


\end_layout

\begin_deeper
\begin_layout Enumerate
discrete variables: several options, we should settle on one suggestion
\begin_inset Separator latexpar
\end_inset


\end_layout

\begin_deeper
\begin_layout Enumerate
insert 
\begin_inset Quotes eld
\end_inset

NA
\begin_inset Quotes erd
\end_inset

 in value_new column
\end_layout

\begin_layout Enumerate
delete the value_old entry (not the best way, maybe don't mention it)
\end_layout

\begin_layout Enumerate
list a vector of missing values in the recode column.
 This vector applies to data as it was imported, before applying the value_old
 to value_new transition.
 PROBLEM: if input data's class does not match class_old, there may be a
 problem.
\end_layout

\end_deeper
\begin_layout Enumerate
numeric variables
\end_layout

\end_deeper
\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset CommandInset bibtex
LatexCommand bibtex
bibfiles "kutils"
options "chicago"

\end_inset


\end_layout

\end_body
\end_document

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{deduper}
\alias{deduper}
\title{Removes redundant words from beginnings of character strings}
\usage{
deduper(x, sep = ",_\\\\s-", n = NULL)
}
\arguments{
\item{x}{Character vector}

\item{sep}{Delimiter. A regular expression indicating the point at
which to split the strings before checking for
duplicates. Default will look for repeat separated by comma,
underscore, or one space character.}

\item{n}{Limit on number of duplicates to remove. Default, NULL,
means delete all duplicates at the beginning of a string.}
}
\value{
Cleaned up vector.
}
\description{
In Qualtrix data, we sometimes find repeated words in column
names. For whatever reason, the variable names have repeated words
like "Philadelphia_Philadelphia_3".  This function changes a
vector c("Philadelphia_Philadelphia_3", "Denver_Denver_4") to
c("Philadelphia_3", "Denver_4"). It is non destructive, so that
other values will not be altered.
}
\details{
See \url{https://stackoverflow.com/questions/43711240/r-regular-expression-match-omit-several-repeats}
}
\examples{
x <- c("Philadelphia_Philadelphia_3", "Denver_Denver_4",
        "Den_Den_Den_Den_Den_Den_Den_5")
deduper(x)
deduper(x, n = 2)
deduper(x, n = 3)
deduper(x, n = 4)
x <- c("Philadelphia,Philadelphia_3", "Denver Denver_4")
## Shows comma also detected by default
deduper(x)
## Works even if delimiter is inside matched string,
## or separators vary
 x <- c("Den_5_Den_5_Den_5,Den_5 Den_5")
deduper(x)
## generate vector
x <- replicate(10, paste(sample(letters, 5), collapse = ""))
n <- c(paste0("_", sample(1:10, 5)), rep("", 5))
x <- paste0(x, "_", x, n, n)
x
deduper(x)

}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{wide2long}
\alias{wide2long}
\title{Convert a key object from wide to long format}
\usage{
wide2long(key, sep = c(character = "\\\\|", logical = "\\\\|", integer =
  "\\\\|", factor = "\\\\|", ordered = "[\\\\|<]", numeric = "\\\\|"))
}
\arguments{
\item{key}{A variable key in the wide format}

\item{sep}{Default separator is the pipe, "\\|" for most
variables, while ordered accepts pipe or less than, "\\|<". If
the key did not follow those customs, other sep values may be
specified for each variable class.}
}
\value{
A long format variable key
}
\description{
This is not flexible, assumes columns are named in our canonical
style, which means the columns are named c("name_old", "name_new",
"class_old", "class_new", "value_old", "value_new").
}
\examples{
mydf.path <- system.file("extdata", "mydf.csv", package = "kutils")
mydf <- read.csv(mydf.path, stringsAsFactors=FALSE)
## Target we are trying to match:
mydf.keylong <- keyTemplate(mydf, long = TRUE, sort = FALSE)

mydf.key <- keyTemplate(mydf)
mydf.keywide2long <- wide2long(mydf.key)

## rownames not meaningful in long key, so remove in both versions
row.names(mydf.keywide2long) <- NULL
row.names(mydf.keylong) <- NULL
all.equal(mydf.keylong, mydf.keywide2long)
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/factors.R
\name{reverse}
\alias{reverse}
\title{Reverse the levels in a factor}
\usage{
reverse(x, eol = c("Skip", "DNP"))
}
\arguments{
\item{x}{a factor variable}

\item{eol}{values to be kept at the end of the list. Does not
accept regular expresssions, just literal text strings
representing values.}
}
\value{
a new factor variable with reversed values
}
\description{
Simple literal reversal. Will stop with an error message if x is
not a factor (or ordered) variable.
}
\details{
Sometimes people want to
reverse some levels, excluding others and leaving them at the end
of the list. The "eol" argument sets aside some levels and puts
them at the end of the list of levels.

The use case for the \code{eol} argument is a factor
with several missing value labels, as appears in SPSS. With
up to 18 different missing codes, we want to leave them
at the end. In the case for which this was designed, the
researcher did not want to designate those values as
missing before inspecting the pattern of observed values.
}
\examples{
## Consider alphabetication of upper and lower
x <- factor(c("a", "b", "c", "C", "a", "c"))
levels(x)
xr1 <- reverse(x)
xr1
## Keep "C" at end of list, after reverse others
xr2 <- reverse(x, eol = "C")
xr2
y <- ordered(x, levels = c("a", "b", "c", "C"))
yr1 <- reverse(y)
class(yr1)[1] == "ordered"
yr1
## Hmm. end of list amounts to being "maximal".
## Unintended side-effect, but interesting.
yr2 <- reverse(y, eol = "C")
yr2
## What about a period as a value (SAS missing)
z <- factor(c("a", "b", "c", "b", "c", "."))
reverse(z)
z <- factor(c(".", "a", "b", "c", "b", "c", "."))
reverse(z)
## How about R NA's
z <- factor(c(".", "a", NA, "b", "c", "b", NA, "c", "."))
z
reverse(z)
z <- ordered(c(".", "a", NA, "b", "c", "b", NA, "c", "."))
z
str(z)
## Put "." at end of list
zr <- reverse(z, eol = ".")
zr
str(zr)
z <- ordered(c(".", "c", NA, "e", "a", "c", NA, "e", "."),
         levels = c(".", "c", "e", "a"))
reverse(z, eol = ".")
reverse(z, eol = c("a", "."))
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{wide2long}
\alias{wide2long}
\title{Convert a key object from wide to long format}
\usage{
wide2long(
  key,
  sep = c(character = "\\\\|", logical = "\\\\|", integer = "\\\\|", factor = "\\\\|",
    ordered = "[\\\\|<]", numeric = "\\\\|")
)
}
\arguments{
\item{key}{A variable key in the wide format}

\item{sep}{Default separator is the pipe, "\\|" for most
variables, while ordered accepts pipe or less than, "\\|<". If
the key did not follow those customs, other sep values may be
specified for each variable class.}
}
\value{
A long format variable key
}
\description{
This is not flexible, assumes columns are named in our canonical
style, which means the columns are named c("name_old", "name_new",
"class_old", "class_new", "value_old", "value_new").
}
\examples{
mydf.path <- system.file("extdata", "mydf.csv", package = "kutils")
mydf <- read.csv(mydf.path, stringsAsFactors=FALSE)
## Target we are trying to match:
mydf.keylong <- keyTemplate(mydf, long = TRUE, sort = FALSE)

mydf.key <- keyTemplate(mydf)
mydf.keywide2long <- wide2long(mydf.key)

## rownames not meaningful in long key, so remove in both versions
row.names(mydf.keywide2long) <- NULL
row.names(mydf.keylong) <- NULL
all.equal(mydf.keylong, mydf.keywide2long)
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{assignRecode}
\alias{assignRecode}
\title{A variable is transformed in an indicated way}
\usage{
assignRecode(x, recode = NULL)
}
\arguments{
\item{x}{A column to be recoded}

\item{recode}{A character string using placeholder "x". See
examples}
}
\value{
A new column
}
\description{
In the variable key framework, the user might request
transformations such as the logarithm, exponential, or square
root. This is done by including strings in the recodes column,
such as "log(x + 1)" or "3 + 1.1 * x + 0.5 * x ^ 2". This
function implements the user's request by parsing the character
string and applying the indicated re-calculation.
}
\details{
In the variable key framework, this is applied to the raw data,
after missings are imposed.
}
\examples{
set.seed(234234)
x <- rpois(100, lambda = 3)
x <- x[order(x)]
str1 <- "log(x + 1)"
xlog <- assignRecode(x, recode = str1)
plot(xlog ~ x, type = "l")
mean(xlog, na.rm = TRUE)
str2 <- "x^2"
xsq <- assignRecode(x, recode = str2)
plot(xsq ~ x, type = "l")
str3 <- "sqrt(x)"
xsrt <- assignRecode(x, recode = str3)
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{starsig}
\alias{starsig}
\title{How many stars would we need for this p value?}
\usage{
starsig(pval, alpha = c(0.05, 0.01, 0.001), symbols = c("*", "**",
  "***"))
}
\arguments{
\item{pval}{P value}

\item{alpha}{alpha vector, defaults as c(0.05, 0.01, 0.001).}

\item{symbols}{The default is c("*", "**", "***"), corresponding
to mean that p values smaller than 0.05 receive one star,
values smaller than 0.01 get two stars, and so forth.  Must be
same number of elements as alpha. These need not be asterixes,
could be any character strings that users desire. See example.}
}
\value{
a character vector of symbols (eg asterixes), same length as pval
}
\description{
Regression table makers need to know how many stars
to attach to parameter estimates. This takes
p values and a vector which indicates how many stars
are deserved.  It returns a required number of asterixes.
Was named "stars" in previous version, but renamed due to
conflict with R base function \code{stars}
}
\details{
Recently, we have requests for different symbols. Some people want
a "+" symbol if the p value is smaller than 0.10 but greater than
0.05, while some want tiny smiley faces if p is smaller than
0.001. We accomodate that by allowing a user specified vector of
symbols, which defaults to c("*", "**", "***")
}
\examples{
starsig(0.06)
starsig(0.021)
starsig(0.001)
alpha.ex <- c(0.10, 0.05, 0.01, 0.001)
symb.ex <- c("+", "*", "**", ":)!")
starsig(0.07, alpha = alpha.ex, symbols = symb.ex)
starsig(0.04, alpha = alpha.ex, symbols = symb.ex)
starsig(0.009, alpha = alpha.ex, symbols = symb.ex)
starsig(0.0009, alpha = alpha.ex, symbols = symb.ex)

}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/likert.R
\name{print.likert}
\alias{print.likert}
\title{print method for likert tables}
\usage{
\method{print}{likert}(x, ...)
}
\arguments{
\item{x}{likert object, 1st item will be printed}

\item{...}{Arguments passed to print method}
}
\value{
Nothing
}
\description{
Nothing fancy here. \code{cat} is called on first item in list
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{starsig}
\alias{starsig}
\title{How many stars would we need for this p value?}
\usage{
starsig(pval, alpha = c(0.05, 0.01, 0.001), symbols = c("*", "**", "***"))
}
\arguments{
\item{pval}{P value}

\item{alpha}{alpha vector, defaults as c(0.05, 0.01, 0.001).}

\item{symbols}{The default is c("*", "**", "***"), corresponding
to mean that p values smaller than 0.05 receive one star,
values smaller than 0.01 get two stars, and so forth.  Must be
same number of elements as alpha. These need not be asterixes,
could be any character strings that users desire. See example.}
}
\value{
a character vector of symbols (eg asterixes), same length as pval
}
\description{
Regression table makers need to know how many stars
to attach to parameter estimates. This takes
p values and a vector which indicates how many stars
are deserved.  It returns a required number of asterixes.
Was named "stars" in previous version, but renamed due to
conflict with R base function \code{stars}
}
\details{
Recently, we have requests for different symbols. Some people want
a "+" symbol if the p value is smaller than 0.10 but greater than
0.05, while some want tiny smiley faces if p is smaller than
0.001. We accomodate that by allowing a user specified vector of
symbols, which defaults to c("*", "**", "***")
}
\examples{
starsig(0.06)
starsig(0.021)
starsig(0.001)
alpha.ex <- c(0.10, 0.05, 0.01, 0.001)
symb.ex <- c("+", "*", "**", ":)!")
starsig(0.07, alpha = alpha.ex, symbols = symb.ex)
starsig(0.04, alpha = alpha.ex, symbols = symb.ex)
starsig(0.009, alpha = alpha.ex, symbols = symb.ex)
starsig(0.0009, alpha = alpha.ex, symbols = symb.ex)

}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{writeCSV}
\alias{writeCSV}
\title{Write CSV files with quotes same as MS Excel 2013 or newer}
\usage{
writeCSV(x, file, row.names = FALSE)
}
\arguments{
\item{x}{a data frame}

\item{file}{character string for file name}

\item{row.names}{Default FALSE for row.names}
}
\value{
the return from write.table, using revised quotes
}
\description{
R's write.csv inserts quotes around all elements in a character
vector (if quote = TRUE).  In contrast, MS Excel CSV export no
longer inserts quotation marks on all elements in character
variables, except when the cells include commas or quotation
marks.  This function generates CSV files that are, so far as we
know, exactly the same "quoted style" as MS Excel CSV export
files.
}
\details{
This works by manually inserting quotation marks where necessary and
turning FALSE R's own method to insert quotation marks.
}
\examples{
set.seed(234)
x1 <- data.frame(x1 = c("a", "b,c", "b", "The \\"Washington, DC\\""),
      x2 = rnorm(4), stringsAsFactors = FALSE)
x1
fn <- tempfile(pattern = "testcsv", fileext = ".csv")
writeCSV(x1, file = fn)
readLines(fn)
x2 <- read.table(fn, sep = ",", header = TRUE, stringsAsFactors = FALSE)
all.equal(x1,x2)
}
\author{
Paul Johnson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelcomparison.R
\name{print.kutable}
\alias{print.kutable}
\title{A print method for kutable objects}
\usage{
\method{print}{kutable}(x, ...)
}
\arguments{
\item{x}{object to be printed}

\item{...}{optional arguments, corrently ignored}
}
\value{
x unchanged
}
\description{
A print method for kutable objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keyUpdate}
\alias{keyUpdate}
\title{Update a key in light of a new data frame (add variables and
values)}
\usage{
keyUpdate(key, dframe, append = TRUE, safeNumericToInteger = TRUE)
}
\arguments{
\item{key}{A variable key}

\item{dframe}{A data.frame object.}

\item{append}{If long key, should new rows be added to the end of
the updated key? Default is TRUE. If FALSE, new rows will be
sorted with the original values.}

\item{safeNumericToInteger}{Default TRUE: Should we treat
    variables which appear to be integers as integers? In many csv
    data sets, the values coded \code{c(1, 2, 3)} are really
    integers, not floats \code{c(1.0, 2.0, 3.0)}. See
    \code{safeInteger}.
## Need to consider implementing this:
## @param ignoreCase}
}
\value{
Updated variable key.
}
\description{
The following chores must be handled.
1. If the data.frame has variables which are not currently
listed in the variable key's "name_old" variable, then new
variables are added to the key.
2. If the data.frame has new values for the previously
existing variables, then those values must be added to the
keys.
3. If the old key has "name_new" or "class_new" designated
for variables, those MUST be preserved in the new key
for all new values of those variables.
}
\details{
This function will not alter key values for "class_old",
"value_old" or "value_new" for variables that have no new
information.

This function deduces if the key provided is in the wide or long
format from the class of the object.
}
\examples{
## Original data frame has 2 variables
dat1 <- data.frame("Score" = c(1, 2, 3, 42, 4, 2),
                   "Gender" = c("M", "M", "M", "F", "F", "F"))
## New data has all of original dat1, plus a new variable "Weight"
#and has new values for "Gender" and "Score"
dat2 <- plyr::rbind.fill(dat1, data.frame("Score" = 7,
           "Gender" = "other", "Weight" = rnorm(3)))
## Create a long key for the original data, specify some
## recodes for Score and Gender in value_new
key1.long <- keyTemplate(dat1, long = TRUE, varlab = TRUE)

key1.long$value_new <- gsub("42", "10", key1.long$value_new)
key1.long$value_new[key1.long$name_new == "Gender"] <-
       mgsub(c("F", "M"), c("female", "male"),
       key1.long$value_new[key1.long$name_new == "Gender"])  
key1.long[key1.long$name_old == "Score", "name_new"] <- "NewScore"
keyUpdate(key1.long, dat2, append = TRUE)
## Throw away one row, make sure key still has Score values
dat2 <- dat2[-1,]
(key1.long.u <- keyUpdate(key1.long, dat2, append = FALSE))
## Key change Score to character variable
key1.longc <- key1.long
key1.longc[key1.longc$name_old == "Score", "class_new"] <- "character"
keyUpdate(key1.longc, dat2, append = TRUE)
str(dat3 <- keyApply(dat2, key1.longc))
## Now try a wide key
key1.wide <- keyTemplate(dat1)
## Put in new values, same as in key1.long
key1.wide[key1.wide$name_old == "Score", c("name_new", "value_new")] <-
                            c("NewScore", "1|2|3|4|10|.")
key1.wide[key1.wide$name_old == "Gender", "value_new"] <- "female|male|."
## Make sure key1.wide equivalent to key1.long:
## If this is not true, it is a fail
all.equal(long2wide(key1.long), key1.wide, check.attributes = FALSE)
(key1.wide.u <- keyUpdate(key1.wide, dat2))
key1.long.to.wide <- long2wide(key1.long.u)
all.equal(key1.long.to.wide, key1.wide.u, check.attributes = FALSE)
str(keyApply(dat2, key1.wide.u))

mydf.key.path <- system.file("extdata", "mydf.key.csv", package = "kutils")
mydf.key <-  keyImport(mydf.key.path)
##'
set.seed(112233)
N <- 20
## The new Jan data arrived!
mydf2 <- data.frame(x5 = rnorm(N),
                    x4 = rpois(N, lambda = 3),
                    x3 = ordered(sample(c("lo", "med", "hi"),
                                       size = N, replace=TRUE),
                                levels = c("med", "lo", "hi")),
                    x2 = letters[sample(c(1:4,6), N, replace = TRUE)],
                    x1 = factor(sample(c("jan"), N, replace = TRUE)),
                    x7 = ordered(letters[sample(c(1:4,6), N, replace = TRUE)]),
                    x6 = sample(c(1:5), N, replace = TRUE),
                    stringsAsFactors = FALSE)
mydf.key2 <- keyUpdate(mydf.key, mydf2)
mydf.key2
mydf.key2["x1", "value_old"] <- "cindy|bobby|jan|peter|marcia|greg|."
mydf.key2["x1", "value_new"] <- "Cindy<Bobby<Jan<Peter<Marcia<Greg<."
##'
mydf.key.path <- system.file("extdata", "mydf.key.csv", package = "kutils")
mydf.path <- system.file("extdata", "mydf.csv", package = "kutils")
mydf <- read.csv(mydf.path, stringsAsFactors=FALSE)
mydf3 <- rbind(mydf, mydf2)
## Now recode with revised key
mydf4 <- keyApply(mydf3, mydf.key2)
rockchalk::summarize(mydf4)
}
\author{
Ben Kite <bakite@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/semTable.R
\name{markupConvert}
\alias{markupConvert}
\title{Convert marked-up semTable structures to latex, html, or csv}
\usage{
markupConvert(marked, type = c("latex", "html", "csv"),
  table.float = FALSE, longtable = FALSE, caption = NULL,
  label = NULL, file = NULL, columns, centering = "siunitx")
}
\arguments{
\item{marked}{A character string}

\item{type}{Output type, latex", "html", or "csv".}

\item{table.float}{TRUE if you want insertion of '\\begin{table}'}

\item{longtable}{should a tabular or a longtable object be
created?}

\item{caption}{A caption to use if either longtable or table is
TRUE}

\item{label}{A LaTeX label for cross-references}

\item{file}{A file stub, to which ".tex", ".html", or ".csv" can
be added}

\item{columns}{For SEM table, the list of columns objects}

\item{centering}{Default "siunitx". Specify "none" to return to
behavior of semTable before 1.50.}
}
\value{
a list of marked up character objects
}
\description{
The conversion key tables are included in the code of the function.
}
\details{
The semTable uses a customized markup framework that uses
character sequences that begin and end with underscores, such as
"_BOMC2_ for "begin of multi-column entity that will use 2
columns". These special markups need to be converted into "tex", "html", or "csv" formats.
}
\author{
Paul Johnson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{writeCSV}
\alias{writeCSV}
\title{Write CSV files with quotes same as MS Excel 2013 or newer}
\usage{
writeCSV(x, file, row.names = FALSE)
}
\arguments{
\item{x}{a data frame}

\item{file}{character string for file name}

\item{row.names}{Default FALSE for row.names}
}
\value{
the return from write.table, using revised quotes
}
\description{
R's write.csv inserts quotes around all elements in a character
vector (if quote = TRUE).  In contrast, MS Excel CSV export no
longer inserts quotation marks on all elements in character
variables, except when the cells include commas or quotation
marks.  This function generates CSV files that are, so far as we
know, exactly the same "quoted style" as MS Excel CSV export
files.
}
\details{
This works by manually inserting quotation marks where necessary and
turning FALSE R's own method to insert quotation marks.
}
\examples{
set.seed(234)
x1 <- data.frame(x1 = c("a", "b,c", "b", "The \"Washington, DC\""),
      x2 = rnorm(4), stringsAsFactors = FALSE)
x1
fn <- tempfile(pattern = "testcsv", fileext = ".csv")
writeCSV(x1, file = fn)
readLines(fn)
x2 <- read.table(fn, sep = ",", header = TRUE, stringsAsFactors = FALSE)
all.equal(x1,x2)
}
\author{
Paul Johnson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keysPool}
\alias{keysPool}
\title{Homogenize class values and create a long key by pooling variable
keys.}
\usage{
keysPool(keylong = NULL, keysplit = NULL, classes = list(c("logical",
  "integer"), c("integer", "numeric"), c("ordered", "factor"), c("factor",
  "character")), colnames = c("class_old", "class_new"),
  textre = "TEXT$")
}
\arguments{
\item{keylong}{A list of long keys, or just one long key,
presumably a result of rbinding several long keys.}

\item{keysplit}{Not often needed for user-level code. A list of
key blocks, each of which is to be inspected and
homogenized. Not used if a keylong argument is provided.}

\item{classes}{A list of vectors specifying legal promotions.}

\item{colnames}{Either c("class_old","class_new), ""class_old", or
"class_new".  The former is the default.}

\item{textre}{A regular expression matching a column name to be
treated as character. Default matches any variable name ending
in "TEXT"}
}
\value{
A class-corrected version of the same format as the input,
    either a long key or a list of key elements.
}
\description{
For long-format keys, this is one way to correct for errors in
"class_old" or "class_new" for common variables. For a long key
created by stacking together several long keys, or for a list of
long keys, this will try to homogenize the classes by using a
"highest common denominator" approach.  If one key has x1 as a
floating point, but another block of rows in the key has x1 as
integer, then class must be changed to floating point
(numeric). If another section of a key has x1 as a character, then
character becomes the class.
}
\details{
Users might run keyTemplate on several data sets, arriving
at keys that need to be combined.  The long versions of the
keys can be stacked together by a function like \code{rbind}.
If the values class_old and class_new for a single variable are
inconsistent, then the "key stack" will fail the tests in keyCheck.
This function automates the process of fixing the class variables by
"promoting" classes where possible.

Begin with a simple example.  In one data set, the value of x is
drawn from integers 1L, 2L, 3L, while in another set it is
floating values like 1.1, 2.2. After creating long format keys,
and stacking them together, the values of class_old will clash.
For x, we will observe both "integer" and "numeric" in the
class_old column.  In that situation, the class_old for all of the
rows under consideration should be set as "numeric".

The promotion schemes are described by the variable classes, where
we have the most conservative changes first. The most destructive
change is when variables are converted from integer to character,
for example. The conservative conversion strategies are specified
in the classes variable, in which the last element in a vector
will be used to replace the preceeding classes.  For example,
c("ordered", "factor", "character") means that the class_old
values of "ordered" and "factor" will be replaced by "character".

The conversions specified by classes are tried, in order.
1. logical -> integer
2. integer -> numeric
3. ordered -> factor

If their application fails to homogenize a vector, then class is
changed to "character". For example, when the value of class_old
observed is c("ordered", "numeric", "character"). In that case,
the class is promoted to "character", it is the least common
denominator.
}
\examples{
dat1 <- data.frame(x1 = as.integer(rnorm(100)), x2 = sample(c("Apple", "Orange"),
                   100, replace = TRUE), x3 = ifelse(rnorm(100) < 0, TRUE, FALSE))
dat2 <- data.frame(x1 = rnorm(100), x2 = ordered(sample(c("Apple", "Orange"),
                   100, replace = TRUE)), x3 = rbinom(100, 1, .5),
                   stringsAsFactors = FALSE)
key1 <- keyTemplate(dat1, long = TRUE)
key2 <- keyTemplate(dat2, long = TRUE)
keys2stack <- rbind(key1, key2)
keys2stack.fix <- keysPool(keys2stack)
keys2stack.fix2 <- keysPool(keys2stack.fix, colname = "class_new")
## Sometimes this will not be able to homogenize
dat1 <- data.frame(x1 = as.integer(rnorm(100)),
                   x2 = sample(c("Apple", "Orange"), 100, replace = TRUE))
dat2 <- data.frame(x1 = rnorm(100),
                   x2 = sample(c("Apple", "Orange"), 100, replace = TRUE),
                   stringsAsFactors = FALSE)
key1 <- keyTemplate(dat1, long = TRUE)
key2 <- keyTemplate(dat2, long = TRUE)
## Create a stack of keys for yourself
keys2stack <- rbind(key1, key2)
keys.fix <- keysPool(keys2stack)
## We will create stack of keys for you
keys.fix2 <- keysPool(list(key1, key2)) 
## View(keys.fix)
## View(keys.fix2)


## If you have wide keys, convert them with wide2long, either by
key1 <- keyTemplate(dat1)
key2 <- keyTemplate(dat2)
keysstack.wide <- rbind(wide2long(key1), wide2long(key2))
keys.fix <- keysPool(keysstack.wide)
## or
keysPool(list(wide2long(key1), wide2long(key2)))
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{all.equal.key}
\alias{all.equal.key}
\title{An all.equal method for variable wide keys}
\usage{
\method{all.equal}{key}(target, current, ..., check.attributes = FALSE)
}
\arguments{
\item{target}{A wide variable key}

\item{current}{A wide variable key}

\item{...}{Other arguments that are ignored}

\item{check.attributes}{Default FALSE}
}
\description{
Disregards attributes by defaults. Before comparing the two keys,
the values are sorted by \code{"name_new")}.
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{removeMatches}
\alias{removeMatches}
\title{Remove elements if they are in a target vector, possibly replacing with NA}
\usage{
removeMatches(x, y, padNA = FALSE)
}
\arguments{
\item{x}{vector from which elements are to be removed}

\item{y}{shorter vector of elements to be removed}

\item{padNA}{Default FALSE, Should removed items be replaced with NA values?}
}
\value{
a vector with elements in y removed
}
\description{
If a vector has c("A", "b", "c") and we want to remove "b" and
"c", this function can do the work. It can also replace "b" and
"c" with the NA symbol.
}
\details{
If elements in y are not members of x, they are silently ignored.

The code for this is not complicated, but it is
difficult to remember.  Here's the recipe to remove
elements y from x: \code{x <- x[!x \%in\% y[y \%in\% x]]}. It is
easy to get that wrong when in a hurry, so we use this function
instead.  The \code{padNA} was an afterthought, but it helps sometimes.
}
\examples{
x <- c("a", "b", "c", "d", "e", "f")
y <- c("e", "a")
removeMatches(x, y)
y <- c("q", "r", "s")
removeMatches(x, y)
}
\author{
Ben Kite <bakite@ku.edu> and Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keysPool}
\alias{keysPool}
\title{Homogenize class values and create a long key by pooling variable
keys.}
\usage{
keysPool(
  keylong = NULL,
  keysplit = NULL,
  classes = list(c("logical", "integer"), c("integer", "numeric"), c("ordered",
    "factor"), c("factor", "character")),
  colnames = c("class_old", "class_new"),
  textre = "TEXT$"
)
}
\arguments{
\item{keylong}{A list of long keys, or just one long key,
presumably a result of rbinding several long keys.}

\item{keysplit}{Not often needed for user-level code. A list of
key blocks, each of which is to be inspected and
homogenized. Not used if a keylong argument is provided.}

\item{classes}{A list of vectors specifying legal promotions.}

\item{colnames}{Either c("class_old","class_new), ""class_old", or
"class_new".  The former is the default.}

\item{textre}{A regular expression matching a column name to be
treated as character. Default matches any variable name ending
in "TEXT"}
}
\value{
A class-corrected version of the same format as the input,
    either a long key or a list of key elements.
}
\description{
For long-format keys, this is one way to correct for errors in
"class_old" or "class_new" for common variables. For a long key
created by stacking together several long keys, or for a list of
long keys, this will try to homogenize the classes by using a
"highest common denominator" approach.  If one key has x1 as a
floating point, but another block of rows in the key has x1 as
integer, then class must be changed to floating point
(numeric). If another section of a key has x1 as a character, then
character becomes the class.
}
\details{
Users might run keyTemplate on several data sets, arriving
at keys that need to be combined.  The long versions of the
keys can be stacked together by a function like \code{rbind}.
If the values class_old and class_new for a single variable are
inconsistent, then the "key stack" will fail the tests in keyCheck.
This function automates the process of fixing the class variables by
"promoting" classes where possible.

Begin with a simple example.  In one data set, the value of x is
drawn from integers 1L, 2L, 3L, while in another set it is
floating values like 1.1, 2.2. After creating long format keys,
and stacking them together, the values of class_old will clash.
For x, we will observe both "integer" and "numeric" in the
class_old column.  In that situation, the class_old for all of the
rows under consideration should be set as "numeric".

The promotion schemes are described by the variable classes, where
we have the most conservative changes first. The most destructive
change is when variables are converted from integer to character,
for example. The conservative conversion strategies are specified
in the classes variable, in which the last element in a vector
will be used to replace the preceeding classes.  For example,
c("ordered", "factor", "character") means that the class_old
values of "ordered" and "factor" will be replaced by "character".

The conversions specified by classes are tried, in order.
1. logical -> integer
2. integer -> numeric
3. ordered -> factor

If their application fails to homogenize a vector, then class is
changed to "character". For example, when the value of class_old
observed is c("ordered", "numeric", "character"). In that case,
the class is promoted to "character", it is the least common
denominator.
}
\examples{
dat1 <- data.frame(x1 = as.integer(rnorm(100)), x2 = sample(c("Apple", "Orange"),
                   100, replace = TRUE), x3 = ifelse(rnorm(100) < 0, TRUE, FALSE))
dat2 <- data.frame(x1 = rnorm(100), x2 = ordered(sample(c("Apple", "Orange"),
                   100, replace = TRUE)), x3 = rbinom(100, 1, .5),
                   stringsAsFactors = FALSE)
key1 <- keyTemplate(dat1, long = TRUE)
key2 <- keyTemplate(dat2, long = TRUE)
keys2stack <- rbind(key1, key2)
keys2stack.fix <- keysPool(keys2stack)
keys2stack.fix2 <- keysPool(keys2stack.fix, colname = "class_new")
## Sometimes this will not be able to homogenize
dat1 <- data.frame(x1 = as.integer(rnorm(100)),
                   x2 = sample(c("Apple", "Orange"), 100, replace = TRUE))
dat2 <- data.frame(x1 = rnorm(100),
                   x2 = sample(c("Apple", "Orange"), 100, replace = TRUE),
                   stringsAsFactors = FALSE)
key1 <- keyTemplate(dat1, long = TRUE)
key2 <- keyTemplate(dat2, long = TRUE)
## Create a stack of keys for yourself
keys2stack <- rbind(key1, key2)
keys.fix <- keysPool(keys2stack)
## We will create stack of keys for you
keys.fix2 <- keysPool(list(key1, key2)) 
## View(keys.fix)
## View(keys.fix2)


## If you have wide keys, convert them with wide2long, either by
key1 <- keyTemplate(dat1)
key2 <- keyTemplate(dat2)
keysstack.wide <- rbind(wide2long(key1), wide2long(key2))
keys.fix <- keysPool(keysstack.wide)
## or
keysPool(list(wide2long(key1), wide2long(key2)))
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/anonomize.R
\name{anonomize}
\alias{anonomize}
\title{Create unique anonymous id values}
\usage{
anonomize(x, prefix = "anon")
}
\arguments{
\item{x}{A column of "confidential" names, possibly with repeats}

\item{prefix}{Character string to use as prefix in result.
Default is "anon"}
}
\value{
Named character vector of anonymized id names.
}
\description{
Obscure participant id values by replacing them with "anon-1" and
so forth.
}
\details{
Caution: the true "confidential" names are used as names in the
output vector
}
\author{
Paul Johnson <pauljohn@ku.edu>
x <- c("bill", "bob", "fred", "bill")
(anonomize(x, prefix = "id"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dir.create.unique.R
\name{dir.create.unique}
\alias{dir.create.unique}
\title{Create a uniquely named directory. Appends number & optionally date to directory
name.}
\usage{
dir.create.unique(path, usedate = TRUE, showWarnings = TRUE,
  recursive = TRUE, mode = "0777")
}
\arguments{
\item{path}{A character string for the base name of the directory.}

\item{usedate}{TRUE or FALSE: Insert YYYYMMDD information?}

\item{showWarnings}{default TRUE. Show warnings? Will be passed on
to dir.create}

\item{recursive}{default TRUE. Will be passed on to dir.create}

\item{mode}{Default permissions on unix-alike systems. Will be
passed on to dir.create}
}
\value{
a character string with the directory name
}
\description{
Checks if the requested directory exists. If so, will create new
directory name. My favorite method is to have the target directory
with a date-based subdirectory, but set usedate as FALSE if you
don't like that. Arguments showWarnings, recursive, and mode are
passed along to R's dir.create, which does the actual work here.
}
\details{
Default response to dir = "../output/" fixes the directory name
like this, "../output/20151118-1/" because usedate is assumed
TRUE. If usedate = FALSE, then output names will be like
"../output-1/", "../output-2/", and so forth.
}
\author{
Paul E Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{checkValue_old}
\alias{checkValue_old}
\title{Compare observed values with the values listed
(presumably from a variable key).}
\usage{
checkValue_old(x, value_old, xname, diagnostic = FALSE)
}
\arguments{
\item{x}{a variable, either character or factor}

\item{value_old}{a vector of old values for which we are checking}

\item{xname}{character string to use for x's name when printing output}

\item{diagnostic}{prints messages about variables if TRUE}
}
\description{
This is purely diagnostic. It prints warnings in
either of 2 cases. 1)  observed data has values that
are not in the value_old, or 2) that value_old has
values that are not in the data.
}
\author{
Paul Johnson <pauljohn@ku.edu>
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deleteBogusRows.R
\name{deleteBogusColumns}
\alias{deleteBogusColumns}
\title{Remove columns in which the proportion of missing data exceeds
a threshold.}
\usage{
deleteBogusColumns(dframe, pm = 0.9, drop = FALSE, verbose = TRUE, n = 25)
}
\arguments{
\item{dframe}{A data frame or matrix}

\item{pm}{"proportion missing data" to be tolerated.}

\item{drop}{Default FALSE: if data frame result is reduced to one
column, should R's default drop behavior "demote" this to a
column vector.}

\item{verbose}{Default TRUE. Should a report be printed
summarizing information to be delted?}

\item{n}{Default 25: limit on number of values to print in
diagnostic output. If set to NULL or NA, then all of the
column values will be printed for the bogus rows.}
}
\value{
a data frame, invisibly
}
\description{
This is a column version of \code{deleteBogusRows}. Use the pm
argument to set the proportion of missing required before a column
is flagged for deletion
}
\seealso{
\code{deleteBogusRows}
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dir.create.unique.R
\name{dir.create.unique}
\alias{dir.create.unique}
\title{Create a uniquely named directory. Appends number & optionally date to directory
name.}
\usage{
dir.create.unique(
  path,
  usedate = TRUE,
  showWarnings = TRUE,
  recursive = TRUE,
  mode = "0777"
)
}
\arguments{
\item{path}{A character string for the base name of the directory.}

\item{usedate}{TRUE or FALSE: Insert YYYYMMDD information?}

\item{showWarnings}{default TRUE. Show warnings? Will be passed on
to dir.create}

\item{recursive}{default TRUE. Will be passed on to dir.create}

\item{mode}{Default permissions on unix-alike systems. Will be
passed on to dir.create}
}
\value{
a character string with the directory name
}
\description{
Checks if the requested directory exists. If so, will create new
directory name. My favorite method is to have the target directory
with a date-based subdirectory, but set usedate as FALSE if you
don't like that. Arguments showWarnings, recursive, and mode are
passed along to R's dir.create, which does the actual work here.
}
\details{
Default response to dir = "../output/" fixes the directory name
like this, "../output/20151118-1/" because usedate is assumed
TRUE. If usedate = FALSE, then output names will be like
"../output-1/", "../output-2/", and so forth.
}
\author{
Paul E Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{all.equal.keylong}
\alias{all.equal.keylong}
\title{An all.equal method for variable long keys}
\usage{
\method{all.equal}{keylong}(target, current, ...,
  check.attributes = FALSE)
}
\arguments{
\item{target}{A long variable key}

\item{current}{A long variable key}

\item{...}{Other arguments that are ignored}

\item{check.attributes}{Default FALSE}
}
\description{
Disregards attributes by defaults. Before comparing the two keys,
the values are sorted by \code{"name_new")}.
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keyApply}
\alias{keyApply}
\title{Apply variable key to data frame (generate recoded data frame)}
\usage{
keyApply(dframe, key, diagnostic = TRUE, safeNumericToInteger = TRUE,
  trimws = "both", ignoreCase = TRUE, drop = TRUE, debug = FALSE)
}
\arguments{
\item{dframe}{An R data frame}

\item{key}{A variable key object, of class either "key" or
"keylong"}

\item{diagnostic}{Default TRUE: Compare the old and new data
frames carefully with the keyDiagnostic function.}

\item{safeNumericToInteger}{Default TRUE: Should we treat values
which appear to be integers as integers? If a column is
numeric, it might be safe to treat it as an integer.  In many
csv data sets, the values coded c(1, 2, 3) are really
integers, not floats c(1.0, 2.0, 3.0). See \code{safeInteger}.}

\item{trimws}{Default is "both", can change to "left", "right", or
set as NULL to avoid any trimming.}

\item{ignoreCase}{Default TRUE. If column name is capitalized
differently than name_old in the key, but the two are
otherwise identical, then the difference in capitalization
will be ignored.}

\item{drop}{Default TRUE. True implies drop = c("vars",
"vals"). TRUE applies to both variables ("vars") and values
("vals"). "vars" means that a column will be omitted from data
if it is not in the key "name_old". Similarly, if anything
except "." appears in value_old, then setting drop="vals"
means omission of a value from key "value_old" causes
observations with those values to become NA.  This is the
original variable key behavior.  The drop argument allows
"partial keys", beginning with kutils version 1.12. drop =
FALSE means that neither values nor variables are omitted.
Rather than TRUE, one can specify either drop = "vars", or
drop = "vals".}

\item{debug}{Default FALSE. If TRUE, emit some warnings.}
}
\value{
A new data.frame object, with renamed and recoded variables
}
\description{
This is the main objective of the variable key system.
}
\examples{
mydf.key.path <- system.file("extdata", "mydf.key.csv", package = "kutils")
mydf.key <-  keyImport(mydf.key.path)
mydf.path <- system.file("extdata", "mydf.csv", package = "kutils")

mydf <- read.csv(mydf.path, stringsAsFactors = FALSE)
mydf2 <- keyApply(mydf, mydf.key)

nls.keylong.path <- system.file("extdata", "natlongsurv.key_long.csv", package = "kutils")
nls.keylong <- keyImport(nls.keylong.path, long = TRUE)
data(natlongsurv)
nls.dat <- keyApply(natlongsurv, nls.keylong)

}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keySave}
\alias{keySave}
\title{Save key as file after deducing type from suffix}
\usage{
keySave(obj, file, na_ = ".", varlab)
}
\arguments{
\item{obj}{a variable key object}

\item{file}{file name. must end in "csv", "xlsx" or "rds"}

\item{na_}{Value to insert to represent a missing score. Default
".".}

\item{varlab}{FALSE or TRUE. Default is FALSE, no new labels will
be created. If a key object has a varlab already, it is saved
with the key, always. This parameter controls whether a new
varlab template should be created when the object is saved.
If TRUE and obj has no varlab attribute, a new varlab template
is created by the \code{varlabTemplate} function. If TRUE and
a varlab attribute currently exists, but some variables are
missing labels, then \code{varlabTemplate} is called to fill
in new variable labels.}
}
\value{
NULL if no file is created. Otherwise, a key object with
    an attribute varlab is returned.
}
\description{
This is specialized to saving of key objects, it is not a
general purpose function for saving things.  It scans the
suffix of the file name and then does the right thing.
}
\details{
In updates 2017-09, a varlab element was introduced.  The varlab
attribute of the object is saved.  The files created incorporate
the variable labels object in different ways. 1) XLSX: variable
labels a worksheet named "varlab" 2) CSV: variable labels saved in
a separate file suffixed "-varlab.csv". 3) RDS: varlab is an
attribute of the key object.
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{makeKeylist}
\alias{makeKeylist}
\title{Convert the variable key into a list of long keys, one for each variable.}
\usage{
makeKeylist(key, sep = c(character = "\\\\|", logical = "\\\\|", integer
  = "\\\\|", factor = "\\\\|", ordered = "[\\\\|<]", numeric = "\\\\|"),
  na.strings = c("\\\\.", "", "\\\\s+", "N/A"))
}
\arguments{
\item{key}{A key object or a file name, csv, xlsx or rds.}

\item{sep}{Separator regular expressions}

\item{na.strings}{Strings that will be treated as NA. This will be
used only if the key object does not have an na.strings
attribute.}
}
\value{
A list with one element per variable name, along with some
    attributes like class_old and class_new. The class is set as
    well, "keylist"
}
\description{
A keylist element is a long key for one new variable. The new
variables are named by "name_old.name_new" for uniqueness.
}
\details{
If the key has one old variable being recoded 6 ways, that begets
6 elements in the resulting list. Attributes including the classes
of the old and new variables are included.
}
\author{
Paul Johnson <pauljohn@ku.edu>
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keyApply}
\alias{keyApply}
\title{Apply variable key to data frame (generate recoded data frame)}
\usage{
keyApply(
  dframe,
  key,
  diagnostic = TRUE,
  safeNumericToInteger = TRUE,
  trimws = "both",
  ignoreCase = TRUE,
  drop = TRUE,
  debug = FALSE
)
}
\arguments{
\item{dframe}{An R data frame}

\item{key}{A variable key object, of class either "key" or
"keylong"}

\item{diagnostic}{Default TRUE: Compare the old and new data
frames carefully with the keyDiagnostic function.}

\item{safeNumericToInteger}{Default TRUE: Should we treat values
which appear to be integers as integers? If a column is
numeric, it might be safe to treat it as an integer.  In many
csv data sets, the values coded c(1, 2, 3) are really
integers, not floats c(1.0, 2.0, 3.0). See \code{safeInteger}.}

\item{trimws}{Default is "both", can change to "left", "right", or
set as NULL to avoid any trimming.}

\item{ignoreCase}{Default TRUE. If column name is capitalized
differently than name_old in the key, but the two are
otherwise identical, then the difference in capitalization
will be ignored.}

\item{drop}{Default TRUE. True implies drop = c("vars",
"vals"). TRUE applies to both variables ("vars") and values
("vals"). "vars" means that a column will be omitted from data
if it is not in the key "name_old". Similarly, if anything
except "." appears in value_old, then setting drop="vals"
means omission of a value from key "value_old" causes
observations with those values to become NA.  This is the
original variable key behavior.  The drop argument allows
"partial keys", beginning with kutils version 1.12. drop =
FALSE means that neither values nor variables are omitted.
Rather than TRUE, one can specify either drop = "vars", or
drop = "vals".}

\item{debug}{Default FALSE. If TRUE, emit some warnings.}
}
\value{
A new data.frame object, with renamed and recoded variables
}
\description{
This is the main objective of the variable key system.
}
\examples{
mydf.key.path <- system.file("extdata", "mydf.key.csv", package = "kutils")
mydf.key <-  keyImport(mydf.key.path)
mydf.path <- system.file("extdata", "mydf.csv", package = "kutils")

mydf <- read.csv(mydf.path, stringsAsFactors = FALSE)
mydf2 <- keyApply(mydf, mydf.key)

nls.keylong.path <- system.file("extdata", "natlongsurv.key_long.csv", package = "kutils")
nls.keylong <- keyImport(nls.keylong.path, long = TRUE)
data(natlongsurv)
nls.dat <- keyApply(natlongsurv, nls.keylong)

}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{makeKeylist}
\alias{makeKeylist}
\title{Convert the variable key into a list of long keys, one for each variable.}
\usage{
makeKeylist(
  key,
  sep = c(character = "\\\\|", logical = "\\\\|", integer = "\\\\|", factor = "\\\\|",
    ordered = "[\\\\|<]", numeric = "\\\\|"),
  na.strings = c("\\\\.", "", "\\\\s+", "N/A")
)
}
\arguments{
\item{key}{A key object or a file name, csv, xlsx or rds.}

\item{sep}{Separator regular expressions}

\item{na.strings}{Strings that will be treated as NA. This will be
used only if the key object does not have an na.strings
attribute.}
}
\value{
A list with one element per variable name, along with some
    attributes like class_old and class_new. The class is set as
    well, "keylist"
}
\description{
A keylist element is a long key for one new variable. The new
variables are named by "name_old.name_new" for uniqueness.
}
\details{
If the key has one old variable being recoded 6 ways, that begets
6 elements in the resulting list. Attributes including the classes
of the old and new variables are included.
}
\author{
Paul Johnson <pauljohn@ku.edu>
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/project.R
\name{initProject}
\alias{initProject}
\title{Create project directories, initialize a git repo, create README.md
ChangeLog, and R template file in R directory}
\usage{
initProject(dir = NULL, ddir = "data", wdir = "workingdata",
  odir = "output", tdir = "tmp", ldir = "lit",
  writedir = "writeup", rdir = "R", ..., gitArgs = "--shared=group")
}
\arguments{
\item{dir}{Default NULL, otherwise a legal directory name to serve
as the top level directory for this project}

\item{ddir}{Data directory, place where "read only" unadjusted
data files are kept. Default is "data". If user sets it as NA
or NULL, the directory will not be created.}

\item{wdir}{Working data directory, where recorded, revised, and
cleaned data files are kept. Default is "workingdata".}

\item{odir}{Output directory. Default is "output".}

\item{tdir}{Temporary directory, where trash files can be kept for
safe keeping. Default is "tmp".}

\item{ldir}{Literature directory, where material about the project
can be stored. Default is "lit".}

\item{writedir}{The folder where the project writeup will be
stored. Default is "writeup".}

\item{rdir}{The name to be used for the R files. Defaults to "R".}

\item{...}{A list of other directories that the user would like to
create. For example, \code{adir = "admin"}, \code{cdir =
"client_provided"}, \code{bdir = "codebooks"}, \code{sdir =
"Stata"}, \code{mdir = "Mplus"}. These may be grouped in a
named vector or list, if user convenience dictates.}

\item{gitArgs}{This function tries to run "git init" and in our
center we add "--shared=group" on a network file server. If
that is undesirable in a user's context, put the argument
gitArgs as "".}
}
\value{
Name of project top level directory. Leaves the R
    working directory unchanged.
}
\description{
This creates folders for the separate parts of a project. It tries
to be clever about which directories are created and where they are
placed. Please see details for 3 scenarios for which we have
planned. If a directory already exists, it will not be damaged or
re-created.
}
\details{
If the dir argument is NULL, as default, then the current working
directory will be the place where new directories and the git
repository will be created.  Assuming the current working
directory's base name is not "R", then folders named "R", "data",
and so forth will be created in the current working directory.

If one has a current R working directory with a basename "R"
(suppose it is \code{"/tmp/whatever/R"}), and the user runs
\code{initProject()}, something different happens. The function
assumes we don't want to create subdirectories inside R. We don't
want to end up with \code{"/tmp/whatever/R/R"}. We don't
want \code{"/tmp/whatever/R/data"} either.  Instead, it assumes we
want the new directories created on same level as R, so it creates
\code{"/tmp/whatever/data"}, \code{"/tmp/whatever/workingdata"},
and so forth.  From within the R directory, these new directories
are seen as \code{"../data"}, \code{"../workingdata"}, and so
forth. That is, we should end up with directories and a git repo
in \code{"/tmp/whatever"}.

If the \code{dir} argument is provided by the user, then that is
used as the folder in which directories \code{"R"}, \code{"data"},
\code{"workingdate"}, and so forth are created.  All materials are
created in \code{dir}, no matter what the current working
directory is named (even if it is \code{"R"}).

The examples demonstrate all three of these scenarios.
}
\examples{
projdir1 <- file.path(tempdir(), "test1")
dir.create(projdir1, recursive = TRUE)
initProject(dir = projdir1)
list.files(projdir1, all.files = TRUE)
projdir2 <- file.path(tempdir(), "test2")
dir.create(projdir2, recursive = TRUE)
## demonstrate ability to create other directories
initProject(dir = projdir2, admin = "admin", clientfiles = "client")
list.files(projdir2, all.files = TRUE)
## demonstrate ability to nullify standard directories
projdir3 <- file.path(tempdir(), "test3")
dir.create(projdir3, recursive = TRUE)
initProject(projdir3, odir = NA, tdir = NA, writedir = NA)
list.files(projdir3, all.files = TRUE)
unlink(c("projdir1", "projdir2", "projdir3"), recursive = TRUE)
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/updatePkgDepends.R
\name{updatePackages}
\alias{updatePackages}
\title{Update packages, spot new dependencies, and install them}
\usage{
updatePackages(ask = FALSE, checkBuilt = TRUE,
  dependencies = c("Depends", "Imports", "LinkingTo"),
  libnew = "/usr/share/R/library/", repos = options("repos"), ...)
}
\arguments{
\item{ask}{If TRUE, asks user to select packages to update}

\item{checkBuilt}{If TRUE, packages built under earlier versions
of R are to be considered 'old'}

\item{dependencies}{A vector specifying which type of dependencies
need to be taken into account. We default to c("Depends",
"Imports", "LinkingTo").}

\item{libnew}{The R library folder into which the new packages
must be installed.  Defaults to "/usr/share/R/library", which
is where EL7 likes those things.  To install packages in
personal user directory, put libnew = NULL.}

\item{repos}{A vector of repositories on which to search for
packages. Same definition as in R's install.packages or
install.packages.}

\item{...}{additional arguments passed to update.packages and
install.packages}
}
\value{
A vector of new packages being installed
}
\description{
Addresses the problem that updates for previously installed R
packages may insert new dependencies. R's update.packages does not
trigger the installation of packages that are added as new
requirements in existing packages.
}
\details{
This function checks for existence of updates, ascertains whether
those packages impose new requirements, and installs any new
requirements. Then it conducts the update.

This function is valuable in system maintenance because sometimes
existing packages adopt new requirements and the update.packages
function does not notice.  Another possible case would be that a user
accidentally deletes some packages without realizing other packages
depend on them.

If this is run as the root/administrator privileged, then base R
packages may be updated, but if user is not root/administrator,
there will be a warning that packages were not updated because
permissions were lacking. For example

"Warning: package 'boot' in library '/usr/lib/R/library' will not
be updated.

This warning does not interfere with rest of purpose of this
function, since the new dependencies can be installed in a place
where the user has privileges, either by specifying libnew as a
full directory name or by setting it to NULL, which puts new
packages in $R_LIBS_USER
}
\examples{
\dontrun{
myrepos <- c("http://rweb.crmda.ku.edu/cran",
             "http://www.bioconductor.org/packages/3.3/bioc")
updatePackages(repos = myrepos)
## libnew defaults to "/usr/share/R/library". Specify NULL
## so that new packages will go to user's directory
updatePackages(libnew = NULL)
}
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/project.R
\name{initProject}
\alias{initProject}
\title{Create project directories, initialize a git repo, create README.md
ChangeLog, and R template file in R directory}
\usage{
initProject(
  dir = NULL,
  ddir = "data",
  wdir = "workingdata",
  odir = "output",
  tdir = "tmp",
  ldir = "lit",
  writedir = "writeup",
  rdir = "R",
  ...,
  gitArgs = "--shared=group"
)
}
\arguments{
\item{dir}{Default NULL, otherwise a legal directory name to serve
as the top level directory for this project}

\item{ddir}{Data directory, place where "read only" unadjusted
data files are kept. Default is "data". If user sets it as NA
or NULL, the directory will not be created.}

\item{wdir}{Working data directory, where recorded, revised, and
cleaned data files are kept. Default is "workingdata".}

\item{odir}{Output directory. Default is "output".}

\item{tdir}{Temporary directory, where trash files can be kept for
safe keeping. Default is "tmp".}

\item{ldir}{Literature directory, where material about the project
can be stored. Default is "lit".}

\item{writedir}{The folder where the project writeup will be
stored. Default is "writeup".}

\item{rdir}{The name to be used for the R files. Defaults to "R".}

\item{...}{A list of other directories that the user would like to
create. For example, \code{adir = "admin"}, \code{cdir =
"client_provided"}, \code{bdir = "codebooks"}, \code{sdir =
"Stata"}, \code{mdir = "Mplus"}. These may be grouped in a
named vector or list, if user convenience dictates.}

\item{gitArgs}{This function tries to run "git init" and in our
center we add "--shared=group" on a network file server. If
that is undesirable in a user's context, put the argument
gitArgs as "".}
}
\value{
Name of project top level directory. Leaves the R
    working directory unchanged.
}
\description{
This creates folders for the separate parts of a project. It tries
to be clever about which directories are created and where they are
placed. Please see details for 3 scenarios for which we have
planned. If a directory already exists, it will not be damaged or
re-created.
}
\details{
If the dir argument is NULL, as default, then the current working
directory will be the place where new directories and the git
repository will be created.  Assuming the current working
directory's base name is not "R", then folders named "R", "data",
and so forth will be created in the current working directory.

If one has a current R working directory with a basename "R"
(suppose it is \code{"/tmp/whatever/R"}), and the user runs
\code{initProject()}, something different happens. The function
assumes we don't want to create subdirectories inside R. We don't
want to end up with \code{"/tmp/whatever/R/R"}. We don't
want \code{"/tmp/whatever/R/data"} either.  Instead, it assumes we
want the new directories created on same level as R, so it creates
\code{"/tmp/whatever/data"}, \code{"/tmp/whatever/workingdata"},
and so forth.  From within the R directory, these new directories
are seen as \code{"../data"}, \code{"../workingdata"}, and so
forth. That is, we should end up with directories and a git repo
in \code{"/tmp/whatever"}.

If the \code{dir} argument is provided by the user, then that is
used as the folder in which directories \code{"R"}, \code{"data"},
\code{"workingdate"}, and so forth are created.  All materials are
created in \code{dir}, no matter what the current working
directory is named (even if it is \code{"R"}).

The examples demonstrate all three of these scenarios.
}
\examples{
projdir1 <- file.path(tempdir(), "test1")
dir.create(projdir1, recursive = TRUE)
initProject(dir = projdir1)
list.files(projdir1, all.files = TRUE)
projdir2 <- file.path(tempdir(), "test2")
dir.create(projdir2, recursive = TRUE)
## demonstrate ability to create other directories
initProject(dir = projdir2, admin = "admin", clientfiles = "client")
list.files(projdir2, all.files = TRUE)
## demonstrate ability to nullify standard directories
projdir3 <- file.path(tempdir(), "test3")
dir.create(projdir3, recursive = TRUE)
initProject(projdir3, odir = NA, tdir = NA, writedir = NA)
list.files(projdir3, all.files = TRUE)
unlink(c("projdir1", "projdir2", "projdir3"), recursive = TRUE)
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{safeInteger}
\alias{safeInteger}
\title{If a numeric variable has only integer values, then
make it an integer.}
\usage{
safeInteger(
  x,
  tol = .Machine$double.eps,
  digits = 7,
  vmax = .Machine$integer.max,
  verbose = FALSE
)
}
\arguments{
\item{x}{a numeric variable}

\item{tol}{Tolerance value. Defaults to Machine$double.eps. See
details.}

\item{digits}{Digits value passed to the zapsmall
function. Defaults to 7.}

\item{vmax}{Maximum value allowed for an integer. Defaults to
Machine$integer.max.}

\item{verbose}{Default FALSE: print warnings about x}
}
\value{
Either an integer vector or the original variable
}
\description{
Users often accidentally create floating point numeric variables
when they really mean integers, such as c(1, 2, 3), when they
should have done c(1L, 2L, 3L). Before running \code{as.integer()}
to coerce the variable, we'd rather be polite and ask the variable
"do you mind being treated as if you are an integer?"  This
function checks to see if the variable is "close enough" to being
an integer, and then coerces as integer. Otherwise, it returns
NULL. And issues a warning.
}
\details{
First, calculate absolute value of differences between \code{x}
and \code{as.integer(x)}. Second, find out if the sum of those
differences is smaller than \code{tol}. If so, then x can
reasonably be coerced to an integer.

Be careful with the return. The correct return value for variables
that should not be coerced as integer is uncertain at this
point. We've tested various strategies, sometimes returning FALSE,
NULL, or just the original variable.
}
\examples{
x1 <- c(1, 2, 3, 4, 5, 6)
is.integer(x1)
is.double(x1)
is.numeric(x1)
(x1int <- safeInteger(x1))
is.integer(x1int)
is.double(x1int)
is.numeric(x1int)
x2 <- rnorm(100)
x2int <- safeInteger(x2)
head(x2int)
x3 <- factor(x1, labels = c(LETTERS[1:6]))
x3int <- safeInteger(x3)

}
\author{
Paul Johnson <pauljohn@ku.edu> and Ben Kite
    <bakite@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/updatePkgDepends.R
\name{updatePackages}
\alias{updatePackages}
\title{Update packages, spot new dependencies, and install them}
\usage{
updatePackages(
  ask = FALSE,
  checkBuilt = TRUE,
  dependencies = c("Depends", "Imports", "LinkingTo"),
  libnew = "/usr/share/R/library/",
  repos = options("repos"),
  ...
)
}
\arguments{
\item{ask}{If TRUE, asks user to select packages to update}

\item{checkBuilt}{If TRUE, packages built under earlier versions
of R are to be considered 'old'}

\item{dependencies}{A vector specifying which type of dependencies
need to be taken into account. We default to c("Depends",
"Imports", "LinkingTo").}

\item{libnew}{The R library folder into which the new packages
must be installed.  Defaults to "/usr/share/R/library", which
is where EL7 likes those things.  To install packages in
personal user directory, put libnew = NULL.}

\item{repos}{A vector of repositories on which to search for
packages. Same definition as in R's install.packages or
install.packages.}

\item{...}{additional arguments passed to update.packages and
install.packages}
}
\value{
A vector of new packages being installed
}
\description{
Addresses the problem that updates for previously installed R
packages may insert new dependencies. R's update.packages does not
trigger the installation of packages that are added as new
requirements in existing packages.
}
\details{
This function checks for existence of updates, ascertains whether
those packages impose new requirements, and installs any new
requirements. Then it conducts the update.

This function is valuable in system maintenance because sometimes
existing packages adopt new requirements and the update.packages
function does not notice.  Another possible case would be that a user
accidentally deletes some packages without realizing other packages
depend on them.

If this is run as the root/administrator privileged, then base R
packages may be updated, but if user is not root/administrator,
there will be a warning that packages were not updated because
permissions were lacking. For example

"Warning: package 'boot' in library '/usr/lib/R/library' will not
be updated.

This warning does not interfere with rest of purpose of this
function, since the new dependencies can be installed in a place
where the user has privileges, either by specifying libnew as a
full directory name or by setting it to NULL, which puts new
packages in $R_LIBS_USER
}
\examples{
\dontrun{
myrepos <- c("http://rweb.crmda.ku.edu/cran",
             "http://www.bioconductor.org/packages/3.3/bioc")
updatePackages(repos = myrepos)
## libnew defaults to "/usr/share/R/library". Specify NULL
## so that new packages will go to user's directory
updatePackages(libnew = NULL)
}
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelcomparison.R
\name{compareLavaan}
\alias{compareLavaan}
\title{Prepare a table to compare fit measures of confirmatory factor analyses (CFA)}
\usage{
compareLavaan(models, fitmeas = c("chisq", "df", "pvalue", "rmsea",
  "cfi", "tli", "srmr", "aic", "bic"), nesting = NULL, scaled = TRUE,
  chidif = TRUE, digits = 3, ...)
}
\arguments{
\item{models}{list of lavaan cfa or sem models. Model names can be
supplied. See examples.}

\item{fitmeas}{A vector of fit measures. One or more of these
\code{c("chisq", "df", "pvalue", "rmsea", "cfi", "tli",
"srmr", "aic", "bic", "srmr", "aic", "bic",
"srmr_mplus")}. Other fit measures present in the lavaan
objects will be allowed; fit measures that are requested but
not found are ignored.}

\item{nesting}{character string indicating the nesting structure
of the models.  Must only contain model names, ">", and "+"
separated by spaces.  The model to the left of a ">" is the
parent model for all models to the right of the same ">", up
until another ">" is reached. When multiple models are nested
in the same parent, they are separated by a "+".}

\item{scaled}{should scaled versions of the fit measures requested
be used if available?  The scaled statistic is determined by
the model estimation method.  The defaul value is TRUE.}

\item{chidif}{should the nested models be compared by using the
anova function? The anova function may pass the model
comparison on to another lavaan function.  The results are
added to the last three columns of the comparison table. The
default value is TRUE.}

\item{digits}{The digits argument that will be passed to xtable.}

\item{...}{Arguments that will be passed to print.xtable. These
arguments can be used to control table caption, label, and so
forth. See \code{?print.xtable}.  If \code{type = "latex"} or
\code{"html"}, this function sets additional default values
for print.xtable that can be overridden by specifying
arguments here. Default type is an R data.frame, which is printed on
screen. Note the print.xtable parameter \code{print.results} determines
whether the markup is displayed before it is returned. The \code{file}
parameter can specify a file into which the markup is to be saved.}
}
\value{
If type = NULL, a data.frame object includes an attribute
    called "noteinfo". If type = "tex", return is a character
    vector created by xtable. If type = "html", a vector of HTML
    markup created by xtable.
}
\description{
If the parameter \code{nesting} is not specified, then this
function attempts to discern which models are nested and they are
ordered accordingly. The user can override that by specifing a
nesting structure. This uses a new notation to represent nesting
of models.  See the parameter \code{nesting}.  The syntax uses the
symbols ">" and "+" in an obvious way to indicate that one model
is the superset or on the same level as another. If the
}
\details{
In May 2018, the output approach was changed. The functions
\code{xtable} and \code{print.xtable} are used to render the final
result and any of the arguments allowed by \code{print.xtable} can
be used here (via the \code{...} argument). We have some default
settings for some \code{print.xtable}, such as \code{type = NULL},
\code{file = NULL}, \code{print.results = TRUE}, and
\code{math.style.exponents = TRUE}. There are some other specific
defaults for type = "latex" and type = "html", but they can all be
overridden by the user. We include a model legend at the bottom of
the table, indicating which models are compared by the Chi-squared
statistic.

If the type argument is not specified, then the output will be a
simple text display of the model table.  If type is either "latex"
or "html", then a marked up table will be displayed and the file
argument can be used to ask for a saved version. If the user wants
to simply save the result in a file, and not display on screen,
insert the argument \code{print.results = FALSE}.
}
\examples{
\donttest{
## These run longer than 5 seconds
library(lavaan)
library(xtable)
set.seed(123)
genmodel <- "f1 =~ .7*v1 + .7*v2 + .7*v3 + .7*v4 + .7*v5 + .7*v6
f1 ~~ 1*f1"
genmodel2 <- "f1 =~ .7*v1 + .7*v2 + .7*v3 + .7*v4 + .7*v5 + .2*v6
f1 ~~ 1*f1"

dat1 <- simulateData(genmodel, sample.nobs = 300)
dat2 <- simulateData(genmodel2, sample.nobs = 300)
dat1$group <- 0
dat2$group <- 1
dat <- rbind(dat1, dat2)

congModel <- "
              f1 =~ 1*v1 + v2 + v3 + v4 + v5 + v6
    		  f1 ~~ f1
    		  f1 ~0*1
    		 "
weakModel <- "
              f1 =~ 1*v1 + c(L2,L2)*v2 + c(L3,L3)*v3 + c(L4,L4)*v4 + c(L5,L5)*v5 + c(L6,L6)*v6
    		  f1 ~~ f1
    		  f1 ~0*1
    		"
partialweakModel <- "
              f1 =~ 1*v1 + c(L2,L2)*v2 + c(L3,L3)*v3 + c(L4,L4)*v4 + c(L5,L5)*v5 + v6
    		  f1 ~~ f1
    		  f1 ~0*1
    		"
partialweakModel2 <- "
              f1 =~ 1*v1 + c(L2,L2)*v2 + c(L3,L3)*v3 + c(L4,L4)*v4 + v5 + v6
    		  f1 ~~ f1
    		  f1 ~0*1
    		"
partialstrongModel1 <- "
              f1 =~ 1*v1 + c(L2,L2)*v2 + c(L3,L3)*v3 + c(L4,L4)*v4 + c(L5,L5)*v5 + v6
    		  f1 ~~ f1
    		  f1 ~ c(0,NA)*1
    		  v1 ~ c(I1,I1)*1
    		  v2 ~ c(I2,I2)*1
    		  v3 ~ c(I3,I3)*1
    		  v4 ~ c(I4,I4)*1
    		  v5 ~ c(I5,I5)*1
    		  v6 ~ c(I6,I6)*1
    		"
cc1 <- cfa(congModel, data=dat, group="group", meanstructure=TRUE, estimator = "MLR")
cc2 <- cfa(weakModel, data=dat, group="group", meanstructure=TRUE, estimator = "MLR")
cc21 <- cfa(partialweakModel, data=dat, group="group", meanstructure=TRUE, estimator = "MLR")
cc3 <- cfa(partialstrongModel1, data=dat, group="group", meanstructure=TRUE, estimator = "MLR")

models <- list(cc1, cc2, cc21, cc3)
## Note, nesting is not specified, so built-in nesting detection applies
compareLavaan(models)
compareLavaan(models, type = "latex")
compareLavaan(models, type = "html")

## Now we specify model labels in the list
models <- list("Configural" = cc1, "Metric" = cc2, "PartialMetric" = cc21, "Scalar" = cc3)
## The model labels are used in the nesting parameter
compareLavaan(models, nesting = "Configural > Metric + PartialMetric > Scalar")

compareLavaan(models, fitmeas = c("chisq", "df", "cfi", "rmsea", "tli"),
            nesting = "Configural > Metric + PartialMetric > Scalar")

## Creates output file
## compareLavaan(models, fitmeas = c("chisq", "df", "cfi", "rmsea", "tli"),
## nesting = "Configural > Metric + PartialMetric > Scalar", type = "tex",
## file = "/tmp/table.tex")
}
}
\author{
Ben Kite <bakite@ku.edu> and Paul Johnson
    <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/textUtils.R
\name{alphaOnly}
\alias{alphaOnly}
\title{Keep only alpha-numeric symbols}
\usage{
alphaOnly(x, also)
}
\arguments{
\item{x}{text string, or vector of strings (each of which is
processed separately)}

\item{also}{A named vector of other symbols that the user wants to
remove, along with replacements. For example, c(" " = "_",
"-" = "", "+" = "") to replace space with underscore and minus
and plus signs with nothing.}
}
\value{
cleaned text string
}
\description{
From a text string, keep ASCII letters, numbers, as well as "'",
" ", "_" "(", ")", "-", and "+". For maximum compatability with
the cross-platform file-naming standard. Obliterates all
characters that migth be mistaken for shell symbols, like "\^",
"\$", "\@" and so forth.
}
\details{
Removes trailing spaces.

This version allows internal spaces in the string, by default. The
also argument can be used to eliminate spaces or other hated symbols.
}
\examples{
x <- c("[]kansas(city) Missouri", "percent\%slash/",
       "\\back{squiggle}_under(paren)", "*star-minus+plus")
alphaOnly(x)
alphaOnly(x, also = c(" " = "_", "+" = "_"))
alphaOnly(x, also = c("(" = "[", ")" = "]"))
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keyTemplateSPSS}
\alias{keyTemplateSPSS}
\title{Import an SPSS file, create a key representing the numeric ->
factor transition}
\usage{
keyTemplateSPSS(dat, long = TRUE)
}
\arguments{
\item{dat}{A character string path to the SPSS file}

\item{long}{TRUE returns a long key, otherwise a wide key}
}
\value{
A variable key (long or wide)
}
\description{
This is a way to keep track of the scores that are used in the SPSS file.
It is also an easy way to start a new variable key that makes it convenient
to work on the value_new column with R text functions.
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{assignMissing}
\alias{assignMissing}
\title{Set missing values}
\usage{
assignMissing(x, missings = NULL, sep = ";")
}
\arguments{
\item{x}{A variable}

\item{missings}{A string vector of semi-colon separated values,
    ranges, and/or inequalities.  For strings and factors, only an
    enumeration of values (or factor levels) to be excluded is
    allowed. For numeric variables (integers or floating point
    variables), one can specify open and double-sided intervals as
    well as particular values to be marked as missing. One can
    append particular values and ranges by
    "1;2;3;(8,10);[22,24];> 99;< 2". The double-sided interval is
    represented in the usual mathematical way, where hard
    bracketes indicate "closed" intervals and parentheses indicate
    open intervals.\enumerate{

\item "(a,b)" means values of x greater than a and smaller than b
    will be set as missing.

\item "[a,b]" is a closed interval, one which includes the
    endpoints, so a <= x <= b will be set as NA

\item "(a,b]" and "[a,b)" are acceptable.
\item "< a"  indicates all values smaller than a will be missing
\item  "<= a" means values smaller than or equal to a will be
    excluded
\item "> a" and ">= a" have comparable
    interpretations.
\item "8;9;10" Mark off specific values by
an enumeration. Be aware, however, that this is
useful only for integer variables.  As demonstrated in the
example, for floating point numbers, one must specify intervals.
\item For factors and character variables, the argument missings
can be written either as "lo;med;hi" or "c('lo','med','hi')" }}

\item{sep}{A separator symbol, ";" (semicolon) by default}
}
\value{
A cleaned column in which R's NA symbol replaces values
    that should be missing
}
\description{
The missings values have to be carefully written, depending on the
type of variable that is being processed.
}
\details{
Version 0.95 of kutils introduced a new style for specification of
missing values.
}
\examples{
## 1.  Integers.
x <- seq.int(-2L, 22L, by = 2L)
## Exclude scores 8, 10, 18
assignMissing(x, "8;10;18")
## Specify range, 4 to 12 inclusive
missings <- "[4,12]"
assignMissing(x, missings)
## Not inclusive
assignMissing(x,  "(4,12)")
## Set missing for any value smaller that 7
assignMissing(x, "< 7")
assignMissing(x, "<= 8")
assignMissing(x, "> 11")
assignMissing(x, "< -1;2;4;(7, 9);> 20")


## 2. strings
x <- c("low", "low", "med", "high")
missings <- "low;high"
assignMissing(x, missings)
missings <- "med;doesnot exist"
assignMissing(x, missings)
## Test alternate separator
assignMissing(x, "low|med", sep = "|")

## 3. factors (same as strings, really)
x <- factor(c("low", "low", "med", "high"), levels = c("low", "med", "high"))
missings <- "low;high"
assignMissing(x, missings)
## Previous same as
missings <- c("low", "high")
assignMissing(x, missings)

missings <- c("med", "doesnot exist")
assignMissing(x, missings)
## ordered factor:
x <- ordered(c("low", "low", "med", "high"), levels = c("low", "med", "high"))
missings <- c("low", "high")
assignMissing(x, missings)

## 4. Real-valued variable
set.seed(234234)
x <- rnorm(10)
x
missings <- "< 0"
assignMissing(x, missings)
missings <- "> -0.2"
assignMissing(x, missings)
## values above 0.1 and below 0.7 are missing
missings <- "(0.1,0.7)"
assignMissing(x, missings)
## Note that in floating point numbers, it is probably
## futile to specify specific values for missings. Even if we
## type out values to 7 decimals, nothing gets excluded
assignMissing(x, "-0.4879708;0.1435791")
## Can mark a range, however
assignMissing(x, "(-0.487971,-0.487970);(0.14357, 0.14358)")
x
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/textUtils.R
\name{alphaOnly}
\alias{alphaOnly}
\title{Keep only alpha-numeric symbols}
\usage{
alphaOnly(x, also)
}
\arguments{
\item{x}{text string, or vector of strings (each of which is
processed separately)}

\item{also}{A named vector of other symbols that the user wants to
remove, along with replacements. For example, c(" " = "_",
"-" = "", "+" = "") to replace space with underscore and minus
and plus signs with nothing.}
}
\value{
cleaned text string
}
\description{
From a text string, keep ASCII letters, numbers, as well as "'",
" ", "_" "(", ")", "-", and "+". For maximum compatability with
the cross-platform file-naming standard. Obliterates all
characters that migth be mistaken for shell symbols, like "^",
"$", "@" and so forth.
}
\details{
Removes trailing spaces.

This version allows internal spaces in the string, by default. The
also argument can be used to eliminate spaces or other hated symbols.
}
\examples{
x <- c("[]kansas(city) Missouri", "percent\%slash/",
       "\back{squiggle}_under(paren)", "*star-minus+plus")
alphaOnly(x)
alphaOnly(x, also = c(" " = "_", "+" = "_"))
alphaOnly(x, also = c("(" = "[", ")" = "]"))
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/likert.R
\name{likert}
\alias{likert}
\title{Percentage tables for Likert Scale variables}
\usage{
likert(data, vlist, columnlabels, valuelabels, rows = FALSE, digits = 2, ...)
}
\arguments{
\item{data}{A data frame. Function will try to include all
variables in data, unless vlist is provided.}

\item{vlist}{A vector of column names in data that should be
displayed}

\item{columnlabels}{Column labels, optional to beautify variable
names. If not supplied, column names will be used as column
labels. Provide either 1) A named vector that replaces one or
more columns, \code{c(oldname1 = "newlabel1", oldname2 =
"newlabel2")} where oldnames are in colnames(data), or 2) a
vector of same length as vlist (or data if vlist is not
supplied) that will replace them one for one.}

\item{valuelabels}{A vector of values to beautify existing
levels. If not supplied, factor levels will be used as row
labels}

\item{rows}{Should output be transposed. This may help if there
are many variables that need to fit on the page.  Percentages
will appear on the rows, rather than columns.}

\item{digits}{Number of decimals to display in percentages}

\item{...}{Arguments to pass to R's table function. We suggest
\code{useNA = "always"} to add missing value information and
\code{exclude = original.value.label} to exclude values
observed. Currently, \code{useNA = "ifany"} does not work as
expected, the number of missings will be displayed, even if
there are none.}
}
\value{
A list, including a frequency table (called "freqTab"),
    column counts ("counts"), column sums ("sums"), and
    column percents ("pcts").
}
\description{
Creates a table with columns for allowed values and rows for
variables.
}
\examples{
vvector <- c("Strongly Disagree", "Disagree", "Neutral",
              "Agree", "Strongly Agree")
set.seed(2342234)
N <- 28
scales <-
    data.frame(Vegas = factor(sample(1:5, N, replace = TRUE),
        levels = 1:5, labels = vvector),
               NewYork = factor(sample(1:5, N, replace = TRUE),
        levels = 1:5, labels = vvector),
               Paris = factor(sample(1:5, N, replace = TRUE),
        levels = 1:5, labels = vvector),
               Berlin = factor(sample(1:5, N, replace = TRUE),
        levels = 1:5, labels = vvector))

likert(scales)

likert(scales, exclude = "Disagree")

likert(scales, exclude = "Strongly Disagree", useNA = "ifany")

(mySummary1 <- likert(data = scales, vlist = c("Vegas", "NewYork", "Paris")))
mySummary1[["pcts"]]

(mySummary2 <- likert(scales, vlist = c("Vegas", "NewYork", "Paris"),
                    valuelabels = c("SD", "D", "N", "A", "SA")))
(mySummary3 <- likert(scales, vlist = c("Vegas", "NewYork", "Paris"),
                    valuelabels = c("Strongly Disagree" = "Strong Disagreement")))

(mySummary5 <- likert(scales, vlist = c("Vegas", "NewYork", "Paris"),
      valuelabels = c("SD", "D", "N", "A", "SA"),
      columnlabels = c("Vegas" = "Sin City"), rows = TRUE))

 ## Example of how one might write this in a file. 
 ## print(xtable::xtable(mySummary1[[1]], digits = 1),
 ##       type = "html", file = "varCount-1.html")       
  
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{padW0}
\alias{padW0}
\title{Insert 0's in the front of existing digits or characters so that
all elements of a vector have the same number of characters.}
\usage{
padW0(x, n = 0)
}
\arguments{
\item{x}{vector to be converted to a character variable by
inserting 0's at the front. Should be integer or character,
floating point numbers will be rounded down. All other
variable types will return errors.}

\item{n}{Optional parameter.  The desired final length of
character vector.  This parameter is a guideline to determine
how many 0's must be inserted.  This will be ignored if
\code{n} is smaller than the number of characters in the
longest element of \code{x}.}
}
\value{
A character vector
}
\description{
The main purpose was to correct ID numbers in studies that are
entered as integers with leading 0's as in 000001 or 034554.  R's
default coercion of integers will throw away the preceding 0's,
and reduce that to 1 or 34554. The user might want to re-insert
the 0's, thereby creating a character vector with values "000001"
and "045665".
}
\details{
If x is an integer or a character vector, the result is the
more-or-less expected outcome (see examples). If x is numeric,
but not an integer, then x will be rounded to the lowest integer.

The previous versions of this function failed when there were
missing values (NA) in the vector x.  This version returns NA for
those values.

One of the surprises in this development was that sprintf() in R
does not have a known consequence when applied to a character
variable. It is platform-dependent (unredictable). On Ubuntu Linux
16.04, for example \code{sprintf("\%05s", 2)} gives back
\code{" 2"}, rather than (what I expected) \code{"00002"}. The
problem is mentioned in the documentation for \code{sprintf}. The
examples show this does work now, but please test your results.
}
\examples{
x1 <- c(0001, 0022, 3432, NA)
padW0(x1)
padW0(x1, n = 10)
x2 <- c("1", "22", "3432", NA)
padW0(x2)
## There's been a problem with strings, but this works now.
## It even preserves non-leading spaces. Hope that's what you want.
x3 <- c("1", "22 4", "34323  42", NA)
padW0(x3)
x4 <- c(1.1, 334.52, NA)
padW0(x4)
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{modifyVector}
\alias{modifyVector}
\title{Use new information to update a vector. Similar in concept to
R's modify list}
\usage{
modifyVector(x, y, augment = FALSE, warnings = FALSE)
}
\arguments{
\item{x}{vector to be updated, may be named or not.}

\item{y}{possibly a named vector. If unnamed, must match
length of x. If named, and length is shorter than x, then
name-value pairs in x will be replaced with name-value pairs
with y. If names in y are not in x, the augment argument
determines the result.}

\item{augment}{If TRUE, add new items in x from y. Otherwise,
ignore named items in y that are not in x.}

\item{warnings}{Defaults as FALSE. Show warnings about augmentation
of the target vector.}
}
\value{
an updated vector
}
\description{
Original purpose was to receive 2 named vectors, x and y, and copy
"updated" named values from y into x. If x or y are not named,
however, this will do something useful.
\itemize{
\item Both vectors are named: values in x for which y names match will be
    updated with values from y. If \code{augment} is true, then named
    values in y that are not present in x will be added to x.
\item If neither vector is named: returns a new vector with x as the values
    and y as the names. Same as returning \code{names(x) <- y}.
\item If x is not named, y is named: replaces elements in x with values of y
    where suitable (x matches names(y)). For matches, returns x = y[x]
    if names(y) include x.
\item If x is named, y is not named: returns y, but with names from x. Lengths
    of x and y must be identical.
\item If y is NULL or not provided, x is returned unaltered.
}
}
\examples{
x <- c(a = 1, b = 2, c = 3)
y <- c(b = 22)
modifyVector(x, y)
y <- c(c = 7, a = 13, e = 8)
## If augment = TRUE, will add more elements to x
modifyVector(x, y, augment = TRUE)
modifyVector(x, y)
x <- c("a", "b", "c")
y <- c("income", "education", "sei")
## Same as names(x) <- y
modifyVector(x, y)
x <- c("a", "b", "c")
y <- c(a = "happy")
modifyVector(x, y)
y <- c(a = "happy", g = "glum")
## Will give error unless augment = TRUE
modifyVector(x, y, augment = TRUE)
}
\author{
Paul Johnson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/file.backup.R
\name{file.backup}
\alias{file.backup}
\title{Create a backup version of a file by renaming it.}
\usage{
file.backup(name, fullpath = FALSE, keep.old = FALSE, verbose = FALSE)
}
\arguments{
\item{name}{A character string for the name of the file.}

\item{fullpath}{Return the full directory path to the
file. Default FALSE, return only the file name.}

\item{keep.old}{If FALSE (default), rename the file. Otherwise, keep old copy.}

\item{verbose}{If TRUE, output warnings and list the files in the output directory when done.}
}
\value{
The name of the newly created file.
}
\description{
Inserts the date-time of the most recent modification at the
end of the file name, before the extension.
}
\details{
Return is the new file name that was created, using whatever
path information was provided in the file's original name. However,
the fullpath argument can be set to TRUE, so a path with the
full directory name will be created and returned.
}
\examples{
tdir <- tempdir()
owd <- getwd()

setwd(tdir)
system("touch test.1.txt")
system("touch test.2.txt")
system("touch test.3.txt")
system("touch test.4.txt")
system("touch test.5.txt")
## note: no extension next
system("touch test.6")
list.files()
file.backup("test.1.txt")
file.backup("test.2.txt", fullpath=TRUE)
list.files()
setwd(owd)
file.backup(file.path(tdir, "test.3.txt"))
## Next should be same path because input had a full path
file.backup(file.path(tdir, "test.4.txt"), fullpath=TRUE)
file.backup(file.path(tdir, "test.5.txt"), fullpath = TRUE, verbose = TRUE)
file.backup(file.path(tdir, "test.6"))
}
\author{
Shadi Pirhosseinloo <shadi@ku.edu> Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colnamesReplace.R
\name{colnamesReplace}
\alias{colnamesReplace}
\title{Replace column names with new names from a named vector}
\usage{
colnamesReplace(dat, newnames, oldnames = NULL, ..., lowercase = FALSE,
  verbose = FALSE)
}
\arguments{
\item{dat}{a data frame}

\item{newnames}{Can be a named vector of the form c(oldname1 =
"newname1", oldname2 = "newname") or it may be simply
c("newname1", "newname2") to correspond with the oldname
vector.}

\item{oldnames}{Optional. If supplied, must be same length as
newnames.}

\item{...}{Additional arguments that will be passed to R's
\code{gsub} function, which is used term-by-term inside this
function.}

\item{lowercase}{Default FALSE. Should all column names be
converted to lower case.}

\item{verbose}{Default FALSE. Want diagnostic output about column
name changes?}
}
\value{
a data frame
}
\description{
A convenience function to alter column names. Can be called from
code cleanup in the variable key system.
}
\examples{
set.seed(234234)
N <- 200
mydf <- data.frame(x5 = rnorm(N), x4 = rnorm(N), x3 = rnorm(N),
                   x2 = letters[sample(1:24, 200, replace = TRUE)],
                   x1 = factor(sample(c("cindy", "bobby", "marsha",
                                        "greg", "chris"), 200, replace = TRUE)),
                   x11 = 7,
                   x12 = 18,
                   x13 = 33,
                   stringsAsFactors = FALSE)
mydf2 <- colnamesReplace(mydf, newnames = c("x4" = "GLOPPY"))
mydf2 <- colnamesReplace(mydf, newnames = c("x4" = "GLOPPY", "USA" = "Interesting"), verbose = TRUE)
colnames(mydf2)
head(mydf3 <- colnamesReplace(mydf, newnames = c(x11 = "x12", x12 = "x13", x13 = "x20")))
head(mydf4 <- colnamesReplace(mydf, newnames = c(x12 = "x11", x11 = "x99", x13 = "x20")))
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkCoercion.R
\name{checkCoercion}
\alias{checkCoercion}
\title{Check if values can be safely coerced without introduction of
missing values}
\usage{
checkCoercion(value, targetclass, na.strings = c("\\\\.", "", "\\\\s+", "N/A"))
}
\arguments{
\item{value}{Character vector of values, such as value_new or
value_old for one variable in a key.}

\item{targetclass}{R class name}

\item{na.strings}{Values that should be interpreted as R NA.
These are ignored in the coercion check.}
}
\value{
either TRUE, or a vector of values which are not
    successfully coerced
}
\description{
This might be named "coercesSafely" or such.  If values cannot be
coerced into class specified, then values must be incorrect.
}
\examples{
x1 <- c("TRUE", "FALSE", FALSE, TRUE, NA, ".", "N/A", " ", "")
checkCoercion(x1, "logical")
x1 <- c(x1, "TRUE.FALSE", "Has a space")
## Should fail:
checkCoercion(x1, "logical")
x2 <- c(4, 5, 6, 9.2, ".", " ")
## Should fail
checkCoercion(x2, "logical")
x3 <- factor(c("bob", "emily", "bob", "jane", "N/A", " ", NA, "NA"))
checkCoercion(x3, "ordered")
checkCoercion(x3, "integer")
## Should fail:
checkCoercion(x3, "logical")

}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{is.data.frame.simple}
\alias{is.data.frame.simple}
\title{Check if a data frame is a simple collection of columns (no lists
or matrices within)}
\usage{
is.data.frame.simple(dframe)
}
\arguments{
\item{dframe}{A data frame}
}
\value{
Boolean, TRUE or FALSE. An attribute "not_a_simple_column"
    is created, indicating which of the elements in the dframe
    have dimensions
}
\description{
Checks for the existence of dimensions within the data
frame. Returns FALSE if any object within dframe has non-null dim
value.
}
\details{
See: http://stackoverflow.com/questions/38902880/data-frame-in-which-elements-are-not-single-columns
}
\examples{
N <- 100
mydf <- data.frame(x5 = rnorm(N),
                   x4 = rpois(N, lambda = 3),
                   x3 = ordered(sample(c("lo", "med", "hi"),
                   size = N, replace=TRUE)))
is.data.frame.simple(mydf)
mydf$amatr <- matrix(0, ncol = 2, nrow = NROW(mydf))
is.data.frame.simple(mydf)
mydf$amatr <- NULL
is.data.frame.simple(mydf)
mydf$adf <- mydf
is.data.frame.simple(mydf)
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colnamesReplace.R
\name{colnamesReplace}
\alias{colnamesReplace}
\title{Replace column names with new names from a named vector}
\usage{
colnamesReplace(
  dat,
  newnames,
  oldnames = NULL,
  ...,
  lowercase = FALSE,
  verbose = FALSE
)
}
\arguments{
\item{dat}{a data frame}

\item{newnames}{Can be a named vector of the form c(oldname1 =
"newname1", oldname2 = "newname") or it may be simply
c("newname1", "newname2") to correspond with the oldname
vector.}

\item{oldnames}{Optional. If supplied, must be same length as
newnames.}

\item{...}{Additional arguments that will be passed to R's
\code{gsub} function, which is used term-by-term inside this
function.}

\item{lowercase}{Default FALSE. Should all column names be
converted to lower case.}

\item{verbose}{Default FALSE. Want diagnostic output about column
name changes?}
}
\value{
a data frame
}
\description{
A convenience function to alter column names. Can be called from
code cleanup in the variable key system.
}
\examples{
set.seed(234234)
N <- 200
mydf <- data.frame(x5 = rnorm(N), x4 = rnorm(N), x3 = rnorm(N),
                   x2 = letters[sample(1:24, 200, replace = TRUE)],
                   x1 = factor(sample(c("cindy", "bobby", "marsha",
                                        "greg", "chris"), 200, replace = TRUE)),
                   x11 = 7,
                   x12 = 18,
                   x13 = 33,
                   stringsAsFactors = FALSE)
mydf2 <- colnamesReplace(mydf, newnames = c("x4" = "GLOPPY"))
mydf2 <- colnamesReplace(mydf, newnames = c("x4" = "GLOPPY", "USA" = "Interesting"), verbose = TRUE)
colnames(mydf2)
head(mydf3 <- colnamesReplace(mydf, newnames = c(x11 = "x12", x12 = "x13", x13 = "x20")))
head(mydf4 <- colnamesReplace(mydf, newnames = c(x12 = "x11", x11 = "x99", x13 = "x20")))
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{varlabTemplate}
\alias{varlabTemplate}
\title{Create Variable Label Template}
\usage{
varlabTemplate(obj, varlab = TRUE)
}
\arguments{
\item{obj}{A variable key}

\item{varlab}{Default NULL, function will start from clean slate,
a set of column labels that match \code{name_new}. User can
specify values by providing a named vector of labels, e.g.,
\code{c("x1" = "happiness", "x2" = "wealth")}, where the names
are values to be matched against "name_new" in key.}
}
\value{
Character matrix with columns \code{name_new} and \code{varlab}.
}
\description{
Receive a key, create a varlab object, with columns
    \code{name_old} \code{name_new}, and \code{varlab}.
}
\details{
If not specified, a matrix is created with empty variable labels.
}
\examples{
mydf.path <- system.file("extdata", "mydf.csv", package = "kutils")
mydf <- read.csv(mydf.path, stringsAsFactors=FALSE)
mydf.keywide1 <- keyTemplate(mydf, long = FALSE, sort = FALSE,
                    varlab = TRUE)
attr(mydf.keywide1, "varlab")
mydf.keywide2 <- keyTemplate(mydf, long = FALSE, sort = FALSE,
                    varlab = c("x3" = "fun"))
attr(mydf.keywide2, "varlab")
attr(mydf.keywide2, "varlab") <- varlabTemplate(mydf.keywide2,
                  varlab = c("x5" = "wealth", "x10" = "happy"))
attr(mydf.keywide2, "varlab")
attr(mydf.keywide2, "varlab") <- varlabTemplate(mydf.keywide2,
                  varlab = TRUE)
attr(mydf.keywide2, "varlab")
## Target we are trying to match:
mydf.keylong <- keyTemplate(mydf, long = TRUE, sort = FALSE, varlab = TRUE)
attr(mydf.keylong, "varlab")
attr(mydf.keylong, "varlab") <- NULL
varlabTemplate(mydf.keylong)
attr(mydf.keylong, "varlab") <- varlabTemplate(mydf.keylong,
                   varlab = c("x3" = "wealth", "x10" = "happy"))
attr(mydf.keylong, "varlab")
attr(mydf.keylong, "varlab") <- varlabTemplate(mydf.keylong, varlab = TRUE)
attr(mydf.keylong, "varlab")
}
\author{
Paul Johnson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{dts}
\alias{dts}
\title{Delete trailing slash}
\usage{
dts(name)
}
\arguments{
\item{name}{A path}
}
\value{
Same path with trailing "/" removed.
}
\description{
This function cleans up a path string by removing the trailing
slash.  This is necessary on MS Windows, \code{file.exists(fn)} fails
if "/" is on end of file name. Deleting the trailing slash is thus
required on Windows and it is not harmful on other platforms.
}
\details{
All usages of \code{file.exists(fn)} in R should be revised
to be multi-platform safe by writing \code{file.exists(dts(fn))}.

This version also removes repeated adjacent slashes, so that
\code{"/tmp///paul//test/"} becomes \code{"/tmp/paul/test"}.
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/importQualtrics.R
\name{qualtricsBlockStack}
\alias{qualtricsBlockStack}
\title{Create meta data frame to align identical questions}
\usage{
qualtricsBlockStack(meta, questionname = "question")
}
\arguments{
\item{meta}{A meta data structure retrieved from importQualtrics}

\item{questionname}{Character string for name of column in meta data that holds the questions}
}
\value{
A new meta data table that horizontally aligns equivalent questions.
}
\description{
Qualtrics returns a data frame that has vertical "blocks", one for
each "treatment condition" in an experimental condition.
Researchers often want to align the questions from the blocks
vertically, essentially converting the Qualtrics "wide" format to
a "long" format. This is a helper function that identifies
questions that may need to be stacked together.  The input is a
meta data structure (can be retrieved as an attribute from
importQualtrics). It will find out which questions are identical
and prepare to re-align ("stack") the columns.
}
\author{
Paul Johnson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/semTable.R
\name{semTable}
\alias{semTable}
\title{Creates Structural Equation Modeling Tables}
\usage{
semTable(object, file = NULL, paramSets = "all", paramSetLabels,
  columns = c(est = "Estimate", se = "SE", z = "z", p = "p"),
  columnLabels, fits = c("chisq", "cfi", "tli", "rmsea"),
  fitLabels = toupper(fits), varLabels = NULL, groups = NULL,
  type = "latex", table.float = FALSE, caption = NULL,
  label = NULL, longtable = FALSE, print.results = TRUE,
  centering = "siunitx", alpha = c(0.05, 0.01, 0.001))
}
\arguments{
\item{object}{A lavaan object (e.g., returned by cfa() or sem()),
or a named list of lavaan objects, e.g., \code{list("Model A"
= obj1, "Model B" = obj2)}. Results will be displayed side by
side.}

\item{file}{Base name for output file. This function will insert
suffix, either "tex", "html" and "csv".}

\item{paramSets}{Parameter sets to be included for each fitted
object.  Valid values of the vector are \code{"all"} or a any
of the following: \code{c("loadings", "slopes", "intercepts",
"residualvariances", "residualcovariances", "latentmeans",
"latentvariances", "latentcovariances", "thresholds",
"constructed", "fits")}. Default is "all", any of the
estimates present in the fitted model that are listed in the
previous sentence will be included in the output. For the sake
of simplicity, we now allow one vector here, which applies to
all models in the object list.}

\item{paramSetLabels}{Named vector, used to supply alternative
pretty printing labels for parameter sets. The default values
are \code{c("loadings"= "Factor Loadings", "slopes" =
"Regression Slopes", "intercepts" = "Intercepts", "means"=
"Means", "residualvariances" = "Residual Variances",
"residualcovariances" = "Residual Covariances", "variances" =
"Variances", "latentvariances" = "Latent Variances",
"latentcovariances" = "Latent Covariances", "latentmeans" =
"Latent Intercepts", "thresholds" = "Thresholds",
"constructed" = "Constructed", "fits" = "Fit Indices")}.  The
paramSetLabels argument must be a named vector that overrides
some or all of the default names.}

\item{columns}{A vector naming estimates to appear for each model.
The allowed columns are "est", "se", "z", "p", "rsquare",
"estse", "eststars", "estsestars". The first 5 have the usual
meanings, while "estse" (can also be written \code{"est(se)"})
displays as, for example "1.21(0.23)", and the last 2 are to
include "significance stars".  \code{"eststars"} shows as
"1.21***" and \code{"estsestars"} (or \code{"est(se)stars"})
displays as "1.21(0.23)**". See parameter \code{alpha}. One
may request different columns for each model by providing a
named list of vectors.  Use model names in the list,
\code{list("Model A" = c("est", "se"), "Model B" = c("estse",
"p"))}.}

\item{columnLabels}{A named vector of "pretty labels" for the
headings in the table. The default labels are \code{c("est" =
"Estimate", se = "Std. Err.", z = "z", p = "p", rsquare =
"R Square", estse = "Estimate(Std.Err."), eststars =
"Estimate", estsestars = "Estimate(Std.Err.)")}.}

\item{fits}{Summary indicators to be included. May be a list, one
for each model provided, otherwise the same fit indicators
will be presented for each model. Any of the fit indicators
provided by \code{lavaan::fitMeasures(object)} are allowed:
\code{c("npar", "fmin", "chisq", "df", "pvalue",
"baseline.chisq", "baseline.df", "baseline.pvalue", "cfi",
"tli", "nnfi", "rfi", "nfi", "pnfi", "ifi", "rni", "logl",
"unrestricted.logl", "aic", "bic", "ntotal", "bic2", "rmsea",
"rmsea.ci.lower", "rmsea.ci.upper", "rmsea.pvalue", "rmr",
"rmr_nomean", "srmr", "srmr_bentler", "srmr_bentler_nomean",
"srmr_bollen", "srmr_bollen_nomean", "srmr_mplus",
"srmr_mplus_nomean", "cn_05", "cn_01", "gfi", "agfi", "pgfi",
"mfi", "ecvi")}. The return for "chisq" will include markup
for degrees of freedom and p value. If user specifies
\code{NULL}, or if "fits" is excluded from \code{paramSets},
all fit indicators are omitted.}

\item{fitLabels}{Labels for some or all of the fit measures
requested by the fits parameter, e.g. \code{c(rmsea =
"Root Mean Square Error of Approximation", cli = "CLI")}. The
default labels are the upper-case fits names (except for
"chisq", where a Greek letter is supplied when possible).}

\item{varLabels}{Named vector of labels to replace variable names
in column 1 of SEM table.}

\item{groups}{All groups will be printed, unless a subset is
requested here. Estimates for all groups will be displayed
side by side. If ONLY SOME groups should be included, then
specify groups as either names of fit objects or as integers
for elements of the groups vector.}

\item{type}{Choose "latex", "html", or "csv"}

\item{table.float}{If TRUE, create a LaTeX floating table object
in which the tabular created here will reside. Default is
FALSE.}

\item{caption}{Caption for table (if table.float=TRUE) or
longtable output. Ignored otherwise.}

\item{label}{LaTeX label for this object (for
cross-references). Only used if table.float = TRUE or
longtable = TRUE.}

\item{longtable}{If TRUE, use longtable for LaTeX
documents. Default is FALSE. If true, \code{table.float}
argument is ignored.}

\item{print.results}{If TRUE, marked up result will be displayed
within the session. Otherwise, result is returned silently and
user can use \code{cat} to dislay it. Don't use \code{print}
because it inserts unwanted decorations.}

\item{centering}{Default "siunitx". For method used in previous
editions, replace with "none".}

\item{alpha}{Thresholds for p-values that determine number of
stars.  Defaults as \code{c(0.05, 0.01, 0.001)} for
\code{c("*", "**", "***")}.}
}
\value{
Markup for SEM table. Includes an attribute
    "markedResults", which can be converted to other markup
    formats by the function markupConvert.
}
\description{
Creates LaTeX markup for structural equation modeling output
tables in the style of the American Psychological
Association(APA). Input objects should be created by the
"\code{lavaan}" package.
}
\details{
The argument paramSets determines the inclusion of estimate sections.
One can specify "all", which means that all types of parameters that
we can find in the fitted model are presented.  Otherwise, a subset
of parameter sets can be chosen by the user.
\itemize{
\item "loadings" are the factor loadings in the model.
\item "slopes" are the regression slopes in the model.
\item "intercepts" are the estimated constants in the measurement
     models.
\item "residualvariances" are the observed variable residual variances.
\item "residualcovariances" are the observed covariances among
residuals of observed variables.
\item "latentvariances" are the variances of unobserved variables.
\item "latentcovariances" are the covariances between unobserved variables.
\item "latentmeans" are means of unobserved variables
\item "thresholds" arise in latent
response variates (non-numeric indicator data).
\item "constructed" are parameters that are calculated from a formula
      in the model specification, such as an indirect path c=a*b.
\item "fits" the summary indicators of the mismatch between
the theoretical and observed covariance matrices, such as
RMSEA, CLI, TFI. While the fits are not technically parameters, they
are displayed in the same block style as parameters
}

The columns parameter is used to specify different columns,
while columnLabels will alter the displayed labels for them.
}
\examples{
\donttest{
## These run longer than 5 seconds
## CFA model
require(lavaan)

tempdir <- tempdir()
## The example from lavaan's docs
HS.model <- ' visual  =~ x1 + x2 + x3
              textual =~ x4 + x5 + x6
              speed   =~ x7 + x8 + x9'
fit1 <- cfa(HS.model, data = HolzingerSwineford1939,
            std.lv = TRUE, meanstructure = TRUE)
## Try a LaTeX file first
fit1.t1 <- semTable(fit1, columns = c("estse", "p"),
                    fits = c("chisq", "rmsea"), file = file.path(tempdir, "fit1.t1"),
                    varLabels = c("x1" = "hello"), type = "latex", print.results = FALSE)
## If you have a working version of pdflatex in your system path, 
if (interactive()) testtable("fit1.t1.tex", tempdir)
fit1.t1 <- semTable(fit1, columns = c("estse", "p"),
                    fits = c("chisq", "rmsea"), file = file.path(tempdir, "fit1.t1.html"),
                    varLabels = c("x1" = "hello"), type = "html", print.results = FALSE)
## Use file attribute to get full file name
browseURL(attr(fit1.t1, "file"))
## Try CSV output next
fit1.t1 <- semTable(fit1, columns = c("estse", "p"),
                    fits = c("chisq", "rmsea"), file = file.path(tempdir, "fit1.t1"),
                    varLabels = c("x1" = "hello"), type = "csv", print.results = FALSE)
## Go inspect this file with a spread sheet program:
attr(fit1.t1, "file")
## Now demonstrate variable labels
vl <- c(visual = "Visual", textual = "Textual", speed = "Speed",
       x1 = "V1", x2 = "V2", x3 = "V3")
fit1.t2 <- semTable(fit1, columns = c("est", "estse"),
                    fits = c("chisq", "rmsea"), file = file.path(tempdir, "fit1.t2"),
                    varLabels = vl)
if (interactive()) testtable("fit1.t2", tempdir)
## floating table
fit1.t3 <- semTable(fit1, columns = c("est", "estse"),
                    fits = c("chisq", "rmsea"), file = file.path(tempdir, "fit1.t3"),
                    varLabels = vl, table.float = TRUE,
                    caption = "Holzinger Swineford 1939",
                    label = "tab:hs1939")
if (interactive()) testtable("fit1.t3", tempdir)
fit1.t4 <- semTable(fit1, columns = c("est", "estse"),
                    fits = c("chisq", "rmsea"), file = file.path(tempdir, "fit1.t3"),
                    varLabels = vl, longtable = TRUE,
                    caption = "Holzinger Swineford 1939",
                    label = "tab:hs1939")
if (interactive()) testtable("fit1.t4", tempdir)
fit1.t5 <- semTable(fit1, fits = c("chisq", "rmsea"),
               columns = c("est", "se"), columnLabels = c(se = "S.E."),
               file = file.path(tempdir, "fit1.t5"))
if (interactive()) testtable("fit1.t5", tempdir)
fit1.t6 <- semTable(fit1, fits = c("chisq", "rmsea"),
               columns = c("estsestars"),
               columnLabels = c("estsestars" = "Est(SE)"),
               file = file.path(tempdir, "fit1.t6"))
if (interactive()) testtable("fit1.t6", tempdir)
vl <- c(x1 = "happy 1", x2 = "happy 2", x3 = "happy 3",
        visual = "Seeing", textual = "Thumb Texting")
fit1.t7 <- semTable(fit1, fits = c("chisq", "rmsea"),
               columns = c("eststars", "p"),
               columnLabels = c("eststars" = "Est(SE)"),
               file = file.path(tempdir, "fit1.t7"),
               varLabels = vl, longtable = TRUE, type = "latex")

if (interactive()) testtable("fit1.t7", tempdir)
# Change output format to csv
cat(markupConvert(attr(fit1.t7, "markedResults"), type = "csv"))

## 2 groups
fit1.g <- cfa(HS.model, data = HolzingerSwineford1939,
                        std.lv = TRUE, group = "school", estimator = "MLR")
fit1.gt1 <- semTable(fit1.g, columns = c("estsestars", "p"),
               columnLabels = c(estsestars = "Est w/stars", p = "p-value"),
               file = file.path(tempdir, "fit1.gt1"))
if (interactive()) testtable("fit1.gt1", tempdir)
## Now name particular group by name
fit1.gt2 <- semTable(fit1.g, columns = c("estsestars", "p"),
               columnLabels = c(estsestars = "Est w/stars", p = "p-value"),
               file = file.path(tempdir, "fit1.gt2"), groups = "Pasteur")
if (interactive()) testtable("fit1.gt2", tempdir)
## Name particular group by number
fit1.gt3 <- semTable(fit1.g, columns = c("estsestars", "p"),
               columnLabels = c(estsestars = "Est w/stars", p = "p-value"),
               file = file.path(tempdir, "fit1.gt3"), groups = 1)
if (interactive()) testtable("fit1.gt3", tempdir)

## Fit same model with standardization
fit1.std <- update(fit1, std.lv = TRUE, std.ov = TRUE, meanstructure = TRUE) 
## include 2 models in table request
fit1.t2 <- semTable(list("Ordinary" = fit1, "Standardized" = fit1.std),
                    file = file.path(tempdir, "fit1.2.1"))
semTable(list("Ordinary" = fit1, "Standardized" = fit1.std),
    columns = list("Ordinary" = c("est", "se"), "Standardized" = c("est")),
    columnLabels = c(est = "Est", se = "SE"), file = file.path(tempdir, "fit1.2.2"))
if (interactive()) testtable("fit1.2.2", tempdir)

fit1.t2 <- semTable(fit1, fits = c("chisq", "rmsea"), print.results = FALSE)
cat(fit1.t2)
fit1.t3 <- semTable(fit1, fits = c("chisq", "rmsea", "tli"),
               columns = c("est", "se"))

## Can create file with cat
cat(fit1.t3, file = file.path(tempdir, "fit1.t3.tex"))

## Basic SEM
regmodel1 <- 'visual  =~ x1 + x2 + x3 
             textual =~ x4 + x5 + x6
             speed   =~ x7 + x8 + x9
             visual ~ textual + speed
'

fit2 <- sem(regmodel1, data = HolzingerSwineford1939, std.lv = TRUE,
            meanstructure = TRUE)

fit2.std <- update(fit2, std.lv = TRUE, std.ov = TRUE, meanstructure = TRUE) 

fit2.t <- semTable(list("Ordinary" = fit2, "Standardized" = fit2.std), fits = "rmsea",
                   columns = list("Ordinary" = c("est", "se", "p"),
                                   "Standardized" = c("estsestars")),
                   columnLabels = c("est" = "Est", "se" = "Std.Err.", "p" = "p",
                                    "estsestars" = "Standardized Est."),
                   paramSets = c("loadings", "intercepts", "slopes", "latentcovariances"),
                   file = file.path(tempdir, "fit2.t1"), type = "latex")

if (interactive()) testtable("fit2.t1", tempdir)

# Change output format to csv
cat(markupConvert(attr(fit2.t, "markedResults"), type = "csv"))
cat(markupConvert(attr(fit2.t, "markedResults"), type = "html"))

fit2.t <- semTable(list("Ordinary" = fit2, "Standardized" = fit2.std),
              type = c("html"),
              file = file.path(tempdir, "fit2.t"),
              varLabels = c(x1 = "happy 1", x2 = "happy 2", x3 = "happy 3"),
              print.results = FALSE)

if (interactive()) browseURL(file.path(tempdir, "fit2.t.html"))


regmodel2 <- 'visual  =~ x1 + x2 + x3
              textual =~ x4 +  x6
              speed   =~  x8 + x9
              visual ~ textual 
'
fit3 <- sem(regmodel2, data = HolzingerSwineford1939, std.lv = TRUE,
            meanstructure = TRUE)

fit3.t1 <-  semTable(fit3, type = c("latex"),
                     columns = c("estsestars", "rsquare"), 
                     file = file.path(tempdir, "fit3.1"),
                     print.results = FALSE)

cat(fit3.t1)
if (interactive()) testtable("fit3.1", tempdir)

fit3.std <- update(fit2, std.lv = TRUE, std.ov = TRUE)

fit3.std.t1 <- semTable(list("Mod 1" = fit2, "Mod 1 std" = fit2.std, "Mod 2" = fit3,
              "Mod 3 std" = fit3.std), columns = c("estsestars"), type = "html",
               file = file.path(tempdir, "fit3.std.t1"), print.results = FALSE)
if(interactive()) browseURL(file.path(tempdir, "fit3.std.t1.html"))

fit3 <- sem(regmodel1, data = HolzingerSwineford1939, group = "school")
## if specify 2 types, get a list of them back
fit3.t1 <- semTable(fit3, type = "latex")
fit3.t2 <- semTable(fit3, columns = c("est", "se"),
                     columnLabels = c(est = "Est.", se = "S.E."))

fit3.t2 <- semTable(fit3, fits = c("chisq", "rmsea", "cfi"), print.results = FALSE)
cat(fit3.t2)

fit3.t2 <- semTable(fit3, columns = c("estsestars"),
            fits = c("chisq", "rmsea", "cfi"), type = "html",
            file = file.path(tempdir, "fit3.t2"))
if(interactive()) browseURL(file.path(tempdir, "fit3.t2.html"))
 
fit3.t2 <- semTable(fit3, fits = c("rmsea", "cfi"))

model <- "factor =~ .7*y1 + .7*y2 + .7*y3 + .7*y4
                 y1 | -1*t1 + 1*t2
                 y2 | -.5*t1 + 1*t2
                 y3 | -.2*t1 + 1*t2
                 y4 | -1*t1 + 1*t2"
dat <- simulateData(model, sample.nobs = 300)

testmodel <- "ExampleFactor =~ y1 + y2 + y3 + y4"

fit4 <- cfa(testmodel, data = dat, ordered = colnames(dat),
            std.lv = FALSE)

fit4.t1 <- semTable(fit4, paramSets = c("loadings", "thresholds",
    "residualvariances"), fits = c("tli", "chisq"),
    fitLabels = c(tli = "TLI", chisq = "chisq"), type = "html")

fit4.t2 <- semTable(fit4, fits = c("rmsea", "cfi", "chisq"),
              fitLabels = c(rmsea = "Root M.SQ.E.A", cfi = "CompFitIdx", chisq = "chisq"),
              type = "latex")

## Model 5 - Mediation model with equality constraints
model5 <-
    '
    # latent variable definitions
    ind60 =~ x1 + x2 + x3
    dem60 =~ y1 + e*y2 + d*y3 + y4
    dem65 =~ y5 + e*y6 + d*y7 + y8
    # regressions
    dem60 ~ a*ind60
    dem65 ~ c*ind60 + b*dem60
    # residual correlations
    y1 ~~ y5
    y2 ~~ y4 + y6
    y3 ~~ y7
    y4 ~~ y8
    y6 ~~ y8

    # indirect effect (a*b)
    ## := operator defines new parameters
    ab := a*b

    ## total effect
    total := c + (a*b)
    '

fit5 <- sem(model5, data=PoliticalDemocracy)
fit5boot <- sem(model5, data=PoliticalDemocracy, se = "bootstrap", bootstrap = 100)

fit5.t1 <- semTable(list("Democracy" = fit5), columns = c("estsestars", "rsquare"),
           file = file.path(tempdir, "fit5.1t"), type = "html",
           print.results = FALSE)
if(interactive()) browseURL(file.path(tempdir, "fit5.t1.html"))

## convert same object to LaTeX, save file, then test
cat(markupConvert(attr(fit5.t1, "markedResults"), type = "latex"),
         file = file.path(tempdir, "fit5.t1.tex"))
if (interactive()) testtable("fit5.t1", tempdir)

semTable(list("Democracy" = fit5, "Bootstrapped SE" = fit5boot),
         columns = c("estsestars", "rsquare"),
         file = file.path(tempdir, "fit5.2"), type = "latex",
         longtable = TRUE)

semTable(list("Democracy" = fit5, "Bootstrapped SE" = fit5boot),
         columns = c("estsestars", "rsquare"),
         paramSets = c("loadings", "slopes", "residualvariances", "constructed"),
         file = file.path(tempdir, "fit5.3"), type = "latex",
         longtable = TRUE)

## Model 5b - Revision of Model 5s
model5b <-
    '
    # Cut some indicators from the measurement model
    ind60 =~ x1 + x2 
    dem60 =~ y1 + e*y2 + d*y3 + y4
    dem65 =~ y5 + e*y6 + d*y7 
    # regressions
    dem60 ~ a*ind60
    dem65 ~ c*ind60 + b*dem60
    # cut out the residual correlations
    # indirect effect (a*b)
    ## := operator defines new parameters
    ab := a*b

    ## total effect
    total := c + (a*b)
    '

fit5b <- sem(model5b, data=PoliticalDemocracy, se = "bootstrap",
bootstrap = 100)
semTable(list("Model 5" = fit5boot, "Model 5b" = fit5b),
         columns = c("estsestars", "rsquare"),
         file = file.path(tempdir, "fit5.5"),
          type = "latex",
         longtable = TRUE)
if(interactive()) testtable("fit5.5", tempdir)

list.files(tempdir)
}
}
\author{
Ben Kite <bakite@ku.edu> Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/peek.R
\name{peek}
\alias{peek}
\alias{histOMatic}
\title{Show variables, one at a time, QUICKLY and EASILY.}
\usage{
peek(dat, sort = TRUE, file = NULL, textout = FALSE, ask, ...,
  xlabstub = "kutils peek: ", freq = FALSE,
  histargs = list(probability = !freq), barargs = list(horiz = TRUE,
  las = 1))
}
\arguments{
\item{dat}{An R data frame or something that can be coerced to a
data frame by \code{as.data.frame}}

\item{sort}{Default TRUE. Do you want display of the columns in
alphabetical order?}

\item{file}{Should output go in file rather than to the screen.
Default is NULL, meaning show on screen. If you supply a file
name, we will write PDF output into it.}

\item{textout}{If TRUE, counts from histogram bins and tables will
appear in the console.}

\item{ask}{As in the old style R \code{par(ask = TRUE)}: should
keyboard interaction advance to the next plot.  Will default
to false if the file argument is non-null.  If file is null,
setting ask = FALSE will cause graphs to whir bye without
pausing.}

\item{...}{Additional arguments for the pdf, histogram, table, or
barplot functions. Please see Details below.}

\item{xlabstub}{A text stub that will appear in the x axis
label. Currently it includes advertising for this package.}

\item{freq}{As in the histogram frequency argument. Should graphs
show counts (freq = TRUE) or proportions (AKA densities) (freq
= FALSE)}

\item{histargs}{A list of arguments to be passed to the
\code{hist} function.}

\item{barargs}{A list of arguments to be passed to the
\code{barplot} function.}
}
\value{
A vector of column names that were plotted
}
\description{
This makes it easy to quickly scan through all of the columns in a
data frame to spot unexpected patterns or data entry errors.  Numeric variables are depicted as
histograms, while factor and character variables are summarized by
the R table function and then presented as barplots. This is most
useful with a large screen graphic device (try running the function
provided with this package, \code{dev.create(height=7, width=7)})
or any other method you prefer to create a large device.
}
\section{Try the Defaults}{
 Every effort has been made to make this
    simple and easy to use. Please run the examples as they are
    before becoming too concerned about customization.  This
    function is intended for getting a quick look at each
    variable, one-by-one, it is not intended to create publication
    quality histograms.  For sake of the fastidious users, a lot
    of settings can be adjusted. Users can control the parameters
    for presentation of histograms (parameters for \code{hist})
    and barplots (parameters for \code{barplot}). The function also
    can create frequency tables (which users can control by providing
    additional named arguments).
}

\section{Style}{
 The histograms are standard, upright histograms.
    The barplots are horizontal. I chose to make the bars
    horizontal because long value labels are more easily
    accomodated on the left axis.  The code measures the length
    (in inches) for strings and the margin is increased
    accordingly.  The examples have a demonstration of that
    effect.
}

\section{Dealing with Dots}{
 additional named arguments,
    \code{...}, are inspected and sorted into groups intended to
    control use of R functions \code{hist}, \code{barplot},
    \code{table} and \code{pdf}.  \cr \cr The parameters
    c("exclude", "dnn", "useNA", "deparse.level") and will go to
    the \code{table} function, which is used to make barplots for
    factor and character variables. These named arguments are
    extracted and sent to the pdf function: c("width", "height",
    "onefile", "family", "title", "fonts", "version", "paper",
    "encoding", "bg", "fg", "pointsize", "pagecentre",
    "colormodel", "useDingbats", "useKerning", "fillOddEven",
    "compress"). Any other arguments that are unique to
    \code{hist} or \code{barplot} are sorted out and sent only to
    those functions.  \cr \cr Any other arguments, including
    graphical parameters will be sent to both the histogram and
    barplot functions, so it is a convenient way to obtain uniform
    appearance. Additional arguments that are common to
    \code{barplot} and \code{hist} will work, and so will any
    graphics parameters (named arguments of \code{par}, for
    example). However, if one wants to target some arguments to
    \code{hist}, but not \code{barplot}, then the \code{histargs}
    list argument should be used. Similarly, \code{barargs} should
    be used to send argument to the \code{barplot}
    function. Warning: the defaults for \code{histargs} and
    \code{barargs} include some settings that are needed for the
    existing design.  If new lists for \code{histargs} or
    \code{barargs} are supplied, the previously specified defaults
    are lost.  Hence, users should include the existing members of
    those lists, possibly with revised values.  \cr \cr All of
    this argument sorting effort is done in order to reduce a
    prolific number of warnings that were observed in previous
    editions of this function.
}

\examples{
\donttest{
set.seed(234234)
N <- 200
mydf <- data.frame(x5 = rnorm(N), x4 = rnorm(N), x3 = rnorm(N),
                   x2 = letters[sample(1:24, 200, replace = TRUE)],
                   x1 = factor(sample(c("cindy", "bobby", "marsha",
                                        "greg", "chris"), 200, replace = TRUE)),
                   stringsAsFactors = FALSE)
## Insert 16 missings
mydf$x1[sample(1:150, 16,)] <- NA
mydf$adate <- as.Date(c("1jan1960", "2jan1960", "31mar1960", "30jul1960"), format = "\%d\%b\%y")
peek(mydf)
peek(mydf, sort = FALSE)
## Demonstrate the dot-dot-dot usage to pass in hist params
peek(mydf, breaks = 30, ylab = "These are Counts, not Densities", freq = TRUE)
## Not Run: file output
## peek(mydf, sort = FALSE, file = "three_histograms.pdf")
## Use some objects from the datasets package
library(datasets)
peek(cars, xlabstub = "R cars data: ")
peek(EuStockMarkets, xlabstub = "Euro Market Data: ")
peek(EuStockMarkets, xlabstub = "Euro Market Data: ", breaks = 50,
     freq = TRUE)
## Not run: file output
## peek(EuStockMarkets, breaks = 50, file = "myeuro.pdf",
##      height = 4, width=3, family = "Times")
## peek(EuStockMarkets, breaks = 50, file = "myeuro-\%d3.pdf",
##      onefile = FALSE, family = "Times", textout = TRUE)
## xlab goes into "..." and affects both histograms and barplots
peek(mydf, breaks = 30, ylab = "These are Counts, not Densities",
    freq = TRUE)
## xlab is added in the barargs list.
peek(mydf, breaks = 30, ylab = "These are Counts, not Densities",
    freq = TRUE, barargs = list(horiz = TRUE, las = 1, xlab = "I'm in barargs"))
peek(mydf, breaks = 30, ylab = "These are Counts, not Densities", freq = TRUE,
     barargs = list(horiz = TRUE, las = 1, xlim = c(0, 100),
     xlab = "I'm in barargs, not in histargs"))
levels(mydf$x1) <- c(levels(mydf$x1), "arthur philpot smythe")
mydf$x1[4] <- "arthur philpot smythe"
mydf$x2[1] <- "I forgot what letter"
peek(mydf, breaks = 30,
     barargs = list(horiz = TRUE, las = 1))
}
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{cleanDataFrame}
\alias{cleanDataFrame}
\title{Check and Clean data.frame for usage with variable key functions}
\usage{
cleanDataFrame(dframe, safeNumericToInteger = TRUE, trimws = "both")
}
\arguments{
\item{dframe}{A data frame}

\item{safeNumericToInteger}{Default TRUE: Should we treat values
which appear to be integers as integers? If a column is
numeric, it might be safe to treat it as an integer.  In many
csv data sets, the values coded c(1, 2, 3) are really
integers, not floats c(1.0, 2.0, 3.0). See \code{safeInteger}.}

\item{trimws}{Defaults as "both", in meaning of \code{which}
argument in \code{trimws} function.  Set as NULL if character
variables must not be trimmed to eliminate white
space. Otherwise, value should be one of \code{c("left",
"right", "both")}.}
}
\value{
A checked and cleaned data frame
}
\description{
Checks that the data.frame is made up of simple individual
columns. Checks numeric columns to find out if they are acceptable
to treat as integers. If they are acceptable to treat as integers,
then convert those numeric to integer class variables.
}
\author{
Paul Johnson <pauljohn@ku.edu>
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deleteBogusRows.R
\name{deleteBogusRows}
\alias{deleteBogusRows}
\title{Remove rows in which the proportion of missing data exceeds
a threshold.}
\usage{
deleteBogusRows(dframe, pm = 0.9, drop = FALSE, verbose = TRUE,
  n = 25)
}
\arguments{
\item{dframe}{A data frame or matrix}

\item{pm}{"proportion missing data" to be tolerated.}

\item{drop}{Default FALSE: if data frame result is reduced to one
row, should R's default drop behavior "demote" this to a
column vector.}

\item{verbose}{Default TRUE. Should a report be printed
summarizing information to be delted?}

\item{n}{Default 25: limit on number of values to print in verbose
diagnostic output. If set to NULL or NA, then all of the
column values will be printed for the bogus rows.}
}
\value{
a data frame, invisibly
}
\description{
If cases are mostly missing, delete them. It often happens that
when data is imported from other sources, some noise rows exist at
the bottom of the input.  Anything that is missing in more than,
say, 90\% of cases is probably useless information.  We invented
this to deal with problem that MS Excel users often include a
marginal note at the bottom of a spread sheet.
}
\examples{
mymat <- matrix(rnorm(10*100), nrow = 10, ncol = 100,
               dimnames = list(1:10, paste0("x", 1:100)))
mymat <- rbind(mymat, c(32, rep(NA, 99)))
mymat2 <- deleteBogusRows(mymat)
mydf <- as.data.frame(mymat)
mydf$someFactor <- factor(sample(c("A", "B"), size = NROW(mydf), replace = TRUE))
mydf2 <- deleteBogusRows(mydf, n = "all")
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{dms}
\alias{dms}
\title{Delete multiple slashes, replace with one}
\usage{
dms(name)
}
\arguments{
\item{name}{A character string to clean}
}
\description{
Sometimes paths end up with "/too//many//slashes".
While harmless, this is untidy. Clean it up.
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/peek.R
\name{peek}
\alias{peek}
\alias{histOMatic}
\title{Show variables, one at a time, QUICKLY and EASILY.}
\usage{
peek(
  dat,
  sort = TRUE,
  file = NULL,
  textout = FALSE,
  ask,
  ...,
  xlabstub = "kutils peek: ",
  freq = FALSE,
  histargs = list(probability = !freq),
  barargs = list(horiz = TRUE, las = 1)
)
}
\arguments{
\item{dat}{An R data frame or something that can be coerced to a
data frame by \code{as.data.frame}}

\item{sort}{Default TRUE. Do you want display of the columns in
alphabetical order?}

\item{file}{Should output go in file rather than to the screen.
Default is NULL, meaning show on screen. If you supply a file
name, we will write PDF output into it.}

\item{textout}{If TRUE, counts from histogram bins and tables will
appear in the console.}

\item{ask}{As in the old style R \code{par(ask = TRUE)}: should
keyboard interaction advance to the next plot.  Will default
to false if the file argument is non-null.  If file is null,
setting ask = FALSE will cause graphs to whir bye without
pausing.}

\item{...}{Additional arguments for the pdf, histogram, table, or
barplot functions. Please see Details below.}

\item{xlabstub}{A text stub that will appear in the x axis
label. Currently it includes advertising for this package.}

\item{freq}{As in the histogram frequency argument. Should graphs
show counts (freq = TRUE) or proportions (AKA densities) (freq
= FALSE)}

\item{histargs}{A list of arguments to be passed to the
\code{hist} function.}

\item{barargs}{A list of arguments to be passed to the
\code{barplot} function.}
}
\value{
A vector of column names that were plotted
}
\description{
This makes it easy to quickly scan through all of the columns in a
data frame to spot unexpected patterns or data entry errors.  Numeric variables are depicted as
histograms, while factor and character variables are summarized by
the R table function and then presented as barplots. This is most
useful with a large screen graphic device (try running the function
provided with this package, \code{dev.create(height=7, width=7)})
or any other method you prefer to create a large device.
}
\section{Try the Defaults}{
 Every effort has been made to make this
    simple and easy to use. Please run the examples as they are
    before becoming too concerned about customization.  This
    function is intended for getting a quick look at each
    variable, one-by-one, it is not intended to create publication
    quality histograms.  For sake of the fastidious users, a lot
    of settings can be adjusted. Users can control the parameters
    for presentation of histograms (parameters for \code{hist})
    and barplots (parameters for \code{barplot}). The function also
    can create frequency tables (which users can control by providing
    additional named arguments).
}

\section{Style}{
 The histograms are standard, upright histograms.
    The barplots are horizontal. I chose to make the bars
    horizontal because long value labels are more easily
    accomodated on the left axis.  The code measures the length
    (in inches) for strings and the margin is increased
    accordingly.  The examples have a demonstration of that
    effect.
}

\section{Dealing with Dots}{
 additional named arguments,
    \code{...}, are inspected and sorted into groups intended to
    control use of R functions \code{hist}, \code{barplot},
    \code{table} and \code{pdf}.  \cr \cr The parameters
    c("exclude", "dnn", "useNA", "deparse.level") and will go to
    the \code{table} function, which is used to make barplots for
    factor and character variables. These named arguments are
    extracted and sent to the pdf function: c("width", "height",
    "onefile", "family", "title", "fonts", "version", "paper",
    "encoding", "bg", "fg", "pointsize", "pagecentre",
    "colormodel", "useDingbats", "useKerning", "fillOddEven",
    "compress"). Any other arguments that are unique to
    \code{hist} or \code{barplot} are sorted out and sent only to
    those functions.  \cr \cr Any other arguments, including
    graphical parameters will be sent to both the histogram and
    barplot functions, so it is a convenient way to obtain uniform
    appearance. Additional arguments that are common to
    \code{barplot} and \code{hist} will work, and so will any
    graphics parameters (named arguments of \code{par}, for
    example). However, if one wants to target some arguments to
    \code{hist}, but not \code{barplot}, then the \code{histargs}
    list argument should be used. Similarly, \code{barargs} should
    be used to send argument to the \code{barplot}
    function. Warning: the defaults for \code{histargs} and
    \code{barargs} include some settings that are needed for the
    existing design.  If new lists for \code{histargs} or
    \code{barargs} are supplied, the previously specified defaults
    are lost.  Hence, users should include the existing members of
    those lists, possibly with revised values.  \cr \cr All of
    this argument sorting effort is done in order to reduce a
    prolific number of warnings that were observed in previous
    editions of this function.
}

\examples{
\donttest{
set.seed(234234)
N <- 200
mydf <- data.frame(x5 = rnorm(N), x4 = rnorm(N), x3 = rnorm(N),
                   x2 = letters[sample(1:24, 200, replace = TRUE)],
                   x1 = factor(sample(c("cindy", "bobby", "marsha",
                                        "greg", "chris"), 200, replace = TRUE)),
                   stringsAsFactors = FALSE)
## Insert 16 missings
mydf$x1[sample(1:150, 16,)] <- NA
mydf$adate <- as.Date(c("1jan1960", "2jan1960", "31mar1960", "30jul1960"), format = "\%d\%b\%y")
peek(mydf)
peek(mydf, sort = FALSE)
## Demonstrate the dot-dot-dot usage to pass in hist params
peek(mydf, breaks = 30, ylab = "These are Counts, not Densities", freq = TRUE)
## Not Run: file output
## peek(mydf, sort = FALSE, file = "three_histograms.pdf")
## Use some objects from the datasets package
library(datasets)
peek(cars, xlabstub = "R cars data: ")
peek(EuStockMarkets, xlabstub = "Euro Market Data: ")
peek(EuStockMarkets, xlabstub = "Euro Market Data: ", breaks = 50,
     freq = TRUE)
## Not run: file output
## peek(EuStockMarkets, breaks = 50, file = "myeuro.pdf",
##      height = 4, width=3, family = "Times")
## peek(EuStockMarkets, breaks = 50, file = "myeuro-\%d3.pdf",
##      onefile = FALSE, family = "Times", textout = TRUE)
## xlab goes into "..." and affects both histograms and barplots
peek(mydf, breaks = 30, ylab = "These are Counts, not Densities",
    freq = TRUE)
## xlab is added in the barargs list.
peek(mydf, breaks = 30, ylab = "These are Counts, not Densities",
    freq = TRUE, barargs = list(horiz = TRUE, las = 1, xlab = "I'm in barargs"))
peek(mydf, breaks = 30, ylab = "These are Counts, not Densities", freq = TRUE,
     barargs = list(horiz = TRUE, las = 1, xlim = c(0, 100),
     xlab = "I'm in barargs, not in histargs"))
levels(mydf$x1) <- c(levels(mydf$x1), "arthur philpot smythe")
mydf$x1[4] <- "arthur philpot smythe"
mydf$x2[1] <- "I forgot what letter"
peek(mydf, breaks = 30,
     barargs = list(horiz = TRUE, las = 1))
}
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keyCrossRef}
\alias{keyCrossRef}
\title{keyCrossRef}
\usage{
keyCrossRef(key, ignoreClass = NULL, verbose = FALSE,
  lowercase = FALSE)
}
\arguments{
\item{key}{A variable key, ideally a long key. If a wide key is
provided it is converted to long.}

\item{ignoreClass}{Classes that should be excluded from
check. Useful when many integer variables are being reverse-
coded. Takes a string or vector.}

\item{verbose}{Should a statement about the number of issues
detected be returned? Defaults to FALSE.}

\item{lowercase}{Should old and new values be passed through
tolower function? Defaults to FALSE.}
}
\value{
Presents a warning for potentially problematic key
    sections. Return is dependent on verbose argument.
}
\description{
Checks a key for dangerous matches of old and new values in a key
for different levels.
}
\details{
Positions in a long key are referred to as levels. If a value is
mismatched at levels 1 and 3, this means that issues are in rows 1
and 3 of the section of the given variable in a long key.
}
\examples{
dat <- data.frame(x1 = sample(c("a", "b", "c", "d"), 100, replace = TRUE),
                  x2 = sample(c("Apple", "Orange"), 100, replace = TRUE),
                  x3 = ordered(sample(c("low", "medium", "high"), 100, replace = TRUE),
                  levels = c("low", "medium", "high")),
                  stringsAsFactors = FALSE)
key <- keyTemplate(dat, long = TRUE)
## No errors with a fresh key.
kutils:::keyCrossRef(key, verbose = TRUE)
key[1:2, "value_new"] <- c("b", "a")
key[5, "value_new"]
key[7:9, "value_new"] <- c("high", "medium", "low")
kutils:::keyCrossRef(key)
kutils:::keyCrossRef(key, ignoreClass = c("ordered", "character"), verbose = TRUE)
}
\author{
Ben Kite <bakite@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deleteBogusRows.R
\name{deleteBogusRows}
\alias{deleteBogusRows}
\title{Remove rows in which the proportion of missing data exceeds
a threshold.}
\usage{
deleteBogusRows(dframe, pm = 0.9, drop = FALSE, verbose = TRUE, n = 25)
}
\arguments{
\item{dframe}{A data frame or matrix}

\item{pm}{"proportion missing data" to be tolerated.}

\item{drop}{Default FALSE: if data frame result is reduced to one
row, should R's default drop behavior "demote" this to a
column vector.}

\item{verbose}{Default TRUE. Should a report be printed
summarizing information to be delted?}

\item{n}{Default 25: limit on number of values to print in verbose
diagnostic output. If set to NULL or NA, then all of the
column values will be printed for the bogus rows.}
}
\value{
a data frame, invisibly
}
\description{
If cases are mostly missing, delete them. It often happens that
when data is imported from other sources, some noise rows exist at
the bottom of the input.  Anything that is missing in more than,
say, 90\% of cases is probably useless information.  We invented
this to deal with problem that MS Excel users often include a
marginal note at the bottom of a spread sheet.
}
\examples{
mymat <- matrix(rnorm(10*100), nrow = 10, ncol = 100,
               dimnames = list(1:10, paste0("x", 1:100)))
mymat <- rbind(mymat, c(32, rep(NA, 99)))
mymat2 <- deleteBogusRows(mymat)
mydf <- as.data.frame(mymat)
mydf$someFactor <- factor(sample(c("A", "B"), size = NROW(mydf), replace = TRUE))
mydf2 <- deleteBogusRows(mydf, n = "all")
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{shorten}
\alias{shorten}
\title{Reduce each in a vector of strings to a given length}
\usage{
shorten(x, k = 20, unique = FALSE)
}
\arguments{
\item{x}{character string}

\item{k}{integer limit on length of string. Default is 20}

\item{unique}{Default FALSE}
}
\value{
vector of character variables no longer than k
}
\description{
This is a simple "chop" at k characters, no fancy truncation at
spaces or such. Optionally, this will make unique the resulting
truncated strings. That way, truncation at character 4 of
"Washington" and "Wash" and "Washingham" will not result in 3
values of "Wash", but rather "Wash", "Wash.1", and "Wash.2"
}
\examples{
x <- c("Washington", "Washingham", "Washmylaundry")
shorten(x, 4)
shorten(x, 4, unique = TRUE)
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keyLookup}
\alias{keyLookup}
\title{Look for old (or new) names in variable key}
\usage{
keyLookup(x, key, get = "name_old")
}
\arguments{
\item{x}{A variable name. If \code{get = "name_old"}, then
\code{x} is a value for \code{name_new}. If \code{get =
"name_new"}, \code{x} should be a value for \code{name_old}.}

\item{key}{Which key should be used}

\item{get}{Either "name_old" (to retrieve the original name) or
"name_new" (to get the new name)}
}
\value{
A vector or list of matches between x and either name_new
    or name_old elements in the key.
}
\description{
Use the key to find the original name of a variable that has been
renamed, or find the new name of an original variable.  The
\code{get} argument indicates if the \code{name_old} or
\code{name_new} is desired.
}
\details{
If \code{get = "name_old"}, the return is a character vector, with
one element per value of \code{x}.  If there is no match for a
value of \code{x}, the value NA is returned for that
value. However, if \code{get = "name_new"}, the return might be
either a vector (one element per value of \code{x}) or a list with
one element for each value of \code{x}.  The list is returned when
a value of \code{x} corresponds to more than one element in
\code{name_old}.
}
\examples{
mydf.key.path <- system.file("extdata", "mydf.key.csv", package = "kutils")
mydf.key <-  keyImport(mydf.key.path)
mydf.key$name_new <- paste0("new_", mydf.key$name_new)
keyLookup("new_x5", mydf.key, get = "name_old")
keyLookup(c("new_x6", "new_x1"), mydf.key, get = "name_old")
keyLookup(c("x6", "x1"), mydf.key, get = "name_new")
keyLookup(c("asdf", "new_x1"), mydf.key, get = "name_old")

mydf.key <- rbind(mydf.key,
                 c("x3", "x3f",  "ordered", "factor", "","","",""))
keyLookup(c("x3"), mydf.key, get = "name_new")
keyLookup(c("x1", "x3", "x5"), mydf.key, get = "name_new")
}
\author{
Paul Johnson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keyCrossRef}
\alias{keyCrossRef}
\title{keyCrossRef}
\usage{
keyCrossRef(key, ignoreClass = NULL, verbose = FALSE, lowercase = FALSE)
}
\arguments{
\item{key}{A variable key, ideally a long key. If a wide key is
provided it is converted to long.}

\item{ignoreClass}{Classes that should be excluded from
check. Useful when many integer variables are being reverse-
coded. Takes a string or vector.}

\item{verbose}{Should a statement about the number of issues
detected be returned? Defaults to FALSE.}

\item{lowercase}{Should old and new values be passed through
tolower function? Defaults to FALSE.}
}
\value{
Presents a warning for potentially problematic key
    sections. Return is dependent on verbose argument.
}
\description{
Checks a key for dangerous matches of old and new values in a key
for different levels.
}
\details{
Positions in a long key are referred to as levels. If a value is
mismatched at levels 1 and 3, this means that issues are in rows 1
and 3 of the section of the given variable in a long key.
}
\examples{
dat <- data.frame(x1 = sample(c("a", "b", "c", "d"), 100, replace = TRUE),
                  x2 = sample(c("Apple", "Orange"), 100, replace = TRUE),
                  x3 = ordered(sample(c("low", "medium", "high"), 100, replace = TRUE),
                  levels = c("low", "medium", "high")),
                  stringsAsFactors = FALSE)
key <- keyTemplate(dat, long = TRUE)
## No errors with a fresh key.
kutils:::keyCrossRef(key, verbose = TRUE)
key[1:2, "value_new"] <- c("b", "a")
key[5, "value_new"]
key[7:9, "value_new"] <- c("high", "medium", "low")
kutils:::keyCrossRef(key)
kutils:::keyCrossRef(key, ignoreClass = c("ordered", "character"), verbose = TRUE)
}
\author{
Ben Kite <bakite@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{long2wide}
\alias{long2wide}
\title{convert a key object from long to wide format}
\usage{
long2wide(keylong, na.strings = c("\\\\.", "", "\\\\s+", "N/A"),
  missSymbol = ".")
}
\arguments{
\item{keylong}{A variable key in the long format}

\item{na.strings}{Strings to be treated as missings in value_new}

\item{missSymbol}{Default is ".", character to insert in value when R NA is found.}
}
\value{
A wide format variable key
}
\description{
##' This is not flexible, assumes columns are named in our canonical
style, which means the columns are named c("name_old", "name_new",
"class_old", "class_new", "value_old", "value_new").
}
\examples{
mydf.path <- system.file("extdata", "mydf.csv", package = "kutils")
mydf <- read.csv(mydf.path, stringsAsFactors=FALSE)
## A wide key we are trying to match:
mydf.key <- keyTemplate(mydf, long = FALSE, sort = TRUE)
mydf.key["x4", "missings"] <- "999"
## A long key we will convert next
mydf.keylong <- keyTemplate(mydf, long = TRUE, sort = TRUE)
mydf.keylong[mydf.keylong[ , "name_old"] == "x4" &
    mydf.keylong[ , "value_old"] == "999", "missings"] <- "999"
mydf.long2wide <- long2wide(mydf.keylong)
all.equal(mydf.key, mydf.long2wide)

mydf.keylong.path <- system.file("extdata", "mydf.key_long.csv", package = "kutils")
mydf.keylong <- keyImport(mydf.keylong.path)
mydf.keywide <- long2wide(mydf.keylong)
mydf.keylong2 <- wide2long(mydf.keywide)
## Is error if following not TRUE
all.equal(mydf.keylong2, mydf.keylong)

}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{statdatKey}
\alias{statdatKey}
\title{keyFactors: private function that does work for keyTemplateSPSS and
key template Stata}
\usage{
statdatKey(datf, datn, long = TRUE)
}
\arguments{
\item{datf}{Data frame with factors}

\item{datn}{Numeric data frame}

\item{long}{Should the result be a long or wide key}
}
\description{
keyFactors: private function that does work for keyTemplateSPSS and
key template Stata
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{sortStanza}
\alias{sortStanza}
\title{Move missing values to last row in long key block}
\usage{
sortStanza(keyblock, valvar = "value_new", na.strings = c("\\\\.", "",
  "\\\\s+", "N/A"))
}
\arguments{
\item{keyblock}{a variable key, or section of rows}

\item{valvar}{Default is "value_new", the column for sorting.}

\item{na.strings}{Stings to be treated as missing, along with R's
NA symbol}
}
\value{
keyblock, row re-arranged with missings last
}
\description{
Receive key stanza and sort the rows so that the missing
values are last in the list. Leaves ordering of other rows
unchanged otherwise.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{long2wide}
\alias{long2wide}
\title{convert a key object from long to wide format}
\usage{
long2wide(
  keylong,
  na.strings = c("\\\\.", "", "\\\\s+", "N/A"),
  missSymbol = "."
)
}
\arguments{
\item{keylong}{A variable key in the long format}

\item{na.strings}{Strings to be treated as missings in value_new}

\item{missSymbol}{Default is ".", character to insert in value when R NA is found.}
}
\value{
A wide format variable key
}
\description{
##' This is not flexible, assumes columns are named in our canonical
style, which means the columns are named c("name_old", "name_new",
"class_old", "class_new", "value_old", "value_new").
}
\examples{
mydf.path <- system.file("extdata", "mydf.csv", package = "kutils")
mydf <- read.csv(mydf.path, stringsAsFactors=FALSE)
## A wide key we are trying to match:
mydf.key <- keyTemplate(mydf, long = FALSE, sort = TRUE)
mydf.key["x4", "missings"] <- "999"
## A long key we will convert next
mydf.keylong <- keyTemplate(mydf, long = TRUE, sort = TRUE)
mydf.keylong[mydf.keylong[ , "name_old"] == "x4" &
    mydf.keylong[ , "value_old"] == "999", "missings"] <- "999"
mydf.long2wide <- long2wide(mydf.keylong)
all.equal(mydf.key, mydf.long2wide)

mydf.keylong.path <- system.file("extdata", "mydf.key_long.csv", package = "kutils")
mydf.keylong <- keyImport(mydf.keylong.path)
mydf.keywide <- long2wide(mydf.keylong)
mydf.keylong2 <- wide2long(mydf.keywide)
## Is error if following not TRUE
all.equal(mydf.keylong2, mydf.keylong)

}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{isNA}
\alias{isNA}
\title{Check if values are R NA symbol or any one of the na.strings
elements}
\usage{
isNA(x, na.strings = c("\\\\.", "", "\\\\s+", "N/A"))
}
\arguments{
\item{x}{Input data vector}

\item{na.strings}{Vector of string values to be considered as
missing. Defaults will match values that are equal to ., empty
string, any number of white space elements, or charcter
string N/A. We do not include `NA` by default because some
projects use NA to mean "not appropriate".}
}
\value{
Logical vector, TRUE if a value is either NA or in
    na.strings.
}
\description{
A value vector in the key will generally be a character
vector.  This utility is used to check if the characters
are either R missing or values in a list of characters that
represent missings.
}
\examples{
x1 <- c("TRUE", "FALSE", FALSE, TRUE, NA, "NA", ".", "N/A", " ", "")
x1na <- kutils:::isNA(x1)
cbind(x1, x1na)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keysPoolCheck}
\alias{keysPoolCheck}
\title{Compares keys from different data sets; finds differences classes of variables.
This used to check for similarity of keys from various data sets,
one precursor to either combining the keys or merging the data
sets themselves.}
\usage{
keysPoolCheck(keys, col = "class_old", excludere = "TEXT$")
}
\arguments{
\item{keys}{A list with variable keys.}

\item{col}{Name of key column to check for equivalence. Default is "class_old", but
"class_new" can be checked as well.}

\item{excludere}{Exclude variables matching a regular expression
(re). Default example shows exclusion of variables that end in
the symbol "TEXT".}
}
\value{
Data.frame summarizing class differences among keys
}
\description{
When several supposedly "equivalent" data sets are used
to generate variable keys, there may be trouble. If variables
with same name have different classes, keyApply might fail
when applied to one of the data sets.
}
\details{
This reports on differences in classes among keys. By default, it
looks for differences in "class_old", because that's where we
usually see trouble.

The output here is diagnostic. The keys can be fixed manually, or the
function keysPool can implement an automatic correction.
}
\examples{
set.seed(234)
dat1 <- data.frame(x1 = rnorm(100),
                   x2 = sample(c("Male", "Female"), 100, replace = TRUE),
                   x3_TEXT = "A", x4 = sample(1:10000, 100))
dat2 <- data.frame(x1 = rnorm(100), x2 = sample(c("Male", "Female"),
                   100, replace = TRUE),
                   x3_TEXT = sample(1:100, 100),
                   stringsAsFactors = FALSE)
key1 <- keyTemplate(dat1)
key2 <- keyTemplate(dat2)
keys <- list(key1, key2)
keysPoolCheck(keys)
## See problem in class_old
keysPoolCheck(keys, col = "class_old")
## problems in class_new
keysPoolCheck(keys, col = "class_new")
keysPoolCheck(keys, excludere = "TEXT$")
}
\author{
Paul Johnson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{sortStanza}
\alias{sortStanza}
\title{Move missing values to last row in long key block}
\usage{
sortStanza(
  keyblock,
  valvar = "value_new",
  na.strings = c("\\\\.", "", "\\\\s+", "N/A")
)
}
\arguments{
\item{keyblock}{a variable key, or section of rows}

\item{valvar}{Default is "value_new", the column for sorting.}

\item{na.strings}{Stings to be treated as missing, along with R's
NA symbol}
}
\value{
keyblock, row re-arranged with missings last
}
\description{
Receive key stanza and sort the rows so that the missing
values are last in the list. Leaves ordering of other rows
unchanged otherwise.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{zapspace}
\alias{zapspace}
\title{Convert leading or trailing white space and tab characters to nothing.}
\usage{
zapspace(x)
}
\arguments{
\item{x}{A character vector}
}
\value{
If x is a character vector, return is a character vector
    with leading and trailing white space values removed. If x is
    not a character vector, an unaltered x is returned.
}
\description{
This eliminates any characters matched by the regular expression
`\\s` if they appear at the beginning of a string or at its
end. It does not alter spaces in the interior of a string. This
was created when I was not aware of R's \code{trimws} and the purpose
is the same. On our TODO list, we intend to eliminate this function
and replace its use with \code{trimws}
}
\examples{
x <-  c("", " ", "   ", "\\t", "\\t some", "some\\t", " space first")
zapspace(x)
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{stringbreak}
\alias{stringbreak}
\title{Insert "\\n" after the k'th character in a string. This IS vectorized,
so can receive just one or many character strings in a vector.}
\usage{
stringbreak(x, k = 20)
}
\arguments{
\item{x}{Character string.}

\item{k}{Number of characters after which to insert "\\n". Default is 20}
}
\value{
Character with "\\n" inserted
}
\description{
If a string is long, insert linebreak "\\n"
}
\details{
If x is not a character string, x is returned without alteration. And
without a warning
}
\examples{
x <- "abcdef ghijkl mnopqrs tuvwxyz abc def ghi jkl mno pqr stv"
stringbreak(x, 10)
stringbreak(x, 20)
stringbreak(x, 25)
x <- c("asdf asdfjl asfdjkl asdfjklasdfasd", "qrweqwer qwerqwerjklqw erjqwe")
stringbreak(x, 5)
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/peek.R
\name{dev.create}
\alias{dev.create}
\title{Create a graphics device}
\usage{
dev.create(...)
}
\arguments{
\item{...}{Currently, height and width parameters that would be
suitable with dev.new}
}
\description{
This is a way to create a graphic device on screen that can
display R plots. It is performing the same purpose as R's dev.new,
but it overcomes the limitations of RStudio.  It is needed because
RStudio does not implement fully the functionality of
dev.new. This is suitable for Windows, Linux, and
Macintosh operating systems.
}
\details{
The argument in dev.new named noRStudioGD seems to be aimed at same
purpose. But it does not do what I want and documentation is too
sparse.
}
\examples{
\donttest{
if(interactive()) dev.create(height = 7, width = 3)
dev.off()
}
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{zapspace}
\alias{zapspace}
\title{Convert leading or trailing white space and tab characters to nothing.}
\usage{
zapspace(x)
}
\arguments{
\item{x}{A character vector}
}
\value{
If x is a character vector, return is a character vector
    with leading and trailing white space values removed. If x is
    not a character vector, an unaltered x is returned.
}
\description{
This eliminates any characters matched by the regular expression
`\\s` if they appear at the beginning of a string or at its
end. It does not alter spaces in the interior of a string. This
was created when I was not aware of R's \code{trimws} and the purpose
is the same. On our TODO list, we intend to eliminate this function
and replace its use with \code{trimws}
}
\examples{
x <-  c("", " ", "   ", "\t", "\t some", "some\t", " space first")
zapspace(x)
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mergeCheck.R
\name{mergeCheck}
\alias{mergeCheck}
\title{First draft of function to diagnose problems in merges and key variables}
\usage{
mergeCheck(x, y, by, by.x = by, by.y = by, incomparables = c(NULL,
  NA, NaN, Inf, "\\\\s+", ""))
}
\arguments{
\item{x}{data frame}

\item{y}{data frame}

\item{by}{Commonly called the "key" variable. A column name to be
used for merging (common to both \code{x} and \code{y})}

\item{by.x}{Column name in \code{x} to be used for merging. If not
supplied, then \code{by.x} is assumed to be same as \code{by}.}

\item{by.y}{Column name in \code{y} to be used for merging. If not
supplied, then \code{by.y} is assumed to be same as \code{by}.}

\item{incomparables}{values in the key (by) variable that are
ignored for matching. We default to include these values as
incomparables: c(NULL, NA, NaN, Inf, "\\s+", ""). Note this is
a larger list of incomparables than assumed by R merge (which
assumes only NULL).}
}
\value{
A list of data structures that are displayed for keys and
    data sets.  The return is \code{list(keysBad, keysDuped,
    unmatched)}. \code{unmatched} is a list with 2 elements, the
    unmatched cases from \code{x} and \code{y}.
}
\description{
This is a first effort. It works with 2 data frames and 1 key
variable in each. It does not work if the by parameter includes
more than one column name (but may work in future). The return is
a list which includes full copies of the rows from the data frames
in which trouble is observed.
}
\examples{
df1 <- data.frame(id = 1:7, x = rnorm(7))
df2 <- data.frame(id = c(2:6, 9:10), x = rnorm(7))
mc1 <- mergeCheck(df1, df2, by = "id")
## Use mc1 objects mc1$keysBad, mc1$keysDuped, mc1$unmatched
df1 <- data.frame(id = c(1:3, NA, NaN, "", " "), x = rnorm(7))
df2 <- data.frame(id = c(2:6, 5:6), x = rnorm(7))
mergeCheck(df1, df2, by = "id")
df1 <- data.frame(idx = c(1:5, NA, NaN), x = rnorm(7))
df2 <- data.frame(idy = c(2:6, 9:10), x = rnorm(7))
mergeCheck(df1, df2, by.x = "idx", by.y = "idy")
}
\author{
Paul Johnson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{isNA}
\alias{isNA}
\title{Check if values are R NA symbol or any one of the na.strings
elements}
\usage{
isNA(x, na.strings = c("\\\\.", "", "\\\\s+", "N/A"))
}
\arguments{
\item{x}{Input data vector}

\item{na.strings}{Vector of string values to be considered as
missing. Defaults will match values that are equal to ., empty
string "", any number of white space elements, or charcter
string "N/A". We do not include "NA" by default because some
projects use "NA" to mean "not appropriate".}
}
\value{
Logical vector, TRUE if a value is either NA or in
    na.strings.
}
\description{
A value vector in the key will generally be a character
vector.  This utility is used to check if the characters
are either R missing or values in a list of characters that
represent missings.
}
\keyword{"}
\keyword{"")}
\keyword{",}
\keyword{".",}
\keyword{"FALSE",}
\keyword{"N/A",}
\keyword{"NA",}
\keyword{##}
\keyword{<-}
\keyword{@examples}
\keyword{FALSE,}
\keyword{NA,}
\keyword{TRUE,}
\keyword{c("TRUE",}
\keyword{cbind(x1,}
\keyword{internal}
\keyword{isNA(x1)}
\keyword{x1}
\keyword{x1na}
\keyword{x1na)}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keyImport}
\alias{keyImport}
\title{Import/validate a key object or import/validate a key from a file.}
\usage{
keyImport(key, ignoreCase = TRUE, sep = c(character = "\\\\|", logical
  = "\\\\|", integer = "\\\\|", factor = "\\\\|", ordered = "[\\\\|<]",
  numeric = "\\\\|"), na.strings = c("\\\\.", "", "\\\\s+", "N/A"),
  missSymbol = ".", ..., keynames = NULL)
}
\arguments{
\item{key}{A key object (class key or keylong) or a file name
character string (ending in csv, xlsx or rds).}

\item{ignoreCase}{In the use of this key, should we ignore
differences in capitalization of the "name_old" variable?
Sometimes there are inadvertent misspellings due to changes in
capitalization. Columns named "var01" and "Var01" and "VAR01"
probably should receive the same treatment, even if the key
has name_old equal to "Var01".}

\item{sep}{Character separator in \code{value_old} and
\code{value_new} strings in a wide key. Default is are "|".
It is also allowed to use "<" for ordered variables. Use
regular expressions in supplying separator values.}

\item{na.strings}{Values that should be converted to missing data.
This is relevant in \code{name_new} as well as
\code{value_new}. In spreadsheet cells, we treat "empty" cells
(the string ""), or values like "." or "N/A", as missing with
defaults ".", "", "\\s" (white space), and "N/A". Change that
if those are not to be treated as missings.}

\item{missSymbol}{Defaults to period "." as missing value
indicator.}

\item{...}{additional arguments for read.csv or read.xlsx.}

\item{keynames}{Don't use this unless you are very careful. In our
current scheme, the column names in a key should be
c("name_old", "name_new", "class_old", "class_new",
"value_old", "value_new", "missings", "recodes"). If your key
does not use those column names, it is necessary to provide
keynames in a format "our_name"="your_name". For example,
keynames = c(name_old = "oldvar", name_new = "newname",
class_old = "vartype", class_new = "class", value_old =
"score", value_new = "val").}
}
\value{
key object, should be same "wide" or "long" as the input
    Missing symbols in value_old and value_new converted to ".".
}
\description{
After the researcher has updated the key by filling in new names
and values, we import that key file. This function can import the
file by its name, after deducing the file type from the suffix, or
it can receive a key object from memory.
}
\details{
This can be either a wide or long format key file.

This cleans up variables in following ways.  1) \code{name_old}
and \code{name_new} have leading and trailing spaces removed 2)
\code{value_old} and \code{value_new} have leading and trailing
spaces removed, and if they are empty or blank spaces, then new
values are set as NA.

Policy change concerning empty "value_new" cells in input keys
(20170929).

There is confusion about what ought to happen in a wide key when
the user leaves value_new as empty or missing. Literally, this
means all values are converted to missing, which does not seem
reasonable. Hence, when a key is wide, and value_new is one of the
na.strings elements, we assume the value_new is to be copied
from value_old. That is to say, if value_new is not supplied,
the values remain same as in old data.

In a long key, the behavior is different.  Since the user can
specify each value for a variable in a separate row, the na.strings
appearing in value_new are treated as missing scores in the new
data set to be created.
}
\examples{
mydf.key.path <- system.file("extdata", "mydf.key.csv", package = "kutils")
mydf.key <-  keyImport(mydf.key.path)
## Create some dupes
mydf.key <- rbind(mydf.key, mydf.key[c(1,7), ])
mydf.key2 <- keyImport(mydf.key)
mydf.key2
## create some empty value_new cells
mydf.key[c(3, 5, 7) , "value_new"] <- ""
mydf.key3 <- keyImport(mydf.key)
mydf.key3
mydf.keylong.path <- system.file("extdata", "mydf.key_long.csv", package = "kutils")
mydf.keylong <- keyImport(mydf.keylong.path)

## testDF is a slightly more elaborate version created for unit testing:
testdf.path <- system.file("extdata", "testDF.csv", package = "kutils")
testdf <- read.csv(testdf.path, header = TRUE)
keytemp <- keyTemplate(testdf, long = TRUE)
## A "hand edited key file"
keyPath <- system.file("extdata", "testDF-key.csv", package="kutils")
key <- keyImport(keyPath)
keydiff <- keyDiff(keytemp, key)
key2 <- rbind(key, keydiff$neworaltered)
key2 <- unique(key)
if(interactive())View(key2)

}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{naLast}
\alias{naLast}
\title{Sort key so that non missing values are first in the
value vector.}
\usage{
naLast(key, byvar = "name_new", valvar = "value_new",
  na.strings = c("\\\\.", "", "\\\\s+", "N/A"))
}
\arguments{
\item{key}{key, long or wide}

\item{byvar}{Default is "name_new", the column for sorting into blocks.}

\item{valvar}{The value variable on which sorting is to be done, "value_new".}

\item{na.strings}{vector of characters to be treated as NA}
}
\value{
sorted key, with blocks that have the missings last
}
\description{
Sort key so that non missing values are first in the
value vector.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mergeCheck.R
\name{mergeCheck}
\alias{mergeCheck}
\title{First draft of function to diagnose problems in merges and key variables}
\usage{
mergeCheck(
  x,
  y,
  by,
  by.x = by,
  by.y = by,
  incomparables = c(NULL, NA, NaN, Inf, "\\\\s+", "")
)
}
\arguments{
\item{x}{data frame}

\item{y}{data frame}

\item{by}{Commonly called the "key" variable. A column name to be
used for merging (common to both \code{x} and \code{y})}

\item{by.x}{Column name in \code{x} to be used for merging. If not
supplied, then \code{by.x} is assumed to be same as \code{by}.}

\item{by.y}{Column name in \code{y} to be used for merging. If not
supplied, then \code{by.y} is assumed to be same as \code{by}.}

\item{incomparables}{values in the key (by) variable that are
ignored for matching. We default to include these values as
incomparables: c(NULL, NA, NaN, Inf, "\\s+", ""). Note this is
a larger list of incomparables than assumed by R merge (which
assumes only NULL).}
}
\value{
A list of data structures that are displayed for keys and
    data sets.  The return is \code{list(keysBad, keysDuped,
    unmatched)}. \code{unmatched} is a list with 2 elements, the
    unmatched cases from \code{x} and \code{y}.
}
\description{
This is a first effort. It works with 2 data frames and 1 key
variable in each. It does not work if the by parameter includes
more than one column name (but may work in future). The return is
a list which includes full copies of the rows from the data frames
in which trouble is observed.
}
\examples{
df1 <- data.frame(id = 1:7, x = rnorm(7))
df2 <- data.frame(id = c(2:6, 9:10), x = rnorm(7))
mc1 <- mergeCheck(df1, df2, by = "id")
## Use mc1 objects mc1$keysBad, mc1$keysDuped, mc1$unmatched
df1 <- data.frame(id = c(1:3, NA, NaN, "", " "), x = rnorm(7))
df2 <- data.frame(id = c(2:6, 5:6), x = rnorm(7))
mergeCheck(df1, df2, by = "id")
df1 <- data.frame(idx = c(1:5, NA, NaN), x = rnorm(7))
df2 <- data.frame(idy = c(2:6, 9:10), x = rnorm(7))
mergeCheck(df1, df2, by.x = "idx", by.y = "idy")
}
\author{
Paul Johnson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{natlongsurv}
\alias{natlongsurv}
\title{Smoking, Happiness, and other survey responses}
\format{A data frame with 2867 rows and 29 variables:

\itemize{
  \item	R0000100	IDENTIFICATION CODE
  \item	R0003300	MARITAL STATUS, 1968
  \item	R0005700	AGE WHEN STOPPED ATTENDING SCHOOL, 1968
  \item	R0060300	IQ SCORE, 1968
  \item	R1051600	HIGHEST GRADE COMPLETED
  \item	R1302000	SMOKING - DOES R SMOKE, 1991
  \item	R1302100	SMOKING - NUMBER OF CIGARETTES R SMOKES PER DAY, 91 (PRESENT SMOKER)
  \item	R6235600	HIGHEST GRADE COMPLETED
  \item	R6502300	IS RESIDENCE/LIVING QUARTERS HOME/APARTMENT/OTHER?
  \item	R6513700	HOUSEHOLD RECORD - HOUSEHOLD MEMBER - AGE CALCULATED FROM BIRTH DATE
  \item	R6516200	CURRENT MARITAL STATUS
  \item	R6520300	HIGHEST GRADE COMPLETED OF HUSBAND
  \item	R6553600	HIGHEST GRADE COMPLETED OF PARTNER
  \item	R7289200	SMOKING - CURRENTLY SMOKE CIGARETTES
  \item	R7289400	ALCOHOL USE - HAS R CONSUMED ANY ALCOHOLIC BEVERAGES IN PAST MONTH?
  \item	R7293430	YOUNG WOMEN 20-ITEM CES-D ITEM RESPONSE SCORE
  \item	R7312300	INCOME FROM WAGES/SALARY IN PAST YEAR
  \item	R7329900	INCOME ADEQUACY: R OPINION OF HER HAPPINESS WITH HER/FAMILY INCOME
  \item	R7330000	INCOME ADEQUACY: R OPINION OF AMOUNT NEEDED TO MAKE ENDS MEET \$ AMOUNT
  \item	R7337600	R HAS ATTENDED/COMPLETED TWO/MORE YEARS OF COLLEGE
  \item	R7344600	ATTITUDE TOWARD FEELINGS OVERALL
  \item	R7344700	DID R DO ANY UNPAID VOLUNTEER WORK IN PAST YEAR?
  \item	R7347500	ATTITUDE TOWARD SOCIAL SECURITY - PERCENT WOULD INVEST IN STOCKS? 2004
  \item	R7347600	ATTITUDE TOWARD SOCIAL SECURITY - PERCENT WOULD INVEST IN BONDS OF PRIVATE COMPANIES? 2004
  \item	R7347700	ATTITUDE TOWARD SOCIAL SECURITY - PERCENT WOULD INVEST IN U.S. GOVERNMENT BONDS? 2004
  \item	R7477700	TOTAL CHILDREN IN ROSTER
  \item	R7477800	COUNT ELIGIBLE HOUSEHOLD CHILDREN
  \item	R7610300	REGION OF RESIDENCE
}}
\source{
National Longitudinal Surveys public-use data set (Bureau of Labor Statistics, 2018).
}
\usage{
data(natlongsurv)
}
\description{
An idiosyncratic selection of 29 variables from the Original
Cohort-Young Women 1968-2003 edition of the US National
Longitudinal Survey.  This originally included 5159 rows, but
subset includes only 2867 rows, so sample frequencies will not
match the values listed in the codebook.  A snapshot of the
codebook, "natlongsurv.cdb.txt", which we have trimmed down, is
included in the package.
}
\details{
All variables are for the 2003 year, except where otherwise noted.
}
\examples{
data(natlongsurv)
peek(natlongsurv, ask = FALSE, file = paste0(tempdir(), "/","peek.pdf"))
}
\references{
Bureau of Labor Statistics. 2018.  NLS Original Cohort: Mature and Young Women, US National Longitudinal Surveys Public Use Data Sets \url{https://www.bls.gov/nls/home.htm}.
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keyImport}
\alias{keyImport}
\title{Import/validate a key object or import/validate a key from a file.}
\usage{
keyImport(
  key,
  ignoreCase = TRUE,
  sep = c(character = "\\\\|", logical = "\\\\|", integer = "\\\\|", factor = "\\\\|",
    ordered = "[\\\\|<]", numeric = "\\\\|"),
  na.strings = c("\\\\.", "", "\\\\s+", "N/A"),
  missSymbol = ".",
  ...,
  keynames = NULL
)
}
\arguments{
\item{key}{A key object (class key or keylong) or a file name
character string (ending in csv, xlsx or rds).}

\item{ignoreCase}{In the use of this key, should we ignore
differences in capitalization of the "name_old" variable?
Sometimes there are inadvertent misspellings due to changes in
capitalization. Columns named "var01" and "Var01" and "VAR01"
probably should receive the same treatment, even if the key
has name_old equal to "Var01".}

\item{sep}{Character separator in \code{value_old} and
\code{value_new} strings in a wide key. Default is are "|".
It is also allowed to use "<" for ordered variables. Use
regular expressions in supplying separator values.}

\item{na.strings}{Values that should be converted to missing data.
This is relevant in \code{name_new} as well as
\code{value_new}. In spreadsheet cells, we treat "empty" cells
(the string ""), or values like "." or "N/A", as missing with
defaults ".", "", "\\s" (white space), and "N/A". Change that
if those are not to be treated as missings.}

\item{missSymbol}{Defaults to period "." as missing value
indicator.}

\item{...}{additional arguments for read.csv or read.xlsx.}

\item{keynames}{Don't use this unless you are very careful. In our
current scheme, the column names in a key should be
c("name_old", "name_new", "class_old", "class_new",
"value_old", "value_new", "missings", "recodes"). If your key
does not use those column names, it is necessary to provide
keynames in a format "our_name"="your_name". For example,
keynames = c(name_old = "oldvar", name_new = "newname",
class_old = "vartype", class_new = "class", value_old =
"score", value_new = "val").}
}
\value{
key object, should be same "wide" or "long" as the input
    Missing symbols in value_old and value_new converted to ".".
}
\description{
After the researcher has updated the key by filling in new names
and values, we import that key file. This function can import the
file by its name, after deducing the file type from the suffix, or
it can receive a key object from memory.
}
\details{
This can be either a wide or long format key file.

This cleans up variables in following ways.  1) \code{name_old}
and \code{name_new} have leading and trailing spaces removed 2)
\code{value_old} and \code{value_new} have leading and trailing
spaces removed, and if they are empty or blank spaces, then new
values are set as NA.

Policy change concerning empty "value_new" cells in input keys
(20170929).

There is confusion about what ought to happen in a wide key when
the user leaves value_new as empty or missing. Literally, this
means all values are converted to missing, which does not seem
reasonable. Hence, when a key is wide, and value_new is one of the
na.strings elements, we assume the value_new is to be copied
from value_old. That is to say, if value_new is not supplied,
the values remain same as in old data.

In a long key, the behavior is different.  Since the user can
specify each value for a variable in a separate row, the na.strings
appearing in value_new are treated as missing scores in the new
data set to be created.
}
\examples{
mydf.key.path <- system.file("extdata", "mydf.key.csv", package = "kutils")
mydf.key <-  keyImport(mydf.key.path)
## Create some dupes
mydf.key <- rbind(mydf.key, mydf.key[c(1,7), ])
mydf.key2 <- keyImport(mydf.key)
mydf.key2
## create some empty value_new cells
mydf.key[c(3, 5, 7) , "value_new"] <- ""
mydf.key3 <- keyImport(mydf.key)
mydf.key3
mydf.keylong.path <- system.file("extdata", "mydf.key_long.csv", package = "kutils")
mydf.keylong <- keyImport(mydf.keylong.path)

## testDF is a slightly more elaborate version created for unit testing:
testdf.path <- system.file("extdata", "testDF.csv", package = "kutils")
testdf <- read.csv(testdf.path, header = TRUE)
keytemp <- keyTemplate(testdf, long = TRUE)
## A "hand edited key file"
keyPath <- system.file("extdata", "testDF-key.csv", package="kutils")
key <- keyImport(keyPath)
keydiff <- keyDiff(keytemp, key)
key2 <- rbind(key, keydiff$neworaltered)
key2 <- unique(key)
if(interactive())View(key2)

}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{mgsub}
\alias{mgsub}
\title{apply a vector of replacements, one after the other.}
\usage{
mgsub(pattern, replacement, x, ...)
}
\arguments{
\item{pattern}{vector of values to be replaced. A vector filled
with patterns as documented in the \code{gsub} pattern
argument}

\item{replacement}{vector of replacements, otherwise same as
\code{gsub}.  Length of replacement must be either 1 or same
as pattern, otherwise an error results.}

\item{x}{the vector in which elements are to be replaced, same as
\code{gsub}}

\item{...}{Additional arguments to be passed to gsub}
}
\value{
vector with pattern replaced by replacement
}
\description{
This is multi-gsub.  Use it when it is necessary to process
many patterns and replacements in a given order on a vector.
}
\examples{
x <- c("Tom", "Jerry", "Elmer", "Bugs")
pattern <- c("Tom", "Bugs")
replacement <- c("Thomas", "Bugs Bunny")
(y <- mgsub(pattern, replacement, x))
x[1] <- "tom"
(y <- mgsub(pattern, replacement, x, ignore.case = TRUE))
(y <- mgsub(c("Elmer", "Bugs"), c("Looney Characters"), x, ignore.case = TRUE))
}
\author{
Jared Harpole <jared.harpole@gmail.com> and Paul Johnson
    <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{naLast}
\alias{naLast}
\title{Sort key so that non missing values are first in the
value vector.}
\usage{
naLast(
  key,
  byvar = "name_new",
  valvar = "value_new",
  na.strings = c("\\\\.", "", "\\\\s+", "N/A")
)
}
\arguments{
\item{key}{key, long or wide}

\item{byvar}{Default is "name_new", the column for sorting into blocks.}

\item{valvar}{The value variable on which sorting is to be done, "value_new".}

\item{na.strings}{vector of characters to be treated as NA}
}
\value{
sorted key, with blocks that have the missings last
}
\description{
Sort key so that non missing values are first in the
value vector.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/textUtils.R
\name{escape}
\alias{escape}
\title{Text that is to be included as content in documents is cleaned
(escaped) to prevent errors}
\usage{
escape(x, type = "tex")
}
\arguments{
\item{x}{a string, or vector of strings (each of which is
processed separately)}

\item{type}{"tex" is default, could be "filename" or "html"}
}
\value{
corrected character vector
}
\description{
This is for fixing up "untrusted text" that is to be passed into a
file as content. It protects against "bad" text strings in 3
contexts, 1) LaTeX documents, 2) HTML documents, or 3) text in a
file name. It converts content text to an improved string that
will not cause failures in the eventual document.
}
\details{
The special in-document LaTeX symbols like percent sign or dollar sign
are "%" and "$". *Warning*: In the R
session, these will appear as double-backslashed symbols, while in
a saved text file, there will only be the one desired slash.

If type = "html", we only clean up <, >, / and &, and quote
characters.  If document is in unicode, we don't need to do the
gigantic set anymore.

If type = "filename", then symbols that are not allowed in file
names, such as "\", "*", are replaced. Do not use this on a
full path, since it will obliterate path separators.
}
\examples{
x1 <- c("_asdf&_&$", "asd adf asd_", "^ \% & $asdf_")
escape(x1)
x2 <- c("a>b", "a<b", "a < c", 'Paul "pj" Johnson')
escape(x2, type = "tex")
escape(x2, type = "html")
escape(x2, type = "filename")
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{natlongsurv}
\alias{natlongsurv}
\title{Smoking, Happiness, and other survey responses}
\format{
A data frame with 2867 rows and 29 variables:

\itemize{
  \item	R0000100	IDENTIFICATION CODE
  \item	R0003300	MARITAL STATUS, 1968
  \item	R0005700	AGE WHEN STOPPED ATTENDING SCHOOL, 1968
  \item	R0060300	IQ SCORE, 1968
  \item	R1051600	HIGHEST GRADE COMPLETED
  \item	R1302000	SMOKING - DOES R SMOKE, 1991
  \item	R1302100	SMOKING - NUMBER OF CIGARETTES R SMOKES PER DAY, 91 (PRESENT SMOKER)
  \item	R6235600	HIGHEST GRADE COMPLETED
  \item	R6502300	IS RESIDENCE/LIVING QUARTERS HOME/APARTMENT/OTHER?
  \item	R6513700	HOUSEHOLD RECORD - HOUSEHOLD MEMBER - AGE CALCULATED FROM BIRTH DATE
  \item	R6516200	CURRENT MARITAL STATUS
  \item	R6520300	HIGHEST GRADE COMPLETED OF HUSBAND
  \item	R6553600	HIGHEST GRADE COMPLETED OF PARTNER
  \item	R7289200	SMOKING - CURRENTLY SMOKE CIGARETTES
  \item	R7289400	ALCOHOL USE - HAS R CONSUMED ANY ALCOHOLIC BEVERAGES IN PAST MONTH?
  \item	R7293430	YOUNG WOMEN 20-ITEM CES-D ITEM RESPONSE SCORE
  \item	R7312300	INCOME FROM WAGES/SALARY IN PAST YEAR
  \item	R7329900	INCOME ADEQUACY: R OPINION OF HER HAPPINESS WITH HER/FAMILY INCOME
  \item	R7330000	INCOME ADEQUACY: R OPINION OF AMOUNT NEEDED TO MAKE ENDS MEET $ AMOUNT
  \item	R7337600	R HAS ATTENDED/COMPLETED TWO/MORE YEARS OF COLLEGE
  \item	R7344600	ATTITUDE TOWARD FEELINGS OVERALL
  \item	R7344700	DID R DO ANY UNPAID VOLUNTEER WORK IN PAST YEAR?
  \item	R7347500	ATTITUDE TOWARD SOCIAL SECURITY - PERCENT WOULD INVEST IN STOCKS? 2004
  \item	R7347600	ATTITUDE TOWARD SOCIAL SECURITY - PERCENT WOULD INVEST IN BONDS OF PRIVATE COMPANIES? 2004
  \item	R7347700	ATTITUDE TOWARD SOCIAL SECURITY - PERCENT WOULD INVEST IN U.S. GOVERNMENT BONDS? 2004
  \item	R7477700	TOTAL CHILDREN IN ROSTER
  \item	R7477800	COUNT ELIGIBLE HOUSEHOLD CHILDREN
  \item	R7610300	REGION OF RESIDENCE
}
}
\source{
National Longitudinal Surveys public-use data set (Bureau of Labor Statistics, 2018).
}
\usage{
data(natlongsurv)
}
\description{
An idiosyncratic selection of 29 variables from the Original
Cohort-Young Women 1968-2003 edition of the US National
Longitudinal Survey.  This originally included 5159 rows, but
subset includes only 2867 rows, so sample frequencies will not
match the values listed in the codebook.  A snapshot of the
codebook, "natlongsurv.cdb.txt", which we have trimmed down, is
included in the package.
}
\details{
All variables are for the 2003 year, except where otherwise noted.
}
\examples{
data(natlongsurv)
peek(natlongsurv, ask = FALSE, file = paste0(tempdir(), "/","peek.pdf"))
}
\references{
Bureau of Labor Statistics. 2018.  NLS Original Cohort: Mature and Young Women, US National Longitudinal Surveys Public Use Data Sets (www.bls.gov/nls/home.htm).
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{n2NA}
\alias{n2NA}
\title{Convert nothing to R missing(NA).}
\usage{
n2NA(x, nothings = c("\\\\.", "\\\\s"), zapspace = TRUE)
}
\arguments{
\item{x}{A character vector. If x is not a character vector, it is
returned unaltered without warning.}

\item{nothings}{A vector of values to be matched by regular
expressions as missing.  The default vector is c("\\.",
"\\s"), where "\\." means a literal period (backslashes needed
to escape the symbol which would otherwise match anything in a
regular expression).}

\item{zapspace}{Should leading and trailing white space be
ignored, so that, for example " . " and "." are both treated
as missing.}
}
\value{
A vector with "nothing" values replaced by R's NA symbol.
    Does not alter other values in the vector. Previous version
    had applied zapspace to non-missing values, but it no longer
    does so.
}
\description{
By "nothing", we mean white space or other indications of
nothingness.  Goal is to find character strings that users
might insert in a key to indicate missing values. Those things,
which are given default values in the argument nothings, will be
changed to NA.
}
\details{
Using regular expression matching, any value that has nothing
except for the indicated "nothing" values is converted to NA.  The
"nothing" values included by default are a period by itself (A SAS
missing value), an empty string, or white space, meaning " ", or
any number of spaces, or a tab.
}
\examples{
gg <- c("", " ", "   ", "\\t", "\\t some", "some\\t", " space first", ".",
       " . ")
n2NA(x = gg)
n2NA(x = gg, zapspace = FALSE)
n2NA(x = gg, nothings = c("\\\\s"), zapspace = FALSE)
n2NA(x = gg, nothings = c("\\\\."), zapspace = TRUE)
n2NA(x = gg, nothings = c("\\\\."), zapspace = FALSE)
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keyTemplate}
\alias{keyTemplate}
\title{Create variable key template (in memory or in a file)}
\usage{
keyTemplate(dframe, long = FALSE, sort = FALSE, file = NULL,
  max.levels = 15, missings = NULL, missSymbol = ".",
  safeNumericToInteger = TRUE, trimws = "both", varlab = FALSE)
}
\arguments{
\item{dframe}{A data frame}

\item{long}{Default FALSE.}

\item{sort}{Default FALSE. Should the rows representing the
variables be sorted alphabetically? Otherwise, they appear in
the order in which they were included in the original dataset.}

\item{file}{DEFAULT NULL, meaning no file is produced. Choose a
file name ending in either "csv" (for comma separated
variables), "xlsx" (compatible with Microsoft Excel), or "rds"
(R serialization data). The file name will be used to select
among the 3 storage formats. XLSX output requires the openxlsx
package.}

\item{max.levels}{How high is the limit on the number of values
for discrete (integer, character, and Date) variables?
Default = 15. If observed number exceeds max.levels, we
conclude the author should not assign new values in the key
and only the missing value will be included in the key as a
"placeholder". This does not affect variables declared as
factor or ordered variables, for which all levels are included
in all cases.}

\item{missings}{Values in exising data which should be treated as
missing in the new key. Character string in format acceptable
to the \code{assignMissing} function. Can be a string with
several missing indicators"1;2;3;(8,10);[22,24];> 99;< 2".}

\item{missSymbol}{Default ".".  A character string used to
represent missing values in the key that is created.  Relevant
(mostly) for the key's \code{value_new} column. Default is the
period, ".". Because R's symbol \code{NA} can be mistaken for
the character string \code{"NA"}, we use a different
(hopefully unmistakable) symbol in the key.}

\item{safeNumericToInteger}{Default TRUE: Should we treat values
which appear to be integers as integers? If a column is
numeric, it might be safe to treat it as an integer.  In many
csv data sets, the values coded c(1, 2, 3) are really
integers, not floats c(1.0, 2.0, 3.0). See \code{safeInteger}.}

\item{trimws}{Default is "both", user can change to "left", "right", or
set as NULL to avoid any trimming.}

\item{varlab}{A key can have a companion data structure for
variable labels. Default is FALSE, but the value may also be
TRUE or a named vector of variable labels, such as
\code{c("x1" = "happiness", "x2" = "wealth")}. The labels
become an attribute of the key object. See Details for
information on storage of varlabs in saved key files.}
}
\value{
A key in the form of a data frame. May also be saved on
    disk if the file argument is supplied. The key may have an
    attribute "varlab", variable labels.
}
\description{
A variable key is a human readable document that describes the
variables in a data set. A key can be revised and re-imported by R
to recode data. This might also be referred to as a
"programmable codebook."  This function inspects a data frame,
takes notice of its variable names, their classes, and legal
values, and then it creates a table summarizing that
information. The aim is to create a document that principal
investigators and research assistants can use to keep a project
well organized.  Please see the vignette in this package.
}
\details{
The variable key can be created in two formats, wide and long.
The original style of the variable key, wide, has one row per
variable. It has a style for compact notation about current values
and required recodes.  That is more compact, probably easier for
experts to read, but perhaps more difficult to edit. The long
style variable key has one row per value per variable.  Thus, in a
larger project, the long key can have many rows. However, in a
larger project, the long style key is easier to edit with a spread
sheet program.

After a key is created, it should be re-imported into R with the
\code{kutils::keyImport} function.  Then the key structure can
guide the importation and recoding of the data set.

Concerning the varlab attribute. Run \code{attr(key, "varlab"} to
review existing labels, if any.

Storing the variable labels in files requires some care because
the \code{rds}, \code{xlsx}, and \code{csv} formats have different
capabilities.  The \code{rds} storage format saves all attributes without
difficulty. In contrast, because \code{csv} and \code{xlsx} do not save
attributes, the varlabs are stored as separate character
matrices. For \code{xlsx} files, the varlab object is saved as a second
sheet in \code{xlsx} file, while in \code{csv} a second file suffixed
"-varlab.csv" is created.
}
\examples{
set.seed(234234)
N <- 200
mydf <- data.frame(x5 = rnorm(N),
                   x4 = rpois(N, lambda = 3),
                   x3 = ordered(sample(c("lo", "med", "hi"),
                   size = N, replace=TRUE),
                   levels = c("med", "lo", "hi")),
                   x2 = letters[sample(c(1:4,6), N, replace = TRUE)],
                   x1 = factor(sample(c("cindy", "bobby", "marcia",
                                        "greg", "peter"), N,
                   replace = TRUE)),
                   x7 = ordered(letters[sample(c(1:4,6), N, replace = TRUE)]),
                   x6 = sample(c(1:5), N, replace = TRUE),
                   stringsAsFactors = FALSE)
mydf$x4[sample(1:N, 10)] <- 999
mydf$x5[sample(1:N, 10)] <- -999

## Note: If we change this example data, we need to save a copy in
## "../inst/extdata" for packacing
dn <- tempdir()
write.csv(mydf, file = file.path(dn, "mydf.csv"), row.names = FALSE)
mydf.templ <- keyTemplate(mydf, file = file.path(dn, "mydf.templ.csv"),
                          varlab = TRUE)
mydf.templ_long <- keyTemplate(mydf, long = TRUE,
                            file = file.path(dn, "mydf.templlong.csv"),
                            varlab = TRUE)

mydf.templx <- keyTemplate(mydf, file = file.path(dn, "mydf.templ.xlsx"),
                            varlab = TRUE)
mydf.templ_longx <- keyTemplate(mydf, long = TRUE,
                             file = file.path(dn, "mydf.templ_long.xlsx"),
                             varlab = TRUE)
## Check the varlab attribute
attr(mydf.templ, "varlab")
mydf.tmpl2 <- keyTemplate(mydf,
                         varlab = c(x5 = "height", x4 = "age",
                         x3 = "intelligence", x1 = "Name"))
## Check the varlab attribute
attr(mydf.tmpl2, "varlab")

## Try with the national longitudinal study data
data(natlongsurv)
natlong.templ <- keyTemplate(natlongsurv,
                          file = file.path(dn, "natlongsurv.templ.csv"),
                          max.levels = 15, varlab = TRUE, sort = TRUE)

natlong.templlong <- keyTemplate(natlongsurv, long = TRUE,
                   file = file.path(dn, "natlongsurv.templ_long.csv"), sort = TRUE)
if(interactive()) View(natlong.templlong)
natlong.templlong2 <- keyTemplate(natlongsurv, long = TRUE,
                      missings = "<0", max.levels = 50, sort = TRUE,
                      varlab = TRUE)
if(interactive()) View(natlong.templlong2)

natlong.templwide2 <- keyTemplate(natlongsurv, long = FALSE,
                      missings = "<0", max.levels = 50, sort = TRUE)
if(interactive()) View(natlong.templwide2)

all.equal(wide2long(natlong.templwide2), natlong.templlong2)

head(keyTemplate(natlongsurv, file = file.path(dn, "natlongsurv.templ.xlsx"),
             max.levels = 15, varlab = TRUE, sort = TRUE), 10)
head(keyTemplate(natlongsurv, file = file.path(dn, "natlongsurv.templ.xlsx"),
             long = TRUE, max.levels = 15, varlab = TRUE, sort = TRUE), 10)

list.files(dn)

}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mergeCheck.R
\name{print.keycheck}
\alias{print.keycheck}
\title{Print out the result of mergeCheck function.}
\usage{
\method{print}{keycheck}(x, ...)
}
\arguments{
\item{x}{keycheck output from mergeCheck}

\item{...}{Other arguments}
}
\value{
None, side effect if print to screen
}
\description{
This is a placeholder for a more elaborate print method
to be prepared in the future. Please advise us what might
be most helpful.
}
\author{
Paul Johnson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{n2NA}
\alias{n2NA}
\title{Convert nothing to R missing(NA).}
\usage{
n2NA(x, nothings = c("\\\\.", "\\\\s"), zapspace = TRUE)
}
\arguments{
\item{x}{A character vector. If x is not a character vector, it is
returned unaltered without warning.}

\item{nothings}{A vector of values to be matched by regular
expressions as missing.  The default vector is c("\\.",
"\\s"), where "\\." means a literal period (backslashes needed
to escape the symbol which would otherwise match anything in a
regular expression).}

\item{zapspace}{Should leading and trailing white space be
ignored, so that, for example " . " and "." are both treated
as missing.}
}
\value{
A vector with "nothing" values replaced by R's NA symbol.
    Does not alter other values in the vector. Previous version
    had applied zapspace to non-missing values, but it no longer
    does so.
}
\description{
By "nothing", we mean white space or other indications of
nothingness.  Goal is to find character strings that users
might insert in a key to indicate missing values. Those things,
which are given default values in the argument nothings, will be
changed to NA.
}
\details{
Using regular expression matching, any value that has nothing
except for the indicated "nothing" values is converted to NA.  The
"nothing" values included by default are a period by itself (A SAS
missing value), an empty string, or white space, meaning " ", or
any number of spaces, or a tab.
}
\examples{
gg <- c("", " ", "   ", "\t", "\t some", "some\t", " space first", ".",
       " . ")
n2NA(x = gg)
n2NA(x = gg, zapspace = FALSE)
n2NA(x = gg, nothings = c("\\\\s"), zapspace = FALSE)
n2NA(x = gg, nothings = c("\\\\."), zapspace = TRUE)
n2NA(x = gg, nothings = c("\\\\."), zapspace = FALSE)
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{print.keyDiff}
\alias{print.keyDiff}
\title{Print a keyDiff object}
\usage{
\method{print}{keyDiff}(x, ...)
}
\arguments{
\item{x}{A keyDiff object}

\item{...}{Other arguments passed through to print}
}
\description{
Print a keyDiff object
}
\author{
Ben Kite <bakite@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keyDiff}
\alias{keyDiff}
\title{Show difference between 2 keys}
\usage{
keyDiff(oldkey, newkey)
}
\arguments{
\item{oldkey}{key, original}

\item{newkey}{key, possibly created by keyUpdate or by user edits}
}
\value{
NULL, or list with as many as 2 key difference data.frames,
 named "deleted" and "neworaltered"
}
\description{
Show difference between 2 keys
}
\examples{

dat1 <- data.frame("Score" = c(1, 2, 3, 42, 4, 2),
                   "Gender" = c("M", "M", "M", "F", "F", "F"))
## First try with a long key
key1 <- keyTemplate(dat1, long = TRUE)
key1$value_new <- gsub("42", "10", key1$value_new)
key1$value_new[key1$name_new == "Gender"] <-
       mgsub(c("F", "M"), c("female", "male"),
       key1$value_new[key1$name_new == "Gender"])  
key1[key1$name_old == "Score", "name_new"] <- "NewScore"
dat2 <- data.frame("Score" = 7, "Gender" = "other", "Weight" = rnorm(3))
dat2 <- plyr::rbind.fill(dat1, dat2)
dat2 <- dat2[-1,]
key2 <- keyUpdate(key1, dat2, append = TRUE)
(kdiff <- keyDiff(key1, key2))
}
\author{
Ben Kite <bakite@ku.edu> and Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keyTemplate}
\alias{keyTemplate}
\title{Create variable key template (in memory or in a file)}
\usage{
keyTemplate(
  dframe,
  long = FALSE,
  sort = FALSE,
  file = NULL,
  max.levels = 15,
  missings = NULL,
  missSymbol = ".",
  safeNumericToInteger = TRUE,
  trimws = "both",
  varlab = FALSE
)
}
\arguments{
\item{dframe}{A data frame}

\item{long}{Default FALSE.}

\item{sort}{Default FALSE. Should the rows representing the
variables be sorted alphabetically? Otherwise, they appear in
the order in which they were included in the original dataset.}

\item{file}{DEFAULT NULL, meaning no file is produced. Choose a
file name ending in either "csv" (for comma separated
variables), "xlsx" (compatible with Microsoft Excel), or "rds"
(R serialization data). The file name will be used to select
among the 3 storage formats. XLSX output requires the openxlsx
package.}

\item{max.levels}{How high is the limit on the number of values
for discrete (integer, character, and Date) variables?
Default = 15. If observed number exceeds max.levels, we
conclude the author should not assign new values in the key
and only the missing value will be included in the key as a
"placeholder". This does not affect variables declared as
factor or ordered variables, for which all levels are included
in all cases.}

\item{missings}{Values in exising data which should be treated as
missing in the new key. Character string in format acceptable
to the \code{assignMissing} function. Can be a string with
several missing indicators"1;2;3;(8,10);[22,24];> 99;< 2".}

\item{missSymbol}{Default ".".  A character string used to
represent missing values in the key that is created.  Relevant
(mostly) for the key's \code{value_new} column. Default is the
period, ".". Because R's symbol \code{NA} can be mistaken for
the character string \code{"NA"}, we use a different
(hopefully unmistakable) symbol in the key.}

\item{safeNumericToInteger}{Default TRUE: Should we treat values
which appear to be integers as integers? If a column is
numeric, it might be safe to treat it as an integer.  In many
csv data sets, the values coded c(1, 2, 3) are really
integers, not floats c(1.0, 2.0, 3.0). See \code{safeInteger}.}

\item{trimws}{Default is "both", user can change to "left", "right", or
set as NULL to avoid any trimming.}

\item{varlab}{A key can have a companion data structure for
variable labels. Default is FALSE, but the value may also be
TRUE or a named vector of variable labels, such as
\code{c("x1" = "happiness", "x2" = "wealth")}. The labels
become an attribute of the key object. See Details for
information on storage of varlabs in saved key files.}
}
\value{
A key in the form of a data frame. May also be saved on
    disk if the file argument is supplied. The key may have an
    attribute "varlab", variable labels.
}
\description{
A variable key is a human readable document that describes the
variables in a data set. A key can be revised and re-imported by R
to recode data. This might also be referred to as a
"programmable codebook."  This function inspects a data frame,
takes notice of its variable names, their classes, and legal
values, and then it creates a table summarizing that
information. The aim is to create a document that principal
investigators and research assistants can use to keep a project
well organized.  Please see the vignette in this package.
}
\details{
The variable key can be created in two formats, wide and long.
The original style of the variable key, wide, has one row per
variable. It has a style for compact notation about current values
and required recodes.  That is more compact, probably easier for
experts to read, but perhaps more difficult to edit. The long
style variable key has one row per value per variable.  Thus, in a
larger project, the long key can have many rows. However, in a
larger project, the long style key is easier to edit with a spread
sheet program.

After a key is created, it should be re-imported into R with the
\code{kutils::keyImport} function.  Then the key structure can
guide the importation and recoding of the data set.

Concerning the varlab attribute. Run \code{attr(key, "varlab"} to
review existing labels, if any.

Storing the variable labels in files requires some care because
the \code{rds}, \code{xlsx}, and \code{csv} formats have different
capabilities.  The \code{rds} storage format saves all attributes without
difficulty. In contrast, because \code{csv} and \code{xlsx} do not save
attributes, the varlabs are stored as separate character
matrices. For \code{xlsx} files, the varlab object is saved as a second
sheet in \code{xlsx} file, while in \code{csv} a second file suffixed
"-varlab.csv" is created.
}
\examples{
set.seed(234234)
N <- 200
mydf <- data.frame(x5 = rnorm(N),
                   x4 = rpois(N, lambda = 3),
                   x3 = ordered(sample(c("lo", "med", "hi"),
                   size = N, replace=TRUE),
                   levels = c("med", "lo", "hi")),
                   x2 = letters[sample(c(1:4,6), N, replace = TRUE)],
                   x1 = factor(sample(c("cindy", "bobby", "marcia",
                                        "greg", "peter"), N,
                   replace = TRUE)),
                   x7 = ordered(letters[sample(c(1:4,6), N, replace = TRUE)]),
                   x6 = sample(c(1:5), N, replace = TRUE),
                   stringsAsFactors = FALSE)
mydf$x4[sample(1:N, 10)] <- 999
mydf$x5[sample(1:N, 10)] <- -999

## Note: If we change this example data, we need to save a copy in
## "../inst/extdata" for packacing
dn <- tempdir()
write.csv(mydf, file = file.path(dn, "mydf.csv"), row.names = FALSE)
mydf.templ <- keyTemplate(mydf, file = file.path(dn, "mydf.templ.csv"),
                          varlab = TRUE)
mydf.templ_long <- keyTemplate(mydf, long = TRUE,
                            file = file.path(dn, "mydf.templlong.csv"),
                            varlab = TRUE)

mydf.templx <- keyTemplate(mydf, file = file.path(dn, "mydf.templ.xlsx"),
                            varlab = TRUE)
mydf.templ_longx <- keyTemplate(mydf, long = TRUE,
                             file = file.path(dn, "mydf.templ_long.xlsx"),
                             varlab = TRUE)
## Check the varlab attribute
attr(mydf.templ, "varlab")
mydf.tmpl2 <- keyTemplate(mydf,
                         varlab = c(x5 = "height", x4 = "age",
                         x3 = "intelligence", x1 = "Name"))
## Check the varlab attribute
attr(mydf.tmpl2, "varlab")

## Try with the national longitudinal study data
data(natlongsurv)
natlong.templ <- keyTemplate(natlongsurv,
                          file = file.path(dn, "natlongsurv.templ.csv"),
                          max.levels = 15, varlab = TRUE, sort = TRUE)

natlong.templlong <- keyTemplate(natlongsurv, long = TRUE,
                   file = file.path(dn, "natlongsurv.templ_long.csv"), sort = TRUE)
if(interactive()) View(natlong.templlong)
natlong.templlong2 <- keyTemplate(natlongsurv, long = TRUE,
                      missings = "<0", max.levels = 50, sort = TRUE,
                      varlab = TRUE)
if(interactive()) View(natlong.templlong2)

natlong.templwide2 <- keyTemplate(natlongsurv, long = FALSE,
                      missings = "<0", max.levels = 50, sort = TRUE)
if(interactive()) View(natlong.templwide2)

all.equal(wide2long(natlong.templwide2), natlong.templlong2)

head(keyTemplate(natlongsurv, file = file.path(dn, "natlongsurv.templ.xlsx"),
             max.levels = 15, varlab = TRUE, sort = TRUE), 10)
head(keyTemplate(natlongsurv, file = file.path(dn, "natlongsurv.templ.xlsx"),
             long = TRUE, max.levels = 15, varlab = TRUE, sort = TRUE), 10)

list.files(dn)

}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/importQualtrics.R
\name{importQualtrics}
\alias{importQualtrics}
\title{Import Qualtrics survey files, apply clean column names}
\usage{
importQualtrics(
  file,
  namerow = 1,
  questionrow = 2,
  importidrow = 3,
  skip = 3,
  dropTEXT = TRUE,
  stringsAsFactors = FALSE
)
}
\arguments{
\item{file}{file name (including path if in another directory) of
a CSV or XLSX file from Qualtrics.}

\item{namerow}{Row number for variable names.  Default 1, the
information to be used as column names (same as HEADER row in
R's read.table function)}

\item{questionrow}{Row number to be treated as the questions in the
survey. Default is 2. If questions do not seem to be present
in this row, there will be a warning.}

\item{importidrow}{Row number to be treated as Qualtrics meta
data. Default is 3. Many CSV created by Qualtrics will have
row 3 with a character string such as
\code{"{""ImportId"":""QID1303_4""}"}.  If importids are not
present in this row, there will be a warning.}

\item{skip}{Number of rows that are meta data. Current Qualtrics
CSV files will usually have 3 metadata rows, 1 = name, 2 =
question, 3 = ImportId. This function will try to guess how
many rows of metadata are present. skip should be at least
as large as \code{max(namerow, questions, and importids)}}

\item{dropTEXT}{Default TRUE, columns ending in "_TEXT" are
omitted.}

\item{stringsAsFactors}{Default FALSE, same meaning as R's
read.csv. Does not affect importation of Excel files.}
}
\value{
Data frame that has attribute "meta"
}
\description{
Defaults are based on most common format received from Qualtrics
downloads to CSV or XLSX (MS Excel) formats.  We assume that the
file has the column names in row 1 and that 3 rows are skipped
before the real data begins. If the parameter questrow is used, it
designates a row that is interpreted as the survey questions
themselves.  Often, this is in row 2.
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keyDiagnostic}
\alias{keyDiagnostic}
\title{Diagnose accuracy of result from applying variable key to data}
\usage{
keyDiagnostic(dfold, dfnew, keylist, max.values = 20, nametrunc = 18,
  wide = 200, confidential = FALSE)
}
\arguments{
\item{dfold}{Original data frame}

\item{dfnew}{The new recoded data frame}

\item{keylist}{The imported variable key that was used to
transform dfold into dfnew.}

\item{max.values}{Show up to this number of values for the old
variable}

\item{nametrunc}{Truncate column and row names. Needed if there
are long factor labels and we want to fit more information on
table. Default = 18 for new name, old name is 10 more
characters (18 + 10 = 28).}

\item{wide}{Number of characters per row in printed
output. Suggest very wide screen, default = 200.}

\item{confidential}{Should numbers in table be rounded to nearest
"10" to comply with security standard enforced by some
American research departments.}
}
\description{
Compare the old and new data frames, checking for accuracy of
calculations in various ways.
}
\details{
CAUTION: This can print WIDE matrices.  Because the on-screen
output will be WIDE, make the display window WIDE!

Crosstabulate new variable versus old variable to see the coding
mismatches.  For tables of up to 10 values or so, that will be
satisfactory.

For numeric variables, it appears there is no good thing to do
except possibly to re-apply any transformations.
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keyCheck}
\alias{keyCheck}
\title{Check a key for consistency of names, values with classes.}
\usage{
keyCheck(key, colname = c("name_new", "class_old", "class_new"),
  na.strings = c("\\\\.", "", "\\\\s+", "N/A"))
}
\arguments{
\item{key}{A variable key object.}

\item{colname}{Leave as default to check consistency between classes, values, and names.
One can specify a check only on "class_old" or "class_new", for example.  But now that
all work correctly, I suggest you leave the default.}

\item{na.strings}{A regular expression of allowed text strings that represent missings.
Now it amounts to any of these: ".", "NA", "N/A", or any white space or tab as signified by \\s+.}
}
\value{
Profuse warnings and a list of failed key blocks.
}
\description{
Split the key into blocks of rows defined by "name_new". Within
these blocks, Perform these checks: 1. name_old must be
homogeneous (identical) within a block of rows. class_old and
class_new must also be identical.
2. elements in "value_new" must be consistent with "class_new".
If values cannot be coerced to match the class specified by
class_new, there must be user error.
Same for "value_old" and "class_old".
}
\author{
Paul Johnson <pauljohn@ku.edu> and Ben Kite <bakite@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/truncsmart.R
\name{truncsmart}
\alias{truncsmart}
\title{Cuts a string at a specified linewidth, trying to align cut with a
separator}
\usage{
truncsmart(
  x,
  target = 20,
  tol = c(5, 3),
  separators = c(" ", "_", ";", ","),
  capwidth = 1
)
}
\arguments{
\item{x}{character or vector of characters}

\item{target}{Goal for result length, in characters}

\item{tol}{number of characters forward/back to check; if single
value then only backwards checking}

\item{separators}{characters at which truncation is preferred,
such as space or underscore.}

\item{capwidth}{penalty for capital characters}
}
\value{
shorter string truncated at acceptable separators when
    found
}
\description{
Some strings are simply too long. We don't want to chop them
exactly at, say, 40 characters, if we could allow 42 and chop on a
space or other separator. We'd rather chop at 37 if there is a
separator, rather than terminate a word exactly at 40. This function
shortens them and attempt to cut at a separator, allowing for a
user specified fudge-factor (the tol parameter).
}
\details{
The default capwidth value is 1, so the calculations treat all
letters equally. In practice, we notice trouble when some strings
are written in ALL CAPS and they are longer than the same
information in lower case letters.  We have decided to allow a
user-specified penalty for capital letters. If each capital
counts for, say 1.2 ordinary letters, then we may end up truncating
the string on an earlier separator.

There's some approximation here. The capital-penalized widths are
calculated for all characters and then we left-shift the target
value so that it is equal to the last penalized value that is
under the target length. Then the "look to the left" and "look
to the right" logic begins.  That looking logic ignores the
capital letter penalty, it is treating all letters the same.
}
\examples{
x <- "Aasdf asdIasdf fW_asd asd aasjdf_as fasdasdfasdfasd"
truncsmart(x, target = 10, tol = c(5, 2))
truncsmart(x, target = 10, tol = c(1, 4))
truncsmart(x, target = 10, tol = c(5, 2), capwidth = 1.2)
truncsmart(x, target = 20, tol = c(5, 2))
truncsmart(x, target = 20, tol = c(10,10), capwidth = 2)
truncsmart(x, target = 20, tol = c(10,10), capwidth = 3)
truncsmart(x, target = 20, tol = c(10,10), capwidth = 4)
truncsmart(x, target = 20, tol = c(10,10), capwidth = 6)
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keyRead}
\alias{keyRead}
\title{Read file after deducing file type from suffix.}
\usage{
keyRead(file, ..., na.strings = c("\\\\s+"))
}
\arguments{
\item{file}{name of file to be imported, including path to
file. file name must end in "csv", "xlsx" or "rds"}

\item{...}{additional arguments for read.csv or read.xlsx.}

\item{na.strings}{Values to be converted to R missing symbol
NA. Default is white space, "\\s+".}
}
\value{
A data frame or matrix.
}
\description{
If the input is XLSX, sheets named "key" and "varlab" are
imported if the exist. If input is CSV, then the key
CSV file is imported and another file suffixed with "-varlab" is
imported if it exists.
}
\details{
The variable lables are a named vector saved as an attribute of
the key object.
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keyDiagnostic}
\alias{keyDiagnostic}
\title{Diagnose accuracy of result from applying variable key to data}
\usage{
keyDiagnostic(
  dfold,
  dfnew,
  keylist,
  max.values = 20,
  nametrunc = 18,
  wide = 200,
  confidential = FALSE
)
}
\arguments{
\item{dfold}{Original data frame}

\item{dfnew}{The new recoded data frame}

\item{keylist}{The imported variable key that was used to
transform dfold into dfnew.}

\item{max.values}{Show up to this number of values for the old
variable}

\item{nametrunc}{Truncate column and row names. Needed if there
are long factor labels and we want to fit more information on
table. Default = 18 for new name, old name is 10 more
characters (18 + 10 = 28).}

\item{wide}{Number of characters per row in printed
output. Suggest very wide screen, default = 200.}

\item{confidential}{Should numbers in table be rounded to nearest
"10" to comply with security standard enforced by some
American research departments.}
}
\description{
Compare the old and new data frames, checking for accuracy of
calculations in various ways.
}
\details{
CAUTION: This can print WIDE matrices.  Because the on-screen
output will be WIDE, make the display window WIDE!

Crosstabulate new variable versus old variable to see the coding
mismatches.  For tables of up to 10 values or so, that will be
satisfactory.

For numeric variables, it appears there is no good thing to do
except possibly to re-apply any transformations.
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/semTable.R
\name{testtable}
\alias{testtable}
\title{Test viewer for tex tables}
\usage{
testtable(tablefile, dir, tmpfn = "tmp.tex")
}
\arguments{
\item{tablefile}{The base name of the table file}

\item{dir}{Directory where table is saved, same will be used for build.}

\item{tmpfn}{File name to be used by example document}
}
\value{
LaTeX log, returned from shell function.
}
\description{
Creates a small latex template file that includes a table
file. Compiles it, then displays in viewer if system has
\code{xdg-open} settings.
}
\examples{
\donttest{
require(lavaan)
tempdir <- tempdir()
HS.model <- ' visual  =~ x1 + x2 + x3
              textual =~ x4 + x5 + x6
              speed   =~ x7 + x8 + x9'
fit1 <- cfa(HS.model, data = HolzingerSwineford1939,
            std.lv = TRUE, meanstructure = TRUE)
fit1.t <- semTable(fit1, fits = c("chisq", "rmsea"),
               columns = c("estsestars", "rsquare"),
               columnLabels = c("estsestars" = "Est(SE)"),
               file = file.path(tempdir, "fit1.t"))
if (interactive()) testtable("fit1.t", tempdir)
}
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/variableKey.R
\name{keyCheck}
\alias{keyCheck}
\title{Check a key for consistency of names, values with classes.}
\usage{
keyCheck(
  key,
  colname = c("name_new", "class_old", "class_new"),
  na.strings = c("\\\\.", "", "\\\\s+", "N/A")
)
}
\arguments{
\item{key}{A variable key object.}

\item{colname}{Leave as default to check consistency between classes, values, and names.
One can specify a check only on "class_old" or "class_new", for example.  But now that
all work correctly, I suggest you leave the default.}

\item{na.strings}{A regular expression of allowed text strings that represent missings.
Now it amounts to any of these: ".", "NA", "N/A", or any white space or tab as signified by \\s+.}
}
\value{
Profuse warnings and a list of failed key blocks.
}
\description{
Split the key into blocks of rows defined by "name_new". Within
these blocks, Perform these checks: 1. name_old must be
homogeneous (identical) within a block of rows. class_old and
class_new must also be identical.
2. elements in "value_new" must be consistent with "class_new".
If values cannot be coerced to match the class specified by
class_new, there must be user error.
Same for "value_old" and "class_old".
}
\author{
Paul Johnson <pauljohn@ku.edu> and Ben Kite <bakite@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelcomparison.R
\name{detectNested}
\alias{detectNested}
\title{Discern nesting pattern of SEM coefficients}
\usage{
detectNested(models)
}
\arguments{
\item{models}{A List of lavaan-fitted SEM models}
}
\value{
matrix indicating nesting relationships
}
\description{
Receives a list of models and orders them by best guess at intended nesting
}
\author{
Ben Kite <bakite@ku.edu>
}
